@@ -56,6 +56,7 @@ typedef struct
 	wListDictionary* files;
 
 	HANDLE thread;
+	BOOL async;
 	wMessageQueue* IrpQueue;
 
 	DEVMAN* devman;
@@ -63,9 +64,9 @@ typedef struct
 	rdpContext* rdpcontext;
 } DRIVE_DEVICE;
 
-static DWORD drive_map_windows_err(DWORD fs_errno)
+static NTSTATUS drive_map_windows_err(DWORD fs_errno)
 {
-	DWORD rc = 0;
+	NTSTATUS rc = 0;
 
 	/* try to return NTSTATUS version of error code */
 
@@ -470,7 +471,10 @@ static UINT drive_process_irp_query_volume_information(DRIVE_DEVICE* drive, IRP*
 			const size_t volumeLabelLen = (_wcslen(volumeLabel) + 1) * sizeof(WCHAR);
 			const size_t length = 17ul + volumeLabelLen;
 
-			Stream_Write_UINT32(output, length); /* Length */
+			if ((length > UINT32_MAX) || (volumeLabelLen > UINT32_MAX))
+				return CHANNEL_RC_NO_BUFFER;
+
+			Stream_Write_UINT32(output, (UINT32)length); /* Length */
 
 			if (!Stream_EnsureRemainingCapacity(output, length))
 			{
@@ -483,7 +487,7 @@ static UINT drive_process_irp_query_volume_information(DRIVE_DEVICE* drive, IRP*
 			Stream_Write_UINT32(output,
 			                    wfad.ftCreationTime.dwHighDateTime);      /* VolumeCreationTime */
 			Stream_Write_UINT32(output, lpNumberOfFreeClusters & 0xffff); /* VolumeSerialNumber */
-			Stream_Write_UINT32(output, volumeLabelLen);                  /* VolumeLabelLength */
+			Stream_Write_UINT32(output, (UINT32)volumeLabelLen);          /* VolumeLabelLength */
 			Stream_Write_UINT8(output, 0);                                /* SupportsObjects */
 			/* Reserved(1), MUST NOT be added! */
 			Stream_Write(output, volumeLabel, volumeLabelLen); /* VolumeLabel (Unicode) */
@@ -511,9 +515,13 @@ static UINT drive_process_irp_query_volume_information(DRIVE_DEVICE* drive, IRP*
 			/* http://msdn.microsoft.com/en-us/library/cc232101.aspx */
 			const WCHAR* diskType =
 			    InitializeConstWCharFromUtf8("FAT32", LabelBuffer, ARRAYSIZE(LabelBuffer));
-			const size_t diskTypeLen = (wcslen(diskType) + 1) * sizeof(WCHAR);
+			const size_t diskTypeLen = (_wcslen(diskType) + 1) * sizeof(WCHAR);
 			const size_t length = 12ul + diskTypeLen;
-			Stream_Write_UINT32(output, length); /* Length */
+
+			if ((length > UINT32_MAX) || (diskTypeLen > UINT32_MAX))
+				return CHANNEL_RC_NO_BUFFER;
+
+			Stream_Write_UINT32(output, (UINT32)length); /* Length */
 
 			if (!Stream_EnsureRemainingCapacity(output, length))
 			{
@@ -524,7 +532,7 @@ static UINT drive_process_irp_query_volume_information(DRIVE_DEVICE* drive, IRP*
 			Stream_Write_UINT32(output, FILE_CASE_SENSITIVE_SEARCH | FILE_CASE_PRESERVED_NAMES |
 			                                FILE_UNICODE_ON_DISK); /* FileSystemAttributes */
 			Stream_Write_UINT32(output, MAX_PATH);                 /* MaximumComponentNameLength */
-			Stream_Write_UINT32(output, diskTypeLen);              /* FileSystemNameLength */
+			Stream_Write_UINT32(output, (UINT32)diskTypeLen);      /* FileSystemNameLength */
 			Stream_Write(output, diskType, diskTypeLen);           /* FileSystemName (Unicode) */
 		}
 		break;
@@ -741,10 +749,25 @@ static UINT drive_process_irp(DRIVE_DEVICE* drive, IRP* irp)
 	return error;
 }
 
+static BOOL drive_poll_run(DRIVE_DEVICE* drive, IRP* irp)
+{
+	WINPR_ASSERT(drive);
+
+	if (irp)
+	{
+		const UINT error = drive_process_irp(drive, irp);
+		if (error)
+		{
+			WLog_ERR(TAG, "drive_process_irp failed with error %" PRIu32 "!", error);
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
 static DWORD WINAPI drive_thread_func(LPVOID arg)
 {
-	IRP* irp = NULL;
-	wMessage message = { 0 };
 	DRIVE_DEVICE* drive = (DRIVE_DEVICE*)arg;
 	UINT error = CHANNEL_RC_OK;
 
@@ -763,26 +786,22 @@ static DWORD WINAPI drive_thread_func(LPVOID arg)
 			break;
 		}
 
+		if (MessageQueue_Size(drive->IrpQueue) < 1)
+			continue;
+
+		wMessage message = { 0 };
 		if (!MessageQueue_Peek(drive->IrpQueue, &message, TRUE))
 		{
 			WLog_ERR(TAG, "MessageQueue_Peek failed!");
-			error = ERROR_INTERNAL_ERROR;
-			break;
+			continue;
 		}
 
 		if (message.id == WMQ_QUIT)
 			break;
 
-		irp = (IRP*)message.wParam;
-
-		if (irp)
-		{
-			if ((error = drive_process_irp(drive, irp)))
-			{
-				WLog_ERR(TAG, "drive_process_irp failed with error %" PRIu32 "!", error);
-				break;
-			}
-		}
+		IRP* irp = (IRP*)message.wParam;
+		if (!drive_poll_run(drive, irp))
+			break;
 	}
 
 fail:
@@ -806,10 +825,18 @@ static UINT drive_irp_request(DEVICE* device, IRP* irp)
 	if (!drive)
 		return ERROR_INVALID_PARAMETER;
 
-	if (!MessageQueue_Post(drive->IrpQueue, NULL, 0, (void*)irp, NULL))
+	if (drive->async)
 	{
-		WLog_ERR(TAG, "MessageQueue_Post failed!");
-		return ERROR_INTERNAL_ERROR;
+		if (!MessageQueue_Post(drive->IrpQueue, NULL, 0, (void*)irp, NULL))
+		{
+			WLog_ERR(TAG, "MessageQueue_Post failed!");
+			return ERROR_INTERNAL_ERROR;
+		}
+	}
+	else
+	{
+		if (!drive_poll_run(drive, irp))
+			return ERROR_INTERNAL_ERROR;
 	}
 
 	return CHANNEL_RC_OK;
@@ -822,7 +849,7 @@ static UINT drive_free_int(DRIVE_DEVICE* drive)
 	if (!drive)
 		return ERROR_INVALID_PARAMETER;
 
-	CloseHandle(drive->thread);
+	(void)CloseHandle(drive->thread);
 	ListDictionary_Free(drive->files);
 	MessageQueue_Free(drive->IrpQueue);
 	Stream_Free(drive->device.data, TRUE);
@@ -945,7 +972,7 @@ static UINT drive_register_drive_path(PDEVICE_SERVICE_ENTRY_POINTS pEntryPoints,
 		}
 		Stream_Write_UINT8(drive->device.data, '\0');
 
-		drive->device.name = (const char*)Stream_Buffer(drive->device.data);
+		drive->device.name = Stream_BufferAs(drive->device.data, char);
 		if (!drive->device.name)
 			goto out_error;
 
@@ -988,14 +1015,19 @@ static UINT drive_register_drive_path(PDEVICE_SERVICE_ENTRY_POINTS pEntryPoints,
 			goto out_error;
 		}
 
-		if (!(drive->thread =
-		          CreateThread(NULL, 0, drive_thread_func, drive, CREATE_SUSPENDED, NULL)))
+		drive->async = !freerdp_settings_get_bool(drive->rdpcontext->settings,
+		                                          FreeRDP_SynchronousStaticChannels);
+		if (drive->async)
 		{
-			WLog_ERR(TAG, "CreateThread failed!");
-			goto out_error;
-		}
+			if (!(drive->thread =
+			          CreateThread(NULL, 0, drive_thread_func, drive, CREATE_SUSPENDED, NULL)))
+			{
+				WLog_ERR(TAG, "CreateThread failed!");
+				goto out_error;
+			}
 
-		ResumeThread(drive->thread);
+			ResumeThread(drive->thread);
+		}
 	}
 
 	return CHANNEL_RC_OK;
@@ -1009,7 +1041,8 @@ out_error:
  *
  * @return 0 on success, otherwise a Win32 error code
  */
-FREERDP_ENTRY_POINT(UINT drive_DeviceServiceEntry(PDEVICE_SERVICE_ENTRY_POINTS pEntryPoints))
+FREERDP_ENTRY_POINT(
+    UINT VCAPITYPE drive_DeviceServiceEntry(PDEVICE_SERVICE_ENTRY_POINTS pEntryPoints))
 {
 	RDPDR_DRIVE* drive = NULL;
 	UINT error = 0;
