@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdint.h>
 
 #include <winpr/crt.h>
 #include <winpr/sysinfo.h>
@@ -514,19 +515,19 @@ typedef struct
  */
 static UINT handle_hotplug(rdpdrPlugin* rdpdr)
 {
-	struct dirent* pDirent;
-	DIR* pDir;
-	char fullpath[PATH_MAX];
+	struct dirent* pDirent = NULL;
+	char fullpath[PATH_MAX] = { 0 };
 	char* szdir = (char*)"/Volumes";
-	struct stat buf;
-	hotplug_dev dev_array[MAX_USB_DEVICES];
-	int count;
-	DEVICE_DRIVE_EXT* device_ext;
+	struct stat buf = { 0 };
+	hotplug_dev dev_array[MAX_USB_DEVICES] = { 0 };
+	int count = 0;
+	DEVICE_DRIVE_EXT* device_ext = NULL;
 	ULONG_PTR* keys = NULL;
 	int size = 0;
-	UINT error;
+	UINT error = ERROR_INTERNAL_ERROR;
 	UINT32 ids[1];
-	pDir = opendir(szdir);
+
+	DIR* pDir = opendir(szdir);
 
 	if (pDir == NULL)
 	{
@@ -538,7 +539,7 @@ static UINT handle_hotplug(rdpdrPlugin* rdpdr)
 	{
 		if (pDirent->d_name[0] != '.')
 		{
-			sprintf_s(fullpath, ARRAYSIZE(fullpath), "%s/%s", szdir, pDirent->d_name);
+			(void)sprintf_s(fullpath, ARRAYSIZE(fullpath), "%s/%s", szdir, pDirent->d_name);
 			if (stat(fullpath, &buf) != 0)
 				continue;
 
@@ -582,7 +583,7 @@ static UINT handle_hotplug(rdpdrPlugin* rdpdr)
 		if (!path)
 			continue;
 
-		/* not plugable device */
+		/* not pluggable device */
 		if (strstr(path, "/Volumes/") == NULL)
 		{
 			free(path);
@@ -706,7 +707,7 @@ static DWORD WINAPI drive_hotplug_thread_func(LPVOID arg)
 out:
 	if (rdpdr->stopEvent)
 	{
-		CloseHandle(rdpdr->stopEvent);
+		(void)CloseHandle(rdpdr->stopEvent);
 		rdpdr->stopEvent = NULL;
 	}
 	ExitThread(CHANNEL_RC_OK);
@@ -737,12 +738,15 @@ static BOOL isAutomountLocation(const char* path)
 		const char* location = automountLocations[x];
 		size_t length = 0;
 
+		WINPR_PRAGMA_DIAG_PUSH
+		WINPR_PRAGMA_DIAG_IGNORED_FORMAT_NONLITERAL
 		if (strstr(location, "%lu"))
 			(void)snprintf(buffer, sizeof(buffer), location, (unsigned long)uid);
 		else if (strstr(location, "%s"))
 			(void)snprintf(buffer, sizeof(buffer), location, uname);
 		else
 			(void)snprintf(buffer, sizeof(buffer), "%s", location);
+		WINPR_PRAGMA_DIAG_POP
 
 		length = strnlen(buffer, sizeof(buffer));
 
@@ -935,6 +939,7 @@ static BOOL hotplug_delete_foreach(ULONG_PTR key, void* element, void* data)
 	WINPR_ASSERT(arg);
 	WINPR_ASSERT(arg->rdpdr);
 	WINPR_ASSERT(arg->dev_array || (arg->dev_array_size == 0));
+	WINPR_ASSERT(key <= UINT32_MAX);
 
 	if (!device_ext || (device_ext->device.type != RDPDR_DTYP_FILESYSTEM) || !device_ext->path ||
 	    !device_ext->automount)
@@ -945,7 +950,7 @@ static BOOL hotplug_delete_foreach(ULONG_PTR key, void* element, void* data)
 	if (!path)
 		return FALSE;
 
-	/* not plugable device */
+	/* not pluggable device */
 	if (isAutomountLocation(path))
 	{
 		for (size_t i = 0; i < arg->dev_array_size; i++)
@@ -965,7 +970,7 @@ static BOOL hotplug_delete_foreach(ULONG_PTR key, void* element, void* data)
 	if (!dev_found)
 	{
 		UINT error = 0;
-		UINT32 ids[1] = { key };
+		UINT32 ids[1] = { (UINT32)key };
 
 		WINPR_ASSERT(arg->rdpdr->devman);
 		devman_unregister_device(arg->rdpdr->devman, (void*)key);
@@ -1080,7 +1085,7 @@ out:
 
 	if (rdpdr->stopEvent)
 	{
-		CloseHandle(rdpdr->stopEvent);
+		(void)CloseHandle(rdpdr->stopEvent);
 		rdpdr->stopEvent = NULL;
 	}
 
@@ -1106,7 +1111,7 @@ static UINT drive_hotplug_thread_terminate(rdpdrPlugin* rdpdr)
 	{
 #if !defined(_WIN32)
 		if (rdpdr->stopEvent)
-			SetEvent(rdpdr->stopEvent);
+			(void)SetEvent(rdpdr->stopEvent);
 #endif
 #ifdef __MACOSX__
 		CFRunLoopStop(rdpdr->runLoop);
@@ -1120,7 +1125,7 @@ static UINT drive_hotplug_thread_terminate(rdpdrPlugin* rdpdr)
 			return error;
 		}
 
-		CloseHandle(rdpdr->hotplugThread);
+		(void)CloseHandle(rdpdr->hotplugThread);
 		rdpdr->hotplugThread = NULL;
 	}
 
@@ -1158,7 +1163,14 @@ static UINT rdpdr_process_connect(rdpdrPlugin* rdpdr)
 	const char* name = freerdp_settings_get_string(settings, FreeRDP_ClientHostname);
 	if (!name)
 		name = freerdp_settings_get_string(settings, FreeRDP_ComputerName);
-	strncpy(rdpdr->computerName, name, sizeof(rdpdr->computerName) - 1);
+	if (!name)
+	{
+		DWORD size = ARRAYSIZE(rdpdr->computerName);
+		if (!GetComputerNameExA(ComputerNameNetBIOS, rdpdr->computerName, &size))
+			return ERROR_INTERNAL_ERROR;
+	}
+	else
+		strncpy(rdpdr->computerName, name, strnlen(name, sizeof(rdpdr->computerName)));
 
 	for (UINT32 index = 0; index < freerdp_settings_get_uint32(settings, FreeRDP_DeviceCount);
 	     index++)
@@ -1173,7 +1185,8 @@ static UINT rdpdr_process_connect(rdpdrPlugin* rdpdr)
 			if (!drive->Path)
 				continue;
 
-			BOOL hotplugAll = strncmp(drive->Path, "*", 2) == 0;
+			const char wildcard[] = "*";
+			BOOL hotplugAll = strncmp(drive->Path, wildcard, sizeof(wildcard)) == 0;
 			BOOL hotplugLater = strncmp(drive->Path, DynamicDrives, sizeof(DynamicDrives)) == 0;
 
 			if (hotplugAll || hotplugLater)
@@ -1284,14 +1297,12 @@ static UINT rdpdr_send_client_name_request(rdpdrPlugin* rdpdr)
 	WINPR_ASSERT(rdpdr->state == RDPDR_CHANNEL_STATE_ANNOUNCE_REPLY);
 	rdpdr_state_advance(rdpdr, RDPDR_CHANNEL_STATE_NAME_REQUEST);
 
-	if (!rdpdr->computerName[0])
-	{
-		DWORD size = sizeof(rdpdr->computerName) - 1;
-		GetComputerNameA(rdpdr->computerName, &size);
-	}
+	const size_t len = strnlen(rdpdr->computerName, sizeof(rdpdr->computerName));
+	if (len == 0)
+		return ERROR_INTERNAL_ERROR;
 
 	WINPR_ASSERT(rdpdr->computerName);
-	computerNameW = ConvertUtf8ToWCharAlloc(rdpdr->computerName, &computerNameLenW);
+	computerNameW = ConvertUtf8NToWCharAlloc(rdpdr->computerName, len, &computerNameLenW);
 	computerNameLenW *= sizeof(WCHAR);
 
 	if (computerNameLenW > 0)
@@ -1505,8 +1516,7 @@ static UINT dummy_irp_response(rdpdrPlugin* rdpdr, wStream* s)
 	Stream_Read_UINT32(s, FileId);       /* FileId (4 bytes) */
 	Stream_Read_UINT32(s, CompletionId); /* CompletionId (4 bytes) */
 
-	if (!rdpdr_write_iocompletion_header(output, DeviceId, CompletionId,
-	                                     (UINT32)STATUS_UNSUCCESSFUL))
+	if (!rdpdr_write_iocompletion_header(output, DeviceId, CompletionId, STATUS_UNSUCCESSFUL))
 		return CHANNEL_RC_NO_MEMORY;
 
 	return rdpdr_send(rdpdr, output);
@@ -1642,7 +1652,7 @@ static const char* state_str(size_t count, va_list ap, char* buffer, size_t size
 static BOOL rdpdr_state_check(rdpdrPlugin* rdpdr, UINT16 packetid, enum RDPDR_CHANNEL_STATE next,
                               size_t count, ...)
 {
-	va_list ap;
+	va_list ap = { 0 };
 	WINPR_ASSERT(rdpdr);
 
 	va_start(ap, count);
@@ -1874,7 +1884,6 @@ static UINT rdpdr_process_receive(rdpdrPlugin* rdpdr, wStream* s)
  */
 UINT rdpdr_send(rdpdrPlugin* rdpdr, wStream* s)
 {
-	UINT status = 0;
 	rdpdrPlugin* plugin = rdpdr;
 
 	if (!s)
@@ -1890,9 +1899,13 @@ UINT rdpdr_send(rdpdrPlugin* rdpdr, wStream* s)
 	}
 
 	const size_t pos = Stream_GetPosition(s);
-	rdpdr_dump_send_packet(rdpdr->log, WLOG_TRACE, s, "[rdpdr-channel] send");
-	status = plugin->channelEntryPoints.pVirtualChannelWriteEx(
-	    plugin->InitHandle, plugin->OpenHandle, Stream_Buffer(s), pos, s);
+	UINT status = ERROR_INTERNAL_ERROR;
+	if (pos <= UINT32_MAX)
+	{
+		rdpdr_dump_send_packet(rdpdr->log, WLOG_TRACE, s, "[rdpdr-channel] send");
+		status = plugin->channelEntryPoints.pVirtualChannelWriteEx(
+		    plugin->InitHandle, plugin->OpenHandle, Stream_Buffer(s), (UINT32)pos, s);
+	}
 
 	if (status != CHANNEL_RC_OK)
 	{
@@ -2025,6 +2038,8 @@ static VOID VCAPITYPE rdpdr_virtual_channel_open_event_ex(LPVOID lpUserParam, DW
 
 		case CHANNEL_EVENT_USER:
 			break;
+		default:
+			break;
 	}
 
 	if (error && rdpdr && rdpdr->rdpcontext)
@@ -2190,7 +2205,7 @@ static UINT rdpdr_virtual_channel_event_disconnected(rdpdrPlugin* rdpdr)
 	}
 
 	if (rdpdr->thread)
-		CloseHandle(rdpdr->thread);
+		(void)CloseHandle(rdpdr->thread);
 	MessageQueue_Free(rdpdr->queue);
 	rdpdr->queue = NULL;
 	rdpdr->thread = NULL;
