@@ -35,6 +35,7 @@
 
 #include "tsmf_constants.h"
 #include "tsmf_decoder.h"
+#include "tsmf_audio.h"
 
 /* Compatibility with older FFmpeg */
 #if LIBAVUTIL_VERSION_MAJOR < 50
@@ -81,7 +82,7 @@ typedef struct
 	enum AVCodecID codec_id;
 #endif
 	AVCodecContext* codec_context;
-	AVCodec* codec;
+	const AVCodec* codec;
 	AVFrame* frame;
 	int prepared;
 
@@ -125,7 +126,11 @@ static BOOL tsmf_ffmpeg_init_audio_stream(ITSMFDecoder* decoder, const TS_AM_MED
 	TSMFFFmpegDecoder* mdecoder = (TSMFFFmpegDecoder*)decoder;
 	mdecoder->codec_context->sample_rate = media_type->SamplesPerSecond.Numerator;
 	mdecoder->codec_context->bit_rate = media_type->BitRate;
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 28, 100)
+	mdecoder->codec_context->ch_layout.nb_channels = media_type->Channels;
+#else
 	mdecoder->codec_context->channels = media_type->Channels;
+#endif
 	mdecoder->codec_context->block_align = media_type->BlockAlign;
 #if LIBAVCODEC_VERSION_MAJOR < 55
 #ifdef AV_CPU_FLAG_SSE2
@@ -153,11 +158,15 @@ static BOOL tsmf_ffmpeg_init_audio_stream(ITSMFDecoder* decoder, const TS_AM_MED
 
 static BOOL tsmf_ffmpeg_init_stream(ITSMFDecoder* decoder, const TS_AM_MEDIA_TYPE* media_type)
 {
-	BYTE* p;
-	UINT32 size;
-	const BYTE* s;
+	BYTE* p = NULL;
+	UINT32 size = 0;
+	const BYTE* s = NULL;
 	TSMFFFmpegDecoder* mdecoder = (TSMFFFmpegDecoder*)decoder;
+
+	WINPR_PRAGMA_DIAG_PUSH
+	WINPR_PRAGMA_DIAG_IGNORED_QUALIFIERS
 	mdecoder->codec = avcodec_find_decoder(mdecoder->codec_id);
+	WINPR_PRAGMA_DIAG_POP
 
 	if (!mdecoder->codec)
 	{
@@ -364,18 +373,20 @@ static BOOL tsmf_ffmpeg_decode_video(ITSMFDecoder* decoder, const BYTE* data, UI
                                      UINT32 extensions)
 {
 	TSMFFFmpegDecoder* mdecoder = (TSMFFFmpegDecoder*)decoder;
-	int decoded;
-	int len;
-	AVFrame* frame;
+	int decoded = 0;
+	int len = 0;
+	AVFrame* frame = NULL;
 	BOOL ret = TRUE;
 #if LIBAVCODEC_VERSION_MAJOR < 52 || \
     (LIBAVCODEC_VERSION_MAJOR == 52 && LIBAVCODEC_VERSION_MINOR <= 20)
 	len = avcodec_decode_video(mdecoder->codec_context, mdecoder->frame, &decoded, data, data_size);
 #else
 	{
-		AVPacket pkt;
+		AVPacket pkt = { 0 };
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(58, 133, 100)
 		av_init_packet(&pkt);
-		pkt.data = (BYTE*)data;
+#endif
+		pkt.data = WINPR_CAST_CONST_PTR_AWAY(data, BYTE*);
 		pkt.size = data_size;
 
 		if (extensions & TSMM_SAMPLE_EXT_CLEANPOINT)
@@ -387,10 +398,9 @@ static BOOL tsmf_ffmpeg_decode_video(ITSMFDecoder* decoder, const BYTE* data, UI
 		len = avcodec_send_packet(mdecoder->codec_context, &pkt);
 		if (len > 0)
 		{
-			do
-			{
-				len = avcodec_receive_frame(mdecoder->codec_context, mdecoder->frame);
-			} while (len == AVERROR(EAGAIN));
+			len = avcodec_receive_frame(mdecoder->codec_context, mdecoder->frame);
+			if (len == AVERROR(EAGAIN))
+				return TRUE;
 		}
 #endif
 	}
@@ -430,9 +440,14 @@ static BOOL tsmf_ffmpeg_decode_video(ITSMFDecoder* decoder, const BYTE* data, UI
 		av_image_fill_arrays(frame->data, frame->linesize, mdecoder->decoded_data,
 		                     mdecoder->codec_context->pix_fmt, mdecoder->codec_context->width,
 		                     mdecoder->codec_context->height, 1);
-		av_image_copy(frame->data, frame->linesize, (const uint8_t**)mdecoder->frame->data,
-		              mdecoder->frame->linesize, mdecoder->codec_context->pix_fmt,
-		              mdecoder->codec_context->width, mdecoder->codec_context->height);
+
+		const uint8_t* ptr[AV_NUM_DATA_POINTERS] = { 0 };
+		for (size_t x = 0; x < AV_NUM_DATA_POINTERS; x++)
+			ptr[x] = mdecoder->frame->data[x];
+
+		av_image_copy(frame->data, frame->linesize, ptr, mdecoder->frame->linesize,
+		              mdecoder->codec_context->pix_fmt, mdecoder->codec_context->width,
+		              mdecoder->codec_context->height);
 		av_free(frame);
 	}
 
@@ -443,17 +458,13 @@ static BOOL tsmf_ffmpeg_decode_audio(ITSMFDecoder* decoder, const BYTE* data, UI
                                      UINT32 extensions)
 {
 	TSMFFFmpegDecoder* mdecoder = (TSMFFFmpegDecoder*)decoder;
-	int len;
-	int frame_size;
-	UINT32 src_size;
-	const BYTE* src;
-	BYTE* dst;
-	int dst_offset;
+	int len = 0;
+	int frame_size = 0;
+
 #if 0
 	WLog_DBG(TAG, ("tsmf_ffmpeg_decode_audio: data_size %"PRIu32"", data_size));
-	int i;
 
-	for (i = 0; i < data_size; i++)
+	for (int i = 0; i < data_size; i++)
 	{
 		WLog_DBG(TAG, ("%02"PRIX8"", data[i]));
 
@@ -472,17 +483,17 @@ static BOOL tsmf_ffmpeg_decode_audio(ITSMFDecoder* decoder, const BYTE* data, UI
 		return FALSE;
 
 	/* align the memory for SSE2 needs */
-	dst = (BYTE*)(((uintptr_t)mdecoder->decoded_data + 15) & ~0x0F);
-	dst_offset = dst - mdecoder->decoded_data;
-	src = data;
-	src_size = data_size;
+	BYTE* dst = (BYTE*)(((uintptr_t)mdecoder->decoded_data + 15) & ~0x0F);
+	size_t dst_offset = (size_t)(dst - mdecoder->decoded_data);
+	const BYTE* src = data;
+	UINT32 src_size = data_size;
 
 	while (src_size > 0)
 	{
 		/* Ensure enough space for decoding */
 		if (mdecoder->decoded_size_max - mdecoder->decoded_size < MAX_AUDIO_FRAME_SIZE)
 		{
-			BYTE* tmp_data;
+			BYTE* tmp_data = NULL;
 			tmp_data = realloc(mdecoder->decoded_data, mdecoder->decoded_size_max * 2 + 16);
 
 			if (!tmp_data)
@@ -492,11 +503,12 @@ static BOOL tsmf_ffmpeg_decode_audio(ITSMFDecoder* decoder, const BYTE* data, UI
 			mdecoder->decoded_data = tmp_data;
 			dst = (BYTE*)(((uintptr_t)mdecoder->decoded_data + 15) & ~0x0F);
 
-			if (dst - mdecoder->decoded_data != dst_offset)
+			const size_t diff = (size_t)(dst - mdecoder->decoded_data);
+			if (diff != dst_offset)
 			{
 				/* re-align the memory if the alignment has changed after realloc */
 				memmove(dst, mdecoder->decoded_data + dst_offset, mdecoder->decoded_size);
-				dst_offset = dst - mdecoder->decoded_data;
+				dst_offset = diff;
 			}
 
 			dst += mdecoder->decoded_size;
@@ -515,9 +527,12 @@ static BOOL tsmf_ffmpeg_decode_audio(ITSMFDecoder* decoder, const BYTE* data, UI
 			AVFrame* decoded_frame = av_frame_alloc();
 #endif
 			int got_frame = 0;
-			AVPacket pkt;
+			AVPacket pkt = { 0 };
+#if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(58, 133, 100)
 			av_init_packet(&pkt);
-			pkt.data = (BYTE*)src;
+#endif
+
+			pkt.data = WINPR_CAST_CONST_PTR_AWAY(src, BYTE*);
 			pkt.size = src_size;
 #if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(57, 48, 101)
 			len = avcodec_decode_audio4(mdecoder->codec_context, decoded_frame, &got_frame, &pkt);
@@ -525,17 +540,20 @@ static BOOL tsmf_ffmpeg_decode_audio(ITSMFDecoder* decoder, const BYTE* data, UI
 			len = avcodec_send_packet(mdecoder->codec_context, &pkt);
 			if (len > 0)
 			{
-				do
-				{
-					len = avcodec_receive_frame(mdecoder->codec_context, decoded_frame);
-				} while (len == AVERROR(EAGAIN));
+				len = avcodec_receive_frame(mdecoder->codec_context, decoded_frame);
+				if (len == AVERROR(EAGAIN))
+					return TRUE;
 			}
 #endif
 
 			if (len >= 0 && got_frame)
 			{
-				frame_size = av_samples_get_buffer_size(NULL, mdecoder->codec_context->channels,
-				                                        decoded_frame->nb_samples,
+#if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(57, 28, 100)
+				const int channels = mdecoder->codec_context->ch_layout.nb_channels;
+#else
+				const int channels = mdecoder->codec_context->channels;
+#endif
+				frame_size = av_samples_get_buffer_size(NULL, channels, decoded_frame->nb_samples,
 				                                        mdecoder->codec_context->sample_fmt, 1);
 				memcpy(dst, decoded_frame->data[0], frame_size);
 			}
@@ -606,7 +624,7 @@ static BOOL tsmf_ffmpeg_decode(ITSMFDecoder* decoder, const BYTE* data, UINT32 d
 
 static BYTE* tsmf_ffmpeg_get_decoded_data(ITSMFDecoder* decoder, UINT32* size)
 {
-	BYTE* buf;
+	BYTE* buf = NULL;
 	TSMFFFmpegDecoder* mdecoder = (TSMFFFmpegDecoder*)decoder;
 	*size = mdecoder->decoded_size;
 	buf = mdecoder->decoded_data;
@@ -657,11 +675,17 @@ static void tsmf_ffmpeg_free(ITSMFDecoder* decoder)
 
 	if (mdecoder->codec_context)
 	{
+		free(mdecoder->codec_context->extradata);
+		mdecoder->codec_context->extradata = NULL;
+
+#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55, 69, 100)
+		avcodec_free_context(&mdecoder->codec_context);
+#else
 		if (mdecoder->prepared)
 			avcodec_close(mdecoder->codec_context);
 
-		free(mdecoder->codec_context->extradata);
 		av_free(mdecoder->codec_context);
+#endif
 	}
 
 	free(decoder);
@@ -676,15 +700,19 @@ static BOOL CALLBACK InitializeAvCodecs(PINIT_ONCE once, PVOID param, PVOID* con
 	return TRUE;
 }
 
-FREERDP_ENTRY_POINT(ITSMFDecoder* ffmpeg_freerdp_tsmf_client_decoder_subsystem_entry(void))
+FREERDP_ENTRY_POINT(UINT VCAPITYPE ffmpeg_freerdp_tsmf_client_decoder_subsystem_entry(void* ptr))
 {
-	TSMFFFmpegDecoder* decoder;
+	ITSMFDecoder** sptr = (ITSMFDecoder**)ptr;
+	WINPR_ASSERT(sptr);
+	*sptr = NULL;
+
+	TSMFFFmpegDecoder* decoder = NULL;
 	InitOnceExecuteOnce(&g_Initialized, InitializeAvCodecs, NULL, NULL);
 	WLog_DBG(TAG, "TSMFDecoderEntry FFMPEG");
 	decoder = (TSMFFFmpegDecoder*)calloc(1, sizeof(TSMFFFmpegDecoder));
 
 	if (!decoder)
-		return NULL;
+		return ERROR_OUTOFMEMORY;
 
 	decoder->iface.SetFormat = tsmf_ffmpeg_set_format;
 	decoder->iface.Decode = tsmf_ffmpeg_decode;
@@ -692,5 +720,6 @@ FREERDP_ENTRY_POINT(ITSMFDecoder* ffmpeg_freerdp_tsmf_client_decoder_subsystem_e
 	decoder->iface.GetDecodedFormat = tsmf_ffmpeg_get_decoded_format;
 	decoder->iface.GetDecodedDimension = tsmf_ffmpeg_get_decoded_dimension;
 	decoder->iface.Free = tsmf_ffmpeg_free;
-	return (ITSMFDecoder*)decoder;
+	*sptr = &decoder->iface;
+	return CHANNEL_RC_OK;
 }
