@@ -23,6 +23,7 @@
 #include <string.h>
 
 #include <winpr/sysinfo.h>
+#include <winpr/cast.h>
 
 #include <urbdrc_helpers.h>
 
@@ -44,18 +45,16 @@ static void usb_process_get_port_status(IUDEVICE* pdev, wStream* out)
 			break;
 
 		case USB_v2_0:
-			Stream_Write_UINT32(out, 0x503);
-			break;
-
 		default:
 			Stream_Write_UINT32(out, 0x503);
 			break;
 	}
 }
 
-static UINT urb_write_completion(IUDEVICE* pdev, GENERIC_CHANNEL_CALLBACK* callback, BOOL noAck,
-                                 wStream* out, UINT32 InterfaceId, UINT32 MessageId,
-                                 UINT32 RequestId, UINT32 usbd_status, UINT32 OutputBufferSize)
+static UINT urb_write_completion(WINPR_ATTR_UNUSED IUDEVICE* pdev,
+                                 GENERIC_CHANNEL_CALLBACK* callback, BOOL noAck, wStream* out,
+                                 UINT32 InterfaceId, UINT32 MessageId, UINT32 RequestId,
+                                 UINT32 usbd_status, UINT32 OutputBufferSize)
 {
 	if (!out)
 		return ERROR_INVALID_PARAMETER;
@@ -124,12 +123,11 @@ static UINT urbdrc_process_register_request_callback(IUDEVICE* pdev,
 {
 	UINT32 NumRequestCompletion = 0;
 	UINT32 RequestCompletion = 0;
-	URBDRC_PLUGIN* urbdrc = NULL;
 
 	if (!callback || !s || !udevman || !pdev)
 		return ERROR_INVALID_PARAMETER;
 
-	urbdrc = (URBDRC_PLUGIN*)callback->plugin;
+	URBDRC_PLUGIN* urbdrc = (URBDRC_PLUGIN*)callback->plugin;
 
 	if (!urbdrc)
 		return ERROR_INVALID_PARAMETER;
@@ -179,7 +177,8 @@ static UINT urbdrc_process_cancel_request(IUDEVICE* pdev, wStream* s, IUDEVMAN*
 	return ERROR_SUCCESS;
 }
 
-static UINT urbdrc_process_retract_device_request(IUDEVICE* pdev, wStream* s, IUDEVMAN* udevman)
+static UINT urbdrc_process_retract_device_request(WINPR_ATTR_UNUSED IUDEVICE* pdev, wStream* s,
+                                                  IUDEVMAN* udevman)
 {
 	UINT32 Reason = 0;
 	URBDRC_PLUGIN* urbdrc = NULL;
@@ -423,7 +422,7 @@ static UINT urb_select_configuration(IUDEVICE* pdev, GENERIC_CHANNEL_CALLBACK* c
 	UINT32 usbd_status = 0;
 	BYTE ConfigurationDescriptorIsValid = 0;
 	wStream* out = NULL;
-	int MsOutSize = 0;
+	size_t MsOutSize = 0;
 	URBDRC_PLUGIN* urbdrc = NULL;
 	const BOOL noAck = (RequestField & 0x80000000U) != 0;
 	const UINT32 RequestId = RequestField & 0x7FFFFFFF;
@@ -472,11 +471,11 @@ static UINT urb_select_configuration(IUDEVICE* pdev, GENERIC_CHANNEL_CALLBACK* c
 	}
 
 	if (MsConfig)
-		MsOutSize = MsConfig->MsOutSize;
+		MsOutSize = WINPR_ASSERTING_INT_CAST(size_t, MsConfig->MsOutSize);
 
 	if (MsOutSize > 0)
 	{
-		if ((size_t)MsOutSize > SIZE_MAX - 36)
+		if (MsOutSize > SIZE_MAX - 36)
 			return ERROR_INVALID_DATA;
 
 		out_size = 36 + MsOutSize;
@@ -497,9 +496,9 @@ static UINT urb_select_configuration(IUDEVICE* pdev, GENERIC_CHANNEL_CALLBACK* c
 	if (MsOutSize > 0)
 	{
 		/** CbTsUrbResult */
-		Stream_Write_UINT32(out, 8 + MsOutSize);
+		Stream_Write_UINT32(out, 8U + (UINT32)MsOutSize);
 		/** TS_URB_RESULT_HEADER Size*/
-		Stream_Write_UINT16(out, 8 + MsOutSize);
+		Stream_Write_UINT16(out, WINPR_ASSERTING_INT_CAST(uint16_t, 8U + (UINT32)MsOutSize));
 	}
 	else
 	{
@@ -605,7 +604,7 @@ static UINT urb_select_interface(IUDEVICE* pdev, GENERIC_CHANNEL_CALLBACK* callb
 	Stream_Write_UINT32(out, RequestId);              /** RequestId */
 	Stream_Write_UINT32(out, 8 + interface_size);     /** CbTsUrbResult */
 	/** TS_URB_RESULT_HEADER */
-	Stream_Write_UINT16(out, 8 + interface_size); /** Size */
+	Stream_Write_UINT16(out, WINPR_ASSERTING_INT_CAST(uint16_t, 8 + interface_size)); /** Size */
 	/** Padding, MUST be ignored upon receipt */
 	Stream_Write_UINT16(out, TS_URB_SELECT_INTERFACE);
 	Stream_Write_UINT32(out, USBD_STATUS_SUCCESS); /** UsbdStatus */
@@ -673,6 +672,8 @@ static UINT urb_control_transfer(IUDEVICE* pdev, GENERIC_CHANNEL_CALLBACK* callb
 
 		case URB_CONTROL_TRANSFER_NONEXTERNAL:
 			break;
+		default:
+			break;
 	}
 
 	/** SetupPacket 8 bytes */
@@ -705,7 +706,10 @@ static UINT urb_control_transfer(IUDEVICE* pdev, GENERIC_CHANNEL_CALLBACK* callb
 	if (transferDir == USBD_TRANSFER_DIRECTION_OUT)
 	{
 		if (!Stream_CheckAndLogRequiredLength(TAG, s, OutputBufferSize))
+		{
+			Stream_Free(out, TRUE);
 			return ERROR_INVALID_DATA;
+		}
 		Stream_Copy(s, out, OutputBufferSize);
 	}
 
@@ -725,8 +729,9 @@ static UINT urb_control_transfer(IUDEVICE* pdev, GENERIC_CHANNEL_CALLBACK* callb
 
 static void urb_bulk_transfer_cb(IUDEVICE* pdev, GENERIC_CHANNEL_CALLBACK* callback, wStream* out,
                                  UINT32 InterfaceId, BOOL noAck, UINT32 MessageId, UINT32 RequestId,
-                                 UINT32 NumberOfPackets, UINT32 status, UINT32 StartFrame,
-                                 UINT32 ErrorCount, UINT32 OutputBufferSize)
+                                 WINPR_ATTR_UNUSED UINT32 NumberOfPackets, UINT32 status,
+                                 WINPR_ATTR_UNUSED UINT32 StartFrame,
+                                 WINPR_ATTR_UNUSED UINT32 ErrorCount, UINT32 OutputBufferSize)
 {
 	if (!pdev->isChannelClosed(pdev))
 		urb_write_completion(pdev, callback, noAck, out, InterfaceId, MessageId, RequestId, status,
@@ -766,13 +771,16 @@ static UINT urb_bulk_or_interrupt_transfer(IUDEVICE* pdev, GENERIC_CHANNEL_CALLB
 	}
 
 	/**  process TS_URB_BULK_OR_INTERRUPT_TRANSFER */
-	return pdev->bulk_or_interrupt_transfer(
+	const int rc = pdev->bulk_or_interrupt_transfer(
 	    pdev, callback, MessageId, RequestId, EndpointAddress, TransferFlags, noAck,
 	    OutputBufferSize, (transferDir == USBD_TRANSFER_DIRECTION_OUT) ? Stream_Pointer(s) : NULL,
 	    urb_bulk_transfer_cb, 10000);
+
+	return (uint32_t)rc;
 }
 
-static void urb_isoch_transfer_cb(IUDEVICE* pdev, GENERIC_CHANNEL_CALLBACK* callback, wStream* out,
+static void urb_isoch_transfer_cb(WINPR_ATTR_UNUSED IUDEVICE* pdev,
+                                  GENERIC_CHANNEL_CALLBACK* callback, wStream* out,
                                   UINT32 InterfaceId, BOOL noAck, UINT32 MessageId,
                                   UINT32 RequestId, UINT32 NumberOfPackets, UINT32 status,
                                   UINT32 StartFrame, UINT32 ErrorCount, UINT32 OutputBufferSize)
@@ -793,7 +801,7 @@ static void urb_isoch_transfer_cb(IUDEVICE* pdev, GENERIC_CHANNEL_CALLBACK* call
 		Stream_Write_UINT32(out, RequestId);       /** RequestId */
 		Stream_Write_UINT32(out, 20 + packetSize); /** CbTsUrbResult */
 		/** TsUrbResult TS_URB_RESULT_HEADER */
-		Stream_Write_UINT16(out, 20 + packetSize); /** Size */
+		Stream_Write_UINT16(out, WINPR_ASSERTING_INT_CAST(uint16_t, 20 + packetSize)); /** Size */
 		Stream_Write_UINT16(out, 0);               /* Padding */
 		Stream_Write_UINT32(out, status);          /** UsbdStatus */
 		Stream_Write_UINT32(out, StartFrame);      /** StartFrame */
@@ -852,7 +860,7 @@ static UINT urb_isoch_transfer(IUDEVICE* pdev, GENERIC_CHANNEL_CALLBACK* callbac
 		return ERROR_INVALID_DATA;
 
 	packetDescriptorData = Stream_Pointer(s);
-	Stream_Seek(s, NumberOfPackets * 12);
+	Stream_Seek(s, 12ULL * NumberOfPackets);
 
 	if (!Stream_CheckAndLogRequiredLength(TAG, s, sizeof(UINT32)))
 		return ERROR_INVALID_DATA;
@@ -947,8 +955,8 @@ static UINT urb_control_descriptor_request(IUDEVICE* pdev, GENERIC_CHANNEL_CALLB
 	/** process get usb device descriptor */
 	if (!pdev->control_transfer(pdev, RequestId, 0, 0, bmRequestType,
 	                            0x06, /* REQUEST_GET_DESCRIPTOR */
-	                            (desc_type << 8) | desc_index, langId, &usbd_status,
-	                            &OutputBufferSize, Stream_Pointer(out), 1000))
+	                            WINPR_ASSERTING_INT_CAST(UINT16, ((desc_type << 8) | desc_index)),
+	                            langId, &usbd_status, &OutputBufferSize, Stream_Pointer(out), 1000))
 	{
 		WLog_Print(urbdrc->log, WLOG_ERROR, "get_descriptor failed");
 		Stream_Free(out, TRUE);
@@ -1035,7 +1043,6 @@ static UINT urb_control_vendor_or_class_request(IUDEVICE* pdev, GENERIC_CHANNEL_
 	BYTE bmRequestType = 0;
 	UINT16 Value = 0;
 	UINT16 Index = 0;
-	UINT16 Padding = 0;
 	wStream* out = NULL;
 	URBDRC_PLUGIN* urbdrc = NULL;
 	const BOOL noAck = (RequestField & 0x80000000U) != 0;
@@ -1058,7 +1065,7 @@ static UINT urb_control_vendor_or_class_request(IUDEVICE* pdev, GENERIC_CHANNEL_
 	Stream_Read_UINT8(s, Request);             /** Request */
 	Stream_Read_UINT16(s, Value);              /** value */
 	Stream_Read_UINT16(s, Index);              /** index */
-	Stream_Read_UINT16(s, Padding);            /** Padding */
+	Stream_Seek_UINT16(s);                     /** Padding */
 	Stream_Read_UINT32(s, OutputBufferSize);
 	if (OutputBufferSize > UINT32_MAX - 36)
 		return ERROR_INVALID_DATA;
@@ -1180,6 +1187,8 @@ static UINT urb_os_feature_descriptor_request(IUDEVICE* pdev, GENERIC_CHANNEL_CA
 
 		case USBD_TRANSFER_DIRECTION_IN:
 			break;
+		default:
+			break;
 	}
 
 	WLog_Print(urbdrc->log, WLOG_DEBUG,
@@ -1531,6 +1540,8 @@ static UINT urb_control_feature_request(IUDEVICE* pdev, GENERIC_CHANNEL_CALLBACK
 		case USBD_TRANSFER_DIRECTION_IN:
 			bmRequestType |= 0x80;
 			break;
+		default:
+			break;
 	}
 
 	switch (command)
