@@ -235,7 +235,7 @@ static size_t udevman_register_udevice(IUDEVMAN* idevman, BYTE bus_number, BYTE
 			addnum++;
 		}
 
-		free(devArray);
+		free((void*)devArray);
 		return addnum;
 	}
 	else
@@ -436,13 +436,13 @@ static IUDEVICE* udevman_get_udevice_by_ChannelID(IUDEVMAN* idevman, UINT32 chan
 static void udevman_loading_lock(IUDEVMAN* idevman)
 {
 	UDEVMAN* udevman = (UDEVMAN*)idevman;
-	WaitForSingleObject(udevman->devman_loading, INFINITE);
+	(void)WaitForSingleObject(udevman->devman_loading, INFINITE);
 }
 
 static void udevman_loading_unlock(IUDEVMAN* idevman)
 {
 	UDEVMAN* udevman = (UDEVMAN*)idevman;
-	ReleaseMutex(udevman->devman_loading);
+	(void)ReleaseMutex(udevman->devman_loading);
 }
 
 BASIC_STATE_FUNC_DEFINED(device_num, UINT32)
@@ -469,14 +469,14 @@ static void udevman_free(IUDEVMAN* idevman)
 	udevman->running = FALSE;
 	if (udevman->thread)
 	{
-		WaitForSingleObject(udevman->thread, INFINITE);
-		CloseHandle(udevman->thread);
+		(void)WaitForSingleObject(udevman->thread, INFINITE);
+		(void)CloseHandle(udevman->thread);
 	}
 
 	udevman_unregister_all_udevices(idevman);
 
 	if (udevman->devman_loading)
-		CloseHandle(udevman->devman_loading);
+		(void)CloseHandle(udevman->devman_loading);
 
 	libusb_exit(udevman->context);
 
@@ -624,7 +624,7 @@ static BOOL udevman_initialize(IUDEVMAN* idevman, UINT32 channelId)
 	if (!udevman)
 		return FALSE;
 
-	idevman->status &= ~URBDRC_DEVICE_CHANNEL_CLOSED;
+	idevman->status &= (uint32_t)~URBDRC_DEVICE_CHANNEL_CLOSED;
 	idevman->controlChannelId = channelId;
 	return TRUE;
 }
@@ -787,18 +787,18 @@ static UINT urbdrc_udevman_parse_addin_args(UDEVMAN* udevman, const ADDIN_ARGV*
 
 static UINT udevman_listener_created_callback(IUDEVMAN* iudevman)
 {
-	UINT status = 0;
 	UDEVMAN* udevman = (UDEVMAN*)iudevman;
+	WINPR_ASSERT(udevman);
 
 	if (udevman->devices_vid_pid)
-	{
-		status = urbdrc_udevman_register_devices(udevman, udevman->devices_vid_pid, FALSE);
-		if (status != CHANNEL_RC_OK)
-			return status;
-	}
+		return urbdrc_udevman_register_devices(udevman, udevman->devices_vid_pid, FALSE)
+		           ? CHANNEL_RC_OK
+		           : CHANNEL_RC_INITIALIZATION_ERROR;
 
 	if (udevman->devices_addr)
-		return urbdrc_udevman_register_devices(udevman, udevman->devices_addr, TRUE);
+		return urbdrc_udevman_register_devices(udevman, udevman->devices_addr, TRUE)
+		           ? CHANNEL_RC_OK
+		           : CHANNEL_RC_INITIALIZATION_ERROR;
 
 	return CHANNEL_RC_OK;
 }
@@ -894,11 +894,10 @@ static DWORD WINAPI poll_thread(LPVOID lpThreadParameter)
 	return 0;
 }
 
-FREERDP_ENTRY_POINT(UINT libusb_freerdp_urbdrc_client_subsystem_entry(
+FREERDP_ENTRY_POINT(UINT VCAPITYPE libusb_freerdp_urbdrc_client_subsystem_entry(
     PFREERDP_URBDRC_SERVICE_ENTRY_POINTS pEntryPoints))
 {
 	wObject* obj = NULL;
-	UINT rc = 0;
 	UINT status = 0;
 	UDEVMAN* udevman = NULL;
 	const ADDIN_ARGV* args = pEntryPoints->args;
@@ -916,15 +915,15 @@ FREERDP_ENTRY_POINT(UINT libusb_freerdp_urbdrc_client_subsystem_entry(
 
 	udevman->next_device_id = BASE_USBDEVICE_NUM;
 	udevman->iface.plugin = pEntryPoints->plugin;
-	rc = libusb_init(&udevman->context);
+	const int res = libusb_init(&udevman->context);
 
-	if (rc != LIBUSB_SUCCESS)
+	if (res != LIBUSB_SUCCESS)
 		goto fail;
 
 #ifdef _WIN32
 #if LIBUSB_API_VERSION >= 0x01000106
 	/* Prefer usbDK backend on windows. Not supported on other platforms. */
-	rc = libusb_set_option(udevman->context, LIBUSB_OPTION_USE_USBDK);
+	const int rc = libusb_set_option(udevman->context, LIBUSB_OPTION_USE_USBDK);
 	switch (rc)
 	{
 		case LIBUSB_SUCCESS:
