@@ -311,8 +311,11 @@ static UINT urdbrc_send_usb_device_add(GENERIC_CHANNEL_CALLBACK* callback, IUDEV
 	                                        TRUE) < 0)
 		goto fail;
 	Stream_Write_UINT16(out, 0);
-	Stream_Write_UINT32(out, HardwareIdsLen[0] + HardwareIdsLen[1] + 3); /* cchHwIds */
-	                                                                     /* HardwareIds 1 */
+	const size_t len = HardwareIdsLen[0] + HardwareIdsLen[1] + 3;
+	if (len > UINT32_MAX)
+		goto fail;
+	Stream_Write_UINT32(out, (UINT32)len); /* cchHwIds */
+	                                       /* HardwareIds 1 */
 	if (Stream_Write_UTF16_String_From_UTF8(out, HardwareIdsLen[0], HardwareIds[0],
 	                                        HardwareIdsLen[0], TRUE) < 0)
 		goto fail;
@@ -357,7 +360,7 @@ static UINT urdbrc_send_usb_device_add(GENERIC_CHANNEL_CALLBACK* callback, IUDEV
 	Stream_Write_UINT32(out, 2); /* UsbBusInterfaceVersion, 0 ,1 or 2 */ // TODO: Get from libusb
 	Stream_Write_UINT32(out, 0x600); /* USBDI_Version, 0x500 or 0x600 */ // TODO: Get from libusb
 	/* Supported_USB_Version, 0x110,0x110 or 0x200(usb2.0) */
-	bcdUSB = pdev->query_device_descriptor(pdev, BCD_USB);
+	bcdUSB = WINPR_ASSERTING_INT_CAST(uint32_t, pdev->query_device_descriptor(pdev, BCD_USB));
 	Stream_Write_UINT32(out, bcdUSB);
 	Stream_Write_UINT32(out, 0x00000000); /* HcdCapabilities, MUST always be zero */
 
@@ -441,7 +444,8 @@ static BOOL urbdrc_announce_devices(IUDEVMAN* udevman)
 	return error == ERROR_SUCCESS;
 }
 
-static UINT urbdrc_device_control_channel(GENERIC_CHANNEL_CALLBACK* callback, wStream* s)
+static UINT urbdrc_device_control_channel(GENERIC_CHANNEL_CALLBACK* callback,
+                                          WINPR_ATTR_UNUSED wStream* s)
 {
 	URBDRC_PLUGIN* urbdrc = (URBDRC_PLUGIN*)callback->plugin;
 	IUDEVMAN* udevman = urbdrc->udevman;
@@ -644,8 +648,9 @@ static UINT urbdrc_on_close(IWTSVirtualChannelCallback* pChannelCallback)
  * @return 0 on success, otherwise a Win32 error code
  */
 static UINT urbdrc_on_new_channel_connection(IWTSListenerCallback* pListenerCallback,
-                                             IWTSVirtualChannel* pChannel, BYTE* pData,
-                                             BOOL* pbAccept,
+                                             IWTSVirtualChannel* pChannel,
+                                             WINPR_ATTR_UNUSED BYTE* pData,
+                                             WINPR_ATTR_UNUSED BOOL* pbAccept,
                                              IWTSVirtualChannelCallback** ppCallback)
 {
 	GENERIC_LISTENER_CALLBACK* listener_callback = (GENERIC_LISTENER_CALLBACK*)pListenerCallback;
@@ -769,10 +774,11 @@ static BOOL urbdrc_register_udevman_addin(IWTSPlugin* pPlugin, IUDEVMAN* udevman
 static UINT urbdrc_load_udevman_addin(IWTSPlugin* pPlugin, LPCSTR name, const ADDIN_ARGV* args)
 {
 	URBDRC_PLUGIN* urbdrc = (URBDRC_PLUGIN*)pPlugin;
-	PFREERDP_URBDRC_DEVICE_ENTRY entry = NULL;
-	FREERDP_URBDRC_SERVICE_ENTRY_POINTS entryPoints;
-	entry = (PFREERDP_URBDRC_DEVICE_ENTRY)freerdp_load_channel_addin_entry(URBDRC_CHANNEL_NAME,
-	                                                                       name, NULL, 0);
+	FREERDP_URBDRC_SERVICE_ENTRY_POINTS entryPoints = { 0 };
+
+	PVIRTUALCHANNELENTRY pvce =
+	    freerdp_load_channel_addin_entry(URBDRC_CHANNEL_NAME, name, NULL, 0);
+	PFREERDP_URBDRC_DEVICE_ENTRY entry = WINPR_FUNC_PTR_CAST(pvce, PFREERDP_URBDRC_DEVICE_ENTRY);
 
 	if (!entry)
 		return ERROR_INVALID_OPERATION;
@@ -781,10 +787,11 @@ static UINT urbdrc_load_udevman_addin(IWTSPlugin* pPlugin, LPCSTR name, const AD
 	entryPoints.pRegisterUDEVMAN = urbdrc_register_udevman_addin;
 	entryPoints.args = args;
 
-	if (entry(&entryPoints) != 0)
+	const UINT error = entry(&entryPoints);
+	if (error)
 	{
 		WLog_Print(urbdrc->log, WLOG_ERROR, "%s entry returns error.", name);
-		return ERROR_INVALID_OPERATION;
+		return error;
 	}
 
 	return CHANNEL_RC_OK;
@@ -950,7 +957,7 @@ BOOL del_device(IUDEVMAN* idevman, UINT32 flags, BYTE busnum, BYTE devnum, UINT1
  *
  * @return 0 on success, otherwise a Win32 error code
  */
-FREERDP_ENTRY_POINT(UINT urbdrc_DVCPluginEntry(IDRDYNVC_ENTRY_POINTS* pEntryPoints))
+FREERDP_ENTRY_POINT(UINT VCAPITYPE urbdrc_DVCPluginEntry(IDRDYNVC_ENTRY_POINTS* pEntryPoints))
 {
 	UINT status = 0;
 	const ADDIN_ARGV* args = NULL;
@@ -998,7 +1005,6 @@ fail:
 
 UINT stream_write_and_free(IWTSPlugin* plugin, IWTSVirtualChannel* channel, wStream* out)
 {
-	UINT rc = 0;
 	URBDRC_PLUGIN* urbdrc = (URBDRC_PLUGIN*)plugin;
 
 	if (!out)
@@ -1017,7 +1023,10 @@ UINT stream_write_and_free(IWTSPlugin* plugin, IWTSVirtualChannel* channel, wStr
 	}
 
 	urbdrc_dump_message(urbdrc->log, TRUE, TRUE, out);
-	rc = channel->Write(channel, Stream_GetPosition(out), Stream_Buffer(out), NULL);
+	const size_t len = Stream_GetPosition(out);
+	UINT rc = ERROR_INTERNAL_ERROR;
+	if (len <= UINT32_MAX)
+		rc = channel->Write(channel, (UINT32)len, Stream_Buffer(out), NULL);
 	Stream_Free(out, TRUE);
 	return rc;
 }
