@@ -59,6 +59,7 @@
 #include "sdl_pointer.hpp"
 #include "sdl_prefs.hpp"
 #include "dialogs/sdl_dialogs.hpp"
+#include "scoped_guard.hpp"
 
 #include <aad/sdl_webview.hpp>
 
@@ -135,7 +136,7 @@ enum SDL_EXIT_CODE
 
 struct sdl_exit_code_map_t
 {
-	DWORD error;
+	UINT32 error;
 	int code;
 	const char* code_tag;
 };
@@ -228,18 +229,18 @@ static void sdl_hide_connection_dialog(SdlContext* sdl)
 		sdl->connection_dialog->hide();
 }
 
-static const struct sdl_exit_code_map_t* sdl_map_entry_by_error(DWORD error)
+static const struct sdl_exit_code_map_t* sdl_map_entry_by_error(UINT32 error)
 {
 	for (const auto& x : sdl_exit_code_map)
 	{
 		const struct sdl_exit_code_map_t* cur = &x;
-		if (cur->error == error)
+		if (cur->error == static_cast<uint32_t>(error))
 			return cur;
 	}
 	return nullptr;
 }
 
-static int sdl_map_error_to_exit_code(DWORD error)
+static int sdl_map_error_to_exit_code(UINT32 error)
 {
 	const struct sdl_exit_code_map_t* entry = sdl_map_entry_by_error(error);
 	if (entry)
@@ -248,7 +249,7 @@ static int sdl_map_error_to_exit_code(DWORD error)
 	return SDL_EXIT_CONN_FAILED;
 }
 
-static const char* sdl_map_error_to_code_tag(DWORD error)
+static const char* sdl_map_error_to_code_tag(UINT32 error)
 {
 	const struct sdl_exit_code_map_t* entry = sdl_map_entry_by_error(error);
 	if (entry)
@@ -272,7 +273,7 @@ static int error_info_to_error(freerdp* instance, DWORD* pcode, char** msg, size
 	const int exit_code = sdl_map_error_to_exit_code(code);
 
 	winpr_asprintf(msg, len, "Terminate with %s due to ERROR_INFO %s [0x%08" PRIx32 "]: %s",
-	               sdl_map_error_to_code_tag(exit_code), name, code, str);
+	               sdl_map_error_to_code_tag(code), name, code, str);
 	WLog_DBG(SDL_TAG, "%s", *msg);
 	if (pcode)
 		*pcode = code;
@@ -332,10 +333,14 @@ class SdlEventUpdateTriggerGuard
 	{
 		_sdl->update_complete.set();
 	}
+	SdlEventUpdateTriggerGuard(const SdlEventUpdateTriggerGuard&) = delete;
+	SdlEventUpdateTriggerGuard(SdlEventUpdateTriggerGuard&&) = delete;
+	SdlEventUpdateTriggerGuard& operator=(const SdlEventUpdateTriggerGuard&) = delete;
+	SdlEventUpdateTriggerGuard& operator=(SdlEventUpdateTriggerGuard&&) = delete;
 };
 
-static bool sdl_draw_to_window_rect(SdlContext* sdl, SdlWindow& window, SDL_Surface* surface,
-                                    SDL_Point offset, SDL_Rect srcRect)
+static bool sdl_draw_to_window_rect([[maybe_unused]] SdlContext* sdl, SdlWindow& window,
+                                    SDL_Surface* surface, SDL_Point offset, SDL_Rect srcRect)
 {
 	SDL_Rect dstRect = { offset.x + srcRect.x, offset.y + srcRect.y, srcRect.w, srcRect.h };
 	return window.blit(surface, srcRect, dstRect);
@@ -583,7 +588,7 @@ static BOOL sdl_pre_connect(freerdp* instance)
 		return FALSE;
 	/* OrderSupport is initialized at this point.
 	 * Only override it if you plan to implement custom order
-	 * callbacks or deactiveate certain features. */
+	 * callbacks or deactivate certain features. */
 	/* Register the channel listeners.
 	 * They are required to set up / tear down channels if they are loaded. */
 	PubSub_SubscribeChannelConnected(instance->context->pubSub, sdl_OnChannelConnectedEventHandler);
@@ -660,7 +665,7 @@ static const char* sdl_window_get_title(rdpSettings* settings)
 
 	addPort = (port != 3389);
 
-	char buffer[MAX_PATH + 64] = { 0 };
+	char buffer[MAX_PATH + 64] = {};
 
 	if (!addPort)
 		(void)sprintf_s(buffer, sizeof(buffer), "%s %s", prefix, name);
@@ -672,7 +677,8 @@ static const char* sdl_window_get_title(rdpSettings* settings)
 	return freerdp_settings_get_string(settings, FreeRDP_WindowTitle);
 }
 
-static void sdl_term_handler(int signum, const char* signame, void* context)
+static void sdl_term_handler([[maybe_unused]] int signum, [[maybe_unused]] const char* signame,
+                             [[maybe_unused]] void* context)
 {
 	sdl_push_quit();
 }
@@ -699,7 +705,6 @@ static BOOL sdl_create_windows(SdlContext* sdl)
 
 	auto settings = sdl->context()->settings;
 	auto title = sdl_window_get_title(settings);
-	BOOL rc = FALSE;
 
 	UINT32 windowCount = freerdp_settings_get_uint32(settings, FreeRDP_MonitorCount);
 
@@ -711,8 +716,8 @@ static BOOL sdl_create_windows(SdlContext* sdl)
 		auto monitor = static_cast<rdpMonitor*>(
 		    freerdp_settings_get_pointer_array_writable(settings, FreeRDP_MonitorDefArray, x));
 
-		Uint32 w = monitor->width;
-		Uint32 h = monitor->height;
+		Uint32 w = WINPR_ASSERTING_INT_CAST(uint32_t, monitor->width);
+		Uint32 h = WINPR_ASSERTING_INT_CAST(uint32_t, monitor->height);
 		if (!(freerdp_settings_get_bool(settings, FreeRDP_UseMultimon) ||
 		      freerdp_settings_get_bool(settings, FreeRDP_Fullscreen)))
 		{
@@ -721,8 +726,8 @@ static BOOL sdl_create_windows(SdlContext* sdl)
 		}
 
 		Uint32 flags = SDL_WINDOW_SHOWN;
-		Uint32 startupX = SDL_WINDOWPOS_CENTERED_DISPLAY(id);
-		Uint32 startupY = SDL_WINDOWPOS_CENTERED_DISPLAY(id);
+		auto startupX = SDL_WINDOWPOS_CENTERED_DISPLAY(id);
+		auto startupY = SDL_WINDOWPOS_CENTERED_DISPLAY(id);
 
 		if (monitor->attributes.desktopScaleFactor > 100)
 		{
@@ -751,8 +756,11 @@ static BOOL sdl_create_windows(SdlContext* sdl)
 			              static_cast<int>(w),
 			              static_cast<int>(h),
 			              flags };
+
+		ScopeGuard guard([&]() { sdl->windows_created.set(); });
+
 		if (!window.window())
-			goto fail;
+			return FALSE;
 
 		if (freerdp_settings_get_bool(settings, FreeRDP_UseMultimon))
 		{
@@ -764,11 +772,7 @@ static BOOL sdl_create_windows(SdlContext* sdl)
 		sdl->windows.insert({ window.id(), std::move(window) });
 	}
 
-	rc = TRUE;
-fail:
-
-	sdl->windows_created.set();
-	return rc;
+	return TRUE;
 }
 
 static BOOL sdl_wait_create_windows(SdlContext* sdl)
@@ -834,7 +838,7 @@ static int sdl_run(SdlContext* sdl)
 
 	while (!shall_abort(sdl))
 	{
-		SDL_Event windowEvent = { 0 };
+		SDL_Event windowEvent = {};
 		while (!shall_abort(sdl) && SDL_WaitEventTimeout(nullptr, 1000))
 		{
 			/* Only poll standard SDL events and SDL_USEREVENTS meant to create dialogs.
@@ -1023,6 +1027,14 @@ static int sdl_run(SdlContext* sdl)
 						window->fullscreen(enter);
 				}
 				break;
+				case SDL_USEREVENT_WINDOW_MINIMIZE:
+				{
+					for (auto& window : sdl->windows)
+					{
+						window.second.minimize();
+					}
+				}
+				break;
 				case SDL_USEREVENT_POINTER_NULL:
 					SDL_ShowCursor(SDL_DISABLE);
 					break;
@@ -1069,7 +1081,7 @@ static int sdl_run(SdlContext* sdl)
 }
 
 /* Called after a RDP connection was successfully established.
- * Settings might have changed during negociation of client / server feature
+ * Settings might have changed during negotiation of client / server feature
  * support.
  *
  * Set up local framebuffers and paing callbacks.
@@ -1154,23 +1166,58 @@ static void sdl_post_final_disconnect(freerdp* instance)
 
 	if (!instance->context)
 		return;
-
-	auto context = get_context(instance->context);
 }
 
-/* RDP main loop.
- * Connects RDP, loops while running and handles event and dispatch, cleans up
- * after the connection ends. */
-static DWORD WINAPI sdl_client_thread_proc(SdlContext* sdl)
+static void sdl_client_cleanup(SdlContext* sdl, int exit_code, const std::string& error_msg)
 {
-	DWORD nCount = 0;
-	DWORD status = 0;
-	int exit_code = SDL_EXIT_SUCCESS;
-	char* error_msg = nullptr;
-	size_t error_msg_len = 0;
+	WINPR_ASSERT(sdl);
 
-	HANDLE handles[MAXIMUM_WAIT_OBJECTS] = {};
+	rdpContext* context = sdl->context();
+	WINPR_ASSERT(context);
 
+	rdpSettings* settings = context->settings;
+	WINPR_ASSERT(settings);
+
+	sdl->rdp_thread_running = false;
+	bool showError = false;
+	if (freerdp_settings_get_bool(settings, FreeRDP_AuthenticationOnly))
+		WLog_Print(sdl->log, WLOG_INFO, "Authentication only, exit status %s [%" PRId32 "]",
+		           sdl_map_to_code_tag(exit_code), exit_code);
+	else
+	{
+		switch (exit_code)
+		{
+			case SDL_EXIT_SUCCESS:
+			case SDL_EXIT_DISCONNECT:
+			case SDL_EXIT_LOGOFF:
+			case SDL_EXIT_DISCONNECT_BY_USER:
+			case SDL_EXIT_CONNECT_CANCELLED:
+				break;
+			default:
+			{
+				std::lock_guard<CriticalSection> lock(sdl->critical);
+				if (sdl->connection_dialog && !error_msg.empty())
+				{
+					sdl->connection_dialog->showError(error_msg.c_str());
+					showError = true;
+				}
+			}
+			break;
+		}
+	}
+
+	if (!showError)
+		sdl_hide_connection_dialog(sdl);
+
+	sdl->exit_code = exit_code;
+	sdl_push_user_event(SDL_USEREVENT_QUIT);
+#if SDL_VERSION_ATLEAST(2, 0, 16)
+	SDL_TLSCleanup();
+#endif
+}
+
+static int sdl_client_thread_connect(SdlContext* sdl, std::string& error_msg)
+{
 	WINPR_ASSERT(sdl);
 
 	auto instance = sdl->context()->instance;
@@ -1180,9 +1227,12 @@ static DWORD WINAPI sdl_client_thread_proc(SdlContext* sdl)
 	BOOL rc = freerdp_connect(instance);
 
 	rdpContext* context = sdl->context();
+	WINPR_ASSERT(context);
+
 	rdpSettings* settings = context->settings;
 	WINPR_ASSERT(settings);
 
+	int exit_code = SDL_EXIT_SUCCESS;
 	if (!rc)
 	{
 		UINT32 error = freerdp_get_last_error(context);
@@ -1195,21 +1245,33 @@ static DWORD WINAPI sdl_client_thread_proc(SdlContext* sdl)
 		freerdp_abort_connect_context(context);
 		WLog_Print(sdl->log, WLOG_ERROR, "Authentication only, %s [0x%08" PRIx32 "] %s",
 		           freerdp_get_last_error_name(code), code, freerdp_get_last_error_string(code));
-		goto terminate;
+		return exit_code;
 	}
 
 	if (!rc)
 	{
 		DWORD code = freerdp_error_info(instance);
 		if (exit_code == SDL_EXIT_SUCCESS)
-			exit_code = error_info_to_error(instance, &code, &error_msg, &error_msg_len);
+		{
+			char* msg = nullptr;
+			size_t len = 0;
+			exit_code = error_info_to_error(instance, &code, &msg, &len);
+			if (msg)
+				error_msg = msg;
+			free(msg);
+		}
 
 		auto last = freerdp_get_last_error(context);
-		if (!error_msg)
+		if (error_msg.empty())
 		{
-			winpr_asprintf(&error_msg, &error_msg_len, "%s [0x%08" PRIx32 "]\n%s",
+			char* msg = nullptr;
+			size_t len = 0;
+			winpr_asprintf(&msg, &len, "%s [0x%08" PRIx32 "]\n%s",
 			               freerdp_get_last_error_name(last), last,
 			               freerdp_get_last_error_string(last));
+			if (msg)
+				error_msg = msg;
+			free(msg);
 		}
 
 		if (exit_code == SDL_EXIT_SUCCESS)
@@ -1221,11 +1283,24 @@ static DWORD WINAPI sdl_client_thread_proc(SdlContext* sdl)
 		}
 
 		sdl_hide_connection_dialog(sdl);
-		goto terminate;
 	}
+	return exit_code;
+}
 
+static int sdl_client_thread_run(SdlContext* sdl, std::string& error_msg)
+{
+	WINPR_ASSERT(sdl);
+
+	auto context = sdl->context();
+	WINPR_ASSERT(context);
+
+	auto instance = context->instance;
+	WINPR_ASSERT(instance);
+
+	int exit_code = SDL_EXIT_SUCCESS;
 	while (!freerdp_shall_disconnect_context(context))
 	{
+		HANDLE handles[MAXIMUM_WAIT_OBJECTS] = {};
 		/*
 		 * win8 and server 2k12 seem to have some timing issue/race condition
 		 * when a initial sync request is send to sync the keyboard indicators
@@ -1241,7 +1316,7 @@ static DWORD WINAPI sdl_client_thread_proc(SdlContext* sdl)
 				break;
 		}
 
-		nCount = freerdp_get_event_handles(context, handles, ARRAYSIZE(handles));
+		const DWORD nCount = freerdp_get_event_handles(context, handles, ARRAYSIZE(handles));
 
 		if (nCount == 0)
 		{
@@ -1249,7 +1324,7 @@ static DWORD WINAPI sdl_client_thread_proc(SdlContext* sdl)
 			break;
 		}
 
-		status = WaitForMultipleObjects(nCount, handles, FALSE, INFINITE);
+		const DWORD status = WaitForMultipleObjects(nCount, handles, FALSE, INFINITE);
 
 		if (status == WAIT_FAILED)
 			break;
@@ -1284,18 +1359,26 @@ static DWORD WINAPI sdl_client_thread_proc(SdlContext* sdl)
 	if (exit_code == SDL_EXIT_SUCCESS)
 	{
 		DWORD code = 0;
-		exit_code = error_info_to_error(instance, &code, &error_msg, &error_msg_len);
+		{
+			char* msg = nullptr;
+			size_t len = 0;
+			exit_code = error_info_to_error(instance, &code, &msg, &len);
+			if (msg)
+				error_msg = msg;
+			free(msg);
+		}
 
 		if ((code == ERRINFO_LOGOFF_BY_USER) &&
 		    (freerdp_get_disconnect_ultimatum(context) == Disconnect_Ultimatum_user_requested))
 		{
 			const char* msg = "Error info says user did not initiate but disconnect ultimatum says "
 			                  "they did; treat this as a user logoff";
-			free(error_msg);
-			error_msg = nullptr;
-			error_msg_len = 0;
-			winpr_asprintf(&error_msg, &error_msg_len, "%s", msg);
-
+			char* emsg = nullptr;
+			size_t len = 0;
+			winpr_asprintf(&emsg, &len, "%s", msg);
+			if (emsg)
+				error_msg = emsg;
+			free(emsg);
 			/* This situation might be limited to Windows XP. */
 			WLog_Print(sdl->log, WLOG_INFO, "%s", msg);
 			exit_code = SDL_EXIT_LOGOFF;
@@ -1303,45 +1386,23 @@ static DWORD WINAPI sdl_client_thread_proc(SdlContext* sdl)
 	}
 
 	freerdp_disconnect(instance);
+	return exit_code;
+}
 
-terminate:
-	sdl->rdp_thread_running = false;
-	bool showError = false;
-	if (freerdp_settings_get_bool(settings, FreeRDP_AuthenticationOnly))
-		WLog_Print(sdl->log, WLOG_INFO, "Authentication only, exit status %s [%" PRId32 "]",
-		           sdl_map_to_code_tag(exit_code), exit_code);
-	else
-	{
-		switch (exit_code)
-		{
-			case SDL_EXIT_SUCCESS:
-			case SDL_EXIT_DISCONNECT:
-			case SDL_EXIT_LOGOFF:
-			case SDL_EXIT_DISCONNECT_BY_USER:
-			case SDL_EXIT_CONNECT_CANCELLED:
-				break;
-			default:
-			{
-				std::lock_guard<CriticalSection> lock(sdl->critical);
-				if (sdl->connection_dialog && error_msg)
-				{
-					sdl->connection_dialog->showError(error_msg);
-					showError = true;
-				}
-			}
-			break;
-		}
-	}
-	free(error_msg);
-	if (!showError)
-		sdl_hide_connection_dialog(sdl);
+/* RDP main loop.
+ * Connects RDP, loops while running and handles event and dispatch, cleans up
+ * after the connection ends. */
+static DWORD WINAPI sdl_client_thread_proc(SdlContext* sdl)
+{
+	WINPR_ASSERT(sdl);
 
-	sdl->exit_code = exit_code;
-	sdl_push_user_event(SDL_USEREVENT_QUIT);
-#if SDL_VERSION_ATLEAST(2, 0, 16)
-	SDL_TLSCleanup();
-#endif
-	return 0;
+	std::string error_msg;
+	int exit_code = sdl_client_thread_connect(sdl, error_msg);
+	if (exit_code == SDL_EXIT_SUCCESS)
+		exit_code = sdl_client_thread_run(sdl, error_msg);
+	sdl_client_cleanup(sdl, exit_code, error_msg);
+
+	return static_cast<DWORD>(exit_code);
 }
 
 /* Optional global initializer.
@@ -1350,7 +1411,7 @@ terminate:
 static BOOL sdl_client_global_init()
 {
 #if defined(_WIN32)
-	WSADATA wsaData = { 0 };
+	WSADATA wsaData = {};
 	const DWORD wVersionRequested = MAKEWORD(1, 1);
 	const int rc = WSAStartup(wVersionRequested, &wsaData);
 	if (rc != 0)
@@ -1360,10 +1421,7 @@ static BOOL sdl_client_global_init()
 	}
 #endif
 
-	if (freerdp_handle_signals() != 0)
-		return FALSE;
-
-	return TRUE;
+	return freerdp_handle_signals() != 0;
 }
 
 /* Optional global tear down */
@@ -1407,7 +1465,7 @@ static BOOL sdl_client_new(freerdp* instance, rdpContext* context)
 	return TRUE;
 }
 
-static void sdl_client_free(freerdp* instance, rdpContext* context)
+static void sdl_client_free([[maybe_unused]] freerdp* instance, rdpContext* context)
 {
 	auto sdl = reinterpret_cast<sdl_rdp_context*>(context);
 
@@ -1579,58 +1637,6 @@ static void SDLCALL winpr_LogOutputFunction(void* userdata, int category, SDL_Lo
 	                  category2str(category), message);
 }
 
-static void print_config_file_help()
-{
-#if defined(WITH_WINPR_JSON)
-	std::cout << "CONFIGURATION FILE" << std::endl;
-	std::cout << std::endl;
-	std::cout << "  The SDL client supports some user defined configuration options." << std::endl;
-	std::cout << "  Settings are stored in JSON format" << std::endl;
-	std::cout << "  The location is a per user file. Location for current user is "
-	          << SdlPref::instance()->get_pref_file() << std::endl;
-	std::cout
-	    << "  The XDG_CONFIG_HOME environment variable can be used to override the base directory."
-	    << std::endl;
-	std::cout << std::endl;
-	std::cout << "  The following configuration options are supported:" << std::endl;
-	std::cout << std::endl;
-	std::cout << "    SDL_KeyModMask" << std::endl;
-	std::cout << "      Defines the key combination required for SDL client shortcuts."
-	          << std::endl;
-	std::cout << "      Default KMOD_RSHIFT" << std::endl;
-	std::cout << "      An array of SDL_Keymod strings as defined at "
-	             "https://wiki.libsdl.org/SDL2/SDL_Keymod"
-	          << std::endl;
-	std::cout << std::endl;
-	std::cout << "    SDL_Fullscreen" << std::endl;
-	std::cout << "      Toggles client fullscreen state." << std::endl;
-	std::cout << "      Default SDL_SCANCODE_RETURN." << std::endl;
-	std::cout << "      A string as "
-	             "defined at https://wiki.libsdl.org/SDL2/SDLScancodeLookup"
-	          << std::endl;
-	std::cout << std::endl;
-	std::cout << "    SDL_Resizeable" << std::endl;
-	std::cout << "      Toggles local window resizeable state." << std::endl;
-	std::cout << "      Default SDL_SCANCODE_R." << std::endl;
-	std::cout << "      A string as "
-	             "defined at https://wiki.libsdl.org/SDL2/SDLScancodeLookup"
-	          << std::endl;
-	std::cout << std::endl;
-	std::cout << "    SDL_Grab" << std::endl;
-	std::cout << "      Toggles keyboard and mouse grab state." << std::endl;
-	std::cout << "      Default SDL_SCANCODE_G." << std::endl;
-	std::cout << "      A string as "
-	             "defined at https://wiki.libsdl.org/SDL2/SDLScancodeLookup"
-	          << std::endl;
-	std::cout << std::endl;
-	std::cout << "    SDL_Disconnect" << std::endl;
-	std::cout << "      Disconnects from the RDP session." << std::endl;
-	std::cout << "      Default SDL_SCANCODE_D." << std::endl;
-	std::cout << "      A string as defined at https://wiki.libsdl.org/SDL2/SDLScancodeLookup"
-	          << std::endl;
-#endif
-}
-
 int main(int argc, char* argv[])
 {
 	int rc = -1;
@@ -1638,6 +1644,8 @@ int main(int argc, char* argv[])
 	RDP_CLIENT_ENTRY_POINTS clientEntryPoints = {};
 
 	freerdp_client_warn_experimental(argc, argv);
+	WLog_WARN(SDL_TAG,
+	          "SDL2 client does not support clipboard! Only SDL3 client has (partial) support");
 
 	RdpClientEntry(&clientEntryPoints);
 	std::unique_ptr<sdl_rdp_context, void (*)(sdl_rdp_context*)> sdl_rdp(
@@ -1655,9 +1663,22 @@ int main(int argc, char* argv[])
 	if (status)
 	{
 		rc = freerdp_client_settings_command_line_status_print(settings, status, argc, argv);
-		print_config_file_help();
 		if (freerdp_settings_get_bool(settings, FreeRDP_ListMonitors))
 			sdl_list_monitors(sdl);
+		else
+		{
+			switch (status)
+			{
+				case COMMAND_LINE_STATUS_PRINT:
+				case COMMAND_LINE_STATUS_PRINT_VERSION:
+				case COMMAND_LINE_STATUS_PRINT_BUILDCONFIG:
+					break;
+				case COMMAND_LINE_STATUS_PRINT_HELP:
+				default:
+					SdlPref::print_config_file_help(2);
+					break;
+			}
+		}
 		return rc;
 	}
 
@@ -1679,7 +1700,8 @@ int main(int argc, char* argv[])
 	if (freerdp_client_stop(context) != 0)
 		return -1;
 
-	rc = sdl->exit_code;
+	if (sdl->exit_code != 0)
+		rc = sdl->exit_code;
 
 	return rc;
 }
@@ -1696,6 +1718,12 @@ BOOL SdlContext::update_fullscreen(BOOL enter)
 	return TRUE;
 }
 
+BOOL SdlContext::update_minimize()
+{
+	std::lock_guard<CriticalSection> lock(critical);
+	return sdl_push_user_event(SDL_USEREVENT_WINDOW_MINIMIZE);
+}
+
 BOOL SdlContext::update_resizeable(BOOL enable)
 {
 	std::lock_guard<CriticalSection> lock(critical);
