@@ -25,6 +25,7 @@
 #include <freerdp/config.h>
 
 #include <math.h>
+#include <winpr/cast.h>
 #include <winpr/assert.h>
 #include <winpr/sspicli.h>
 
@@ -101,6 +102,7 @@
 #include "xf_graphics.h"
 #include "xf_keyboard.h"
 #include "xf_channels.h"
+#include "xf_client.h"
 #include "xfreerdp.h"
 #include "xf_utils.h"
 
@@ -168,20 +170,17 @@ static BOOL xf_get_pixmap_info(xfContext* xfc);
 #ifdef WITH_XRENDER
 static void xf_draw_screen_scaled(xfContext* xfc, int x, int y, int w, int h)
 {
-	XTransform transform;
+	XTransform transform = { 0 };
 	Picture windowPicture = 0;
 	Picture primaryPicture = 0;
 	XRenderPictureAttributes pa;
 	XRenderPictFormat* picFormat = NULL;
-	double xScalingFactor = NAN;
-	double yScalingFactor = NAN;
 	int x2 = 0;
 	int y2 = 0;
 	const char* filter = NULL;
-	rdpSettings* settings = NULL;
 	WINPR_ASSERT(xfc);
 
-	settings = xfc->common.context.settings;
+	rdpSettings* settings = xfc->common.context.settings;
 	WINPR_ASSERT(settings);
 
 	if (xfc->scaledWidth <= 0 || xfc->scaledHeight <= 0)
@@ -197,16 +196,22 @@ static void xf_draw_screen_scaled(xfContext* xfc, int x, int y, int w, int h)
 		return;
 	}
 
-	xScalingFactor =
-	    freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth) / (double)xfc->scaledWidth;
-	yScalingFactor =
-	    freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight) / (double)xfc->scaledHeight;
+	const double xScalingFactor = 1.0 *
+	                              freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth) /
+	                              (double)xfc->scaledWidth;
+	const double yScalingFactor = 1.0 *
+	                              freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight) /
+	                              (double)xfc->scaledHeight;
 	XSetFillStyle(xfc->display, xfc->gc, FillSolid);
 	XSetForeground(xfc->display, xfc->gc, 0);
 	/* Black out possible space between desktop and window borders */
 	{
-		XRectangle box1 = { 0, 0, xfc->window->width, xfc->window->height };
-		XRectangle box2 = { xfc->offset_x, xfc->offset_y, xfc->scaledWidth, xfc->scaledHeight };
+		XRectangle box1 = { 0, 0, WINPR_ASSERTING_INT_CAST(UINT16, xfc->window->width),
+			                WINPR_ASSERTING_INT_CAST(UINT16, xfc->window->height) };
+		XRectangle box2 = { WINPR_ASSERTING_INT_CAST(INT16, xfc->offset_x),
+			                WINPR_ASSERTING_INT_CAST(INT16, xfc->offset_y),
+			                WINPR_ASSERTING_INT_CAST(UINT16, xfc->scaledWidth),
+			                WINPR_ASSERTING_INT_CAST(UINT16, xfc->scaledHeight) };
 		Region reg1 = XCreateRegion();
 		Region reg2 = XCreateRegion();
 		XUnionRectWithRegion(&box1, reg1, reg1);
@@ -215,8 +220,9 @@ static void xf_draw_screen_scaled(xfContext* xfc, int x, int y, int w, int h)
 		if (XSubtractRegion(reg1, reg2, reg1) && !XEmptyRegion(reg1))
 		{
 			XSetRegion(xfc->display, xfc->gc, reg1);
-			XFillRectangle(xfc->display, xfc->window->handle, xfc->gc, 0, 0, xfc->window->width,
-			               xfc->window->height);
+			XFillRectangle(xfc->display, xfc->window->handle, xfc->gc, 0, 0,
+			               WINPR_ASSERTING_INT_CAST(UINT16, xfc->window->width),
+			               WINPR_ASSERTING_INT_CAST(UINT16, xfc->window->height));
 			XSetClipMask(xfc->display, xfc->gc, None);
 		}
 
@@ -254,13 +260,19 @@ static void xf_draw_screen_scaled(xfContext* xfc, int x, int y, int w, int h)
 	/* calculate and fix up scaled coordinates */
 	x2 = x + w;
 	y2 = y + h;
-	x = ((int)floor(x / xScalingFactor)) - 1;
-	y = ((int)floor(y / yScalingFactor)) - 1;
-	w = ((int)ceil(x2 / xScalingFactor)) + 1 - x;
-	h = ((int)ceil(y2 / yScalingFactor)) + 1 - y;
+
+	const double dx1 = floor(x / xScalingFactor);
+	const double dy1 = floor(y / yScalingFactor);
+	const double dx2 = ceil(x2 / xScalingFactor);
+	const double dy2 = ceil(y2 / yScalingFactor);
+	x = ((int)dx1) - 1;
+	y = ((int)dy1) - 1;
+	w = ((int)dx2) + 1 - x;
+	h = ((int)dy2) + 1 - y;
 	XRenderSetPictureTransform(xfc->display, primaryPicture, &transform);
 	XRenderComposite(xfc->display, PictOpSrc, primaryPicture, 0, windowPicture, x, y, 0, 0,
-	                 xfc->offset_x + x, xfc->offset_y + y, w, h);
+	                 xfc->offset_x + x, xfc->offset_y + y, WINPR_ASSERTING_INT_CAST(uint32_t, w),
+	                 WINPR_ASSERTING_INT_CAST(uint32_t, h));
 	XRenderFreePicture(xfc->display, primaryPicture);
 	XRenderFreePicture(xfc->display, windowPicture);
 }
@@ -285,8 +297,8 @@ BOOL xf_picture_transform_required(xfContext* xfc)
 }
 #endif /* WITH_XRENDER defined */
 
-void xf_draw_screen_(xfContext* xfc, int x, int y, int w, int h, const char* fkt, const char* file,
-                     int line)
+void xf_draw_screen_(xfContext* xfc, int x, int y, int w, int h, const char* fkt,
+                     WINPR_ATTR_UNUSED const char* file, WINPR_ATTR_UNUSED int line)
 {
 	if (!xfc)
 	{
@@ -315,7 +327,8 @@ void xf_draw_screen_(xfContext* xfc, int x, int y, int w, int h, const char* fkt
 	}
 
 #endif
-	XCopyArea(xfc->display, xfc->primary, xfc->window->handle, xfc->gc, x, y, w, h, x, y);
+	XCopyArea(xfc->display, xfc->primary, xfc->window->handle, xfc->gc, x, y,
+	          WINPR_ASSERTING_INT_CAST(uint32_t, w), WINPR_ASSERTING_INT_CAST(uint32_t, h), x, y);
 }
 
 static BOOL xf_desktop_resize(rdpContext* context)
@@ -334,10 +347,11 @@ static BOOL xf_desktop_resize(rdpContext* context)
 		XFreePixmap(xfc->display, xfc->primary);
 
 		WINPR_ASSERT(xfc->depth != 0);
-		if (!(xfc->primary = XCreatePixmap(
-		          xfc->display, xfc->drawable,
-		          freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
-		          freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight), xfc->depth)))
+		if (!(xfc->primary =
+		          XCreatePixmap(xfc->display, xfc->drawable,
+		                        freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
+		                        freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight),
+		                        WINPR_ASSERTING_INT_CAST(uint32_t, xfc->depth))))
 			return FALSE;
 
 		if (same)
@@ -348,17 +362,22 @@ static BOOL xf_desktop_resize(rdpContext* context)
 
 	if (!freerdp_settings_get_bool(settings, FreeRDP_SmartSizing))
 	{
-		xfc->scaledWidth = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
-		xfc->scaledHeight = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
+		xfc->scaledWidth = WINPR_ASSERTING_INT_CAST(
+		    int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth));
+		xfc->scaledHeight = WINPR_ASSERTING_INT_CAST(
+		    int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
 	}
 
 #endif
 
 	if (!xfc->fullscreen)
 	{
-		xf_ResizeDesktopWindow(xfc, xfc->window,
-		                       freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
-		                       freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
+		xf_ResizeDesktopWindow(
+		    xfc, xfc->window,
+		    WINPR_ASSERTING_INT_CAST(int,
+		                             freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth)),
+		    WINPR_ASSERTING_INT_CAST(int,
+		                             freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight)));
 	}
 	else
 	{
@@ -369,8 +388,10 @@ static BOOL xf_desktop_resize(rdpContext* context)
 		{
 			/* Update the saved width and height values the window will be
 			 * resized to when toggling out of fullscreen */
-			xfc->savedWidth = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
-			xfc->savedHeight = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
+			xfc->savedWidth = WINPR_ASSERTING_INT_CAST(
+			    int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth));
+			xfc->savedHeight = WINPR_ASSERTING_INT_CAST(
+			    int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
 		}
 
 		XSetFunction(xfc->display, xfc->gc, GXcopy);
@@ -391,16 +412,19 @@ static BOOL xf_paint(xfContext* xfc, const GDI_RGN* region)
 
 	if (xfc->remote_app)
 	{
-		const RECTANGLE_16 rect = { .left = region->x,
-			                        .top = region->y,
-			                        .right = region->x + region->w,
-			                        .bottom = region->y + region->h };
+		const RECTANGLE_16 rect = { .left = WINPR_ASSERTING_INT_CAST(UINT16, region->x),
+			                        .top = WINPR_ASSERTING_INT_CAST(UINT16, region->y),
+			                        .right =
+			                            WINPR_ASSERTING_INT_CAST(UINT16, region->x + region->w),
+			                        .bottom =
+			                            WINPR_ASSERTING_INT_CAST(UINT16, region->y + region->h) };
 		xf_rail_paint(xfc, &rect);
 	}
 	else
 	{
 		XPutImage(xfc->display, xfc->primary, xfc->gc, xfc->image, region->x, region->y, region->x,
-		          region->y, region->w, region->h);
+		          region->y, WINPR_ASSERTING_INT_CAST(UINT16, region->w),
+		          WINPR_ASSERTING_INT_CAST(UINT16, region->h));
 		xf_draw_screen(xfc, region->x, region->y, region->w, region->h);
 	}
 	return TRUE;
@@ -473,9 +497,11 @@ static BOOL xf_sw_desktop_resize(rdpContext* context)
 	}
 
 	WINPR_ASSERT(xfc->depth != 0);
-	if (!(xfc->image = XCreateImage(xfc->display, xfc->visual, xfc->depth, ZPixmap, 0,
-	                                (char*)gdi->primary_buffer, gdi->width, gdi->height,
-	                                xfc->scanline_pad, gdi->stride)))
+	if (!(xfc->image = XCreateImage(
+	          xfc->display, xfc->visual, WINPR_ASSERTING_INT_CAST(uint32_t, xfc->depth), ZPixmap, 0,
+	          (char*)gdi->primary_buffer, WINPR_ASSERTING_INT_CAST(uint32_t, gdi->width),
+	          WINPR_ASSERTING_INT_CAST(uint32_t, gdi->height), xfc->scanline_pad,
+	          WINPR_ASSERTING_INT_CAST(int, gdi->stride))))
 	{
 		goto out;
 	}
@@ -541,7 +567,7 @@ static char* xf_window_get_title(rdpSettings* settings)
 	if (!port)
 		(void)sprintf_s(windowTitle, size, "%s %s", prefix, name);
 	else
-		(void)sprintf_s(windowTitle, size, "%s %s:%i", prefix, name,
+		(void)sprintf_s(windowTitle, size, "%s %s:%" PRIu32, prefix, name,
 		                freerdp_settings_get_uint32(settings, FreeRDP_ServerPort));
 
 	return windowTitle;
@@ -557,8 +583,10 @@ BOOL xf_create_window(xfContext* xfc)
 	rdpSettings* settings = xfc->common.context.settings;
 	WINPR_ASSERT(settings);
 
-	int width = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
-	int height = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
+	int width =
+	    WINPR_ASSERTING_INT_CAST(int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth));
+	int height =
+	    WINPR_ASSERTING_INT_CAST(int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
 
 	const XSetWindowAttributes empty = { 0 };
 	xfc->attribs = empty;
@@ -623,10 +651,12 @@ BOOL xf_create_window(xfContext* xfc)
 		if (freerdp_settings_get_bool(settings, FreeRDP_SmartSizing) && !xfc->fullscreen)
 		{
 			if (freerdp_settings_get_uint32(settings, FreeRDP_SmartSizingWidth) > 0)
-				width = freerdp_settings_get_uint32(settings, FreeRDP_SmartSizingWidth);
+				width = WINPR_ASSERTING_INT_CAST(
+				    int, freerdp_settings_get_uint32(settings, FreeRDP_SmartSizingWidth));
 
 			if (freerdp_settings_get_uint32(settings, FreeRDP_SmartSizingHeight) > 0)
-				height = freerdp_settings_get_uint32(settings, FreeRDP_SmartSizingHeight);
+				height = WINPR_ASSERTING_INT_CAST(
+				    int, freerdp_settings_get_uint32(settings, FreeRDP_SmartSizingHeight));
 
 			xfc->scaledWidth = width;
 			xfc->scaledHeight = height;
@@ -663,10 +693,10 @@ BOOL xf_create_window(xfContext* xfc)
 
 	WINPR_ASSERT(xfc->depth != 0);
 	if (!xfc->primary)
-		xfc->primary =
-		    XCreatePixmap(xfc->display, xfc->drawable,
-		                  freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
-		                  freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight), xfc->depth);
+		xfc->primary = XCreatePixmap(xfc->display, xfc->drawable,
+		                             freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
+		                             freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight),
+		                             WINPR_ASSERTING_INT_CAST(uint32_t, xfc->depth));
 
 	xfc->drawing = xfc->primary;
 
@@ -697,11 +727,12 @@ BOOL xf_create_image(xfContext* xfc)
 		WINPR_ASSERT(cgdi);
 
 		WINPR_ASSERT(xfc->depth != 0);
-		xfc->image = XCreateImage(xfc->display, xfc->visual, xfc->depth, ZPixmap, 0,
-		                          (char*)cgdi->primary_buffer,
-		                          freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
-		                          freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight),
-		                          xfc->scanline_pad, cgdi->stride);
+		xfc->image =
+		    XCreateImage(xfc->display, xfc->visual, WINPR_ASSERTING_INT_CAST(uint32_t, xfc->depth),
+		                 ZPixmap, 0, (char*)cgdi->primary_buffer,
+		                 freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
+		                 freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight),
+		                 xfc->scanline_pad, WINPR_ASSERTING_INT_CAST(int, cgdi->stride));
 		xfc->image->byte_order = LSBFirst;
 		xfc->image->bitmap_bit_order = LSBFirst;
 	}
@@ -758,7 +789,7 @@ static void xf_window_free(xfContext* xfc)
 
 void xf_toggle_fullscreen(xfContext* xfc)
 {
-	WindowStateChangeEventArgs e;
+	WindowStateChangeEventArgs e = { 0 };
 	rdpContext* context = (rdpContext*)xfc;
 	rdpSettings* settings = context->settings;
 
@@ -778,33 +809,44 @@ void xf_toggle_fullscreen(xfContext* xfc)
 	PubSub_OnWindowStateChange(context->pubSub, context, &e);
 }
 
-void xf_lock_x11_(xfContext* xfc, const char* fkt)
+void xf_minimize(xfContext* xfc)
 {
+	WindowStateChangeEventArgs e = { 0 };
+	rdpContext* context = (rdpContext*)xfc;
+	WINPR_ASSERT(context);
 
+	/*
+	  when debugging, ungrab keyboard when toggling fullscreen
+	  to allow keyboard usage on the debugger
+	*/
+	if (xfc->debug)
+		xf_ungrab(xfc);
+
+	xf_SetWindowMinimized(xfc, xfc->window);
+
+	e.state = xfc->fullscreen ? FREERDP_WINDOW_STATE_FULLSCREEN : 0;
+	PubSub_OnWindowStateChange(context->pubSub, context, &e);
+}
+
+void xf_lock_x11_(xfContext* xfc, WINPR_ATTR_UNUSED const char* fkt)
+{
 	if (!xfc->UseXThreads)
-		WaitForSingleObject(xfc->mutex, INFINITE);
+		(void)WaitForSingleObject(xfc->mutex, INFINITE);
 	else
 		XLockDisplay(xfc->display);
 
 	xfc->locked++;
-#ifdef WITH_DEBUG_X11
-	WLog_VRB(TAG, "[%" PRIu32 "] from %s", xfc->locked, fkt);
-#endif
 }
 
-void xf_unlock_x11_(xfContext* xfc, const char* fkt)
+void xf_unlock_x11_(xfContext* xfc, WINPR_ATTR_UNUSED const char* fkt)
 {
 	if (xfc->locked == 0)
 		WLog_WARN(TAG, "X11: trying to unlock although not locked!");
 	else
 		xfc->locked--;
 
-#ifdef WITH_DEBUG_X11
-	WLog_VRB(TAG, "[%" PRIu32 "] from %s", xfc->locked, fkt);
-#endif
-
 	if (!xfc->UseXThreads)
-		ReleaseMutex(xfc->mutex);
+		(void)ReleaseMutex(xfc->mutex);
 	else
 		XUnlockDisplay(xfc->display);
 }
@@ -945,10 +987,12 @@ static void xf_get_x11_button_map(xfContext* xfc, unsigned char* x11_map)
 			{
 				for (int i = 0; i < num_devices; ++i)
 				{
-					if ((devices2[i].use == XISlavePointer) &&
-					    (strncmp(devices2[i].name, TEST_PTR_STR, TEST_PTR_LEN) != 0))
+					XIDeviceInfo* dev = &devices2[i];
+					if ((dev->use == XISlavePointer) &&
+					    (strncmp(dev->name, TEST_PTR_STR, TEST_PTR_LEN) != 0))
 					{
-						ptr_dev = XOpenDevice(xfc->display, devices2[i].deviceid);
+						ptr_dev = XOpenDevice(xfc->display,
+						                      WINPR_ASSERTING_INT_CAST(uint32_t, dev->deviceid));
 						if (ptr_dev)
 							break;
 					}
@@ -1019,7 +1063,7 @@ static const button_map xf_button_flags[NUM_BUTTONS_MAPPED] = {
 	{ 112, PTR_XFLAGS_BUTTON2 }
 };
 
-static UINT16 get_flags_for_button(int button)
+static UINT16 get_flags_for_button(size_t button)
 {
 	for (size_t x = 0; x < ARRAYSIZE(xf_button_flags); x++)
 	{
@@ -1032,7 +1076,7 @@ static UINT16 get_flags_for_button(int button)
 	return 0;
 }
 
-static void xf_button_map_init(xfContext* xfc)
+void xf_button_map_init(xfContext* xfc)
 {
 	size_t pos = 0;
 	/* loop counter for array initialization */
@@ -1092,27 +1136,23 @@ static void xf_button_map_init(xfContext* xfc)
  * connection.
  *
  * @param instance - pointer to the rdp_freerdp structure that contains the connection's parameters,
- * and will be filled with the appropriate informations.
+ * and will be filled with the appropriate information.
  *
  * @return TRUE if successful. FALSE otherwise.
  * Can exit with error code XF_EXIT_PARSE_ARGUMENTS if there is an error in the parameters.
  */
 static BOOL xf_pre_connect(freerdp* instance)
 {
-	rdpChannels* channels = NULL;
-	rdpSettings* settings = NULL;
-	rdpContext* context = NULL;
-	xfContext* xfc = NULL;
 	UINT32 maxWidth = 0;
 	UINT32 maxHeight = 0;
 
 	WINPR_ASSERT(instance);
 
-	context = instance->context;
-	xfc = (xfContext*)instance->context;
+	rdpContext* context = instance->context;
 	WINPR_ASSERT(context);
+	xfContext* xfc = (xfContext*)context;
 
-	settings = context->settings;
+	rdpSettings* settings = context->settings;
 	WINPR_ASSERT(settings);
 
 	if (!freerdp_settings_set_bool(settings, FreeRDP_CertificateCallbackPreferPEM, TRUE))
@@ -1124,9 +1164,6 @@ static BOOL xf_pre_connect(freerdp* instance)
 			return FALSE;
 	}
 
-	channels = context->channels;
-	WINPR_ASSERT(channels);
-
 	if (!freerdp_settings_set_uint32(settings, FreeRDP_OsMajorType, OSMAJORTYPE_UNIX))
 		return FALSE;
 	if (!freerdp_settings_set_uint32(settings, FreeRDP_OsMinorType, OSMINORTYPE_NATIVE_XSERVER))
@@ -1165,8 +1202,16 @@ static BOOL xf_pre_connect(freerdp* instance)
 
 	if (!freerdp_settings_get_bool(settings, FreeRDP_AuthenticationOnly))
 	{
+		const char* KeyboardRemappingList = freerdp_settings_get_string(
+		    xfc->common.context.settings, FreeRDP_KeyboardRemappingList);
+
+		xfc->remap_table = freerdp_keyboard_remap_string_to_list(KeyboardRemappingList);
+		if (!xfc->remap_table)
+			return FALSE;
 		if (!xf_keyboard_init(xfc))
 			return FALSE;
+		if (!xf_keyboard_action_script_init(xfc))
+			return FALSE;
 		if (!xf_detect_monitors(xfc, &maxWidth, &maxHeight))
 			return FALSE;
 	}
@@ -1268,7 +1313,7 @@ static BOOL xf_process_pipe(rdpContext* context, const char* pipe)
 		}
 		else
 		{
-			if (!xf_inject_keypress(context, buffer, rd))
+			if (!xf_inject_keypress(context, buffer, WINPR_ASSERTING_INT_CAST(size_t, rd)))
 				break;
 		}
 	}
@@ -1276,7 +1321,8 @@ static BOOL xf_process_pipe(rdpContext* context, const char* pipe)
 	return TRUE;
 }
 
-static void cleanup_pipe(int signum, const char* signame, void* context)
+static void cleanup_pipe(WINPR_ATTR_UNUSED int signum, WINPR_ATTR_UNUSED const char* signame,
+                         void* context)
 {
 	const char* pipe = context;
 	if (!pipe)
@@ -1306,11 +1352,14 @@ static DWORD WINAPI xf_handle_pipe(void* arg)
 		         winpr_strerror(errno, ebuffer, sizeof(ebuffer)), errno);
 		return 0;
 	}
-	freerdp_add_signal_cleanup_handler(pipe, cleanup_pipe);
+
+	void* ctx = WINPR_CAST_CONST_PTR_AWAY(pipe, void*);
+	freerdp_add_signal_cleanup_handler(ctx, cleanup_pipe);
 
 	xf_process_pipe(context, pipe);
 
-	freerdp_del_signal_cleanup_handler(pipe, cleanup_pipe);
+	freerdp_del_signal_cleanup_handler(ctx, cleanup_pipe);
+
 	unlink(pipe);
 	return 0;
 }
@@ -1354,8 +1403,10 @@ static BOOL xf_post_connect(freerdp* instance)
 		return FALSE;
 
 #ifdef WITH_XRENDER
-	xfc->scaledWidth = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
-	xfc->scaledHeight = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
+	xfc->scaledWidth =
+	    WINPR_ASSERTING_INT_CAST(int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth));
+	xfc->scaledHeight =
+	    WINPR_ASSERTING_INT_CAST(int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
 	xfc->offset_x = 0;
 	xfc->offset_y = 0;
 #endif
@@ -1401,8 +1452,10 @@ static BOOL xf_post_connect(freerdp* instance)
 	}
 
 	EventArgsInit(&e, "xfreerdp");
-	e.width = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
-	e.height = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
+	e.width =
+	    WINPR_ASSERTING_INT_CAST(int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth));
+	e.height =
+	    WINPR_ASSERTING_INT_CAST(int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
 	PubSub_OnResizeWindow(context->pubSub, xfc, &e);
 	return TRUE;
 }
@@ -1425,8 +1478,8 @@ static void xf_post_disconnect(freerdp* instance)
 
 	if (xfc->pipethread)
 	{
-		WaitForSingleObject(xfc->pipethread, INFINITE);
-		CloseHandle(xfc->pipethread);
+		(void)WaitForSingleObject(xfc->pipethread, INFINITE);
+		(void)CloseHandle(xfc->pipethread);
 		xfc->pipethread = NULL;
 	}
 	if (xfc->clipboard)
@@ -1446,6 +1499,9 @@ static void xf_post_disconnect(freerdp* instance)
 	else
 		xf_DestroyDummyWindow(xfc, xfc->drawable);
 
+	freerdp_keyboard_remap_free(xfc->remap_table);
+	xfc->remap_table = NULL;
+
 	xf_window_free(xfc);
 }
 
@@ -1521,7 +1577,7 @@ static DWORD WINAPI xf_client_thread(LPVOID param)
 	if (!status)
 	{
 		UINT32 error = freerdp_get_last_error(instance->context);
-		exit_code = xf_map_error_to_exit_code(error);
+		exit_code = (uint32_t)xf_map_error_to_exit_code(error);
 	}
 	else
 		exit_code = XF_EXIT_SUCCESS;
@@ -1618,7 +1674,7 @@ static DWORD WINAPI xf_client_thread(LPVOID param)
 					const UINT32 error = freerdp_get_last_error(instance->context);
 
 					if (freerdp_error_info(instance) == 0)
-						exit_code = xf_map_error_to_exit_code(error);
+						exit_code = (uint32_t)xf_map_error_to_exit_code(error);
 				}
 
 				if (freerdp_get_last_error(context) == FREERDP_ERROR_SUCCESS)
@@ -1655,7 +1711,7 @@ static DWORD WINAPI xf_client_thread(LPVOID param)
 disconnect:
 
 	if (timer)
-		CloseHandle(timer);
+		(void)CloseHandle(timer);
 
 	freerdp_disconnect(instance);
 end:
@@ -1663,11 +1719,11 @@ end:
 	return exit_code;
 }
 
-DWORD xf_exit_code_from_disconnect_reason(DWORD reason)
+int xf_exit_code_from_disconnect_reason(DWORD reason)
 {
 	if (reason == 0 ||
 	    (reason >= XF_EXIT_PARSE_ARGUMENTS && reason <= XF_EXIT_CONNECT_NO_OR_MISSING_CREDENTIALS))
-		return reason;
+		return WINPR_ASSERTING_INT_CAST(int, reason);
 	/* License error set */
 	else if (reason >= 0x100 && reason <= 0x10A)
 		reason -= 0x100 + XF_EXIT_LICENSE_INTERNAL;
@@ -1678,7 +1734,7 @@ DWORD xf_exit_code_from_disconnect_reason(DWORD reason)
 	else if (!(reason <= 0xC))
 		reason = XF_EXIT_UNKNOWN;
 
-	return reason;
+	return WINPR_ASSERTING_INT_CAST(int, reason);
 }
 
 static void xf_TerminateEventHandler(void* context, const TerminateEventArgs* e)
@@ -1718,8 +1774,11 @@ static void xf_ZoomingChangeEventHandler(void* context, const ZoomingChangeEvent
 
 	xfc->scaledWidth = w;
 	xfc->scaledHeight = h;
-	xf_draw_screen(xfc, 0, 0, freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
-	               freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
+	xf_draw_screen(xfc, 0, 0,
+	               WINPR_ASSERTING_INT_CAST(
+	                   int32_t, freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth)),
+	               WINPR_ASSERTING_INT_CAST(
+	                   int32_t, freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight)));
 }
 
 static void xf_PanningChangeEventHandler(void* context, const PanningChangeEventArgs* e)
@@ -1738,8 +1797,11 @@ static void xf_PanningChangeEventHandler(void* context, const PanningChangeEvent
 
 	xfc->offset_x += e->dx;
 	xfc->offset_y += e->dy;
-	xf_draw_screen(xfc, 0, 0, freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
-	               freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
+	xf_draw_screen(xfc, 0, 0,
+	               WINPR_ASSERTING_INT_CAST(
+	                   int32_t, freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth)),
+	               WINPR_ASSERTING_INT_CAST(
+	                   int32_t, freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight)));
 }
 #endif
 
@@ -1749,6 +1811,7 @@ static void xf_PanningChangeEventHandler(void* context, const PanningChangeEvent
 
 static BOOL xfreerdp_client_global_init(void)
 {
+	// NOLINTNEXTLINE(concurrency-mt-unsafe)
 	(void)setlocale(LC_ALL, "");
 
 	if (freerdp_handle_signals() != 0)
@@ -1806,13 +1869,13 @@ void xf_teardown_x11(xfContext* xfc)
 
 	if (xfc->x11event)
 	{
-		CloseHandle(xfc->x11event);
+		(void)CloseHandle(xfc->x11event);
 		xfc->x11event = NULL;
 	}
 
 	if (xfc->mutex)
 	{
-		CloseHandle(xfc->mutex);
+		(void)CloseHandle(xfc->mutex);
 		xfc->mutex = NULL;
 	}
 
@@ -1835,7 +1898,7 @@ BOOL xf_setup_x11(xfContext* xfc)
 	xfc->UseXThreads = TRUE;
 
 #if !defined(NDEBUG)
-	/* uncomment below if debugging to prevent keyboard grap */
+	/* uncomment below if debugging to prevent keyboard grab */
 	xfc->debug = TRUE;
 #endif
 
@@ -1908,15 +1971,27 @@ BOOL xf_setup_x11(xfContext* xfc)
 	    Logging_XInternAtom(xfc->log, xfc->display, "_XWAYLAND_MAY_GRAB_KEYBOARD", False);
 	xfc->_NET_WM_ICON = Logging_XInternAtom(xfc->log, xfc->display, "_NET_WM_ICON", False);
 	xfc->_MOTIF_WM_HINTS = Logging_XInternAtom(xfc->log, xfc->display, "_MOTIF_WM_HINTS", False);
+	xfc->_NET_NUMBER_OF_DESKTOPS =
+	    Logging_XInternAtom(xfc->log, xfc->display, "_NET_NUMBER_OF_DESKTOPS", False);
 	xfc->_NET_CURRENT_DESKTOP =
 	    Logging_XInternAtom(xfc->log, xfc->display, "_NET_CURRENT_DESKTOP", False);
 	xfc->_NET_WORKAREA = Logging_XInternAtom(xfc->log, xfc->display, "_NET_WORKAREA", False);
 	xfc->_NET_WM_STATE = get_supported_atom(xfc, "_NET_WM_STATE");
-	xfc->_NET_WM_STATE_FULLSCREEN = get_supported_atom(xfc, "_NET_WM_STATE_FULLSCREEN");
+	xfc->_NET_WM_STATE_MODAL = get_supported_atom(xfc, "_NET_WM_STATE_MODAL");
+	xfc->_NET_WM_STATE_STICKY = get_supported_atom(xfc, "_NET_WM_STATE_STICKY");
 	xfc->_NET_WM_STATE_MAXIMIZED_HORZ =
 	    Logging_XInternAtom(xfc->log, xfc->display, "_NET_WM_STATE_MAXIMIZED_HORZ", False);
 	xfc->_NET_WM_STATE_MAXIMIZED_VERT =
 	    Logging_XInternAtom(xfc->log, xfc->display, "_NET_WM_STATE_MAXIMIZED_VERT", False);
+	xfc->_NET_WM_STATE_SHADED = get_supported_atom(xfc, "_NET_WM_STATE_SHADED");
+	xfc->_NET_WM_STATE_SKIP_TASKBAR = get_supported_atom(xfc, "_NET_WM_STATE_SKIP_TASKBAR");
+	xfc->_NET_WM_STATE_SKIP_PAGER = get_supported_atom(xfc, "_NET_WM_STATE_SKIP_PAGER");
+	xfc->_NET_WM_STATE_HIDDEN = get_supported_atom(xfc, "_NET_WM_STATE_HIDDEN");
+	xfc->_NET_WM_STATE_FULLSCREEN = get_supported_atom(xfc, "_NET_WM_STATE_FULLSCREEN");
+	xfc->_NET_WM_STATE_ABOVE = get_supported_atom(xfc, "_NET_WM_STATE_ABOVE");
+	xfc->_NET_WM_STATE_BELOW = get_supported_atom(xfc, "_NET_WM_STATE_BELOW");
+	xfc->_NET_WM_STATE_DEMANDS_ATTENTION =
+	    get_supported_atom(xfc, "_NET_WM_STATE_DEMANDS_ATTENTION");
 	xfc->_NET_WM_FULLSCREEN_MONITORS = get_supported_atom(xfc, "_NET_WM_FULLSCREEN_MONITORS");
 	xfc->_NET_WM_NAME = Logging_XInternAtom(xfc->log, xfc->display, "_NET_WM_NAME", False);
 	xfc->_NET_WM_PID = Logging_XInternAtom(xfc->log, xfc->display, "_NET_WM_PID", False);
@@ -2011,7 +2086,7 @@ static BOOL xfreerdp_client_new(freerdp* instance, rdpContext* context)
 	return TRUE;
 }
 
-static void xfreerdp_client_free(freerdp* instance, rdpContext* context)
+static void xfreerdp_client_free(WINPR_ATTR_UNUSED freerdp* instance, rdpContext* context)
 {
 	if (!context)
 		return;
