@@ -24,6 +24,7 @@
 #include <X11/Xutil.h>
 
 #include <string.h>
+#include <math.h>
 
 #include <winpr/assert.h>
 #include <winpr/path.h>
@@ -173,8 +174,8 @@ const char* x11_event_string(int event)
 	} while (0)
 #endif
 
-static BOOL xf_action_script_append(xfContext* xfc, const char* buffer, size_t size, void* user,
-                                    const char* what, const char* arg)
+static BOOL xf_action_script_append(xfContext* xfc, const char* buffer, size_t size,
+                                    WINPR_ATTR_UNUSED void* user, const char* what, const char* arg)
 {
 	WINPR_ASSERT(xfc);
 	WINPR_UNUSED(what);
@@ -193,28 +194,21 @@ static BOOL xf_action_script_append(xfContext* xfc, const char* buffer, size_t s
 
 BOOL xf_event_action_script_init(xfContext* xfc)
 {
-	wObject* obj = NULL;
-	const rdpSettings* settings = NULL;
-
 	WINPR_ASSERT(xfc);
 
-	settings = xfc->common.context.settings;
-	WINPR_ASSERT(settings);
+	xf_event_action_script_free(xfc);
 
 	xfc->xevents = ArrayList_New(TRUE);
 
 	if (!xfc->xevents)
 		return FALSE;
 
-	obj = ArrayList_Object(xfc->xevents);
+	wObject* obj = ArrayList_Object(xfc->xevents);
 	WINPR_ASSERT(obj);
 	obj->fnObjectNew = winpr_ObjectStringClone;
 	obj->fnObjectFree = winpr_ObjectStringFree;
 
-	if (!run_action_script(xfc, "xevent", NULL, xf_action_script_append, NULL))
-		return FALSE;
-
-	return TRUE;
+	return run_action_script(xfc, "xevent", NULL, xf_action_script_append, NULL);
 }
 
 void xf_event_action_script_free(xfContext* xfc)
@@ -307,43 +301,37 @@ static BOOL xf_event_execute_action_script(xfContext* xfc, const XEvent* event)
 	char arg[2048] = { 0 };
 	(void)_snprintf(command, sizeof(command), "xevent %s", xeventName);
 	(void)_snprintf(arg, sizeof(arg), "%lu", (unsigned long)xfc->window->handle);
-	if (!run_action_script(xfc, command, arg, action_script_run, NULL))
-		return FALSE;
-	return TRUE;
+	return run_action_script(xfc, command, arg, action_script_run, NULL);
 }
 
 void xf_adjust_coordinates_to_screen(xfContext* xfc, UINT32* x, UINT32* y)
 {
-	rdpSettings* settings = NULL;
-	INT64 tx = 0;
-	INT64 ty = 0;
-
 	if (!xfc || !xfc->common.context.settings || !y || !x)
 		return;
 
-	settings = xfc->common.context.settings;
-	tx = *x;
-	ty = *y;
+	rdpSettings* settings = xfc->common.context.settings;
+	INT64 tx = *x;
+	INT64 ty = *y;
 	if (!xfc->remote_app)
 	{
 #ifdef WITH_XRENDER
 
 		if (xf_picture_transform_required(xfc))
 		{
-			double xScalingFactor = xfc->scaledWidth / (double)freerdp_settings_get_uint32(
-			                                               settings, FreeRDP_DesktopWidth);
-			double yScalingFactor = xfc->scaledHeight / (double)freerdp_settings_get_uint32(
-			                                                settings, FreeRDP_DesktopHeight);
-			tx = ((tx + xfc->offset_x) * xScalingFactor);
-			ty = ((ty + xfc->offset_y) * yScalingFactor);
+			const double dw = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
+			const double dh = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
+			double xScalingFactor = xfc->scaledWidth / dw;
+			double yScalingFactor = xfc->scaledHeight / dh;
+			tx = (INT64)lround((1.0 * (*x) + xfc->offset_x) * xScalingFactor);
+			ty = (INT64)lround((1.0 * (*y) + xfc->offset_y) * yScalingFactor);
 		}
 
 #endif
 	}
 
 	CLAMP_COORDINATES(tx, ty);
-	*x = tx;
-	*y = ty;
+	*x = (UINT32)tx;
+	*y = (UINT32)ty;
 }
 
 void xf_event_adjust_coordinates(xfContext* xfc, int* x, int* y)
@@ -390,8 +378,10 @@ static BOOL xf_event_Expose(xfContext* xfc, const XExposeEvent* event, BOOL app)
 	{
 		x = 0;
 		y = 0;
-		w = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
-		h = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
+		w = WINPR_ASSERTING_INT_CAST(int,
+		                             freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth));
+		h = WINPR_ASSERTING_INT_CAST(int,
+		                             freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
 	}
 	else
 	{
@@ -405,7 +395,9 @@ static BOOL xf_event_Expose(xfContext* xfc, const XExposeEvent* event, BOOL app)
 	{
 		if (xfc->common.context.gdi->gfx)
 		{
-			xf_OutputExpose(xfc, x, y, w, h);
+			xf_OutputExpose(
+			    xfc, WINPR_ASSERTING_INT_CAST(uint32_t, x), WINPR_ASSERTING_INT_CAST(uint32_t, y),
+			    WINPR_ASSERTING_INT_CAST(uint32_t, w), WINPR_ASSERTING_INT_CAST(uint32_t, h));
 			return TRUE;
 		}
 		xf_draw_screen(xfc, x, y, w, h);
@@ -436,9 +428,6 @@ BOOL xf_generic_MotionNotify(xfContext* xfc, int x, int y, int state, Window win
 	WINPR_ASSERT(xfc);
 	WINPR_ASSERT(xfc->common.context.settings);
 
-	rdpInput* input = xfc->common.context.input;
-	WINPR_ASSERT(input);
-
 	if (!freerdp_settings_get_bool(xfc->common.context.settings, FreeRDP_MouseMotion))
 	{
 		if ((state & (Button1Mask | Button2Mask | Button3Mask)) == 0)
@@ -467,7 +456,8 @@ BOOL xf_generic_MotionNotify(xfContext* xfc, int x, int y, int state, Window win
 	return TRUE;
 }
 
-BOOL xf_generic_RawMotionNotify(xfContext* xfc, int x, int y, Window window, BOOL app)
+BOOL xf_generic_RawMotionNotify(xfContext* xfc, int x, int y, WINPR_ATTR_UNUSED Window window,
+                                BOOL app)
 {
 	WINPR_ASSERT(xfc);
 
@@ -491,7 +481,8 @@ static BOOL xf_event_MotionNotify(xfContext* xfc, const XMotionEvent* event, BOO
 	    (xfc->common.mouse_grabbed && freerdp_client_use_relative_mouse_events(&xfc->common)))
 		return TRUE;
 
-	return xf_generic_MotionNotify(xfc, event->x, event->y, event->state, event->window, app);
+	return xf_generic_MotionNotify(xfc, event->x, event->y,
+	                               WINPR_ASSERTING_INT_CAST(int, event->state), event->window, app);
 }
 
 BOOL xf_generic_ButtonEvent(xfContext* xfc, int x, int y, int button, Window window, BOOL app,
@@ -501,12 +492,14 @@ BOOL xf_generic_ButtonEvent(xfContext* xfc, int x, int y, int button, Window win
 	Window childWindow = None;
 
 	WINPR_ASSERT(xfc);
+	if (button < 0)
+		return FALSE;
 
 	for (size_t i = 0; i < ARRAYSIZE(xfc->button_map); i++)
 	{
 		const button_map* cur = &xfc->button_map[i];
 
-		if (cur->button == button)
+		if (cur->button == (UINT32)button)
 		{
 			flags = cur->flags;
 			break;
@@ -597,7 +590,9 @@ static BOOL xf_event_ButtonPress(xfContext* xfc, const XButtonEvent* event, BOOL
 	if (xfc->xi_event ||
 	    (xfc->common.mouse_grabbed && freerdp_client_use_relative_mouse_events(&xfc->common)))
 		return TRUE;
-	return xf_generic_ButtonEvent(xfc, event->x, event->y, event->button, event->window, app, TRUE);
+	return xf_generic_ButtonEvent(xfc, event->x, event->y,
+	                              WINPR_ASSERTING_INT_CAST(int, event->button), event->window, app,
+	                              TRUE);
 }
 
 static BOOL xf_event_ButtonRelease(xfContext* xfc, const XButtonEvent* event, BOOL app)
@@ -607,7 +602,8 @@ static BOOL xf_event_ButtonRelease(xfContext* xfc, const XButtonEvent* event, BO
 	if (xfc->xi_event ||
 	    (xfc->common.mouse_grabbed && freerdp_client_use_relative_mouse_events(&xfc->common)))
 		return TRUE;
-	return xf_generic_ButtonEvent(xfc, event->x, event->y, event->button, event->window, app,
+	return xf_generic_ButtonEvent(xfc, event->x, event->y,
+	                              WINPR_ASSERTING_INT_CAST(int, event->button), event->window, app,
 	                              FALSE);
 }
 
@@ -725,10 +721,24 @@ static BOOL xf_event_MappingNotify(xfContext* xfc, const XMappingEvent* event, B
 {
 	WINPR_UNUSED(app);
 
-	if (event->request == MappingModifier)
-		return xf_keyboard_update_modifier_map(xfc);
-
-	return TRUE;
+	switch (event->request)
+	{
+		case MappingModifier:
+			return xf_keyboard_update_modifier_map(xfc);
+		case MappingKeyboard:
+			WLog_VRB(TAG, "[%d] MappingKeyboard", event->request);
+			return xf_keyboard_init(xfc);
+		case MappingPointer:
+			WLog_VRB(TAG, "[%d] MappingPointer", event->request);
+			xf_button_map_init(xfc);
+			return TRUE;
+		default:
+			WLog_WARN(TAG,
+			          "[%d] Unsupported MappingNotify::request, must be one "
+			          "of[MappingModifier(%d), MappingKeyboard(%d), MappingPointer(%d)]",
+			          event->request, MappingModifier, MappingKeyboard, MappingPointer);
+			return FALSE;
+	}
 }
 
 static BOOL xf_event_ClientMessage(xfContext* xfc, const XClientMessageEvent* event, BOOL app)
@@ -741,7 +751,7 @@ static BOOL xf_event_ClientMessage(xfContext* xfc, const XClientMessageEvent* ev
 			xfAppWindow* appWindow = xf_AppWindowFromX11Window(xfc, event->window);
 
 			if (appWindow)
-				xf_rail_send_client_system_command(xfc, appWindow->windowId, SC_CLOSE);
+				return xf_rail_send_client_system_command(xfc, appWindow->windowId, SC_CLOSE);
 
 			return TRUE;
 		}
@@ -839,14 +849,19 @@ static BOOL xf_event_ConfigureNotify(xfContext* xfc, const XConfigureEvent* even
 			{
 				xfc->scaledWidth = xfc->window->width;
 				xfc->scaledHeight = xfc->window->height;
-				xf_draw_screen(xfc, 0, 0,
-				               freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth),
-				               freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
+				xf_draw_screen(
+				    xfc, 0, 0,
+				    WINPR_ASSERTING_INT_CAST(
+				        int32_t, freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth)),
+				    WINPR_ASSERTING_INT_CAST(
+				        int32_t, freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight)));
 			}
 			else
 			{
-				xfc->scaledWidth = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
-				xfc->scaledHeight = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
+				xfc->scaledWidth = WINPR_ASSERTING_INT_CAST(
+				    int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth));
+				xfc->scaledHeight = WINPR_ASSERTING_INT_CAST(
+				    int, freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight));
 			}
 
 #endif
@@ -1044,7 +1059,8 @@ static BOOL xf_event_PropertyNotify(xfContext* xfc, const XPropertyEvent* event,
 				if (appWindow->rail_state != WINDOW_SHOW_MAXIMIZED)
 				{
 					appWindow->rail_state = WINDOW_SHOW_MAXIMIZED;
-					xf_rail_send_client_system_command(xfc, appWindow->windowId, SC_MAXIMIZE);
+					return xf_rail_send_client_system_command(xfc, appWindow->windowId,
+					                                          SC_MAXIMIZE);
 				}
 			}
 			else if (appWindow->minimized)
@@ -1052,7 +1068,8 @@ static BOOL xf_event_PropertyNotify(xfContext* xfc, const XPropertyEvent* event,
 				if (appWindow->rail_state != WINDOW_SHOW_MINIMIZED)
 				{
 					appWindow->rail_state = WINDOW_SHOW_MINIMIZED;
-					xf_rail_send_client_system_command(xfc, appWindow->windowId, SC_MINIMIZE);
+					return xf_rail_send_client_system_command(xfc, appWindow->windowId,
+					                                          SC_MINIMIZE);
 				}
 			}
 			else
@@ -1060,7 +1077,7 @@ static BOOL xf_event_PropertyNotify(xfContext* xfc, const XPropertyEvent* event,
 				if (appWindow->rail_state != WINDOW_SHOW && appWindow->rail_state != WINDOW_HIDE)
 				{
 					appWindow->rail_state = WINDOW_SHOW;
-					xf_rail_send_client_system_command(xfc, appWindow->windowId, SC_RESTORE);
+					return xf_rail_send_client_system_command(xfc, appWindow->windowId, SC_RESTORE);
 				}
 			}
 		}
@@ -1155,6 +1172,8 @@ static BOOL xf_event_suppress_events(xfContext* xfc, xfAppWindow* appWindow, con
 		case LMS_TERMINATING:
 			/* Already sent RDP end move to server. Allow events to pass. */
 			break;
+		default:
+			break;
 	}
 
 	return FALSE;
@@ -1311,14 +1330,14 @@ BOOL xf_generic_RawButtonEvent(xfContext* xfc, int button, BOOL app, BOOL down)
 {
 	UINT16 flags = 0;
 
-	if (app)
+	if (app || (button < 0))
 		return FALSE;
 
 	for (size_t i = 0; i < ARRAYSIZE(xfc->button_map); i++)
 	{
 		const button_map* cur = &xfc->button_map[i];
 
-		if (cur->button == button)
+		if (cur->button == (UINT32)button)
 		{
 			flags = cur->flags;
 			break;
