@@ -23,10 +23,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdint.h>
 
 #include <winpr/crt.h>
 #include <winpr/path.h>
 #include <winpr/assert.h>
+#include <winpr/cast.h>
 #include <winpr/collections.h>
 
 #include <freerdp/utils/string.h>
@@ -47,6 +49,26 @@
 #include <freerdp/log.h>
 #define TAG CLIENT_TAG("x11")
 
+typedef struct
+{
+	BOOL Shift;
+	BOOL LeftShift;
+	BOOL RightShift;
+	BOOL Alt;
+	BOOL LeftAlt;
+	BOOL RightAlt;
+	BOOL Ctrl;
+	BOOL LeftCtrl;
+	BOOL RightCtrl;
+	BOOL Super;
+	BOOL LeftSuper;
+	BOOL RightSuper;
+} XF_MODIFIER_KEYS;
+
+static UINT32 xf_keyboard_get_toggle_keys_state(xfContext* xfc);
+static BOOL xf_keyboard_handle_special_keys(xfContext* xfc, KeySym keysym);
+static void xf_keyboard_handle_special_keys_release(xfContext* xfc, KeySym keysym);
+
 static void xf_keyboard_modifier_map_free(xfContext* xfc)
 {
 	WINPR_ASSERT(xfc);
@@ -81,8 +103,8 @@ static void xf_keyboard_clear(xfContext* xfc)
 	ZeroMemory(xfc->KeyboardState, sizeof(xfc->KeyboardState));
 }
 
-static BOOL xf_action_script_append(xfContext* xfc, const char* buffer, size_t size, void* user,
-                                    const char* what, const char* arg)
+static BOOL xf_action_script_append(xfContext* xfc, const char* buffer, size_t size,
+                                    WINPR_ATTR_UNUSED void* user, const char* what, const char* arg)
 {
 	WINPR_ASSERT(xfc);
 	WINPR_UNUSED(what);
@@ -93,30 +115,6 @@ static BOOL xf_action_script_append(xfContext* xfc, const char* buffer, size_t s
 	return ArrayList_Append(xfc->keyCombinations, buffer);
 }
 
-static BOOL xf_keyboard_action_script_init(xfContext* xfc)
-{
-	wObject* obj = NULL;
-	const rdpSettings* settings = NULL;
-
-	settings = xfc->common.context.settings;
-	WINPR_ASSERT(settings);
-
-	xfc->keyCombinations = ArrayList_New(TRUE);
-
-	if (!xfc->keyCombinations)
-		return FALSE;
-
-	obj = ArrayList_Object(xfc->keyCombinations);
-	WINPR_ASSERT(obj);
-	obj->fnObjectNew = winpr_ObjectStringClone;
-	obj->fnObjectFree = winpr_ObjectStringFree;
-
-	if (!run_action_script(xfc, "key", NULL, xf_action_script_append, NULL))
-		return FALSE;
-
-	return xf_event_action_script_init(xfc);
-}
-
 static void xf_keyboard_action_script_free(xfContext* xfc)
 {
 	xf_event_action_script_free(xfc);
@@ -129,6 +127,27 @@ static void xf_keyboard_action_script_free(xfContext* xfc)
 	}
 }
 
+BOOL xf_keyboard_action_script_init(xfContext* xfc)
+{
+	WINPR_ASSERT(xfc);
+
+	xf_keyboard_action_script_free(xfc);
+	xfc->keyCombinations = ArrayList_New(TRUE);
+
+	if (!xfc->keyCombinations)
+		return FALSE;
+
+	wObject* obj = ArrayList_Object(xfc->keyCombinations);
+	WINPR_ASSERT(obj);
+	obj->fnObjectNew = winpr_ObjectStringClone;
+	obj->fnObjectFree = winpr_ObjectStringFree;
+
+	if (!run_action_script(xfc, "key", NULL, xf_action_script_append, NULL))
+		return FALSE;
+
+	return xf_event_action_script_init(xfc);
+}
+
 BOOL xf_keyboard_init(xfContext* xfc)
 {
 	rdpSettings* settings = NULL;
@@ -140,16 +159,11 @@ BOOL xf_keyboard_init(xfContext* xfc)
 
 	xf_keyboard_clear(xfc);
 	xfc->KeyboardLayout = freerdp_settings_get_uint32(settings, FreeRDP_KeyboardLayout);
-	xfc->KeyboardLayout = freerdp_keyboard_init_ex(
-	    xfc->KeyboardLayout, freerdp_settings_get_string(settings, FreeRDP_KeyboardRemappingList));
+	xfc->KeyboardLayout = freerdp_keyboard_init_ex(xfc->KeyboardLayout, NULL);
 	if (!freerdp_settings_set_uint32(settings, FreeRDP_KeyboardLayout, xfc->KeyboardLayout))
 		return FALSE;
 
-	if (!xf_keyboard_update_modifier_map(xfc))
-		return FALSE;
-
-	xf_keyboard_action_script_init(xfc);
-	return TRUE;
+	return xf_keyboard_update_modifier_map(xfc);
 }
 
 void xf_keyboard_free(xfContext* xfc)
@@ -191,11 +205,13 @@ void xf_keyboard_release_all_keypress(xfContext* xfc)
 {
 	WINPR_ASSERT(xfc);
 
+	WINPR_STATIC_ASSERT(ARRAYSIZE(xfc->KeyboardState) <= UINT32_MAX);
 	for (size_t keycode = 0; keycode < ARRAYSIZE(xfc->KeyboardState); keycode++)
 	{
 		if (xfc->KeyboardState[keycode])
 		{
-			const DWORD rdp_scancode = freerdp_keyboard_get_rdp_scancode_from_x11_keycode(keycode);
+			const DWORD rdp_scancode =
+			    freerdp_keyboard_get_rdp_scancode_from_x11_keycode((UINT32)keycode);
 
 			// release tab before releasing the windows key.
 			// this stops the start menu from opening on unfocus event.
@@ -211,25 +227,23 @@ void xf_keyboard_release_all_keypress(xfContext* xfc)
 	xf_sync_kbd_state(xfc);
 }
 
-BOOL xf_keyboard_key_pressed(xfContext* xfc, KeySym keysym)
+static BOOL xf_keyboard_key_pressed(xfContext* xfc, KeySym keysym)
 {
 	KeyCode keycode = XKeysymToKeycode(xfc->display, keysym);
-	WINPR_ASSERT(keycode <= ARRAYSIZE(xfc->KeyboardState));
+	WINPR_ASSERT(keycode < ARRAYSIZE(xfc->KeyboardState));
 	return xfc->KeyboardState[keycode];
 }
 
 void xf_keyboard_send_key(xfContext* xfc, BOOL down, BOOL repeat, const XKeyEvent* event)
 {
-	DWORD rdp_scancode = 0;
-	rdpInput* input = NULL;
-
 	WINPR_ASSERT(xfc);
 	WINPR_ASSERT(event);
 
-	input = xfc->common.context.input;
+	rdpInput* input = xfc->common.context.input;
 	WINPR_ASSERT(input);
 
-	rdp_scancode = freerdp_keyboard_get_rdp_scancode_from_x11_keycode(event->keycode);
+	const DWORD sc = freerdp_keyboard_get_rdp_scancode_from_x11_keycode(event->keycode);
+	const DWORD rdp_scancode = freerdp_keyboard_remap_key(xfc->remap_table, sc);
 	if (rdp_scancode == RDP_SCANCODE_PAUSE && !xf_keyboard_key_pressed(xfc, XK_Control_L) &&
 	    !xf_keyboard_key_pressed(xfc, XK_Control_R))
 	{
@@ -239,7 +253,7 @@ void xf_keyboard_send_key(xfContext* xfc, BOOL down, BOOL repeat, const XKeyEven
 		 */
 		if (down)
 		{
-			freerdp_input_send_keyboard_pause_event(input);
+			(void)freerdp_input_send_keyboard_pause_event(input);
 		}
 	}
 	else
@@ -274,25 +288,33 @@ void xf_keyboard_send_key(xfContext* xfc, BOOL down, BOOL repeat, const XKeyEven
 				if (rdp_scancode == RDP_SCANCODE_UNKNOWN)
 					WLog_ERR(TAG, "Unknown key with X keycode 0x%02" PRIx8 "", event->keycode);
 				else
-					freerdp_input_send_keyboard_event_ex(input, down, repeat, rdp_scancode);
+					(void)freerdp_input_send_keyboard_event_ex(input, down, repeat, rdp_scancode);
 			}
 			else
-				freerdp_input_send_unicode_keyboard_event(input, down ? 0 : KBD_FLAGS_RELEASE,
-				                                          buffer[0]);
+			{
+				char str[3 * ARRAYSIZE(buffer)] = { 0 };
+				// NOLINTNEXTLINE(concurrency-mt-unsafe)
+				const size_t rc = wcstombs(str, buffer, ARRAYSIZE(buffer));
+
+				WCHAR wbuffer[ARRAYSIZE(buffer)] = { 0 };
+				(void)ConvertUtf8ToWChar(str, wbuffer, rc);
+				(void)freerdp_input_send_unicode_keyboard_event(input, down ? 0 : KBD_FLAGS_RELEASE,
+				                                                wbuffer[0]);
+			}
 		}
 		else if (rdp_scancode == RDP_SCANCODE_UNKNOWN)
 			WLog_ERR(TAG, "Unknown key with X keycode 0x%02" PRIx8 "", event->keycode);
 		else
-			freerdp_input_send_keyboard_event_ex(input, down, repeat, rdp_scancode);
+			(void)freerdp_input_send_keyboard_event_ex(input, down, repeat, rdp_scancode);
 
 		if ((rdp_scancode == RDP_SCANCODE_CAPSLOCK) && (down == FALSE))
 		{
-			xf_sync_kbd_state(xfc);
+			(void)xf_sync_kbd_state(xfc);
 		}
 	}
 }
 
-int xf_keyboard_read_keyboard_state(xfContext* xfc)
+static int xf_keyboard_read_keyboard_state(xfContext* xfc)
 {
 	int dummy = 0;
 	Window wdummy = 0;
@@ -309,10 +331,10 @@ int xf_keyboard_read_keyboard_state(xfContext* xfc)
 		              &dummy, &dummy, &dummy, &state);
 	}
 
-	return state;
+	return WINPR_ASSERTING_INT_CAST(int, state);
 }
 
-static int xf_keyboard_get_keymask(xfContext* xfc, int keysym)
+static int xf_keyboard_get_keymask(xfContext* xfc, KeySym keysym)
 {
 	int keysymMask = 0;
 	KeyCode keycode = XKeysymToKeycode(xfc->display, keysym);
@@ -337,7 +359,7 @@ static int xf_keyboard_get_keymask(xfContext* xfc, int keysym)
 	return keysymMask;
 }
 
-BOOL xf_keyboard_get_key_state(xfContext* xfc, int state, int keysym)
+static BOOL xf_keyboard_get_key_state(xfContext* xfc, int state, KeySym keysym)
 {
 	int keysymMask = xf_keyboard_get_keymask(xfc, keysym);
 
@@ -347,7 +369,7 @@ BOOL xf_keyboard_get_key_state(xfContext* xfc, int state, int keysym)
 	return (state & keysymMask) ? TRUE : FALSE;
 }
 
-static BOOL xf_keyboard_set_key_state(xfContext* xfc, BOOL on, int keysym)
+static BOOL xf_keyboard_set_key_state(xfContext* xfc, BOOL on, KeySym keysym)
 {
 	if (!xfc->xkbAvailable)
 		return FALSE;
@@ -359,7 +381,9 @@ static BOOL xf_keyboard_set_key_state(xfContext* xfc, BOOL on, int keysym)
 		return FALSE;
 	}
 
-	return XkbLockModifiers(xfc->display, XkbUseCoreKbd, keysymMask, on ? keysymMask : 0);
+	return XkbLockModifiers(xfc->display, XkbUseCoreKbd,
+	                        WINPR_ASSERTING_INT_CAST(uint32_t, keysymMask),
+	                        on ? WINPR_ASSERTING_INT_CAST(uint32_t, keysymMask) : 0);
 }
 
 UINT32 xf_keyboard_get_toggle_keys_state(xfContext* xfc)
@@ -384,8 +408,8 @@ UINT32 xf_keyboard_get_toggle_keys_state(xfContext* xfc)
 
 static void xk_keyboard_update_modifier_keys(xfContext* xfc)
 {
-	const int keysyms[] = { XK_Shift_L,   XK_Shift_R,   XK_Alt_L,   XK_Alt_R,
-		                    XK_Control_L, XK_Control_R, XK_Super_L, XK_Super_R };
+	const KeySym keysyms[] = { XK_Shift_L,   XK_Shift_R,   XK_Alt_L,   XK_Alt_R,
+		                       XK_Control_L, XK_Control_R, XK_Super_L, XK_Super_R };
 
 	xf_keyboard_clear(xfc);
 
@@ -396,7 +420,7 @@ static void xk_keyboard_update_modifier_keys(xfContext* xfc)
 		if (xf_keyboard_get_key_state(xfc, state, keysyms[i]))
 		{
 			const KeyCode keycode = XKeysymToKeycode(xfc->display, keysyms[i]);
-			WINPR_ASSERT(keycode <= ARRAYSIZE(xfc->KeyboardState));
+			WINPR_ASSERT(keycode < ARRAYSIZE(xfc->KeyboardState));
 			xfc->KeyboardState[keycode] = TRUE;
 		}
 	}
@@ -418,7 +442,7 @@ void xf_keyboard_focus_in(xfContext* xfc)
 	WINPR_ASSERT(input);
 
 	const UINT32 syncFlags = xf_keyboard_get_toggle_keys_state(xfc);
-	freerdp_input_send_focus_in_event(input, syncFlags);
+	freerdp_input_send_focus_in_event(input, WINPR_ASSERTING_INT_CAST(UINT16, syncFlags));
 	xk_keyboard_update_modifier_keys(xfc);
 
 	/* finish with a mouse pointer position like mstsc.exe if required */
@@ -515,7 +539,7 @@ static int xf_keyboard_execute_action_script(xfContext* xfc, XF_MODIFIER_KEYS* m
 	winpr_str_append(keyStr, combination, sizeof(combination), "+");
 
 	for (size_t i = 0; i < strnlen(combination, sizeof(combination)); i++)
-		combination[i] = tolower(combination[i]);
+		combination[i] = WINPR_ASSERTING_INT_CAST(char, tolower(combination[i]));
 
 	const size_t count = ArrayList_Count(xfc->keyCombinations);
 
@@ -588,24 +612,37 @@ BOOL xf_keyboard_handle_special_keys(xfContext* xfc, KeySym keysym)
 
 	if (!xfc->remote_app && xfc->fullscreen_toggle)
 	{
-		if (keysym == XK_Return)
+		switch (keysym)
 		{
-			if (mod.Ctrl && mod.Alt)
-			{
-				/* Ctrl-Alt-Enter: toggle full screen */
-				xf_toggle_fullscreen(xfc);
-				return TRUE;
-			}
+			case XK_Return:
+				if (mod.Ctrl && mod.Alt)
+				{
+					/* Ctrl-Alt-Enter: toggle full screen */
+					xf_toggle_fullscreen(xfc);
+					return TRUE;
+				}
+				break;
+			default:
+				break;
 		}
 	}
 
-	if ((keysym == XK_c) || (keysym == XK_C))
+	if (mod.Ctrl && mod.Alt)
 	{
-		if (mod.Ctrl && mod.Alt)
+		switch (keysym)
 		{
-			/* Ctrl-Alt-C: toggle control */
-			if (freerdp_client_encomsp_toggle_control(xfc->common.encomsp))
+			case XK_m:
+			case XK_M:
+				xf_minimize(xfc);
 				return TRUE;
+			case XK_c:
+			case XK_C:
+				/* Ctrl-Alt-C: toggle control */
+				if (freerdp_client_encomsp_toggle_control(xfc->common.encomsp))
+					return TRUE;
+				break;
+			default:
+				break;
 		}
 	}
 
