@@ -34,15 +34,15 @@ static const DWORD log_level = WLOG_TRACE;
 
 static void write_log(wLog* log, DWORD level, const char* fname, const char* fkt, size_t line, ...)
 {
-	va_list ap;
+	va_list ap = { 0 };
 	va_start(ap, line);
 	WLog_PrintMessageVA(log, WLOG_MESSAGE_TEXT, level, line, fname, fkt, ap);
 	va_end(ap);
 }
 
-char* Safe_XGetAtomNameEx(wLog* log, Display* display, Atom atom, const char* atomvar)
+char* Safe_XGetAtomNameEx(wLog* log, Display* display, Atom atom, const char* varname)
 {
-	WLog_Print(log, log_level, "XGetAtomName(%s, 0x%08" PRIx32 ")", atomvar, atom);
+	WLog_Print(log, log_level, "XGetAtomName(%s, 0x%08" PRIx32 ")", varname, atom);
 	if (atom == None)
 		return strdup("Atom_None");
 	return XGetAtomName(display, atom);
@@ -172,6 +172,7 @@ int LogDynAndXGetWindowProperty_ex(wLog* log, const char* file, const char* fkt,
 
 BOOL IsGnome(void)
 {
+	// NOLINTNEXTLINE(concurrency-mt-unsafe)
 	char* env = getenv("DESKTOP_SESSION");
 	return (env != NULL && strcmp(env, "gnome") == 0);
 }
@@ -191,7 +192,10 @@ BOOL run_action_script(xfContext* xfc, const char* what, const char* arg, fn_act
 	xfc->actionScriptExists = winpr_PathFileExists(ActionScript);
 
 	if (!xfc->actionScriptExists)
+	{
+		WLog_WARN(TAG, "[ActionScript] no such script '%s'", ActionScript);
 		goto fail;
+	}
 
 	char command[2048] = { 0 };
 	(void)sprintf_s(command, sizeof(command), "%s %s", ActionScript, what);
@@ -220,9 +224,20 @@ BOOL run_action_script(xfContext* xfc, const char* what, const char* arg, fn_act
 
 	rc = read_data;
 fail:
-	if (!rc)
-		xfc->actionScriptExists = FALSE;
 	if (keyScript)
 		pclose(keyScript);
-	return rc;
+	const BOOL res = rc || !xfc->actionScriptExists;
+	if (!rc)
+		xfc->actionScriptExists = FALSE;
+	return res;
+}
+
+const char* x11_error_to_string(xfContext* xfc, int error, char* buffer, size_t size)
+{
+	WINPR_ASSERT(xfc);
+	WINPR_ASSERT(size <= INT32_MAX);
+	const int rc = XGetErrorText(xfc->display, error, buffer, (int)size);
+	if (rc)
+		WLog_WARN(TAG, "XGetErrorText returned %d", rc);
+	return buffer;
 }
