@@ -60,7 +60,7 @@ struct S_FREERDP_DSP_CONTEXT
 #endif
 };
 
-static BOOL ffmpeg_codec_is_filtered(enum AVCodecID id, BOOL encoder)
+static BOOL ffmpeg_codec_is_filtered(enum AVCodecID id, WINPR_ATTR_UNUSED BOOL encoder)
 {
 	switch (id)
 	{
@@ -70,19 +70,17 @@ static BOOL ffmpeg_codec_is_filtered(enum AVCodecID id, BOOL encoder)
 		case AV_CODEC_ID_MP3:
 		case AV_CODEC_ID_ADPCM_MS:
 		case AV_CODEC_ID_G723_1:
+		case AV_CODEC_ID_GSM_MS:
+		case AV_CODEC_ID_PCM_ALAW:
+		case AV_CODEC_ID_PCM_MULAW:
 			return TRUE;
 #endif
 
 		case AV_CODEC_ID_NONE:
 			return TRUE;
 
-		case AV_CODEC_ID_GSM_MS:
 		case AV_CODEC_ID_AAC:
 		case AV_CODEC_ID_AAC_LATM:
-#if !defined(WITH_DSP_EXPERIMENTAL)
-			if (encoder)
-				return TRUE;
-#endif
 			return FALSE;
 
 		default:
@@ -277,7 +275,7 @@ static BOOL ffmpeg_open_context(FREERDP_DSP_CONTEXT* WINPR_RESTRICT context)
 #endif
 	context->context->sample_rate = (int)format->nSamplesPerSec;
 	context->context->block_align = format->nBlockAlign;
-	context->context->bit_rate = format->nAvgBytesPerSec * 8;
+	context->context->bit_rate = format->nAvgBytesPerSec * 8LL;
 	context->context->sample_fmt = ffmpeg_sample_format(format);
 	context->context->time_base = av_make_q(1, context->context->sample_rate);
 
@@ -360,7 +358,8 @@ static BOOL ffmpeg_open_context(FREERDP_DSP_CONTEXT* WINPR_RESTRICT context)
 		context->buffered->format = context->resampled->format;
 		context->buffered->nb_samples = context->context->frame_size;
 
-		if ((ret = av_frame_get_buffer(context->buffered, 1)) < 0)
+		ret = av_frame_get_buffer(context->buffered, 1);
+		if (ret < 0)
 			goto fail;
 	}
 
@@ -478,13 +477,14 @@ static BOOL ffmpeg_encode_frame(AVCodecContext* WINPR_RESTRICT context, AVFrame*
 	}
 
 	/* read all the output frames (in general there may be any number of them */
-	while (ret >= 0)
+	while (TRUE)
 	{
 		ret = avcodec_receive_packet(context, packet);
 
 		if ((ret == AVERROR(EAGAIN)) || (ret == AVERROR_EOF))
-			return TRUE;
-		else if (ret < 0)
+			break;
+
+		if (ret < 0)
 		{
 			const char* err = av_err2str(ret);
 			WLog_ERR(TAG, "Error during encoding %s [%d]", err, ret);
@@ -655,7 +655,10 @@ FREERDP_DSP_CONTEXT* freerdp_dsp_ffmpeg_context_new(BOOL encode)
 	return context;
 
 fail:
+	WINPR_PRAGMA_DIAG_PUSH
+	WINPR_PRAGMA_DIAG_IGNORED_MISMATCHED_DEALLOC
 	freerdp_dsp_ffmpeg_context_free(context);
+	WINPR_PRAGMA_DIAG_POP
 	return NULL;
 }
 
@@ -734,7 +737,7 @@ static BOOL freerdp_dsp_channel_mix(FREERDP_DSP_CONTEXT* WINPR_RESTRICT context,
 
 			case 2:  /* We only support stereo, so we can not handle this case. */
 			default: /* Unsupported number of channels */
-				WLog_WARN(TAG, "[%s] unsuported source channel count %" PRIu16, __func__,
+				WLog_WARN(TAG, "[%s] unsupported source channel count %" PRIu16, __func__,
 				          srcFormat->nChannels);
 				return FALSE;
 		}
@@ -763,7 +766,7 @@ static BOOL freerdp_dsp_channel_mix(FREERDP_DSP_CONTEXT* WINPR_RESTRICT context,
 
 		case 1:  /* Invalid, do we want to use a 0 channel sound? */
 		default: /* Unsupported number of channels */
-			WLog_WARN(TAG, "[%s] unsuported channel count %" PRIu16, __func__,
+			WLog_WARN(TAG, "[%s] unsupported channel count %" PRIu16, __func__,
 			          srcFormat->nChannels);
 			return FALSE;
 	}
@@ -773,22 +776,23 @@ static BOOL freerdp_dsp_channel_mix(FREERDP_DSP_CONTEXT* WINPR_RESTRICT context,
 
 BOOL freerdp_dsp_ffmpeg_encode(FREERDP_DSP_CONTEXT* WINPR_RESTRICT context,
                                const AUDIO_FORMAT* WINPR_RESTRICT format,
-                               const BYTE* WINPR_RESTRICT data, size_t length,
+                               const BYTE* WINPR_RESTRICT sdata, size_t length,
                                wStream* WINPR_RESTRICT out)
 {
 	AUDIO_FORMAT fmt = { 0 };
 
-	if (!context || !format || !data || !out || !context->common.encoder)
+	if (!context || !format || !sdata || !out || !context->common.encoder)
 		return FALSE;
 
-	if (!context || !data || !out)
+	if (!context || !sdata || !out)
 		return FALSE;
 
 	/* https://github.com/FreeRDP/FreeRDP/issues/7607
 	 *
 	 * we get noisy data with channel transformation, so do it ourselves.
 	 */
-	if (!freerdp_dsp_channel_mix(context, data, length, format, &data, &length, &fmt))
+	const BYTE* data = NULL;
+	if (!freerdp_dsp_channel_mix(context, sdata, length, format, &data, &length, &fmt))
 		return FALSE;
 
 	/* Create input frame */
@@ -858,7 +862,7 @@ BOOL freerdp_dsp_ffmpeg_decode(FREERDP_DSP_CONTEXT* WINPR_RESTRICT context,
 #if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(58, 133, 100)
 	av_init_packet(context->packet);
 #endif
-	context->packet->data = (uint8_t*)data;
+	context->packet->data = WINPR_CAST_CONST_PTR_AWAY(data, uint8_t*);
 
 	WINPR_ASSERT(length <= INT_MAX);
 	context->packet->size = (int)length;
