@@ -65,7 +65,7 @@ static inline char* error_string(char* errbuf, size_t errbuf_size, int errnum)
 #define av_err2str(errnum) error_string((char[64]){ 0 }, 64, errnum)
 #endif
 
-#ifdef WITH_VAAPI
+#if defined(WITH_VAAPI) || defined(WITH_VAAPI_H264_ENCODING)
 #define VAAPI_DEVICE "/dev/dri/renderD128"
 #endif
 
@@ -81,7 +81,7 @@ typedef struct
 	AVPacket bufferpacket;
 #endif
 	AVPacket* packet;
-#ifdef WITH_VAAPI
+#if defined(WITH_VAAPI) || defined(WITH_VAAPI_H264_ENCODING)
 	AVBufferRef* hwctx;
 	AVFrame* hwVideoFrame;
 	enum AVPixelFormat hw_pix_fmt;
@@ -91,9 +91,9 @@ typedef struct
 #endif
 } H264_CONTEXT_LIBAVCODEC;
 
-static void libavcodec_destroy_encoder(H264_CONTEXT* h264)
+static void libavcodec_destroy_encoder_context(H264_CONTEXT* WINPR_RESTRICT h264)
 {
-	H264_CONTEXT_LIBAVCODEC* sys;
+	H264_CONTEXT_LIBAVCODEC* sys = NULL;
 
 	if (!h264 || !h264->subsystem)
 		return;
@@ -102,22 +102,58 @@ static void libavcodec_destroy_encoder(H264_CONTEXT* h264)
 
 	if (sys->codecEncoderContext)
 	{
-		avcodec_close(sys->codecEncoderContext);
 #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55, 69, 100)
 		avcodec_free_context(&sys->codecEncoderContext);
 #else
+		avcodec_close(sys->codecEncoderContext);
 		av_free(sys->codecEncoderContext);
 #endif
 	}
 
-	sys->codecEncoder = NULL;
 	sys->codecEncoderContext = NULL;
 }
 
-static BOOL libavcodec_create_encoder(H264_CONTEXT* h264)
+#ifdef WITH_VAAPI_H264_ENCODING
+static int set_hw_frames_ctx(H264_CONTEXT* WINPR_RESTRICT h264)
+{
+	H264_CONTEXT_LIBAVCODEC* sys = (H264_CONTEXT_LIBAVCODEC*)h264->pSystemData;
+	AVBufferRef* hw_frames_ref = NULL;
+	AVHWFramesContext* frames_ctx = NULL;
+	int err = 0;
+
+	if (!(hw_frames_ref = av_hwframe_ctx_alloc(sys->hwctx)))
+	{
+		WLog_Print(h264->log, WLOG_ERROR, "Failed to create VAAPI frame context");
+		return -1;
+	}
+	frames_ctx = (AVHWFramesContext*)(hw_frames_ref->data);
+	frames_ctx->format = AV_PIX_FMT_VAAPI;
+	frames_ctx->sw_format = AV_PIX_FMT_NV12;
+	frames_ctx->width = sys->codecEncoderContext->width;
+	frames_ctx->height = sys->codecEncoderContext->height;
+	frames_ctx->initial_pool_size = 20;
+	if ((err = av_hwframe_ctx_init(hw_frames_ref)) < 0)
+	{
+		WLog_Print(h264->log, WLOG_ERROR,
+		           "Failed to initialize VAAPI frame context."
+		           "Error code: %s",
+		           av_err2str(err));
+		av_buffer_unref(&hw_frames_ref);
+		return err;
+	}
+	sys->codecEncoderContext->hw_frames_ctx = av_buffer_ref(hw_frames_ref);
+	if (!sys->codecEncoderContext->hw_frames_ctx)
+		err = AVERROR(ENOMEM);
+
+	av_buffer_unref(&hw_frames_ref);
+	return err;
+}
+#endif
+
+static BOOL libavcodec_create_encoder_context(H264_CONTEXT* WINPR_RESTRICT h264)
 {
 	BOOL recreate = FALSE;
-	H264_CONTEXT_LIBAVCODEC* sys;
+	H264_CONTEXT_LIBAVCODEC* sys = NULL;
 
 	if (!h264 || !h264->subsystem)
 		return FALSE;
@@ -126,9 +162,10 @@ static BOOL libavcodec_create_encoder(H264_CONTEXT* h264)
 		return FALSE;
 
 	sys = (H264_CONTEXT_LIBAVCODEC*)h264->pSystemData;
-	if (!sys)
+	if (!sys || !sys->codecEncoder)
 		return FALSE;
-	recreate = !sys->codecEncoder || !sys->codecEncoderContext;
+
+	recreate = !sys->codecEncoderContext;
 
 	if (sys->codecEncoderContext)
 	{
@@ -140,11 +177,7 @@ static BOOL libavcodec_create_encoder(H264_CONTEXT* h264)
 	if (!recreate)
 		return TRUE;
 
-	libavcodec_destroy_encoder(h264);
-	sys->codecEncoder = avcodec_find_encoder(AV_CODEC_ID_H264);
-
-	if (!sys->codecEncoder)
-		goto EXCEPTION;
+	libavcodec_destroy_encoder_context(h264);
 
 	sys->codecEncoderContext = avcodec_alloc_context3(sys->codecEncoder);
 
@@ -158,7 +191,11 @@ static BOOL libavcodec_create_encoder(H264_CONTEXT* h264)
 			break;
 
 		case H264_RATECONTROL_CQP:
-			/* TODO: sys->codecEncoderContext-> = h264->QP; */
+			if (av_opt_set_int(sys->codecEncoderContext, "qp", h264->QP, AV_OPT_SEARCH_CHILDREN) <
+			    0)
+			{
+				WLog_Print(h264->log, WLOG_ERROR, "av_opt_set_int failed");
+			}
 			break;
 
 		default:
@@ -169,24 +206,43 @@ static BOOL libavcodec_create_encoder(H264_CONTEXT* h264)
 	sys->codecEncoderContext->height = (int)MIN(INT32_MAX, h264->height);
 	sys->codecEncoderContext->delay = 0;
 #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(56, 13, 100)
-	sys->codecEncoderContext->framerate = (AVRational){ h264->FrameRate, 1 };
+	sys->codecEncoderContext->framerate =
+	    (AVRational){ WINPR_ASSERTING_INT_CAST(int, h264->FrameRate), 1 };
 #endif
-	sys->codecEncoderContext->time_base = (AVRational){ 1, h264->FrameRate };
-	av_opt_set(sys->codecEncoderContext, "preset", "medium", AV_OPT_SEARCH_CHILDREN);
+	sys->codecEncoderContext->time_base =
+	    (AVRational){ 1, WINPR_ASSERTING_INT_CAST(int, h264->FrameRate) };
 	av_opt_set(sys->codecEncoderContext, "tune", "zerolatency", AV_OPT_SEARCH_CHILDREN);
+
 	sys->codecEncoderContext->flags |= AV_CODEC_FLAG_LOOP_FILTER;
-	sys->codecEncoderContext->pix_fmt = AV_PIX_FMT_YUV420P;
+
+#ifdef WITH_VAAPI_H264_ENCODING
+	if (sys->hwctx)
+	{
+		av_opt_set(sys->codecEncoderContext, "preset", "veryslow", AV_OPT_SEARCH_CHILDREN);
+
+		sys->codecEncoderContext->pix_fmt = AV_PIX_FMT_VAAPI;
+		/* set hw_frames_ctx for encoder's AVCodecContext */
+		if (set_hw_frames_ctx(h264) < 0)
+			goto EXCEPTION;
+	}
+	else
+#endif
+	{
+		av_opt_set(sys->codecEncoderContext, "preset", "medium", AV_OPT_SEARCH_CHILDREN);
+		sys->codecEncoderContext->pix_fmt = AV_PIX_FMT_YUV420P;
+	}
 
 	if (avcodec_open2(sys->codecEncoderContext, sys->codecEncoder, NULL) < 0)
 		goto EXCEPTION;
 
 	return TRUE;
 EXCEPTION:
-	libavcodec_destroy_encoder(h264);
+	libavcodec_destroy_encoder_context(h264);
 	return FALSE;
 }
 
-static int libavcodec_decompress(H264_CONTEXT* h264, const BYTE* pSrcData, UINT32 SrcSize)
+static int libavcodec_decompress(H264_CONTEXT* WINPR_RESTRICT h264,
+                                 const BYTE* WINPR_RESTRICT pSrcData, UINT32 SrcSize)
 {
 	union
 	{
@@ -194,7 +250,7 @@ static int libavcodec_decompress(H264_CONTEXT* h264, const BYTE* pSrcData, UINT3
 		BYTE* pv;
 	} cnv;
 	int rc = -1;
-	int status;
+	int status = 0;
 	int gotFrame = 0;
 	AVPacket* packet = NULL;
 
@@ -237,15 +293,17 @@ static int libavcodec_decompress(H264_CONTEXT* h264, const BYTE* pSrcData, UINT3
 
 	sys->videoFrame->format = AV_PIX_FMT_YUV420P;
 
-	do
-	{
 #ifdef WITH_VAAPI
 		status = avcodec_receive_frame(sys->codecDecoderContext,
 		                               sys->hwctx ? sys->hwVideoFrame : sys->videoFrame);
 #else
 		status = avcodec_receive_frame(sys->codecDecoderContext, sys->videoFrame);
 #endif
-	} while (status == AVERROR(EAGAIN));
+	    if (status == AVERROR(EAGAIN))
+	    {
+		    rc = 0;
+		    goto fail;
+	    }
 
 	gotFrame = (status == 0);
 #else
@@ -324,8 +382,10 @@ fail:
 	return rc;
 }
 
-static int libavcodec_compress(H264_CONTEXT* h264, const BYTE** pSrcYuv, const UINT32* pStride,
-                               BYTE** ppDstData, UINT32* pDstSize)
+static int libavcodec_compress(H264_CONTEXT* WINPR_RESTRICT h264,
+                               const BYTE** WINPR_RESTRICT pSrcYuv,
+                               const UINT32* WINPR_RESTRICT pStride,
+                               BYTE** WINPR_RESTRICT ppDstData, UINT32* WINPR_RESTRICT pDstSize)
 {
 	union
 	{
@@ -333,7 +393,7 @@ static int libavcodec_compress(H264_CONTEXT* h264, const BYTE** pSrcYuv, const U
 		uint8_t* pv;
 	} cnv;
 	int rc = -1;
-	int status;
+	int status = 0;
 	int gotFrame = 0;
 
 	WINPR_ASSERT(h264);
@@ -341,7 +401,7 @@ static int libavcodec_compress(H264_CONTEXT* h264, const BYTE** pSrcYuv, const U
 	H264_CONTEXT_LIBAVCODEC* sys = (H264_CONTEXT_LIBAVCODEC*)h264->pSystemData;
 	WINPR_ASSERT(sys);
 
-	if (!libavcodec_create_encoder(h264))
+	if (!libavcodec_create_encoder_context(h264))
 		return -1;
 
 #if LIBAVCODEC_VERSION_INT < AV_VERSION_INT(58, 133, 100)
@@ -364,7 +424,7 @@ static int libavcodec_compress(H264_CONTEXT* h264, const BYTE** pSrcYuv, const U
 
 	WINPR_ASSERT(sys->videoFrame);
 	WINPR_ASSERT(sys->codecEncoderContext);
-	sys->videoFrame->format = sys->codecEncoderContext->pix_fmt;
+	sys->videoFrame->format = AV_PIX_FMT_YUV420P;
 	sys->videoFrame->width = sys->codecEncoderContext->width;
 	sys->videoFrame->height = sys->codecEncoderContext->height;
 #if LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(52, 48, 100)
@@ -386,9 +446,37 @@ static int libavcodec_compress(H264_CONTEXT* h264, const BYTE** pSrcYuv, const U
 	sys->videoFrame->linesize[1] = (int)pStride[1];
 	sys->videoFrame->linesize[2] = (int)pStride[2];
 	sys->videoFrame->pts++;
+
+#ifdef WITH_VAAPI_H264_ENCODING
+	if (sys->hwctx)
+	{
+		av_frame_unref(sys->hwVideoFrame);
+		if ((status = av_hwframe_get_buffer(sys->codecEncoderContext->hw_frames_ctx,
+		                                    sys->hwVideoFrame, 0)) < 0 ||
+		    !sys->hwVideoFrame->hw_frames_ctx)
+		{
+			WLog_Print(h264->log, WLOG_ERROR, "av_hwframe_get_buffer failed (%s [%d])",
+			           av_err2str(status), status);
+			goto fail;
+		}
+		sys->videoFrame->format = AV_PIX_FMT_NV12;
+		if ((status = av_hwframe_transfer_data(sys->hwVideoFrame, sys->videoFrame, 0)) < 0)
+		{
+			WLog_Print(h264->log, WLOG_ERROR, "av_hwframe_transfer_data failed (%s [%d])",
+			           av_err2str(status), status);
+			goto fail;
+		}
+	}
+#endif
+
 	/* avcodec_encode_video2 is deprecated with libavcodec 57.48.101 */
 #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 48, 101)
+#ifdef WITH_VAAPI_H264_ENCODING
+	status = avcodec_send_frame(sys->codecEncoderContext,
+	                            sys->hwctx ? sys->hwVideoFrame : sys->videoFrame);
+#else
 	status = avcodec_send_frame(sys->codecEncoderContext, sys->videoFrame);
+#endif
 
 	if (status < 0)
 	{
@@ -481,8 +569,7 @@ static void libavcodec_uninit(H264_CONTEXT* h264)
 #endif
 	}
 
-#ifdef WITH_VAAPI
-
+#if defined(WITH_VAAPI) || defined(WITH_VAAPI_H264_ENCODING)
 	if (sys->hwVideoFrame)
 	{
 #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55, 18, 102)
@@ -501,6 +588,7 @@ static void libavcodec_uninit(H264_CONTEXT* h264)
 		av_buffer_unref(&sys->hw_frames_ctx);
 
 #endif
+
 #endif
 
 	if (sys->codecParser)
@@ -508,15 +596,15 @@ static void libavcodec_uninit(H264_CONTEXT* h264)
 
 	if (sys->codecDecoderContext)
 	{
-		avcodec_close(sys->codecDecoderContext);
 #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55, 69, 100)
 		avcodec_free_context(&sys->codecDecoderContext);
 #else
+		avcodec_close(sys->codecDecoderContext);
 		av_free(sys->codecDecoderContext);
 #endif
 	}
 
-	libavcodec_destroy_encoder(h264);
+	libavcodec_destroy_encoder_context(h264);
 	free(sys);
 	h264->pSystemData = NULL;
 }
@@ -533,9 +621,7 @@ static enum AVPixelFormat libavcodec_get_format(struct AVCodecContext* ctx,
 	H264_CONTEXT_LIBAVCODEC* sys = (H264_CONTEXT_LIBAVCODEC*)h264->pSystemData;
 	WINPR_ASSERT(sys);
 
-	const enum AVPixelFormat* p;
-
-	for (p = fmts; *p != AV_PIX_FMT_NONE; p++)
+	for (const enum AVPixelFormat* p = fmts; *p != AV_PIX_FMT_NONE; p++)
 	{
 		if (*p == sys->hw_pix_fmt)
 		{
@@ -581,7 +667,7 @@ static enum AVPixelFormat libavcodec_get_format(struct AVCodecContext* ctx,
 
 static BOOL libavcodec_init(H264_CONTEXT* h264)
 {
-	H264_CONTEXT_LIBAVCODEC* sys;
+	H264_CONTEXT_LIBAVCODEC* sys = NULL;
 
 	WINPR_ASSERT(h264);
 	sys = (H264_CONTEXT_LIBAVCODEC*)calloc(1, sizeof(H264_CONTEXT_LIBAVCODEC));
@@ -638,6 +724,7 @@ static BOOL libavcodec_init(H264_CONTEXT* h264)
 				goto fail_hwdevice_create;
 			}
 		}
+		WLog_Print(h264->log, WLOG_INFO, "Using VAAPI for accelerated H264 decoding");
 
 		sys->codecDecoderContext->get_format = libavcodec_get_format;
 		sys->hw_pix_fmt = AV_PIX_FMT_VAAPI;
@@ -662,10 +749,45 @@ static BOOL libavcodec_init(H264_CONTEXT* h264)
 			goto EXCEPTION;
 		}
 	}
+	else
+	{
+#ifdef WITH_VAAPI_H264_ENCODING
+		if (h264->hwAccel) /* user requested hw accel */
+		{
+			sys->codecEncoder = avcodec_find_encoder_by_name("h264_vaapi");
+			if (!sys->codecEncoder)
+			{
+				WLog_Print(h264->log, WLOG_ERROR, "H264 VAAPI encoder not found");
+			}
+			else if (av_hwdevice_ctx_create(&sys->hwctx, AV_HWDEVICE_TYPE_VAAPI, VAAPI_DEVICE, NULL,
+			                                0) < 0)
+			{
+				WLog_Print(h264->log, WLOG_ERROR, "av_hwdevice_ctx_create failed");
+				sys->codecEncoder = NULL;
+				sys->hwctx = NULL;
+			}
+			else
+			{
+				WLog_Print(h264->log, WLOG_INFO, "Using VAAPI for accelerated H264 encoding");
+			}
+		}
+#endif
+		if (!sys->codecEncoder)
+		{
+			sys->codecEncoder = avcodec_find_encoder(AV_CODEC_ID_H264);
+			h264->hwAccel = FALSE; /* not supported */
+		}
+
+		if (!sys->codecEncoder)
+		{
+			WLog_Print(h264->log, WLOG_ERROR, "Failed to initialize H264 encoder");
+			goto EXCEPTION;
+		}
+	}
 
 #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(55, 18, 102)
 	sys->videoFrame = av_frame_alloc();
-#ifdef WITH_VAAPI
+#if defined(WITH_VAAPI) || defined(WITH_VAAPI_H264_ENCODING)
 	sys->hwVideoFrame = av_frame_alloc();
 #endif
 #else
@@ -678,8 +800,7 @@ static BOOL libavcodec_init(H264_CONTEXT* h264)
 		goto EXCEPTION;
 	}
 
-#ifdef WITH_VAAPI
-
+#if defined(WITH_VAAPI) || defined(WITH_VAAPI_H264_ENCODING)
 	if (!sys->hwVideoFrame)
 	{
 		WLog_Print(h264->log, WLOG_ERROR, "Failed to allocate libav hw frame");
