@@ -20,8 +20,10 @@
  */
 #include <freerdp/config.h>
 
+#include <winpr/winpr.h>
 #include <winpr/library.h>
 #include <winpr/assert.h>
+#include <winpr/cast.h>
 
 #include <freerdp/log.h>
 #include <freerdp/codec/h264.h>
@@ -63,14 +65,16 @@ static const char* openh264_library_names[] = {
 #elif defined(__APPLE__)
 	"libopenh264.dylib"
 #else
-	"libopenh264.so"
+	"libopenh264.so.7",     "libopenh264.so.2.5.0", "libopenh264.so.2.4.1", "libopenh264.so.2.4.0",
+	"libopenh264.so.2.3.1", "libopenh264.so.2.3.0", "libopenh264.so",
+
 #endif
 };
 #endif
 
-static void openh264_trace_callback(H264_CONTEXT* WINPR_RESTRICT h264, int level,
-                                    const char* WINPR_RESTRICT message)
+static void openh264_trace_callback(void* ctx, int level, const char* message)
 {
+	H264_CONTEXT* h264 = ctx;
 	if (h264)
 		WLog_Print(h264->log, WLOG_TRACE, "%d - %s", level, message);
 }
@@ -108,8 +112,9 @@ static int openh264_decompress(H264_CONTEXT* WINPR_RESTRICT h264,
 	pYUVData[2] = NULL;
 
 	WINPR_ASSERT(sys->pDecoder);
-	state =
-	    (*sys->pDecoder)->DecodeFrame2(sys->pDecoder, pSrcData, SrcSize, pYUVData, &sBufferInfo);
+	state = (*sys->pDecoder)
+	            ->DecodeFrame2(sys->pDecoder, pSrcData, WINPR_ASSERTING_INT_CAST(int, SrcSize),
+	                           pYUVData, &sBufferInfo);
 
 	if (sBufferInfo.iBufferStatus != 1)
 	{
@@ -147,9 +152,9 @@ static int openh264_decompress(H264_CONTEXT* WINPR_RESTRICT h264,
 #endif
 
 	pSystemBuffer = &sBufferInfo.UsrData.sSystemBuffer;
-	iStride[0] = pSystemBuffer->iStride[0];
-	iStride[1] = pSystemBuffer->iStride[1];
-	iStride[2] = pSystemBuffer->iStride[1];
+	iStride[0] = WINPR_ASSERTING_INT_CAST(uint32_t, pSystemBuffer->iStride[0]);
+	iStride[1] = WINPR_ASSERTING_INT_CAST(uint32_t, pSystemBuffer->iStride[1]);
+	iStride[2] = WINPR_ASSERTING_INT_CAST(uint32_t, pSystemBuffer->iStride[1]);
 
 	if (sBufferInfo.iBufferStatus != 1)
 	{
@@ -228,7 +233,7 @@ static int openh264_compress(H264_CONTEXT* WINPR_RESTRICT h264,
 			return status;
 		}
 
-		EUsageType usageType;
+		EUsageType usageType = SCREEN_CONTENT_REAL_TIME;
 
 		switch (h264->UsageType)
 		{
@@ -243,20 +248,21 @@ static int openh264_compress(H264_CONTEXT* WINPR_RESTRICT h264,
 				break;
 			case H264_SCREEN_CONTENT_REAL_TIME:
 			default:
-				usageType = SCREEN_CONTENT_REAL_TIME;
 				break;
 		}
 
 		sys->EncParamExt.iUsageType = usageType;
-		sys->EncParamExt.iPicWidth = (int)h264->width;
-		sys->EncParamExt.iPicHeight = (int)h264->height;
-		sys->EncParamExt.fMaxFrameRate = (int)h264->FrameRate;
+		sys->EncParamExt.iPicWidth = WINPR_ASSERTING_INT_CAST(int, h264->width);
+		sys->EncParamExt.iPicHeight = WINPR_ASSERTING_INT_CAST(int, h264->height);
+		sys->EncParamExt.fMaxFrameRate = WINPR_ASSERTING_INT_CAST(short, h264->FrameRate);
 		sys->EncParamExt.iMaxBitrate = UNSPECIFIED_BIT_RATE;
 		sys->EncParamExt.bEnableDenoise = 0;
 		sys->EncParamExt.bEnableLongTermReference = 0;
 		sys->EncParamExt.iSpatialLayerNum = 1;
-		sys->EncParamExt.iMultipleThreadIdc = (int)h264->NumberOfThreads;
-		sys->EncParamExt.sSpatialLayers[0].fFrameRate = h264->FrameRate;
+		sys->EncParamExt.iMultipleThreadIdc =
+		    WINPR_ASSERTING_INT_CAST(unsigned short, h264->NumberOfThreads);
+		sys->EncParamExt.sSpatialLayers[0].fFrameRate =
+		    WINPR_ASSERTING_INT_CAST(short, h264->FrameRate);
 		sys->EncParamExt.sSpatialLayers[0].iVideoWidth = sys->EncParamExt.iPicWidth;
 		sys->EncParamExt.sSpatialLayers[0].iVideoHeight = sys->EncParamExt.iPicHeight;
 		sys->EncParamExt.sSpatialLayers[0].iMaxSpatialBitrate = sys->EncParamExt.iMaxBitrate;
@@ -276,6 +282,8 @@ static int openh264_compress(H264_CONTEXT* WINPR_RESTRICT h264,
 				sys->EncParamExt.sSpatialLayers[0].iDLayerQp = (int)h264->QP;
 				sys->EncParamExt.bEnableFrameSkip = 0;
 				break;
+			default:
+				break;
 		}
 
 		if (sys->EncParamExt.iMultipleThreadIdc > 1)
@@ -334,9 +342,9 @@ static int openh264_compress(H264_CONTEXT* WINPR_RESTRICT h264,
 					}
 				}
 
-				if (sys->EncParamExt.fMaxFrameRate != (int)h264->FrameRate)
+				if ((uint32_t)sys->EncParamExt.fMaxFrameRate != h264->FrameRate)
 				{
-					sys->EncParamExt.fMaxFrameRate = (int)h264->FrameRate;
+					sys->EncParamExt.fMaxFrameRate = WINPR_ASSERTING_INT_CAST(int, h264->FrameRate);
 
 					WINPR_ASSERT((*sys->pEncoder)->SetOption);
 					status = (*sys->pEncoder)
@@ -371,6 +379,8 @@ static int openh264_compress(H264_CONTEXT* WINPR_RESTRICT h264,
 					}
 				}
 
+				break;
+			default:
 				break;
 		}
 	}
@@ -381,9 +391,9 @@ static int openh264_compress(H264_CONTEXT* WINPR_RESTRICT h264,
 	pic.iStride[0] = (int)iStride[0];
 	pic.iStride[1] = (int)iStride[1];
 	pic.iStride[2] = (int)iStride[2];
-	pic.pData[0] = (unsigned char*)pYUVData[0];
-	pic.pData[1] = (unsigned char*)pYUVData[1];
-	pic.pData[2] = (unsigned char*)pYUVData[2];
+	pic.pData[0] = WINPR_CAST_CONST_PTR_AWAY(pYUVData[0], BYTE*);
+	pic.pData[1] = WINPR_CAST_CONST_PTR_AWAY(pYUVData[1], BYTE*);
+	pic.pData[2] = WINPR_CAST_CONST_PTR_AWAY(pYUVData[2], BYTE*);
 
 	WINPR_ASSERT((*sys->pEncoder)->EncodeFrame);
 	status = (*sys->pEncoder)->EncodeFrame(sys->pEncoder, &pic, &info);
@@ -401,7 +411,8 @@ static int openh264_compress(H264_CONTEXT* WINPR_RESTRICT h264,
 	{
 		for (int j = 0; j < info.sLayerInfo[i].iNalCount; j++)
 		{
-			*pDstSize += info.sLayerInfo[i].pNalLengthInByte[j];
+			const int val = info.sLayerInfo[i].pNalLengthInByte[j];
+			*pDstSize += WINPR_ASSERTING_INT_CAST(uint32_t, val);
 		}
 	}
 
@@ -467,15 +478,15 @@ static BOOL openh264_load_functionpointers(H264_CONTEXT* h264, const char* name)
 		return FALSE;
 
 	sysContexts->WelsGetCodecVersionEx =
-	    (pWelsGetCodecVersionEx)GetProcAddress(sysContexts->lib, "WelsGetCodecVersionEx");
+	    GetProcAddressAs(sysContexts->lib, "WelsGetCodecVersionEx", pWelsGetCodecVersionEx);
 	sysContexts->WelsCreateDecoder =
-	    (pWelsCreateDecoder)GetProcAddress(sysContexts->lib, "WelsCreateDecoder");
+	    GetProcAddressAs(sysContexts->lib, "WelsCreateDecoder", pWelsCreateDecoder);
 	sysContexts->WelsDestroyDecoder =
-	    (pWelsDestroyDecoder)GetProcAddress(sysContexts->lib, "WelsDestroyDecoder");
+	    GetProcAddressAs(sysContexts->lib, "WelsDestroyDecoder", pWelsDestroyDecoder);
 	sysContexts->WelsCreateSVCEncoder =
-	    (pWelsCreateSVCEncoder)GetProcAddress(sysContexts->lib, "WelsCreateSVCEncoder");
+	    GetProcAddressAs(sysContexts->lib, "WelsCreateSVCEncoder", pWelsCreateSVCEncoder);
 	sysContexts->WelsDestroySVCEncoder =
-	    (pWelsDestroySVCEncoder)GetProcAddress(sysContexts->lib, "WelsDestroySVCEncoder");
+	    GetProcAddressAs(sysContexts->lib, "WelsDestroySVCEncoder", pWelsDestroySVCEncoder);
 
 	if (!sysContexts->WelsCreateDecoder || !sysContexts->WelsDestroyDecoder ||
 	    !sysContexts->WelsCreateSVCEncoder || !sysContexts->WelsDestroySVCEncoder ||
@@ -518,7 +529,7 @@ static BOOL openh264_init(H264_CONTEXT* h264)
 #if (OPENH264_MAJOR == 1) && (OPENH264_MINOR <= 5)
 	static EVideoFormatType videoFormat = videoFormatI420;
 #endif
-	static WelsTraceCallback traceCallback = (WelsTraceCallback)openh264_trace_callback;
+	static WelsTraceCallback traceCallback = openh264_trace_callback;
 
 	WINPR_ASSERT(h264);
 
@@ -617,9 +628,9 @@ static BOOL openh264_init(H264_CONTEXT* h264)
 					goto EXCEPTION;
 				}
 
-				status =
-				    (*sys->pDecoder)
-				        ->SetOption(sys->pDecoder, DECODER_OPTION_TRACE_CALLBACK_CONTEXT, &h264);
+				status = (*sys->pDecoder)
+				             ->SetOption(sys->pDecoder, DECODER_OPTION_TRACE_CALLBACK_CONTEXT,
+				                         (void*)&h264);
 
 				if (status != 0)
 				{
@@ -630,9 +641,9 @@ static BOOL openh264_init(H264_CONTEXT* h264)
 					goto EXCEPTION;
 				}
 
-				status =
-				    (*sys->pDecoder)
-				        ->SetOption(sys->pDecoder, DECODER_OPTION_TRACE_CALLBACK, &traceCallback);
+				status = (*sys->pDecoder)
+				             ->SetOption(sys->pDecoder, DECODER_OPTION_TRACE_CALLBACK,
+				                         (void*)&traceCallback);
 
 				if (status != 0)
 				{
@@ -646,6 +657,7 @@ static BOOL openh264_init(H264_CONTEXT* h264)
 		}
 	}
 
+	h264->hwAccel = FALSE; /* not supported */
 	return TRUE;
 EXCEPTION:
 	openh264_uninit(h264);
