@@ -20,6 +20,8 @@
  */
 
 #include <winpr/assert.h>
+#include <winpr/cast.h>
+
 #include <freerdp/config.h>
 
 #include <winpr/crt.h>
@@ -29,8 +31,6 @@
 #include <freerdp/log.h>
 #include "xcrush.h"
 
-#define TAG FREERDP_TAG("codec")
-
 #pragma pack(push, 1)
 
 typedef struct
@@ -161,10 +161,10 @@ static const char* xcrush_get_level_1_compression_flags_string(UINT32 flags)
 }
 #endif
 
-static UINT32 xcrush_update_hash(const BYTE* WINPR_RESTRICT data, UINT32 size)
+static UINT16 xcrush_update_hash(const BYTE* WINPR_RESTRICT data, UINT32 size)
 {
 	const BYTE* end = NULL;
-	UINT32 seed = 5381; /* same value as in djb2 */
+	UINT16 seed = 5381; /* same value as in djb2 */
 
 	WINPR_ASSERT(data);
 	WINPR_ASSERT(size >= 4);
@@ -183,7 +183,7 @@ static UINT32 xcrush_update_hash(const BYTE* WINPR_RESTRICT data, UINT32 size)
 		data += 4;
 	}
 
-	return (UINT16)seed;
+	return seed;
 }
 
 static int xcrush_append_chunk(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush,
@@ -206,8 +206,8 @@ static int xcrush_append_chunk(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush,
 
 	if (size >= 15)
 	{
-		UINT32 seed = xcrush_update_hash(&data[*beg], (UINT16)size);
-		xcrush->Signatures[xcrush->SignatureIndex].size = size;
+		const UINT16 seed = xcrush_update_hash(&data[*beg], WINPR_ASSERTING_INT_CAST(UINT16, size));
+		xcrush->Signatures[xcrush->SignatureIndex].size = WINPR_ASSERTING_INT_CAST(UINT16, size);
 		xcrush->Signatures[xcrush->SignatureIndex].seed = seed;
 		xcrush->SignatureIndex++;
 		*beg = end;
@@ -334,7 +334,6 @@ static int xcrush_find_next_matching_chunk(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush
                                            XCRUSH_CHUNK* WINPR_RESTRICT chunk,
                                            XCRUSH_CHUNK** WINPR_RESTRICT pNextChunk)
 {
-	UINT32 index = 0;
 	XCRUSH_CHUNK* next = NULL;
 
 	WINPR_ASSERT(xcrush);
@@ -344,8 +343,12 @@ static int xcrush_find_next_matching_chunk(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush
 
 	if (chunk->next)
 	{
-		index = (chunk - xcrush->Chunks) / sizeof(XCRUSH_CHUNK);
+		// NOLINTNEXTLINE(bugprone-sizeof-expression)
+		const intptr_t diff = (chunk - xcrush->Chunks);
+		if (diff < 0)
+			return -4011;
 
+		const size_t index = (size_t)diff / sizeof(XCRUSH_CHUNK);
 		if (index >= 65534)
 			return -4002; /* error */
 
@@ -367,9 +370,6 @@ static int xcrush_insert_chunk(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush,
                                XCRUSH_SIGNATURE* WINPR_RESTRICT signature, UINT32 offset,
                                XCRUSH_CHUNK** WINPR_RESTRICT pPrevChunk)
 {
-	UINT32 seed = 0;
-	UINT32 index = 0;
-
 	WINPR_ASSERT(xcrush);
 
 	if (xcrush->ChunkHead >= 65530)
@@ -384,16 +384,13 @@ static int xcrush_insert_chunk(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush,
 		xcrush->ChunkTail += 10000;
 	}
 
-	index = xcrush->ChunkHead++;
+	const UINT32 index = xcrush->ChunkHead++;
 
 	if (xcrush->ChunkHead >= 65534)
 		return -3001; /* error */
 
 	xcrush->Chunks[index].offset = offset;
-	seed = signature->seed;
-
-	if (seed >= 65536)
-		return -3002; /* error */
+	const UINT16 seed = signature->seed;
 
 	if (xcrush->NextChunks[seed])
 	{
@@ -405,7 +402,7 @@ static int xcrush_insert_chunk(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush,
 	}
 
 	xcrush->Chunks[index].next = xcrush->NextChunks[seed] & 0xFFFF;
-	xcrush->NextChunks[seed] = index;
+	xcrush->NextChunks[seed] = WINPR_ASSERTING_INT_CAST(UINT16, index);
 	return 1;
 }
 
@@ -499,8 +496,12 @@ static int xcrush_find_match_length(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush, UINT3
 	if (MatchStartPtr < HistoryBuffer)
 		return -2006; /* error */
 
-	MatchInfo->MatchOffset = MatchStartPtr - HistoryBuffer;
-	MatchInfo->ChunkOffset = ChunkBuffer - ReverseMatchLength - HistoryBuffer;
+	const intptr_t diff = MatchStartPtr - HistoryBuffer;
+	const intptr_t cdiff = ChunkBuffer - ReverseMatchLength - HistoryBuffer;
+	if ((diff > UINT32_MAX) || (diff < 0) || (cdiff < 0) || (cdiff > UINT32_MAX))
+		return -1;
+	MatchInfo->MatchOffset = (UINT32)diff;
+	MatchInfo->ChunkOffset = (UINT32)cdiff;
 	MatchInfo->MatchLength = TotalMatchLength;
 	return (int)TotalMatchLength;
 }
@@ -682,7 +683,6 @@ static int xcrush_generate_output(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush,
 	UINT32 MatchIndex = 0;
 	UINT32 MatchOffset = 0;
 	UINT16 MatchLength = 0;
-	UINT32 MatchCount = 0;
 	UINT32 CurrentOffset = 0;
 	UINT32 MatchOffsetDiff = 0;
 	UINT32 HistoryOffsetDiff = 0;
@@ -693,13 +693,13 @@ static int xcrush_generate_output(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush,
 	WINPR_ASSERT(OutputSize >= 2);
 	WINPR_ASSERT(pDstSize);
 
-	MatchCount = xcrush->OptimizedMatchCount;
+	const UINT32 MatchCount = xcrush->OptimizedMatchCount;
 	OutputEnd = &OutputBuffer[OutputSize];
 
 	if (&OutputBuffer[2] >= &OutputBuffer[OutputSize])
 		return -6001; /* error */
 
-	Data_Write_UINT16(OutputBuffer, MatchCount);
+	winpr_Data_Write_UINT16(OutputBuffer, WINPR_ASSERTING_INT_CAST(UINT16, MatchCount));
 	MatchDetails = (RDP61_MATCH_DETAILS*)&OutputBuffer[2];
 	Literals = (BYTE*)&MatchDetails[MatchCount];
 
@@ -708,12 +708,18 @@ static int xcrush_generate_output(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush,
 
 	for (MatchIndex = 0; MatchIndex < MatchCount; MatchIndex++)
 	{
-		Data_Write_UINT16(&MatchDetails[MatchIndex].MatchLength,
-		                  xcrush->OptimizedMatches[MatchIndex].MatchLength);
-		Data_Write_UINT16(&MatchDetails[MatchIndex].MatchOutputOffset,
-		                  xcrush->OptimizedMatches[MatchIndex].MatchOffset - HistoryOffset);
-		Data_Write_UINT32(&MatchDetails[MatchIndex].MatchHistoryOffset,
-		                  xcrush->OptimizedMatches[MatchIndex].ChunkOffset);
+		const UINT32 len = xcrush->OptimizedMatches[MatchIndex].MatchLength;
+		winpr_Data_Write_UINT16(&MatchDetails[MatchIndex].MatchLength,
+		                        WINPR_ASSERTING_INT_CAST(UINT16, len));
+
+		const UINT32 moff = xcrush->OptimizedMatches[MatchIndex].MatchOffset;
+		WINPR_ASSERT(moff >= HistoryOffset);
+
+		const UINT32 off = moff - HistoryOffset;
+		winpr_Data_Write_UINT16(&MatchDetails[MatchIndex].MatchOutputOffset,
+		                        WINPR_ASSERTING_INT_CAST(UINT16, off));
+		winpr_Data_Write_UINT32(&MatchDetails[MatchIndex].MatchHistoryOffset,
+		                        xcrush->OptimizedMatches[MatchIndex].ChunkOffset);
 	}
 
 	CurrentOffset = HistoryOffset;
@@ -753,7 +759,10 @@ static int xcrush_generate_output(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush,
 		return -6006; /* error */
 
 	CopyMemory(Literals, &xcrush->HistoryBuffer[CurrentOffset], HistoryOffsetDiff);
-	*pDstSize = Literals + HistoryOffsetDiff - OutputBuffer;
+	const intptr_t diff = Literals + HistoryOffsetDiff - OutputBuffer;
+	if ((diff < 0) || (diff > UINT32_MAX))
+		return -1;
+	*pDstSize = (UINT32)diff;
 	return 1;
 }
 
@@ -761,8 +770,8 @@ static INLINE size_t xcrush_copy_bytes_no_overlap(BYTE* WINPR_RESTRICT dst,
                                                   const BYTE* WINPR_RESTRICT src, size_t num)
 {
 	// src and dst overlaps
-	// we should copy the area that doesn't overlap repeatly
-	const size_t diff = (dst > src) ? dst - src : src - dst;
+	// we should copy the area that doesn't overlap repeatedly
+	const size_t diff = WINPR_ASSERTING_INT_CAST(size_t, (dst > src) ? dst - src : src - dst);
 	const size_t rest = num % diff;
 	const size_t end = num - rest;
 
@@ -839,7 +848,7 @@ static int xcrush_decompress_l1(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush,
 		if ((pSrcData + 2) > pSrcEnd)
 			return -1003;
 
-		Data_Read_UINT16(pSrcData, MatchCount);
+		MatchCount = winpr_Data_Get_UINT16(pSrcData);
 		MatchDetails = (const RDP61_MATCH_DETAILS*)&pSrcData[2];
 		Literals = (const BYTE*)&MatchDetails[MatchCount];
 		OutputOffset = 0;
@@ -849,9 +858,10 @@ static int xcrush_decompress_l1(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush,
 
 		for (MatchIndex = 0; MatchIndex < MatchCount; MatchIndex++)
 		{
-			Data_Read_UINT16(&MatchDetails[MatchIndex].MatchLength, MatchLength);
-			Data_Read_UINT16(&MatchDetails[MatchIndex].MatchOutputOffset, MatchOutputOffset);
-			Data_Read_UINT32(&MatchDetails[MatchIndex].MatchHistoryOffset, MatchHistoryOffset);
+			MatchLength = winpr_Data_Get_UINT16(&MatchDetails[MatchIndex].MatchLength);
+			MatchOutputOffset = winpr_Data_Get_UINT16(&MatchDetails[MatchIndex].MatchOutputOffset);
+			MatchHistoryOffset =
+			    winpr_Data_Get_UINT32(&MatchDetails[MatchIndex].MatchHistoryOffset);
 
 			if (MatchOutputOffset < OutputOffset)
 				return -1005;
@@ -896,7 +906,7 @@ static int xcrush_decompress_l1(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush,
 
 	if (Literals < pSrcEnd)
 	{
-		OutputLength = pSrcEnd - Literals;
+		OutputLength = WINPR_ASSERTING_INT_CAST(size_t, pSrcEnd - Literals);
 
 		if ((&HistoryPtr[OutputLength] >= HistoryBufferEnd) || (&Literals[OutputLength] > pSrcEnd))
 			return -1012;
@@ -905,8 +915,15 @@ static int xcrush_decompress_l1(XCRUSH_CONTEXT* WINPR_RESTRICT xcrush,
 		HistoryPtr += OutputLength;
 	}
 
-	xcrush->HistoryOffset = HistoryPtr - HistoryBuffer;
-	*pDstSize = HistoryPtr - xcrush->HistoryPtr;
+	const intptr_t diff = HistoryPtr - HistoryBuffer;
+	if ((diff < 0) || (diff > UINT32_MAX))
+		return -1;
+	xcrush->HistoryOffset = (UINT32)diff;
+
+	const intptr_t sizediff = HistoryPtr - xcrush->HistoryPtr;
+	if ((sizediff < 0) || (sizediff > UINT32_MAX))
+		return -1;
+	*pDstSize = (UINT32)sizediff;
 	*ppDstData = xcrush->HistoryPtr;
 	return 1;
 }
