@@ -22,6 +22,7 @@
 #include "settings.h"
 
 #include <winpr/assert.h>
+#include <winpr/cast.h>
 
 #include "activation.h"
 #include "display.h"
@@ -39,7 +40,8 @@ static BOOL rdp_write_synchronize_pdu(wStream* s, const rdpSettings* settings)
 	if (!Stream_CheckAndLogRequiredCapacity(TAG, (s), 4))
 		return FALSE;
 	Stream_Write_UINT16(s, SYNCMSGTYPE_SYNC); /* messageType (2 bytes) */
-	Stream_Write_UINT16(s, PduSource);        /* targetUser (2 bytes) */
+	Stream_Write_UINT16(s,
+	                    WINPR_ASSERTING_INT_CAST(uint16_t, PduSource)); /* targetUser (2 bytes) */
 	return TRUE;
 }
 
@@ -266,20 +268,16 @@ static BOOL rdp_write_client_persistent_key_list_pdu(wStream* s,
 
 	for (UINT32 index = 0; index < info->keyCount; index++)
 	{
-		const UINT32 key1 = (UINT32)(info->keyList[index] & UINT32_MAX);
-		const UINT32 key2 = (UINT32)(info->keyList[index] >> 32);
-		Stream_Write_UINT32(s, key1);
-		Stream_Write_UINT32(s, key2);
+		const UINT64 val = info->keyList[index];
+		Stream_Write_UINT64(s, val);
 	}
 
 	return TRUE;
 }
 
-static UINT32 rdp_load_persistent_key_list(rdpRdp* rdp, UINT64** pKeyList)
+static UINT16 rdp_load_persistent_key_list(rdpRdp* rdp, UINT64** pKeyList)
 {
-	int count = 0;
-	int status = 0;
-	UINT32 keyCount = 0;
+	UINT16 keyCount = 0;
 	UINT64* keyList = NULL;
 	rdpPersistentCache* persistent = NULL;
 	rdpSettings* settings = rdp->settings;
@@ -297,15 +295,18 @@ static UINT32 rdp_load_persistent_key_list(rdpRdp* rdp, UINT64** pKeyList)
 	if (!persistent)
 		return 0;
 
-	status = persistent_cache_open(persistent, settings->BitmapCachePersistFile, FALSE, 0);
+	const int status =
+	    persistent_cache_open(persistent, settings->BitmapCachePersistFile, FALSE, 0);
 
 	if (status < 1)
 		goto error;
 
-	count = persistent_cache_get_count(persistent);
+	const int count = persistent_cache_get_count(persistent);
+	if ((count < 0) || (count > UINT16_MAX))
+		goto error;
 
-	keyCount = (UINT32)count;
-	keyList = (UINT64*)malloc(keyCount * sizeof(UINT64));
+	keyCount = (UINT16)count;
+	keyList = (UINT64*)calloc(keyCount, sizeof(UINT64));
 
 	if (!keyList)
 		goto error;
@@ -332,13 +333,14 @@ error:
 
 BOOL rdp_send_client_persistent_key_list_pdu(rdpRdp* rdp)
 {
-	UINT32 keyMaxFrag = 2042;
+	UINT16 keyMaxFrag = 2042;
 	UINT64* keyList = NULL;
 	RDP_BITMAP_PERSISTENT_INFO info = { 0 };
+	WINPR_ASSERT(rdp);
 	rdpSettings* settings = rdp->settings;
-	UINT32 keyCount = rdp_load_persistent_key_list(rdp, &keyList);
+	UINT16 keyCount = rdp_load_persistent_key_list(rdp, &keyList);
 
-	WLog_DBG(TAG, "Persistent Key List: TotalKeyCount: %" PRIu32 " MaxKeyFrag: %" PRIu32, keyCount,
+	WLog_DBG(TAG, "Persistent Key List: TotalKeyCount: %" PRIu16 " MaxKeyFrag: %" PRIu16, keyCount,
 	         keyMaxFrag);
 
 	// MS-RDPBCGR recommends sending no more than 169 entries at once.
@@ -350,11 +352,20 @@ BOOL rdp_send_client_persistent_key_list_pdu(rdpRdp* rdp)
 	if (keyCount > keyMaxFrag)
 		keyCount = keyMaxFrag;
 
-	info.totalEntriesCache0 = settings->BitmapCacheV2CellInfo[0].numEntries;
-	info.totalEntriesCache1 = settings->BitmapCacheV2CellInfo[1].numEntries;
-	info.totalEntriesCache2 = settings->BitmapCacheV2CellInfo[2].numEntries;
-	info.totalEntriesCache3 = settings->BitmapCacheV2CellInfo[3].numEntries;
-	info.totalEntriesCache4 = settings->BitmapCacheV2CellInfo[4].numEntries;
+	WINPR_ASSERT(settings->BitmapCacheV2CellInfo[0].numEntries <= UINT16_MAX);
+	info.totalEntriesCache0 = (UINT16)settings->BitmapCacheV2CellInfo[0].numEntries;
+
+	WINPR_ASSERT(settings->BitmapCacheV2CellInfo[1].numEntries <= UINT16_MAX);
+	info.totalEntriesCache1 = (UINT16)settings->BitmapCacheV2CellInfo[1].numEntries;
+
+	WINPR_ASSERT(settings->BitmapCacheV2CellInfo[2].numEntries <= UINT16_MAX);
+	info.totalEntriesCache2 = (UINT16)settings->BitmapCacheV2CellInfo[2].numEntries;
+
+	WINPR_ASSERT(settings->BitmapCacheV2CellInfo[3].numEntries <= UINT16_MAX);
+	info.totalEntriesCache3 = (UINT16)settings->BitmapCacheV2CellInfo[3].numEntries;
+
+	WINPR_ASSERT(settings->BitmapCacheV2CellInfo[4].numEntries <= UINT16_MAX);
+	info.totalEntriesCache4 = (UINT16)settings->BitmapCacheV2CellInfo[4].numEntries;
 
 	info.numEntriesCache0 = MIN(keyCount, info.totalEntriesCache0);
 	keyCount -= info.numEntriesCache0;
@@ -365,7 +376,6 @@ BOOL rdp_send_client_persistent_key_list_pdu(rdpRdp* rdp)
 	info.numEntriesCache3 = MIN(keyCount, info.totalEntriesCache3);
 	keyCount -= info.numEntriesCache3;
 	info.numEntriesCache4 = MIN(keyCount, info.totalEntriesCache4);
-	keyCount -= info.numEntriesCache4;
 
 	info.totalEntriesCache0 = info.numEntriesCache0;
 	info.totalEntriesCache1 = info.numEntriesCache1;
