@@ -130,7 +130,8 @@ static const char* autodetect_request_type_to_string_buffer(UINT32 requestType,
 }
 
 static BOOL autodetect_send_rtt_measure_request(rdpAutoDetect* autodetect,
-                                                RDP_TRANSPORT_TYPE transport, UINT16 sequenceNumber)
+                                                WINPR_ATTR_UNUSED RDP_TRANSPORT_TYPE transport,
+                                                UINT16 sequenceNumber)
 {
 	UINT16 requestType = 0;
 	wStream* s = NULL;
@@ -179,7 +180,7 @@ static BOOL autodetect_send_rtt_measure_response(rdpAutoDetect* autodetect, UINT
 }
 
 static BOOL autodetect_send_bandwidth_measure_start(rdpAutoDetect* autodetect,
-                                                    RDP_TRANSPORT_TYPE transport,
+                                                    WINPR_ATTR_UNUSED RDP_TRANSPORT_TYPE transport,
                                                     UINT16 sequenceNumber)
 {
 	UINT16 requestType = 0;
@@ -206,9 +207,10 @@ static BOOL autodetect_send_bandwidth_measure_start(rdpAutoDetect* autodetect,
 	return rdp_send_message_channel_pdu(autodetect->context->rdp, s, SEC_AUTODETECT_REQ);
 }
 
-static BOOL autodetect_send_bandwidth_measure_payload(rdpAutoDetect* autodetect,
-                                                      RDP_TRANSPORT_TYPE transport,
-                                                      UINT16 sequenceNumber, UINT16 payloadLength)
+static BOOL
+autodetect_send_bandwidth_measure_payload(rdpAutoDetect* autodetect,
+                                          WINPR_ATTR_UNUSED RDP_TRANSPORT_TYPE transport,
+                                          UINT16 sequenceNumber, UINT16 payloadLength)
 {
 	wStream* s = NULL;
 
@@ -246,7 +248,7 @@ static BOOL autodetect_send_bandwidth_measure_payload(rdpAutoDetect* autodetect,
 }
 
 static BOOL autodetect_send_bandwidth_measure_stop(rdpAutoDetect* autodetect,
-                                                   RDP_TRANSPORT_TYPE transport,
+                                                   WINPR_ATTR_UNUSED RDP_TRANSPORT_TYPE transport,
                                                    UINT16 sequenceNumber, UINT16 payloadLength)
 {
 	UINT16 requestType = 0;
@@ -354,7 +356,8 @@ static BOOL autodetect_send_bandwidth_measure_results(rdpAutoDetect* autodetect,
 	return rdp_send_message_channel_pdu(autodetect->context->rdp, s, SEC_AUTODETECT_RSP);
 }
 
-static BOOL autodetect_send_netchar_result(rdpAutoDetect* autodetect, RDP_TRANSPORT_TYPE transport,
+static BOOL autodetect_send_netchar_result(rdpAutoDetect* autodetect,
+                                           WINPR_ATTR_UNUSED RDP_TRANSPORT_TYPE transport,
                                            UINT16 sequenceNumber,
                                            const rdpNetworkCharacteristicsResult* result)
 {
@@ -376,7 +379,9 @@ static BOOL autodetect_send_netchar_result(rdpAutoDetect* autodetect, RDP_TRANSP
 			Stream_Write_UINT8(s, 0x0E);                       /* headerLength (1 byte) */
 			Stream_Write_UINT8(s, TYPE_ID_AUTODETECT_REQUEST); /* headerTypeId (1 byte) */
 			Stream_Write_UINT16(s, sequenceNumber);            /* sequenceNumber (2 bytes) */
-			Stream_Write_UINT16(s, result->type);              /* requestType (2 bytes) */
+			WINPR_ASSERT((result->type <= UINT16_MAX));
+			WINPR_ASSERT((result->type >= 0));
+			Stream_Write_UINT16(s, (UINT16)result->type);      /* requestType (2 bytes) */
 			Stream_Write_UINT32(s, result->baseRTT);           /* baseRTT (4 bytes) */
 			Stream_Write_UINT32(s, result->averageRTT);        /* averageRTT (4 bytes) */
 			break;
@@ -384,7 +389,9 @@ static BOOL autodetect_send_netchar_result(rdpAutoDetect* autodetect, RDP_TRANSP
 			Stream_Write_UINT8(s, 0x0E);                       /* headerLength (1 byte) */
 			Stream_Write_UINT8(s, TYPE_ID_AUTODETECT_REQUEST); /* headerTypeId (1 byte) */
 			Stream_Write_UINT16(s, sequenceNumber);            /* sequenceNumber (2 bytes) */
-			Stream_Write_UINT16(s, result->type);              /* requestType (2 bytes) */
+			WINPR_ASSERT((result->type <= UINT16_MAX));
+			WINPR_ASSERT((result->type >= 0));
+			Stream_Write_UINT16(s, (UINT16)result->type);      /* requestType (2 bytes) */
 			Stream_Write_UINT32(s, result->bandwidth);         /* bandwidth (4 bytes) */
 			Stream_Write_UINT32(s, result->averageRTT);        /* averageRTT (4 bytes) */
 			break;
@@ -392,7 +399,9 @@ static BOOL autodetect_send_netchar_result(rdpAutoDetect* autodetect, RDP_TRANSP
 			Stream_Write_UINT8(s, 0x12);                       /* headerLength (1 byte) */
 			Stream_Write_UINT8(s, TYPE_ID_AUTODETECT_REQUEST); /* headerTypeId (1 byte) */
 			Stream_Write_UINT16(s, sequenceNumber);            /* sequenceNumber (2 bytes) */
-			Stream_Write_UINT16(s, result->type);              /* requestType (2 bytes) */
+			WINPR_ASSERT((result->type <= UINT16_MAX));
+			WINPR_ASSERT((result->type >= 0));
+			Stream_Write_UINT16(s, (UINT16)result->type);      /* requestType (2 bytes) */
 			Stream_Write_UINT32(s, result->baseRTT);           /* baseRTT (4 bytes) */
 			Stream_Write_UINT32(s, result->bandwidth);         /* bandwidth (4 bytes) */
 			Stream_Write_UINT32(s, result->averageRTT);        /* averageRTT (4 bytes) */
@@ -429,35 +438,9 @@ autodetect_on_connect_time_auto_detect_progress_default(rdpAutoDetect* autodetec
 	return autodetect->state;
 }
 
-static BOOL autodetect_send_netchar_sync(rdpAutoDetect* autodetect, RDP_TRANSPORT_TYPE transport,
-                                         UINT16 sequenceNumber)
-{
-	wStream* s = NULL;
-
-	WINPR_ASSERT(autodetect);
-	WINPR_ASSERT(autodetect->context);
-	WINPR_ASSERT(autodetect->context->rdp);
-
-	/* Send the response PDU to the server */
-	s = rdp_message_channel_pdu_init(autodetect->context->rdp);
-
-	if (!s)
-		return FALSE;
-
-	WLog_Print(autodetect->log, WLOG_TRACE,
-	           "sending Network Characteristics Sync PDU -> bandwidth=%" PRIu32 ", rtt=%" PRIu32 "",
-	           autodetect->netCharBandwidth, autodetect->netCharAverageRTT);
-	Stream_Write_UINT8(s, 0x0E);                            /* headerLength (1 byte) */
-	Stream_Write_UINT8(s, TYPE_ID_AUTODETECT_RESPONSE);     /* headerTypeId (1 byte) */
-	Stream_Write_UINT16(s, sequenceNumber);                 /* sequenceNumber (2 bytes) */
-	Stream_Write_UINT16(s, RDP_NETCHAR_SYNC_RESPONSE_TYPE); /* responseType (1 byte) */
-	Stream_Write_UINT32(s, autodetect->netCharBandwidth);   /* bandwidth (4 bytes) */
-	Stream_Write_UINT32(s, autodetect->netCharAverageRTT);  /* rtt (4 bytes) */
-	return rdp_send_message_channel_pdu(autodetect->context->rdp, s, SEC_AUTODETECT_RSP);
-}
-
 static BOOL autodetect_recv_rtt_measure_request(rdpAutoDetect* autodetect,
-                                                RDP_TRANSPORT_TYPE transport, wStream* s,
+                                                WINPR_ATTR_UNUSED RDP_TRANSPORT_TYPE transport,
+                                                WINPR_ATTR_UNUSED wStream* s,
                                                 const AUTODETECT_REQ_PDU* autodetectReqPdu)
 {
 	WINPR_ASSERT(autodetect);
@@ -478,7 +461,8 @@ static BOOL autodetect_recv_rtt_measure_request(rdpAutoDetect* autodetect,
 }
 
 static BOOL autodetect_recv_rtt_measure_response(rdpAutoDetect* autodetect,
-                                                 RDP_TRANSPORT_TYPE transport, wStream* s,
+                                                 RDP_TRANSPORT_TYPE transport,
+                                                 WINPR_ATTR_UNUSED wStream* s,
                                                  const AUTODETECT_RSP_PDU* autodetectRspPdu)
 {
 	BOOL success = TRUE;
@@ -510,7 +494,8 @@ static BOOL autodetect_recv_rtt_measure_response(rdpAutoDetect* autodetect,
 }
 
 static BOOL autodetect_recv_bandwidth_measure_start(rdpAutoDetect* autodetect,
-                                                    RDP_TRANSPORT_TYPE transport, wStream* s,
+                                                    WINPR_ATTR_UNUSED RDP_TRANSPORT_TYPE transport,
+                                                    WINPR_ATTR_UNUSED wStream* s,
                                                     const AUTODETECT_REQ_PDU* autodetectReqPdu)
 {
 	WINPR_ASSERT(autodetect);
@@ -540,9 +525,10 @@ static BOOL autodetect_recv_bandwidth_measure_start(rdpAutoDetect* autodetect,
 	return TRUE;
 }
 
-static BOOL autodetect_recv_bandwidth_measure_payload(rdpAutoDetect* autodetect,
-                                                      RDP_TRANSPORT_TYPE transport, wStream* s,
-                                                      const AUTODETECT_REQ_PDU* autodetectReqPdu)
+static BOOL
+autodetect_recv_bandwidth_measure_payload(rdpAutoDetect* autodetect,
+                                          WINPR_ATTR_UNUSED RDP_TRANSPORT_TYPE transport,
+                                          wStream* s, const AUTODETECT_REQ_PDU* autodetectReqPdu)
 {
 	UINT16 payloadLength = 0;
 
