@@ -207,8 +207,9 @@ static BOOL rdp_client_reset_codecs(rdpContext* context)
 
 	if (!freerdp_settings_get_bool(settings, FreeRDP_DeactivateClientDecoding))
 	{
-		codecs_free(context->codecs);
-		context->codecs = codecs_new(context);
+		const UINT32 flags = freerdp_settings_get_uint32(settings, FreeRDP_ThreadingFlags);
+		freerdp_client_codecs_free(context->codecs);
+		context->codecs = freerdp_client_codecs_new(flags);
 
 		if (!context->codecs)
 			return FALSE;
@@ -256,7 +257,9 @@ static BOOL rdp_client_wait_for_activation(rdpRdp* rdp)
 			return FALSE;
 		}
 
-		wstatus = WaitForMultipleObjectsEx(nevents, events, FALSE, (dueDate - now), TRUE);
+		const UINT64 timeout = (dueDate - now);
+		WINPR_ASSERT(timeout <= UINT32_MAX);
+		wstatus = WaitForMultipleObjectsEx(nevents, events, FALSE, (UINT32)timeout, TRUE);
 		switch (wstatus)
 		{
 			case WAIT_TIMEOUT:
@@ -316,6 +319,7 @@ BOOL rdp_client_connect(rdpRdp* rdp)
 		flags |= WINPR_SSL_INIT_ENABLE_FIPS;
 
 	winpr_InitializeSSL(flags);
+	rdp_log_build_warnings(rdp);
 
 	/* FIPS Mode forces the following and overrides the following(by happening later */
 	/* in the command line processing): */
@@ -332,7 +336,7 @@ BOOL rdp_client_connect(rdpRdp* rdp)
 	UINT32 TcpConnectTimeout = freerdp_settings_get_uint32(settings, FreeRDP_TcpConnectTimeout);
 	if (settings->GatewayArmTransport)
 	{
-		if (!arm_resolve_endpoint(rdp->context, TcpConnectTimeout))
+		if (!arm_resolve_endpoint(rdp->log, rdp->context, TcpConnectTimeout))
 		{
 			WLog_ERR(TAG, "error retrieving ARM configuration");
 			return FALSE;
@@ -346,17 +350,17 @@ BOOL rdp_client_connect(rdpRdp* rdp)
 		return FALSE;
 	}
 
+	const UINT32 port = settings->ServerPort;
+	WINPR_ASSERT(port <= UINT32_MAX);
+
 	nego_init(rdp->nego);
-	nego_set_target(rdp->nego, hostname, settings->ServerPort);
+	nego_set_target(rdp->nego, hostname, (UINT16)port);
 
 	if (settings->GatewayEnabled)
 	{
 		char* user = NULL;
 		char* domain = NULL;
-		char* cookie = NULL;
 		size_t user_length = 0;
-		size_t domain_length = 0;
-		size_t cookie_length = 0;
 
 		if (settings->Username)
 		{
@@ -369,15 +373,16 @@ BOOL rdp_client_connect(rdpRdp* rdp)
 		else
 			domain = settings->ComputerName;
 
-		domain_length = strlen(domain);
-		cookie_length = domain_length + 1 + user_length;
-		cookie = (char*)malloc(cookie_length + 1);
+		const size_t domain_length = strlen(domain);
+		const size_t cookie_length = domain_length + 1 + user_length;
+		char* cookie = malloc(cookie_length + 1);
 
 		if (!cookie)
 			return FALSE;
 
 		CopyMemory(cookie, domain, domain_length);
-		CharUpperBuffA(cookie, domain_length);
+		WINPR_ASSERT(domain_length <= UINT32_MAX);
+		CharUpperBuffA(cookie, (UINT32)domain_length);
 		cookie[domain_length] = '\\';
 
 		if (settings->Username)
@@ -401,6 +406,7 @@ BOOL rdp_client_connect(rdpRdp* rdp)
 	nego_set_preconnection_blob(rdp->nego, settings->PreconnectionBlob);
 	nego_set_negotiation_enabled(rdp->nego, settings->NegotiateSecurityLayer);
 	nego_set_restricted_admin_mode_required(rdp->nego, settings->RestrictedAdminModeRequired);
+	nego_set_RCG_required(rdp->nego, settings->RemoteCredentialGuard);
 	nego_set_gateway_enabled(rdp->nego, settings->GatewayEnabled);
 	nego_set_gateway_bypass_local(rdp->nego, settings->GatewayBypassLocal);
 	nego_enable_rdp(rdp->nego, settings->RdpSecurity);
@@ -486,6 +492,9 @@ BOOL rdp_client_disconnect(rdpRdp* rdp)
 			return FALSE;
 	}
 
+	if (!transport_disconnect(rdp->transport))
+		return FALSE;
+
 	if (!rdp_reset(rdp))
 		return FALSE;
 
@@ -495,7 +504,7 @@ BOOL rdp_client_disconnect(rdpRdp* rdp)
 	if (freerdp_channels_disconnect(context->channels, context->instance) != CHANNEL_RC_OK)
 		return FALSE;
 
-	codecs_free(context->codecs);
+	freerdp_client_codecs_free(context->codecs);
 	context->codecs = NULL;
 	return TRUE;
 }
@@ -622,11 +631,10 @@ BOOL rdp_client_redirect(rdpRdp* rdp)
 	if (!rdp_client_disconnect_and_clear(rdp))
 		return FALSE;
 
+	/* Only disconnect & close the channels here.
+	 * they will be discarded and recreated after the new settings have been applied. */
 	freerdp_channels_disconnect(rdp->context->channels, rdp->context->instance);
 	freerdp_channels_close(rdp->context->channels, rdp->context->instance);
-	freerdp_channels_free(rdp->context->channels);
-	rdp->context->channels = freerdp_channels_new(rdp->context->instance);
-	WINPR_ASSERT(rdp->context->channels);
 
 	if (rdp_redirection_apply_settings(rdp) != 0)
 		return FALSE;
@@ -684,14 +692,10 @@ BOOL rdp_client_redirect(rdpRdp* rdp)
 	if (!IFCALLRESULT(TRUE, rdp->context->instance->Redirect, rdp->context->instance))
 		return FALSE;
 
-	BOOL ok = IFCALLRESULT(TRUE, rdp->context->instance->LoadChannels, rdp->context->instance);
+	BOOL ok = utils_reload_channels(rdp->context);
 	if (!ok)
 		return FALSE;
 
-	if (CHANNEL_RC_OK !=
-	    freerdp_channels_pre_connect(rdp->context->channels, rdp->context->instance))
-		return FALSE;
-
 	status = rdp_client_connect(rdp);
 
 	if (status)
@@ -762,8 +766,11 @@ static BOOL rdp_client_establish_keys(rdpRdp* rdp)
 	crypto_rsa_public_encrypt(settings->ClientRandom, settings->ClientRandomLength, info,
 	                          crypt_client_random, info->ModulusLength);
 	/* send crypt client random to server */
-	const size_t length =
-	    RDP_PACKET_HEADER_MAX_LENGTH + RDP_SECURITY_HEADER_LENGTH + 4 + info->ModulusLength + 8;
+	const size_t length = RDP_PACKET_HEADER_MAX_LENGTH + RDP_SECURITY_HEADER_LENGTH + 4ULL +
+	                      info->ModulusLength + 8ULL;
+	if (length > UINT16_MAX)
+		return FALSE;
+
 	s = Stream_New(NULL, length);
 
 	if (!s)
@@ -782,7 +789,6 @@ static BOOL rdp_client_establish_keys(rdpRdp* rdp)
 	Stream_Zero(s, 8);
 	Stream_SealLength(s);
 	status = transport_write(rdp->mcs->transport, s);
-	Stream_Free(s, TRUE);
 
 	if (status < 0)
 		goto end;
@@ -800,8 +806,9 @@ static BOOL rdp_client_establish_keys(rdpRdp* rdp)
 
 	if (settings->EncryptionMethods == ENCRYPTION_METHOD_FIPS)
 	{
-		rdp->fips_encrypt = winpr_Cipher_New(WINPR_CIPHER_DES_EDE3_CBC, WINPR_ENCRYPT,
-		                                     rdp->fips_encrypt_key, fips_ivec);
+		rdp->fips_encrypt =
+		    winpr_Cipher_NewEx(WINPR_CIPHER_DES_EDE3_CBC, WINPR_ENCRYPT, rdp->fips_encrypt_key,
+		                       sizeof(rdp->fips_encrypt_key), fips_ivec, sizeof(fips_ivec));
 
 		if (!rdp->fips_encrypt)
 		{
@@ -809,8 +816,9 @@ static BOOL rdp_client_establish_keys(rdpRdp* rdp)
 			goto end;
 		}
 
-		rdp->fips_decrypt = winpr_Cipher_New(WINPR_CIPHER_DES_EDE3_CBC, WINPR_DECRYPT,
-		                                     rdp->fips_decrypt_key, fips_ivec);
+		rdp->fips_decrypt =
+		    winpr_Cipher_NewEx(WINPR_CIPHER_DES_EDE3_CBC, WINPR_DECRYPT, rdp->fips_decrypt_key,
+		                       sizeof(rdp->fips_decrypt_key), fips_ivec, sizeof(fips_ivec));
 
 		if (!rdp->fips_decrypt)
 		{
@@ -829,6 +837,7 @@ static BOOL rdp_client_establish_keys(rdpRdp* rdp)
 
 	ret = TRUE;
 end:
+	Stream_Free(s, TRUE);
 	free(crypt_client_random);
 
 	if (!ret)
@@ -927,8 +936,9 @@ BOOL rdp_server_establish_keys(rdpRdp* rdp, wStream* s)
 
 	if (rdp->settings->EncryptionMethods == ENCRYPTION_METHOD_FIPS)
 	{
-		rdp->fips_encrypt = winpr_Cipher_New(WINPR_CIPHER_DES_EDE3_CBC, WINPR_ENCRYPT,
-		                                     rdp->fips_encrypt_key, fips_ivec);
+		rdp->fips_encrypt =
+		    winpr_Cipher_NewEx(WINPR_CIPHER_DES_EDE3_CBC, WINPR_ENCRYPT, rdp->fips_encrypt_key,
+		                       sizeof(rdp->fips_encrypt_key), fips_ivec, sizeof(fips_ivec));
 
 		if (!rdp->fips_encrypt)
 		{
@@ -936,8 +946,9 @@ BOOL rdp_server_establish_keys(rdpRdp* rdp, wStream* s)
 			goto end;
 		}
 
-		rdp->fips_decrypt = winpr_Cipher_New(WINPR_CIPHER_DES_EDE3_CBC, WINPR_DECRYPT,
-		                                     rdp->fips_decrypt_key, fips_ivec);
+		rdp->fips_decrypt =
+		    winpr_Cipher_NewEx(WINPR_CIPHER_DES_EDE3_CBC, WINPR_DECRYPT, rdp->fips_decrypt_key,
+		                       sizeof(rdp->fips_decrypt_key), fips_ivec, sizeof(fips_ivec));
 
 		if (!rdp->fips_decrypt)
 		{
@@ -1128,48 +1139,74 @@ BOOL rdp_client_connect_mcs_channel_join_confirm(rdpRdp* rdp, wStream* s)
 	return TRUE;
 }
 
+BOOL rdp_handle_message_channel(rdpRdp* rdp, wStream* s, UINT16 channelId, UINT16 length)
+{
+	WINPR_ASSERT(rdp);
+	WINPR_ASSERT(rdp->mcs);
+
+	if (!rdp->mcs->messageChannelJoined)
+	{
+		WLog_Print(rdp->log, WLOG_WARN, "MCS message channel not joined!");
+		return FALSE;
+	}
+	const UINT16 messageChannelId = rdp->mcs->messageChannelId;
+	if (messageChannelId == 0)
+	{
+		WLog_Print(rdp->log, WLOG_WARN, "MCS message channel id == 0");
+		return FALSE;
+	}
+
+	if (channelId != messageChannelId)
+	{
+		WLog_Print(rdp->log, WLOG_WARN, "MCS message channel expected id=%" PRIu16 ", got %" PRIu16,
+		           messageChannelId, channelId);
+		return FALSE;
+	}
+
+	UINT16 securityFlags = 0;
+	if (!rdp_read_security_header(rdp, s, &securityFlags, &length))
+		return FALSE;
+
+	if (securityFlags & SEC_ENCRYPT)
+	{
+		if (!rdp_decrypt(rdp, s, &length, securityFlags))
+			return FALSE;
+	}
+
+	if (rdp_recv_message_channel_pdu(rdp, s, securityFlags) != STATE_RUN_SUCCESS)
+		return FALSE;
+
+	return tpkt_ensure_stream_consumed(s, length);
+}
+
 BOOL rdp_client_connect_auto_detect(rdpRdp* rdp, wStream* s)
 {
 	WINPR_ASSERT(rdp);
 	WINPR_ASSERT(rdp->mcs);
 
-	const UINT16 messageChannelId = rdp->mcs->messageChannelId;
-	/* If the MCS message channel has been joined... */
-	if (messageChannelId != 0)
+	const size_t pos = Stream_GetPosition(s);
+	UINT16 length = 0;
+	UINT16 channelId = 0;
+
+	if (rdp_read_header(rdp, s, &length, &channelId))
 	{
+		const UINT16 messageChannelId = rdp->mcs->messageChannelId;
+		/* If the MCS message channel has been joined... */
+
 		/* Process any MCS message channel PDUs. */
-		const size_t pos = Stream_GetPosition(s);
-		UINT16 length = 0;
-		UINT16 channelId = 0;
-
-		if (rdp_read_header(rdp, s, &length, &channelId))
+		if (rdp->mcs->messageChannelJoined && (channelId == messageChannelId))
 		{
-			if (channelId == messageChannelId)
-			{
-				UINT16 securityFlags = 0;
-
-				if (!rdp_read_security_header(rdp, s, &securityFlags, &length))
-					return FALSE;
-
-				if (securityFlags & SEC_ENCRYPT)
-				{
-					if (!rdp_decrypt(rdp, s, &length, securityFlags))
-						return FALSE;
-				}
-
-				if (rdp_recv_message_channel_pdu(rdp, s, securityFlags) == STATE_RUN_SUCCESS)
-					return tpkt_ensure_stream_consumed(s, length);
-			}
+			if (rdp_handle_message_channel(rdp, s, channelId, length))
+				return TRUE;
 		}
 		else
+		{
 			WLog_WARN(TAG, "expected messageChannelId=%" PRIu16 ", got %" PRIu16, messageChannelId,
 			          channelId);
-
-		Stream_SetPosition(s, pos);
+		}
 	}
-	else
-		WLog_WARN(TAG, "messageChannelId == 0");
 
+	Stream_SetPosition(s, pos);
 	return FALSE;
 }
 
@@ -1185,6 +1222,17 @@ state_run_t rdp_client_connect_license(rdpRdp* rdp, wStream* s)
 	if (!rdp_read_header(rdp, s, &length, &channelId))
 		return STATE_RUN_FAILED;
 
+	/* there might be autodetect messages mixed in between licensing messages.
+	 * that has been observed with 2k12 R2 and 2k19
+	 */
+	const UINT16 messageChannelId = rdp->mcs->messageChannelId;
+	if (rdp->mcs->messageChannelJoined && (channelId == messageChannelId))
+	{
+		if (!rdp_handle_message_channel(rdp, s, channelId, length))
+			return STATE_RUN_FAILED;
+		return STATE_RUN_SUCCESS;
+	}
+
 	if (!rdp_read_security_header(rdp, s, &securityFlags, &length))
 		return STATE_RUN_FAILED;
 
@@ -1194,15 +1242,6 @@ state_run_t rdp_client_connect_license(rdpRdp* rdp, wStream* s)
 			return STATE_RUN_FAILED;
 	}
 
-	/* there might be autodetect messages mixed in between licensing messages.
-	 * that has been observed with 2k12 R2 and 2k19
-	 */
-	const UINT16 messageChannelId = rdp->mcs->messageChannelId;
-	if ((channelId == messageChannelId) || (securityFlags & SEC_AUTODETECT_REQ))
-	{
-		return rdp_recv_message_channel_pdu(rdp, s, securityFlags);
-	}
-
 	if (channelId != MCS_GLOBAL_CHANNEL_ID)
 		WLog_WARN(TAG, "unexpected message for channel %u, expected %u", channelId,
 		          MCS_GLOBAL_CHANNEL_ID);
@@ -1264,6 +1303,17 @@ state_run_t rdp_client_connect_demand_active(rdpRdp* rdp, wStream* s)
 	if (freerdp_shall_disconnect_context(rdp->context))
 		return STATE_RUN_QUIT_SESSION;
 
+	if (rdp->mcs->messageChannelId && (channelId == rdp->mcs->messageChannelId))
+	{
+		rdp->inPackets++;
+		if (!rdp_handle_message_channel(rdp, s, channelId, length))
+			return STATE_RUN_FAILED;
+		return STATE_RUN_SUCCESS;
+	}
+
+	if (!rdp_handle_optional_rdp_decryption(rdp, s, &length, NULL))
+		return STATE_RUN_FAILED;
+
 	if (!rdp_read_share_control_header(rdp, s, NULL, NULL, &pduType, &pduSource))
 		return STATE_RUN_FAILED;
 
@@ -1369,7 +1419,7 @@ BOOL rdp_client_transition_to_state(rdpRdp* rdp, CONNECTION_STATE state)
 		ConnectionStateChangeEventArgs stateEvent = { 0 };
 		rdpContext* context = rdp->context;
 		EventArgsInit(&stateEvent, "libfreerdp");
-		stateEvent.state = rdp_get_state(rdp);
+		stateEvent.state = WINPR_ASSERTING_INT_CAST(int32_t, rdp_get_state(rdp));
 		stateEvent.active = rdp_is_active_state(rdp);
 		PubSub_OnConnectionStateChange(rdp->pubSub, context, &stateEvent);
 	}
@@ -1945,7 +1995,7 @@ fail:
 	return status;
 }
 
-const char* rdp_client_connection_state_string(int state)
+const char* rdp_client_connection_state_string(UINT state)
 {
 	switch (state)
 	{
@@ -2083,7 +2133,7 @@ BOOL rdp_channels_from_mcs(rdpSettings* settings, const rdpRdp* rdp)
  * 1. send the CONFIRM_ACTIVE PDU to the server
  * 2. register callbacks, the server can now start sending stuff
  */
-state_run_t rdp_client_connect_confirm_active(rdpRdp* rdp, wStream* s)
+state_run_t rdp_client_connect_confirm_active(rdpRdp* rdp, WINPR_ATTR_UNUSED wStream* s)
 {
 	WINPR_ASSERT(rdp);
 	WINPR_ASSERT(rdp->settings);
@@ -2139,3 +2189,31 @@ state_run_t rdp_client_connect_confirm_active(rdpRdp* rdp, wStream* s)
 		status = STATE_RUN_FAILED;
 	return status;
 }
+
+BOOL rdp_handle_optional_rdp_decryption(rdpRdp* rdp, wStream* s, UINT16* length,
+                                        UINT16* pSecurityFlags)
+{
+	BOOL rc = FALSE;
+	WINPR_ASSERT(rdp);
+	WINPR_ASSERT(rdp->settings);
+
+	UINT16 securityFlags = 0;
+	if (rdp->settings->UseRdpSecurityLayer)
+	{
+		if (!rdp_read_security_header(rdp, s, &securityFlags, length))
+			goto fail;
+
+		if (securityFlags & SEC_ENCRYPT)
+		{
+			if (!rdp_decrypt(rdp, s, length, securityFlags))
+				goto fail;
+		}
+	}
+
+	rc = TRUE;
+
+fail:
+	if (pSecurityFlags)
+		*pSecurityFlags = securityFlags;
+	return rc;
+}
