@@ -88,7 +88,6 @@ static int freerdp_connect_begin(freerdp* instance)
 	rdpRdp* rdp = NULL;
 	BOOL status = TRUE;
 	rdpSettings* settings = NULL;
-	UINT32 KeyboardLayout = 0;
 
 	if (!instance)
 		return -1;
@@ -119,6 +118,11 @@ static int freerdp_connect_begin(freerdp* instance)
 	instance->ConnectionCallbackState = CLIENT_STATE_PRECONNECT_PASSED;
 
 	freerdp_settings_print_warnings(settings);
+	if (status)
+		status = freerdp_settings_enforce_monitor_exists(settings);
+
+	if (status)
+		status = freerdp_settings_check_client_after_preconnect(settings);
 
 	if (status)
 		status = rdp_set_backup_settings(rdp);
@@ -126,7 +130,7 @@ static int freerdp_connect_begin(freerdp* instance)
 		status = utils_reload_channels(instance->context);
 
 	const UINT32 cp = freerdp_settings_get_uint32(settings, FreeRDP_KeyboardCodePage);
-	KeyboardLayout = freerdp_get_keyboard_default_layout_for_locale(cp);
+	int64_t KeyboardLayout = freerdp_get_keyboard_default_layout_for_locale(cp);
 	if (KeyboardLayout == 0)
 		KeyboardLayout = freerdp_settings_get_uint32(settings, FreeRDP_KeyboardLayout);
 
@@ -154,7 +158,8 @@ static int freerdp_connect_begin(freerdp* instance)
 		WINPR_ASSERT(context);
 		freerdp_set_last_error_if_not(context, FREERDP_ERROR_PRE_CONNECT_FAILED);
 
-		WLog_Print(context->log, WLOG_ERROR, "freerdp_pre_connect failed");
+		WLog_Print(context->log, WLOG_ERROR, "freerdp_pre_connect failed: %s",
+		           rdp_client_connection_state_string(instance->ConnectionCallbackState));
 		return 0;
 	}
 
@@ -187,13 +192,8 @@ BOOL freerdp_connect(freerdp* instance)
 	WINPR_ASSERT(rdp);
 	WINPR_ASSERT(rdp->settings);
 
-	if (rc < 0)
-		return FALSE;
-
-	if (rc == 0)
-		goto freerdp_connect_finally;
-
-	/* Pointers might have changed inbetween */
+	if (rc > 0)
+	/* Pointers might have changed in between */
 	{
 		rdp_update_internal* up = update_cast(rdp->update);
 
@@ -205,10 +205,7 @@ BOOL freerdp_connect(freerdp* instance)
 			if (up->pcap_rfx)
 				up->dump_rfx = TRUE;
 		}
-	}
 
-	if (rc > 0)
-	{
 		pointer_cache_register_callbacks(instance->context->update);
 		status = IFCALLRESULT(TRUE, instance->PostConnect, instance);
 		instance->ConnectionCallbackState = CLIENT_STATE_POSTCONNECT_PASSED;
@@ -321,7 +318,8 @@ BOOL freerdp_abort_connect_context(rdpContext* context)
 }
 
 #if defined(WITH_FREERDP_DEPRECATED)
-BOOL freerdp_get_fds(freerdp* instance, void** rfds, int* rcount, void** wfds, int* wcount)
+BOOL freerdp_get_fds(freerdp* instance, void** rfds, int* rcount, WINPR_ATTR_UNUSED void** wfds,
+                     WINPR_ATTR_UNUSED int* wcount)
 {
 	rdpRdp* rdp = NULL;
 
@@ -391,7 +389,11 @@ DWORD freerdp_get_event_handles(rdpContext* context, HANDLE* events, DWORD count
 	else
 		return 0;
 
-	return nCount;
+	const SSIZE_T rc = freerdp_client_channel_get_registered_event_handles(
+	    context->channels, &events[nCount], count - nCount);
+	if (rc < 0)
+		return 0;
+	return nCount + (DWORD)rc;
 }
 
 /* Resend mouse cursor position to prevent session lock in prevent-session-lock mode */
@@ -407,7 +409,8 @@ static BOOL freerdp_prevent_session_lock(rdpContext* context)
 	if (FakeMouseMotionInterval && in->lastInputTimestamp)
 	{
 		const time_t now = time(NULL);
-		if (now - in->lastInputTimestamp > FakeMouseMotionInterval)
+		if (WINPR_ASSERTING_INT_CAST(size_t, now) - in->lastInputTimestamp >
+		    FakeMouseMotionInterval)
 		{
 			WLog_Print(context->log, WLOG_DEBUG,
 			           "fake mouse move: x=%d y=%d lastInputTimestamp=%" PRIu64 " "
@@ -498,6 +501,8 @@ wMessageQueue* freerdp_get_message_queue(freerdp* instance, DWORD id)
 			queue = input->queue;
 		}
 		break;
+		default:
+			break;
 	}
 
 	return queue;
@@ -533,6 +538,8 @@ int freerdp_message_queue_process_message(freerdp* instance, DWORD id, wMessage*
 		case FREERDP_INPUT_MESSAGE_QUEUE:
 			status = input_message_queue_process_message(context->input, message);
 			break;
+		default:
+			break;
 	}
 
 	return status;
@@ -557,6 +564,8 @@ int freerdp_message_queue_process_pending_messages(freerdp* instance, DWORD id)
 		case FREERDP_INPUT_MESSAGE_QUEUE:
 			status = input_message_queue_process_pending_messages(context->input);
 			break;
+		default:
+			break;
 	}
 
 	return status;
@@ -718,12 +727,15 @@ const char* freerdp_get_version_string(void)
 
 const char* freerdp_get_build_config(void)
 {
+	WINPR_PRAGMA_DIAG_PUSH
+	WINPR_PRAGMA_DIAG_IGNORED_OVERLENGTH_STRINGS
 	static const char build_config[] =
 	    "Build configuration: " FREERDP_BUILD_CONFIG "\n"
 	    "Build type:          " FREERDP_BUILD_TYPE "\n"
 	    "CFLAGS:              " FREERDP_CFLAGS "\n"
 	    "Compiler:            " FREERDP_COMPILER_ID ", " FREERDP_COMPILER_VERSION "\n"
 	    "Target architecture: " FREERDP_TARGET_ARCH "\n";
+	WINPR_PRAGMA_DIAG_POP
 	return build_config;
 }
 
@@ -791,7 +803,6 @@ BOOL freerdp_context_new_ex(freerdp* instance, rdpSettings* settings)
 	if (!rdp)
 		goto fail;
 
-	rdp_log_build_warnings(rdp);
 	context->rdp = rdp;
 	context->pubSub = rdp->pubSub;
 
@@ -892,7 +903,7 @@ void freerdp_context_free(freerdp* instance)
 	ctx->metrics = NULL;
 
 	if (ctx->channelErrorEvent)
-		CloseHandle(ctx->channelErrorEvent);
+		(void)CloseHandle(ctx->channelErrorEvent);
 	ctx->channelErrorEvent = NULL;
 
 	free(ctx->errorDescription);
@@ -1062,14 +1073,14 @@ void freerdp_set_last_error_ex(rdpContext* context, UINT32 lastError, const char
 	if (lastError == FREERDP_ERROR_SUCCESS)
 	{
 		if (WLog_IsLevelActive(context->log, WLOG_DEBUG))
-			WLog_PrintMessage(context->log, WLOG_MESSAGE_TEXT, WLOG_DEBUG, line, file, fkt,
+			WLog_PrintMessage(context->log, WLOG_MESSAGE_TEXT, WLOG_DEBUG, (size_t)line, file, fkt,
 			                  "resetting error state");
 	}
 	else if (context->LastError != FREERDP_ERROR_SUCCESS)
 	{
 		if (WLog_IsLevelActive(context->log, WLOG_ERROR))
 		{
-			WLog_PrintMessage(context->log, WLOG_MESSAGE_TEXT, WLOG_ERROR, line, file, fkt,
+			WLog_PrintMessage(context->log, WLOG_MESSAGE_TEXT, WLOG_ERROR, (size_t)line, file, fkt,
 			                  "TODO: Trying to set error code %s, but %s already set!",
 			                  freerdp_get_last_error_name(lastError),
 			                  freerdp_get_last_error_name(context->LastError));
@@ -1210,15 +1221,39 @@ BOOL freerdp_nla_revert_to_self(rdpContext* context)
 
 UINT32 freerdp_get_nla_sspi_error(rdpContext* context)
 {
-	rdpNla* nla = NULL;
-
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(context->rdp);
 	WINPR_ASSERT(context->rdp->transport);
 
-	nla = transport_get_nla(context->rdp->transport);
+	rdpNla* nla = transport_get_nla(context->rdp->transport);
 
-	return nla_get_sspi_error(nla);
+	return (UINT32)nla_get_sspi_error(nla);
+}
+
+BOOL freerdp_nla_encrypt(rdpContext* context, const SecBuffer* inBuffer, SecBuffer* outBuffer)
+{
+	WINPR_ASSERT(context);
+	WINPR_ASSERT(context->rdp);
+
+	rdpNla* nla = context->rdp->nla;
+	return nla_encrypt(nla, inBuffer, outBuffer);
+}
+
+BOOL freerdp_nla_decrypt(rdpContext* context, const SecBuffer* inBuffer, SecBuffer* outBuffer)
+{
+	WINPR_ASSERT(context);
+	WINPR_ASSERT(context->rdp);
+
+	rdpNla* nla = context->rdp->nla;
+	return nla_decrypt(nla, inBuffer, outBuffer);
+}
+
+SECURITY_STATUS freerdp_nla_QueryContextAttributes(rdpContext* context, DWORD ulAttr, PVOID pBuffer)
+{
+	WINPR_ASSERT(context);
+	WINPR_ASSERT(context->rdp);
+
+	return nla_QueryContextAttributes(context->rdp->nla, ulAttr, pBuffer);
 }
 
 HANDLE getChannelErrorEventHandle(rdpContext* context)
@@ -1263,13 +1298,13 @@ void clearChannelError(rdpContext* context)
 	WINPR_ASSERT(context);
 	context->channelErrorNum = 0;
 	memset(context->errorDescription, 0, 500);
-	ResetEvent(context->channelErrorEvent);
+	(void)ResetEvent(context->channelErrorEvent);
 }
 
 WINPR_ATTR_FORMAT_ARG(3, 4)
 void setChannelError(rdpContext* context, UINT errorNum, WINPR_FORMAT_ARG const char* format, ...)
 {
-	va_list ap;
+	va_list ap = { 0 };
 	va_start(ap, format);
 
 	WINPR_ASSERT(context);
@@ -1277,7 +1312,7 @@ void setChannelError(rdpContext* context, UINT errorNum, WINPR_FORMAT_ARG const
 	context->channelErrorNum = errorNum;
 	(void)vsnprintf(context->errorDescription, 499, format, ap);
 	va_end(ap);
-	SetEvent(context->channelErrorEvent);
+	(void)SetEvent(context->channelErrorEvent);
 }
 
 const char* freerdp_nego_get_routing_token(rdpContext* context, DWORD* length)
@@ -1425,3 +1460,11 @@ BOOL freerdp_is_valid_mcs_create_response(const BYTE* data, size_t size)
 	test_mcs_free(mcs);
 	return result;
 }
+
+BOOL freerdp_persist_credentials(rdpContext* context)
+{
+	if (!context)
+		return FALSE;
+	WINPR_ASSERT(context->rdp);
+	return utils_persist_credentials(context->rdp->originalSettings, context->rdp->settings);
+}
