@@ -20,6 +20,7 @@
 #include <freerdp/config.h>
 
 #include <winpr/assert.h>
+#include <winpr/cast.h>
 #include <winpr/crt.h>
 #include <winpr/crypto.h>
 
@@ -321,12 +322,9 @@ static BOOL rts_read_auth_verifier(wStream* s, auth_verifier_co_t* auth,
 	if (!rts_read_auth_verifier_no_checks(s, auth, header, &pos, silent))
 		return FALSE;
 
-	{
-		const size_t expected = header->frag_length - header->auth_length - 8;
-		WINPR_ASSERT(pos + auth->auth_pad_length == expected);
-	}
-
-	return TRUE;
+	const size_t expected = header->frag_length - header->auth_length - 8;
+	WINPR_ASSERT(pos + auth->auth_pad_length == expected);
+	return pos + auth->auth_pad_length == expected;
 }
 
 static BOOL rts_read_auth_verifier_with_stub(wStream* s, auth_verifier_co_t* auth,
@@ -966,7 +964,7 @@ static BOOL rts_read_pdu_fault(wStream* s, rpcconn_fault_hdr_t* ctx, BOOL silent
 	Stream_Read_UINT8(s, ctx->reserved);
 	Stream_Read_UINT32(s, ctx->status);
 
-	WLog_WARN(TAG, "status=%s", Win32ErrorCode2Tag(ctx->status));
+	WLog_WARN(TAG, "status=%s", Win32ErrorCode2Tag(ctx->status & 0xFFFF));
 	return rts_read_auth_verifier_with_stub(s, &ctx->auth_verifier, &ctx->header, silent);
 }
 
@@ -1236,17 +1234,15 @@ static BOOL rts_write_pdu_header(wStream* s, const rpcconn_rts_hdr_t* header)
 	return TRUE;
 }
 
-static BOOL rts_receive_window_size_command_read(rdpRpc* rpc, wStream* buffer,
+static BOOL rts_receive_window_size_command_read(WINPR_ATTR_UNUSED rdpRpc* rpc, wStream* buffer,
                                                  UINT64* ReceiveWindowSize)
 {
-	UINT32 val = 0;
-
 	WINPR_ASSERT(rpc);
 	WINPR_ASSERT(buffer);
 
 	if (!Stream_CheckAndLogRequiredLength(TAG, buffer, 8))
 		return FALSE;
-	Stream_Read_UINT64(buffer, val);
+	const UINT64 val = Stream_Get_UINT64(buffer);
 	if (ReceiveWindowSize)
 		*ReceiveWindowSize = val; /* ReceiveWindowSize (8 bytes) */
 
@@ -1324,17 +1320,16 @@ static BOOL rts_flow_control_ack_command_write(wStream* s, UINT32 BytesReceived,
 	return TRUE;
 }
 
-static BOOL rts_connection_timeout_command_read(rdpRpc* rpc, wStream* buffer,
+static BOOL rts_connection_timeout_command_read(WINPR_ATTR_UNUSED rdpRpc* rpc, wStream* buffer,
                                                 UINT64* ConnectionTimeout)
 {
-	UINT32 val = 0;
 	WINPR_ASSERT(rpc);
 	WINPR_ASSERT(buffer);
 
 	if (!Stream_CheckAndLogRequiredLength(TAG, buffer, 8))
 		return FALSE;
 
-	Stream_Read_UINT64(buffer, val);
+	UINT64 val = Stream_Get_UINT64(buffer);
 	if (ConnectionTimeout)
 		*ConnectionTimeout = val; /* ConnectionTimeout (8 bytes) */
 
@@ -1384,7 +1379,7 @@ static BOOL rts_client_keepalive_command_write(wStream* s, UINT32 ClientKeepaliv
 	return TRUE;
 }
 
-static BOOL rts_version_command_read(rdpRpc* rpc, wStream* buffer)
+static BOOL rts_version_command_read(WINPR_ATTR_UNUSED rdpRpc* rpc, wStream* buffer)
 {
 	WINPR_ASSERT(rpc);
 	WINPR_ASSERT(buffer);
@@ -1473,7 +1468,8 @@ static BOOL rts_association_group_id_command_write(wStream* s, const BYTE* Assoc
 	return TRUE;
 }
 
-static int rts_destination_command_read(rdpRpc* rpc, wStream* buffer, UINT32* Destination)
+static int rts_destination_command_read(WINPR_ATTR_UNUSED rdpRpc* rpc, wStream* buffer,
+                                        UINT32* Destination)
 {
 	UINT32 val = 0;
 	WINPR_ASSERT(rpc);
@@ -1597,16 +1593,17 @@ BOOL rts_recv_CONN_A3_pdu(rdpRpc* rpc, wStream* buffer)
 		return FALSE;
 
 	rc = rts_connection_timeout_command_read(rpc, buffer, &ConnectionTimeout);
-	if (!rc)
+	if (!rc || (ConnectionTimeout > UINT32_MAX))
 		return rc;
 
-	WLog_DBG(TAG, "Receiving CONN/A3 RTS PDU: ConnectionTimeout: %" PRIu32 "", ConnectionTimeout);
+	WLog_DBG(TAG, "Receiving CONN/A3 RTS PDU: ConnectionTimeout: %" PRIu64 "", ConnectionTimeout);
 
 	WINPR_ASSERT(rpc);
 	WINPR_ASSERT(rpc->VirtualConnection);
 	WINPR_ASSERT(rpc->VirtualConnection->DefaultInChannel);
 
-	rpc->VirtualConnection->DefaultInChannel->PingOriginator.ConnectionTimeout = ConnectionTimeout;
+	rpc->VirtualConnection->DefaultInChannel->PingOriginator.ConnectionTimeout =
+	    (UINT32)ConnectionTimeout;
 	return TRUE;
 }
 
@@ -1686,13 +1683,14 @@ BOOL rts_recv_CONN_C2_pdu(rdpRpc* rpc, wStream* buffer)
 	if (!rc)
 		return rc;
 	rc = rts_receive_window_size_command_read(rpc, buffer, &ReceiveWindowSize);
-	if (!rc)
+	if (!rc || (ReceiveWindowSize > UINT32_MAX))
 		return rc;
 	rc = rts_connection_timeout_command_read(rpc, buffer, &ConnectionTimeout);
-	if (!rc)
+	if (!rc || (ConnectionTimeout > UINT32_MAX))
 		return rc;
+
 	WLog_DBG(TAG,
-	         "Receiving CONN/C2 RTS PDU: ConnectionTimeout: %" PRIu32 " ReceiveWindowSize: %" PRIu32
+	         "Receiving CONN/C2 RTS PDU: ConnectionTimeout: %" PRIu64 " ReceiveWindowSize: %" PRIu64
 	         "",
 	         ConnectionTimeout, ReceiveWindowSize);
 
@@ -1700,8 +1698,9 @@ BOOL rts_recv_CONN_C2_pdu(rdpRpc* rpc, wStream* buffer)
 	WINPR_ASSERT(rpc->VirtualConnection);
 	WINPR_ASSERT(rpc->VirtualConnection->DefaultInChannel);
 
-	rpc->VirtualConnection->DefaultInChannel->PingOriginator.ConnectionTimeout = ConnectionTimeout;
-	rpc->VirtualConnection->DefaultInChannel->PeerReceiveWindow = ReceiveWindowSize;
+	rpc->VirtualConnection->DefaultInChannel->PingOriginator.ConnectionTimeout =
+	    (UINT32)ConnectionTimeout;
+	rpc->VirtualConnection->DefaultInChannel->PeerReceiveWindow = (UINT32)ReceiveWindowSize;
 	return TRUE;
 }
 
@@ -2183,7 +2182,7 @@ static int rts_recv_OUT_R1_A2_pdu(rdpRpc* rpc, wStream* buffer)
 	return 1;
 }
 
-static int rts_recv_OUT_R2_A6_pdu(rdpRpc* rpc, wStream* buffer)
+static int rts_recv_OUT_R2_A6_pdu(rdpRpc* rpc, WINPR_ATTR_UNUSED wStream* buffer)
 {
 	int status = 0;
 	RpcVirtualConnection* connection = NULL;
@@ -2218,7 +2217,7 @@ static int rts_recv_OUT_R2_A6_pdu(rdpRpc* rpc, wStream* buffer)
 	return 1;
 }
 
-static int rts_recv_OUT_R2_B3_pdu(rdpRpc* rpc, wStream* buffer)
+static int rts_recv_OUT_R2_B3_pdu(rdpRpc* rpc, WINPR_ATTR_UNUSED wStream* buffer)
 {
 	RpcVirtualConnection* connection = NULL;
 
