@@ -102,7 +102,7 @@
  * 	subInterval			INTEGER (0..MAX)
  * }
  *
- * AttachUserRequest ::= [APPPLICATION 10] IMPLICIT SEQUENCE
+ * AttachUserRequest ::= [APPLICATION 10] IMPLICIT SEQUENCE
  * {
  * }
  *
@@ -300,8 +300,6 @@ static BOOL mcs_read_domain_mcspdu_header(wStream* s, DomainMCSPDU domainMCSPDU,
 
 static int mcs_initialize_client_channels(rdpMcs* mcs, const rdpSettings* settings)
 {
-	UINT32 index;
-
 	if (!mcs || !settings)
 		return -1;
 
@@ -312,7 +310,7 @@ static int mcs_initialize_client_channels(rdpMcs* mcs, const rdpSettings* settin
 
 	ZeroMemory(mcs->channels, sizeof(rdpMcsChannel) * mcs->channelMaxCount);
 
-	for (index = 0; index < mcs->channelCount; index++)
+	for (UINT32 index = 0; index < mcs->channelCount; index++)
 	{
 		const CHANNEL_DEF* defchannel =
 		    freerdp_settings_get_pointer_array(settings, FreeRDP_ChannelDefArray, index);
@@ -337,9 +335,8 @@ static int mcs_initialize_client_channels(rdpMcs* mcs, const rdpSettings* settin
 BOOL mcs_read_domain_mcspdu_header(wStream* s, DomainMCSPDU domainMCSPDU, UINT16* length,
                                    DomainMCSPDU* actual)
 {
-	UINT16 li;
-	BYTE choice;
-	DomainMCSPDU MCSPDU;
+	UINT16 li = 0;
+	BYTE choice = 0;
 
 	if (actual)
 		*actual = DomainMCSPDU_invalid;
@@ -357,7 +354,7 @@ BOOL mcs_read_domain_mcspdu_header(wStream* s, DomainMCSPDU domainMCSPDU, UINT16
 	if (!per_read_choice(s, &choice))
 		return FALSE;
 
-	MCSPDU = (choice >> 2);
+	const DomainMCSPDU MCSPDU = (choice >> 2);
 	if (actual)
 		*actual = MCSPDU;
 
@@ -450,8 +447,8 @@ static BOOL mcs_read_domain_parameters(wStream* s, DomainParameters* domainParam
 
 static BOOL mcs_write_domain_parameters(wStream* s, DomainParameters* domainParameters)
 {
-	size_t length;
-	wStream* tmps;
+	size_t length = 0;
+	wStream* tmps = NULL;
 
 	if (!s || !domainParameters)
 		return FALSE;
@@ -720,8 +717,8 @@ BOOL mcs_recv_connect_initial(rdpMcs* mcs, wStream* s)
 
 BOOL mcs_write_connect_initial(wStream* s, rdpMcs* mcs, wStream* userData)
 {
-	size_t length;
-	wStream* tmps;
+	size_t length = 0;
+	wStream* tmps = NULL;
 	BOOL ret = FALSE;
 
 	if (!s || !mcs || !userData)
@@ -778,8 +775,8 @@ out:
 
 BOOL mcs_write_connect_response(wStream* s, rdpMcs* mcs, wStream* userData)
 {
-	size_t length;
-	wStream* tmps;
+	size_t length = 0;
+	wStream* tmps = NULL;
 	BOOL ret = FALSE;
 
 	if (!s || !mcs || !userData)
@@ -819,12 +816,13 @@ out:
 static BOOL mcs_send_connect_initial(rdpMcs* mcs)
 {
 	int status = -1;
-	size_t length;
+	size_t length = 0;
 	wStream* s = NULL;
-	size_t bm, em;
+	size_t bm = 0;
+	size_t em = 0;
 	wStream* gcc_CCrq = NULL;
 	wStream* client_data = NULL;
-	rdpContext* context;
+	rdpContext* context = NULL;
 
 	if (!mcs)
 		return FALSE;
@@ -898,11 +896,11 @@ out:
 
 BOOL mcs_recv_connect_response(rdpMcs* mcs, wStream* s)
 {
-	size_t length;
-	UINT16 tlength;
-	BYTE result;
-	UINT16 li;
-	UINT32 calledConnectId;
+	size_t length = 0;
+	UINT16 tlength = 0;
+	BYTE result = 0;
+	UINT16 li = 0;
+	UINT32 calledConnectId = 0;
 
 	if (!mcs || !s)
 		return FALSE;
@@ -939,10 +937,11 @@ BOOL mcs_recv_connect_response(rdpMcs* mcs, wStream* s)
 
 BOOL mcs_send_connect_response(rdpMcs* mcs)
 {
-	size_t length;
+	size_t length = 0;
 	int status = -1;
 	wStream* s = NULL;
-	size_t bm, em;
+	size_t bm = 0;
+	size_t em = 0;
 	wStream* gcc_CCrsp = NULL;
 	wStream* server_data = NULL;
 
@@ -1011,11 +1010,11 @@ out:
  * @param s stream
  */
 
-BOOL mcs_recv_erect_domain_request(rdpMcs* mcs, wStream* s)
+BOOL mcs_recv_erect_domain_request(WINPR_ATTR_UNUSED rdpMcs* mcs, wStream* s)
 {
-	UINT16 length;
-	UINT32 subHeight;
-	UINT32 subInterval;
+	UINT16 length = 0;
+	UINT32 subHeight = 0;
+	UINT32 subInterval = 0;
 
 	WINPR_ASSERT(mcs);
 	WINPR_ASSERT(s);
@@ -1040,8 +1039,8 @@ BOOL mcs_recv_erect_domain_request(rdpMcs* mcs, wStream* s)
 
 BOOL mcs_send_erect_domain_request(rdpMcs* mcs)
 {
-	wStream* s;
-	int status;
+	wStream* s = NULL;
+	int status = 0;
 	UINT16 length = 12;
 
 	if (!mcs)
@@ -1073,7 +1072,7 @@ BOOL mcs_send_erect_domain_request(rdpMcs* mcs)
 
 BOOL mcs_recv_attach_user_request(rdpMcs* mcs, wStream* s)
 {
-	UINT16 length;
+	UINT16 length = 0;
 
 	if (!mcs || !s)
 		return FALSE;
@@ -1091,8 +1090,8 @@ BOOL mcs_recv_attach_user_request(rdpMcs* mcs, wStream* s)
 
 BOOL mcs_send_attach_user_request(rdpMcs* mcs)
 {
-	wStream* s;
-	int status;
+	wStream* s = NULL;
+	int status = 0;
 	UINT16 length = 8;
 
 	if (!mcs)
@@ -1121,8 +1120,8 @@ BOOL mcs_send_attach_user_request(rdpMcs* mcs)
 
 BOOL mcs_recv_attach_user_confirm(rdpMcs* mcs, wStream* s)
 {
-	BYTE result;
-	UINT16 length;
+	BYTE result = 0;
+	UINT16 length = 0;
 
 	if (!mcs || !s)
 		return FALSE;
@@ -1144,8 +1143,8 @@ BOOL mcs_recv_attach_user_confirm(rdpMcs* mcs, wStream* s)
 
 BOOL mcs_send_attach_user_confirm(rdpMcs* mcs)
 {
-	wStream* s;
-	int status;
+	wStream* s = NULL;
+	int status = 0;
 	UINT16 length = 11;
 
 	if (!mcs)
@@ -1179,8 +1178,8 @@ BOOL mcs_send_attach_user_confirm(rdpMcs* mcs)
 BOOL mcs_recv_channel_join_request(rdpMcs* mcs, const rdpSettings* settings, wStream* s,
                                    UINT16* channelId)
 {
-	UINT16 length;
-	UINT16 userId;
+	UINT16 length = 0;
+	UINT16 userId = 0;
 
 	if (!mcs || !s || !channelId)
 		return FALSE;
@@ -1215,8 +1214,8 @@ BOOL mcs_recv_channel_join_request(rdpMcs* mcs, const rdpSettings* settings, wSt
 
 BOOL mcs_send_channel_join_request(rdpMcs* mcs, UINT16 channelId)
 {
-	wStream* s;
-	int status;
+	wStream* s = NULL;
+	int status = 0;
 	UINT16 length = 12;
 
 	WINPR_ASSERT(mcs);
@@ -1244,12 +1243,12 @@ BOOL mcs_send_channel_join_request(rdpMcs* mcs, UINT16 channelId)
  * @param mcs mcs module
  */
 
-BOOL mcs_recv_channel_join_confirm(rdpMcs* mcs, wStream* s, UINT16* channelId)
+BOOL mcs_recv_channel_join_confirm(WINPR_ATTR_UNUSED rdpMcs* mcs, wStream* s, UINT16* channelId)
 {
-	UINT16 length;
-	BYTE result;
-	UINT16 initiator;
-	UINT16 requested;
+	UINT16 length = 0;
+	BYTE result = 0;
+	UINT16 initiator = 0;
+	UINT16 requested = 0;
 
 	WINPR_ASSERT(mcs);
 	WINPR_ASSERT(channelId);
@@ -1276,7 +1275,7 @@ BOOL mcs_recv_channel_join_confirm(rdpMcs* mcs, wStream* s, UINT16* channelId)
 
 BOOL mcs_send_channel_join_confirm(rdpMcs* mcs, UINT16 channelId)
 {
-	wStream* s;
+	wStream* s = NULL;
 	int status = -1;
 	UINT16 length = 15;
 
@@ -1313,9 +1312,10 @@ fail:
  * @param mcs mcs module
  */
 
-BOOL mcs_recv_disconnect_provider_ultimatum(rdpMcs* mcs, wStream* s, int* reason)
+BOOL mcs_recv_disconnect_provider_ultimatum(WINPR_ATTR_UNUSED rdpMcs* mcs, wStream* s, int* reason)
 {
-	BYTE b1, b2;
+	BYTE b1 = 0;
+	BYTE b2 = 0;
 
 	WINPR_ASSERT(mcs);
 	WINPR_ASSERT(s);
@@ -1389,7 +1389,7 @@ fail:
 
 BOOL mcs_client_begin(rdpMcs* mcs)
 {
-	rdpContext* context;
+	rdpContext* context = NULL;
 
 	if (!mcs || !mcs->transport)
 		return FALSE;
@@ -1419,7 +1419,7 @@ BOOL mcs_client_begin(rdpMcs* mcs)
 
 rdpMcs* mcs_new(rdpTransport* transport)
 {
-	rdpMcs* mcs;
+	rdpMcs* mcs = NULL;
 
 	mcs = (rdpMcs*)calloc(1, sizeof(rdpMcs));
 
