@@ -25,6 +25,7 @@
 #include "settings.h"
 
 #include <winpr/assert.h>
+#include <winpr/cast.h>
 #include <winpr/crt.h>
 #include <winpr/winsock.h>
 
@@ -93,7 +94,9 @@ static HANDLE freerdp_peer_virtual_channel_open(freerdp_peer* client, const char
 		return (HANDLE)peerChannel;
 	}
 
-	peerChannel = server_channel_common_new(client, index, mcsChannel->ChannelId, 128, NULL, name);
+	WINPR_ASSERT(index <= UINT16_MAX);
+	peerChannel =
+	    server_channel_common_new(client, (UINT16)index, mcsChannel->ChannelId, 128, NULL, name);
 
 	if (peerChannel)
 	{
@@ -105,7 +108,8 @@ static HANDLE freerdp_peer_virtual_channel_open(freerdp_peer* client, const char
 	return (HANDLE)peerChannel;
 }
 
-static BOOL freerdp_peer_virtual_channel_close(freerdp_peer* client, HANDLE hChannel)
+static BOOL freerdp_peer_virtual_channel_close(WINPR_ATTR_UNUSED freerdp_peer* client,
+                                               HANDLE hChannel)
 {
 	rdpMcsChannel* mcsChannel = NULL;
 	rdpPeerChannel* peerChannel = NULL;
@@ -187,7 +191,8 @@ static int freerdp_peer_virtual_channel_write(freerdp_peer* client, HANDLE hChan
 
 		Stream_Write(s, buffer, chunkSize);
 
-		if (!rdp_send(rdp, s, peerChannel->channelId))
+		WINPR_ASSERT(peerChannel->channelId <= UINT16_MAX);
+		if (!rdp_send(rdp, s, (UINT16)peerChannel->channelId))
 			return -1;
 
 		buffer += chunkSize;
@@ -198,7 +203,8 @@ static int freerdp_peer_virtual_channel_write(freerdp_peer* client, HANDLE hChan
 	return 1;
 }
 
-static void* freerdp_peer_virtual_channel_get_data(freerdp_peer* client, HANDLE hChannel)
+static void* freerdp_peer_virtual_channel_get_data(WINPR_ATTR_UNUSED freerdp_peer* client,
+                                                   HANDLE hChannel)
 {
 	rdpPeerChannel* peerChannel = (rdpPeerChannel*)hChannel;
 
@@ -209,7 +215,8 @@ static void* freerdp_peer_virtual_channel_get_data(freerdp_peer* client, HANDLE
 	return peerChannel->extra;
 }
 
-static int freerdp_peer_virtual_channel_set_data(freerdp_peer* client, HANDLE hChannel, void* data)
+static int freerdp_peer_virtual_channel_set_data(WINPR_ATTR_UNUSED freerdp_peer* client,
+                                                 HANDLE hChannel, void* data)
 {
 	rdpPeerChannel* peerChannel = (rdpPeerChannel*)hChannel;
 
@@ -326,7 +333,8 @@ static BOOL freerdp_peer_check_fds(freerdp_peer* peer)
 	return TRUE;
 }
 
-static state_run_t peer_recv_data_pdu(freerdp_peer* client, wStream* s, UINT16 totalLength)
+static state_run_t peer_recv_data_pdu(freerdp_peer* client, wStream* s,
+                                      WINPR_ATTR_UNUSED UINT16 totalLength)
 {
 	BYTE type = 0;
 	UINT16 length = 0;
@@ -449,27 +457,13 @@ static state_run_t peer_recv_tpkt_pdu(freerdp_peer* client, wStream* s)
 
 	if (rdp_get_state(rdp) <= CONNECTION_STATE_LICENSING)
 	{
-		if (!rdp_read_security_header(rdp, s, &securityFlags, &length))
+		if (!rdp_handle_message_channel(rdp, s, channelId, length))
 			return STATE_RUN_FAILED;
-		if (securityFlags & SEC_ENCRYPT)
-		{
-			if (!rdp_decrypt(rdp, s, &length, securityFlags))
-				return STATE_RUN_FAILED;
-		}
-		return rdp_recv_message_channel_pdu(rdp, s, securityFlags);
+		return STATE_RUN_SUCCESS;
 	}
 
-	if (settings->UseRdpSecurityLayer)
-	{
-		if (!rdp_read_security_header(rdp, s, &securityFlags, &length))
-			return STATE_RUN_FAILED;
-
-		if (securityFlags & SEC_ENCRYPT)
-		{
-			if (!rdp_decrypt(rdp, s, &length, securityFlags))
-				return STATE_RUN_FAILED;
-		}
-	}
+	if (!rdp_handle_optional_rdp_decryption(rdp, s, &length, &securityFlags))
+		return STATE_RUN_FAILED;
 
 	if (channelId == MCS_GLOBAL_CHANNEL_ID)
 	{
@@ -755,9 +749,6 @@ static state_run_t rdp_peer_handle_state_active(freerdp_peer* client)
 	WINPR_ASSERT(client);
 	WINPR_ASSERT(client->context);
 
-	rdpRdp* rdp = client->context->rdp;
-	WINPR_ASSERT(rdp);
-
 	if (!client->connected)
 	{
 		/**
@@ -794,7 +785,8 @@ static state_run_t rdp_peer_handle_state_active(freerdp_peer* client)
 	return ret;
 }
 
-static state_run_t peer_recv_callback_internal(rdpTransport* transport, wStream* s, void* extra)
+static state_run_t peer_recv_callback_internal(WINPR_ATTR_UNUSED rdpTransport* transport,
+                                               wStream* s, void* extra)
 {
 	UINT32 SelectedProtocol = 0;
 	freerdp_peer* client = (freerdp_peer*)extra;
@@ -1004,8 +996,8 @@ static state_run_t peer_recv_callback_internal(rdpTransport* transport, wStream*
 					const UINT32 h = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
 					const rdpMonitor primary = { .x = 0,
 						                         .y = 0,
-						                         .width = w,
-						                         .height = h,
+						                         .width = WINPR_ASSERTING_INT_CAST(int32_t, w),
+						                         .height = WINPR_ASSERTING_INT_CAST(int32_t, h),
 						                         .is_primary = TRUE,
 						                         .orig_screen = 0,
 						                         .attributes = { .physicalWidth = w,
@@ -1131,8 +1123,11 @@ static state_run_t peer_recv_callback_internal(rdpTransport* transport, wStream*
 				{
 					if (!rdp_server_transition_to_state(rdp, CONNECTION_STATE_ACTIVE))
 						ret = STATE_RUN_FAILED;
-					update_reset_state(rdp->update);
-					ret = STATE_RUN_CONTINUE;
+					else
+					{
+						update_reset_state(rdp->update);
+						ret = STATE_RUN_CONTINUE;
+					}
 				}
 				else
 					ret = peer_unexpected_client_message(rdp, FINALIZE_CS_FONT_LIST_PDU);
@@ -1248,6 +1243,7 @@ static BOOL freerdp_peer_send_channel_data(freerdp_peer* client, UINT16 channelI
 static BOOL freerdp_peer_send_server_redirection_pdu(freerdp_peer* peer,
                                                      const rdpRedirection* redirection)
 {
+	BOOL rc = FALSE;
 	WINPR_ASSERT(peer);
 	WINPR_ASSERT(peer->context);
 
@@ -1258,11 +1254,10 @@ static BOOL freerdp_peer_send_server_redirection_pdu(freerdp_peer* peer,
 		goto fail;
 	if (!rdp_send_pdu(peer->context->rdp, s, PDU_TYPE_SERVER_REDIRECTION, 0))
 		goto fail;
-
-	return rdp_reset_runtime_settings(peer->context->rdp);
+	rc = rdp_reset_runtime_settings(peer->context->rdp);
 fail:
 	Stream_Release(s);
-	return FALSE;
+	return rc;
 }
 
 static BOOL freerdp_peer_send_channel_packet(freerdp_peer* client, UINT16 channelId,
@@ -1306,7 +1301,8 @@ static BOOL freerdp_peer_has_more_to_read(freerdp_peer* peer)
 	return transport_have_more_bytes_to_read(peer->context->rdp->transport);
 }
 
-static LicenseCallbackResult freerdp_peer_nolicense(freerdp_peer* peer, wStream* s)
+static LicenseCallbackResult freerdp_peer_nolicense(freerdp_peer* peer,
+                                                    WINPR_ATTR_UNUSED wStream* s)
 {
 	rdpRdp* rdp = NULL;
 
@@ -1340,7 +1336,7 @@ void freerdp_peer_context_free(freerdp_peer* client)
 	{
 		rdpContext* ctx = client->context;
 
-		CloseHandle(ctx->channelErrorEvent);
+		(void)CloseHandle(ctx->channelErrorEvent);
 		ctx->channelErrorEvent = NULL;
 		free(ctx->errorDescription);
 		ctx->errorDescription = NULL;
@@ -1377,6 +1373,8 @@ static const char* os_major_type_to_string(UINT16 osMajorType)
 			return "Android platform";
 		case OSMAJORTYPE_CHROME_OS:
 			return "Chrome OS platform";
+		default:
+			break;
 	}
 
 	return "Unknown platform";
@@ -1384,18 +1382,15 @@ static const char* os_major_type_to_string(UINT16 osMajorType)
 
 const char* freerdp_peer_os_major_type_string(freerdp_peer* client)
 {
-	rdpContext* context = NULL;
-	UINT16 osMajorType = 0;
-
 	WINPR_ASSERT(client);
 
-	context = client->context;
+	rdpContext* context = client->context;
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(context->settings);
 
-	osMajorType = freerdp_settings_get_uint32(context->settings, FreeRDP_OsMajorType);
-
-	return os_major_type_to_string(osMajorType);
+	const UINT32 osMajorType = freerdp_settings_get_uint32(context->settings, FreeRDP_OsMajorType);
+	WINPR_ASSERT(osMajorType <= UINT16_MAX);
+	return os_major_type_to_string((UINT16)osMajorType);
 }
 
 static const char* os_minor_type_to_string(UINT16 osMinorType)
@@ -1422,6 +1417,8 @@ static const char* os_minor_type_to_string(UINT16 osMinorType)
 			return "Pseudo X Server";
 		case OSMINORTYPE_WINDOWS_RT:
 			return "Windows RT";
+		default:
+			break;
 	}
 
 	return "Unknown version";
@@ -1429,18 +1426,15 @@ static const char* os_minor_type_to_string(UINT16 osMinorType)
 
 const char* freerdp_peer_os_minor_type_string(freerdp_peer* client)
 {
-	rdpContext* context = NULL;
-	UINT16 osMinorType = 0;
-
 	WINPR_ASSERT(client);
 
-	context = client->context;
+	rdpContext* context = client->context;
 	WINPR_ASSERT(context);
 	WINPR_ASSERT(context->settings);
 
-	osMinorType = freerdp_settings_get_uint32(context->settings, FreeRDP_OsMinorType);
-
-	return os_minor_type_to_string(osMinorType);
+	const UINT32 osMinorType = freerdp_settings_get_uint32(context->settings, FreeRDP_OsMinorType);
+	WINPR_ASSERT(osMinorType <= UINT16_MAX);
+	return os_minor_type_to_string((UINT16)osMinorType);
 }
 
 freerdp_peer* freerdp_peer_new(int sockfd)
