@@ -56,6 +56,72 @@ static const char client_dll[] = "C:\\Windows\\System32\\mstscax.dll";
 #define GLYPH_CACHE_KEY CLIENT_KEY "\\GlyphCache"
 #define POINTER_CACHE_KEY CLIENT_KEY "\\PointerCache"
 
+struct bounds_t
+{
+	INT32 x;
+	INT32 y;
+	INT32 width;
+	INT32 height;
+};
+
+static const char* bounds2str(const struct bounds_t* bounds, char* buffer, size_t len)
+{
+	WINPR_ASSERT(bounds);
+	(void)_snprintf(buffer, len, "{%dx%d-%dx%d}", bounds->x, bounds->y, bounds->x + bounds->width,
+	                bounds->y + bounds->height);
+	return buffer;
+}
+
+static struct bounds_t union_rect(const struct bounds_t* a, const struct bounds_t* b)
+{
+	WINPR_ASSERT(a);
+	WINPR_ASSERT(b);
+
+	struct bounds_t u = *a;
+	if (b->x < u.x)
+		u.x = b->x;
+	if (b->y < u.y)
+		u.y = b->y;
+
+	const INT32 rightA = a->x + a->width;
+	const INT32 rightB = b->x + b->width;
+	const INT32 right = MAX(rightA, rightB);
+	u.width = right - u.x;
+
+	const INT32 bottomA = a->y + a->height;
+	const INT32 bottomB = b->y + b->height;
+	const INT32 bottom = MAX(bottomA, bottomB);
+	u.height = bottom - u.y;
+
+	return u;
+}
+
+static BOOL intersect_rects(const struct bounds_t* r1, const struct bounds_t* r2)
+{
+	WINPR_ASSERT(r1);
+	WINPR_ASSERT(r2);
+
+	const INT32 left = MAX(r1->x, r2->x);
+	const INT32 top = MAX(r1->y, r2->y);
+	const INT32 right = MIN(r1->x + r1->width, r2->x + r2->width);
+	const INT32 bottom = MIN(r1->y + r1->height, r2->y + r2->height);
+
+	return (left < right) && (top < bottom);
+}
+
+static BOOL align_rects(const struct bounds_t* r1, const struct bounds_t* r2)
+{
+	WINPR_ASSERT(r1);
+	WINPR_ASSERT(r2);
+
+	const INT32 left = MAX(r1->x, r2->x);
+	const INT32 top = MAX(r1->y, r2->y);
+	const INT32 right = MIN(r1->x + r1->width, r2->x + r2->width);
+	const INT32 bottom = MIN(r1->y + r1->height, r2->y + r2->height);
+
+	return (left == right) || (top == bottom);
+}
+
 static BOOL settings_reg_query_dword_val(HKEY hKey, const TCHAR* sub, DWORD* value)
 {
 	DWORD dwType = 0;
@@ -252,18 +318,17 @@ static void settings_load_hkey_local_machine(rdpSettings* settings)
 		settings_client_load_hkey_local_machine(settings);
 }
 
-static BOOL settings_get_computer_name(rdpSettings* settings)
+static BOOL settings_init_computer_name(rdpSettings* settings)
 {
-	CHAR computerName[256] = { 0 };
-	DWORD nSize = sizeof(computerName);
+	CHAR computerName[MAX_COMPUTERNAME_LENGTH + 1] = { 0 };
+	DWORD nSize = ARRAYSIZE(computerName);
 
 	if (!GetComputerNameExA(ComputerNameNetBIOS, computerName, &nSize))
 		return FALSE;
 
-	if (nSize > MAX_COMPUTERNAME_LENGTH)
-		computerName[MAX_COMPUTERNAME_LENGTH] = '\0';
-
-	return freerdp_settings_set_string(settings, FreeRDP_ComputerName, computerName);
+	if (!freerdp_settings_set_string(settings, FreeRDP_ComputerName, computerName))
+		return FALSE;
+	return freerdp_settings_set_string(settings, FreeRDP_ClientHostname, computerName);
 }
 
 void freerdp_settings_print_warnings(const rdpSettings* settings)
@@ -277,6 +342,375 @@ void freerdp_settings_print_warnings(const rdpSettings* settings)
 	}
 }
 
+static BOOL monitor_operlaps(const rdpSettings* settings, UINT32 orig, UINT32 start, UINT32 count,
+                             const rdpMonitor* compare)
+{
+	const struct bounds_t rect1 = {
+		.x = compare->x, .y = compare->y, .width = compare->width, .height = compare->height
+	};
+	for (UINT32 x = start; x < count; x++)
+	{
+		const rdpMonitor* monitor =
+		    freerdp_settings_get_pointer_array(settings, FreeRDP_MonitorDefArray, x);
+		const struct bounds_t rect2 = {
+			.x = monitor->x, .y = monitor->y, .width = monitor->width, .height = monitor->height
+		};
+
+		if (intersect_rects(&rect1, &rect2))
+		{
+			char buffer1[32] = { 0 };
+			char buffer2[32] = { 0 };
+
+			WLog_ERR(TAG, "Monitor %" PRIu32 " and %" PRIu32 " are overlapping:", orig, x);
+			WLog_ERR(TAG, "%s overlapps with %s", bounds2str(&rect1, buffer1, sizeof(buffer1)),
+			         bounds2str(&rect2, buffer2, sizeof(buffer2)));
+			WLog_ERR(
+			    TAG,
+			    "Mulitimonitor mode requested, but local layout has gaps or overlapping areas!");
+			WLog_ERR(TAG,
+			         "Please reconfigure your local monitor setup so that there are no gaps or "
+			         "overlapping areas!");
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
+static BOOL monitor_has_gaps(const rdpSettings* settings, UINT32 start, UINT32 count,
+                             const rdpMonitor* compare, UINT32** graph)
+{
+	const struct bounds_t rect1 = {
+		.x = compare->x, .y = compare->y, .width = compare->width, .height = compare->height
+	};
+
+	BOOL hasNeighbor = FALSE;
+	for (UINT32 i = 0; i < count; i++)
+	{
+		if (i == start)
+			continue;
+
+		const rdpMonitor* monitor =
+		    freerdp_settings_get_pointer_array(settings, FreeRDP_MonitorDefArray, i);
+
+		const struct bounds_t rect2 = {
+			.x = monitor->x, .y = monitor->y, .width = monitor->width, .height = monitor->height
+		};
+		if (align_rects(&rect1, &rect2))
+		{
+			hasNeighbor = TRUE;
+			graph[start][i] = 1;
+			graph[i][start] = 1;
+		}
+	}
+
+	if (!hasNeighbor)
+	{
+		WLog_ERR(TAG,
+		         "Monitor configuration has gaps! Monitor %" PRIu32 " does not have any neighbor",
+		         start);
+	}
+
+	return !hasNeighbor;
+}
+
+static void alloc_free(UINT32** ptr)
+{
+	free((void*)ptr);
+}
+
+WINPR_ATTR_MALLOC(alloc_free, 1)
+static UINT32** alloc_array(size_t count)
+{
+	// NOLINTNEXTLINE(clang-analyzer-unix.MallocSizeof)
+	BYTE* array = calloc(count * sizeof(uintptr_t), count * sizeof(UINT32));
+	UINT32** dst = (UINT32**)array;
+	UINT32* val = (UINT32*)(array + count * sizeof(uintptr_t));
+	for (size_t x = 0; x < count; x++)
+		dst[x] = &val[x];
+
+	return dst;
+}
+
+/* Monitors in the array need to:
+ *
+ * 1. be connected to another monitor (edges touch but don't overlap or have gaps)
+ * 2. all monitors need to be connected so there are no separate groups.
+ *
+ * So, what we do here is we use dijkstra algorithm to find a path from any start node
+ * (lazy as we are we always use the first in the array) to each other node.
+ */
+static BOOL find_path_exists_with_dijkstra(UINT32** graph, size_t count, UINT32 start)
+{
+	if (count < 1)
+		return FALSE;
+
+	WINPR_ASSERT(start < count);
+
+	UINT32** cost = alloc_array(count);
+	WINPR_ASSERT(cost);
+
+	UINT32* distance = calloc(count, sizeof(UINT32));
+	WINPR_ASSERT(distance);
+
+	UINT32* visited = calloc(count, sizeof(UINT32));
+	WINPR_ASSERT(visited);
+
+	UINT32* parent = calloc(count, sizeof(UINT32));
+	WINPR_ASSERT(parent);
+
+	for (size_t x = 0; x < count; x++)
+	{
+		for (size_t y = 0; y < count; y++)
+		{
+			if (graph[x][y] == 0)
+				cost[x][y] = UINT32_MAX;
+			else
+				cost[x][y] = graph[x][y];
+		}
+	}
+
+	for (UINT32 x = 0; x < count; x++)
+	{
+		distance[x] = cost[start][x];
+		parent[x] = start;
+		visited[x] = 0;
+	}
+
+	distance[start] = 0;
+	visited[start] = 1;
+
+	size_t pos = 1;
+	UINT32 nextnode = UINT32_MAX;
+	while (pos < count - 1)
+	{
+		UINT32 mindistance = UINT32_MAX;
+
+		for (UINT32 x = 0; x < count; x++)
+		{
+			if ((distance[x] < mindistance) && !visited[x])
+			{
+				mindistance = distance[x];
+				nextnode = x;
+			}
+		}
+
+		visited[nextnode] = 1;
+
+		for (size_t y = 0; y < count; y++)
+		{
+			if (!visited[y])
+			{
+				UINT32 dist = mindistance + cost[nextnode][y];
+				if (dist < distance[y])
+				{
+					distance[y] = dist;
+					parent[y] = nextnode;
+				}
+			}
+		}
+		pos++;
+	}
+
+	BOOL rc = TRUE;
+	for (size_t x = 0; x < count; x++)
+	{
+		if (x != start)
+		{
+			if (distance[x] == UINT32_MAX)
+			{
+				WLog_ERR(TAG, "monitor %" PRIu32 " not connected with monitor %" PRIuz, start, x);
+				rc = FALSE;
+				break;
+			}
+		}
+	}
+	alloc_free(cost);
+	free(distance);
+	free(visited);
+	free(parent);
+
+	return rc;
+}
+
+static BOOL freerdp_settings_client_monitors_have_gaps(const rdpSettings* settings)
+{
+	BOOL rc = TRUE;
+	const UINT32 count = freerdp_settings_get_uint32(settings, FreeRDP_MonitorCount);
+	if (count <= 1)
+		return FALSE;
+
+	UINT32** graph = alloc_array(count);
+	WINPR_ASSERT(graph);
+
+	for (UINT32 x = 0; x < count; x++)
+	{
+		const rdpMonitor* monitor =
+		    freerdp_settings_get_pointer_array(settings, FreeRDP_MonitorDefArray, x);
+		if (monitor_has_gaps(settings, x, count, monitor, graph))
+			goto fail;
+	}
+
+	rc = !find_path_exists_with_dijkstra(graph, count, 0);
+
+fail:
+	alloc_free(graph);
+
+	return rc;
+}
+
+static void log_monitor(UINT32 idx, const rdpMonitor* monitor, wLog* log, DWORD level)
+{
+	WINPR_ASSERT(monitor);
+
+	WLog_Print(log, level,
+	           "[%" PRIu32 "] [%s] {%dx%d-%dx%d} [%" PRIu32 "] {%" PRIu32 "x%" PRIu32
+	           ", orientation: %" PRIu32 ", desktopScale: %" PRIu32 ", deviceScale: %" PRIu32 "}",
+	           idx, monitor->is_primary ? "primary" : "       ", monitor->x, monitor->y,
+	           monitor->width, monitor->height, monitor->orig_screen,
+	           monitor->attributes.physicalWidth, monitor->attributes.physicalHeight,
+	           monitor->attributes.orientation, monitor->attributes.desktopScaleFactor,
+	           monitor->attributes.deviceScaleFactor);
+}
+
+static void log_monitor_configuration(const rdpSettings* settings, wLog* log, DWORD level)
+{
+	const UINT32 count = freerdp_settings_get_uint32(settings, FreeRDP_MonitorCount);
+	WLog_Print(log, level, "[BEGIN] MonitorDefArray[%" PRIu32 "]", count);
+	for (UINT32 x = 0; x < count; x++)
+	{
+		const rdpMonitor* monitor =
+		    freerdp_settings_get_pointer_array(settings, FreeRDP_MonitorDefArray, x);
+		log_monitor(x, monitor, log, level);
+	}
+	WLog_Print(log, level, "[END] MonitorDefArray[%" PRIu32 "]", count);
+}
+
+static BOOL freerdp_settings_client_monitors_overlap(const rdpSettings* settings)
+{
+	const UINT32 count = freerdp_settings_get_uint32(settings, FreeRDP_MonitorCount);
+	for (UINT32 x = 0; x < count; x++)
+	{
+		const rdpMonitor* monitor =
+		    freerdp_settings_get_pointer_array(settings, FreeRDP_MonitorDefArray, x);
+		if (monitor_operlaps(settings, x, x + 1, count, monitor))
+			return TRUE;
+	}
+	return FALSE;
+}
+
+/* See [MS-RDPBCGR] 2.2.1.3.6.1 for details on limits
+ * https://learn.microsoft.com/en-us/openspecs/windows_protocols/ms-rdpbcgr/c3964b39-3d54-4ae1-a84a-ceaed311e0f6
+ */
+static BOOL freerdp_settings_client_monitors_check_primary_and_origin(const rdpSettings* settings)
+{
+	const UINT32 count = freerdp_settings_get_uint32(settings, FreeRDP_MonitorCount);
+	BOOL havePrimary = FALSE;
+	BOOL foundOrigin = FALSE;
+	BOOL primaryIsOrigin = FALSE;
+	BOOL rc = TRUE;
+
+	struct bounds_t bounds = { 0 };
+
+	if (count == 0)
+	{
+		WLog_WARN(TAG, "Monitor configuration empty.");
+		return TRUE;
+	}
+
+	for (UINT32 x = 0; x < count; x++)
+	{
+		const rdpMonitor* monitor =
+		    freerdp_settings_get_pointer_array(settings, FreeRDP_MonitorDefArray, x);
+		struct bounds_t cur = {
+			.x = monitor->x, .y = monitor->y, .width = monitor->width, .height = monitor->height
+		};
+
+		bounds = union_rect(&bounds, &cur);
+
+		if (monitor->is_primary)
+		{
+			if (havePrimary)
+			{
+				WLog_ERR(TAG, "Monitor configuration contains multiple primary monitors!");
+				rc = FALSE;
+			}
+			havePrimary = TRUE;
+		}
+
+		if ((monitor->x == 0) && (monitor->y == 0))
+		{
+			if (foundOrigin)
+			{
+				WLog_ERR(TAG, "Monitor configuration does have multiple origin 0/0");
+				rc = FALSE;
+			}
+			foundOrigin = TRUE;
+			primaryIsOrigin = monitor->is_primary != 0;
+		}
+	}
+
+	if ((bounds.width > 32766) || (bounds.width < 200))
+	{
+		WLog_ERR(TAG,
+		         "Monitor configuration virtual desktop width must be 200 <= %" PRId32 " <= 32766",
+		         bounds.width);
+		rc = FALSE;
+	}
+	if ((bounds.height > 32766) || (bounds.height < 200))
+	{
+		WLog_ERR(TAG,
+		         "Monitor configuration virtual desktop height must be 200 <= %" PRId32 " <= 32766",
+		         bounds.height);
+		rc = FALSE;
+	}
+
+	if (!havePrimary)
+	{
+		WLog_ERR(TAG, "Monitor configuration does not contain a primary monitor!");
+		rc = FALSE;
+	}
+	if (!foundOrigin)
+	{
+		WLog_ERR(TAG, "Monitor configuration must start at 0/0 for first monitor!");
+		rc = FALSE;
+	}
+	if (!primaryIsOrigin)
+	{
+		WLog_ERR(TAG, "Monitor configuration must start at 0/0 for primary monitor!");
+		rc = FALSE;
+	}
+
+	return rc;
+}
+
+BOOL freerdp_settings_check_client_after_preconnect(const rdpSettings* settings)
+{
+	wLog* log = WLog_Get(TAG);
+	BOOL rc = TRUE;
+	log_monitor_configuration(settings, log, WLOG_DEBUG);
+	if (freerdp_settings_client_monitors_overlap(settings))
+		rc = FALSE;
+	if (freerdp_settings_client_monitors_have_gaps(settings))
+		rc = FALSE;
+	if (!freerdp_settings_client_monitors_check_primary_and_origin(settings))
+		rc = FALSE;
+	if (!rc)
+	{
+		DWORD level = WLOG_ERROR;
+		WLog_Print(log, level, "Invalid or unsupported monitor configuration detected");
+		WLog_Print(log, level, "Check if the configuration is valid.");
+		WLog_Print(log, level,
+		           "If you suspect a bug create a new issue at "
+		           "https://github.com/FreeRDP/FreeRDP/issues/new");
+		WLog_Print(
+		    log, level,
+		    "Provide at least the following log lines detailing your monitor configuration:");
+		log_monitor_configuration(settings, log, level);
+	}
+
+	return rc;
+}
+
 BOOL freerdp_settings_set_default_order_support(rdpSettings* settings)
 {
 	BYTE* OrderSupport = freerdp_settings_get_pointer_writable(settings, FreeRDP_OrderSupport);
@@ -334,11 +768,11 @@ BOOL freerdp_capability_buffer_allocate(rdpSettings* settings, UINT32 count)
 	memset(tmp, 0, count * sizeof(BYTE));
 	settings->ReceivedCapabilities = tmp;
 
-	tmp = realloc(settings->ReceivedCapabilityData, count * sizeof(BYTE*));
+	tmp = realloc((void*)settings->ReceivedCapabilityData, count * sizeof(BYTE*));
 	if (!tmp)
 		return FALSE;
 	memset(tmp, 0, count * sizeof(BYTE*));
-	settings->ReceivedCapabilityData = tmp;
+	settings->ReceivedCapabilityData = (BYTE**)tmp;
 
 	tmp = realloc(settings->ReceivedCapabilityDataSizes, count * sizeof(UINT32));
 	if (!tmp)
@@ -356,7 +790,7 @@ static char* freerdp_settings_get_legacy_config_path(void)
 	char product[sizeof(FREERDP_PRODUCT_STRING)] = { 0 };
 
 	for (size_t i = 0; i < sizeof(product); i++)
-		product[i] = tolower(FREERDP_PRODUCT_STRING[i]);
+		product[i] = (char)tolower(FREERDP_PRODUCT_STRING[i]);
 
 	return GetKnownSubPath(KNOWN_PATH_XDG_CONFIG_HOME, product);
 }
@@ -391,6 +825,30 @@ rdpSettings* freerdp_settings_new(DWORD flags)
 	if (!settings)
 		return NULL;
 
+	if (!server && !remote)
+	{
+		if (!freerdp_settings_set_string(settings, FreeRDP_GatewayAvdClientID,
+		                                 "a85cf173-4192-42f8-81fa-777a763e6e2c"))
+			goto out_fail;
+		if (!freerdp_settings_set_string(settings, FreeRDP_GatewayAzureActiveDirectory,
+		                                 "login.microsoftonline.com"))
+			goto out_fail;
+		if (!freerdp_settings_set_string(settings, FreeRDP_GatewayAvdAadtenantid, "common"))
+			goto out_fail;
+		if (!freerdp_settings_set_bool(settings, FreeRDP_GatewayAvdUseTenantid, FALSE))
+			goto out_fail;
+		if (!freerdp_settings_set_uint32(settings, FreeRDP_DesktopPhysicalWidth, 1000))
+			goto out_fail;
+		if (!freerdp_settings_set_uint32(settings, FreeRDP_DesktopPhysicalHeight, 1000))
+			goto out_fail;
+		if (!freerdp_settings_set_uint16(settings, FreeRDP_DesktopOrientation,
+		                                 ORIENTATION_LANDSCAPE))
+			goto out_fail;
+		if (!freerdp_settings_set_uint32(settings, FreeRDP_DeviceScaleFactor, 100))
+			goto out_fail;
+		if (!freerdp_settings_set_uint32(settings, FreeRDP_DesktopScaleFactor, 100))
+			goto out_fail;
+	}
 	if (!freerdp_settings_set_uint32(settings, FreeRDP_SurfaceCommandsSupported,
 	                                 SURFCMDS_SET_SURFACE_BITS | SURFCMDS_STREAM_SURFACE_BITS |
 	                                     SURFCMDS_FRAME_MARKER))
@@ -519,10 +977,10 @@ rdpSettings* freerdp_settings_new(DWORD flags)
 	if (!freerdp_settings_set_pointer_len(settings, FreeRDP_MonitorDefArray, NULL, 32))
 		goto out_fail;
 
-	if (!freerdp_settings_set_uint32(settings, FreeRDP_MonitorLocalShiftX, 0))
+	if (!freerdp_settings_set_int32(settings, FreeRDP_MonitorLocalShiftX, 0))
 		goto out_fail;
 
-	if (!freerdp_settings_set_uint32(settings, FreeRDP_MonitorLocalShiftY, 0))
+	if (!freerdp_settings_set_int32(settings, FreeRDP_MonitorLocalShiftY, 0))
 		goto out_fail;
 
 	if (!freerdp_settings_set_pointer_len(settings, FreeRDP_MonitorIds, NULL, 0))
@@ -534,7 +992,7 @@ rdpSettings* freerdp_settings_new(DWORD flags)
 	if (!freerdp_settings_set_bool(settings, FreeRDP_SupportMultitransport, TRUE))
 		goto out_fail;
 
-	if (!settings_get_computer_name(settings))
+	if (!settings_init_computer_name(settings))
 		goto out_fail;
 
 	if (!freerdp_settings_set_pointer_len(settings, FreeRDP_RdpServerCertificate, NULL, 1))
@@ -549,14 +1007,6 @@ rdpSettings* freerdp_settings_new(DWORD flags)
 			goto out_fail;
 	}
 
-	{
-		char ClientHostname[33] = { 0 };
-		DWORD size = sizeof(ClientHostname) - 2;
-		GetComputerNameA(ClientHostname, &size);
-		if (!freerdp_settings_set_string(settings, FreeRDP_ClientHostname, ClientHostname))
-			goto out_fail;
-	}
-
 	/* [MS-RDPBCGR] 2.2.7.1.5 Pointer Capability Set (TS_POINTER_CAPABILITYSET)
 	 *
 	 * if we are in server mode send a reasonable large cache size,
@@ -661,6 +1111,8 @@ rdpSettings* freerdp_settings_new(DWORD flags)
 			case 9:
 				cache.cacheMaximumCellSize = 256;
 				break;
+			default:
+				goto out_fail;
 		}
 
 		if (!freerdp_settings_set_pointer_array(settings, FreeRDP_GlyphCache, x, &cache))
@@ -800,8 +1252,10 @@ rdpSettings* freerdp_settings_new(DWORD flags)
 		goto out_fail;
 	settings->OrderSupport = calloc(1, 32);
 
-	freerdp_settings_set_uint16(settings, FreeRDP_TLSMinVersion, TLS1_VERSION);
-	freerdp_settings_set_uint16(settings, FreeRDP_TLSMaxVersion, 0);
+	if (!freerdp_settings_set_uint16(settings, FreeRDP_TLSMinVersion, TLS1_VERSION))
+		goto out_fail;
+	if (!freerdp_settings_set_uint16(settings, FreeRDP_TLSMaxVersion, 0))
+		goto out_fail;
 
 	if (!settings->OrderSupport)
 		goto out_fail;
@@ -831,7 +1285,10 @@ rdpSettings* freerdp_settings_new(DWORD flags)
 
 	return settings;
 out_fail:
+	WINPR_PRAGMA_DIAG_PUSH
+	WINPR_PRAGMA_DIAG_IGNORED_MISMATCHED_DEALLOC
 	freerdp_settings_free(settings);
+	WINPR_PRAGMA_DIAG_POP
 	return NULL;
 }
 
@@ -865,34 +1322,41 @@ static BOOL freerdp_settings_int_buffer_copy(rdpSettings* _settings, const rdpSe
 	if (!_settings || !settings)
 		return FALSE;
 
+	typedef struct
 	{
-		const void* data = freerdp_settings_get_pointer(settings, FreeRDP_LoadBalanceInfo);
-		const UINT32 len = freerdp_settings_get_uint32(settings, FreeRDP_LoadBalanceInfoLength);
-		if (!freerdp_settings_set_pointer_len(_settings, FreeRDP_LoadBalanceInfo, data, len))
-			return FALSE;
-	}
+		FreeRDP_Settings_Keys_UInt32 lenKey;
+		FreeRDP_Settings_Keys_Pointer pointerKey;
+	} PointerDefs;
+
+	PointerDefs pointerDefs[] = {
+		/* */
+		{ FreeRDP_LoadBalanceInfoLength, FreeRDP_LoadBalanceInfo },
+		{ FreeRDP_ServerRandomLength, FreeRDP_ServerRandom },
+		{ FreeRDP_ClientRandomLength, FreeRDP_ClientRandom },
+		{ FreeRDP_ServerCertificateLength, FreeRDP_ServerCertificate },
+		{ FreeRDP_ChannelDefArraySize, FreeRDP_ChannelDefArray },
+		{ FreeRDP_NumMonitorIds, FreeRDP_MonitorIds },
+		{ FreeRDP_BitmapCacheV2NumCells, FreeRDP_BitmapCacheV2CellInfo },
+		{ FreeRDP_RedirectionPasswordLength, FreeRDP_RedirectionPassword },
+		{ FreeRDP_RedirectionTsvUrlLength, FreeRDP_RedirectionTsvUrl },
+		{ FreeRDP_RedirectionGuidLength, FreeRDP_RedirectionGuid },
+		{ FreeRDP_MonitorDefArraySize, FreeRDP_MonitorDefArray }
+	};
+
+	for (size_t i = 0; i < ARRAYSIZE(pointerDefs); i++)
 	{
-		const void* data = freerdp_settings_get_pointer(settings, FreeRDP_ServerRandom);
-		const UINT32 len = freerdp_settings_get_uint32(settings, FreeRDP_ServerRandomLength);
-		if (!freerdp_settings_set_pointer_len(_settings, FreeRDP_ServerRandom, data, len))
-			return FALSE;
-	}
-	{
-		const void* data = freerdp_settings_get_pointer(settings, FreeRDP_ClientRandom);
-		const UINT32 len = freerdp_settings_get_uint32(settings, FreeRDP_ClientRandomLength);
-		if (!freerdp_settings_set_pointer_len(_settings, FreeRDP_ClientRandom, data, len))
-			return FALSE;
+		const PointerDefs* keys = &pointerDefs[i];
+		UINT32 n = freerdp_settings_get_uint32(settings, keys->lenKey);
+
+		const void* ptr = freerdp_settings_get_pointer(settings, keys->pointerKey);
+		if (!freerdp_settings_set_pointer_len(_settings, keys->pointerKey, ptr, n))
+			goto out_fail;
 	}
+
 	if (!freerdp_server_license_issuers_copy(_settings, settings->ServerLicenseProductIssuers,
 	                                         settings->ServerLicenseProductIssuersCount))
 		return FALSE;
 
-	{
-		const void* data = freerdp_settings_get_pointer(settings, FreeRDP_ServerCertificate);
-		const UINT32 len = freerdp_settings_get_uint32(settings, FreeRDP_ServerCertificateLength);
-		if (!freerdp_settings_set_pointer_len(_settings, FreeRDP_ServerCertificate, data, len))
-			return FALSE;
-	}
 	if (settings->RdpServerCertificate)
 	{
 		rdpCertificate* cert = freerdp_certificate_clone(settings->RdpServerCertificate);
@@ -924,36 +1388,8 @@ static BOOL freerdp_settings_int_buffer_copy(rdpSettings* _settings, const rdpSe
 	if (!freerdp_settings_set_uint32(_settings, FreeRDP_ChannelCount,
 	                                 freerdp_settings_get_uint32(settings, FreeRDP_ChannelCount)))
 		goto out_fail;
-	if (!freerdp_settings_set_uint32(
-	        _settings, FreeRDP_ChannelDefArraySize,
-	        freerdp_settings_get_uint32(settings, FreeRDP_ChannelDefArraySize)))
-		goto out_fail;
-
-	const UINT32 defArraySize = freerdp_settings_get_uint32(settings, FreeRDP_ChannelDefArraySize);
-	const CHANNEL_DEF* defArray = freerdp_settings_get_pointer(settings, FreeRDP_ChannelDefArray);
-	if (!freerdp_settings_set_pointer_len(_settings, FreeRDP_ChannelDefArray, defArray,
-	                                      defArraySize))
-		goto out_fail;
-
-	{
-		const UINT32 count = freerdp_settings_get_uint32(settings, FreeRDP_MonitorDefArraySize);
-		const rdpMonitor* monitors =
-		    freerdp_settings_get_pointer(settings, FreeRDP_MonitorDefArray);
-
-		if (!freerdp_settings_set_pointer_len(_settings, FreeRDP_MonitorDefArray, monitors, count))
-			goto out_fail;
-	}
-
-	if (!freerdp_settings_set_pointer_len(_settings, FreeRDP_MonitorIds, NULL, 16))
-		goto out_fail;
-
-	const UINT32 monitorIdSize = freerdp_settings_get_uint32(settings, FreeRDP_NumMonitorIds);
-	const UINT32* monitorIds = freerdp_settings_get_pointer(settings, FreeRDP_MonitorIds);
-	if (!freerdp_settings_set_pointer_len(_settings, FreeRDP_MonitorIds, monitorIds, monitorIdSize))
-		goto out_fail;
 
 	_settings->OrderSupport = malloc(32);
-
 	if (!_settings->OrderSupport)
 		goto out_fail;
 
@@ -961,14 +1397,6 @@ static BOOL freerdp_settings_int_buffer_copy(rdpSettings* _settings, const rdpSe
 		goto out_fail;
 	CopyMemory(_settings->OrderSupport, settings->OrderSupport, 32);
 
-	const UINT32 cellInfoSize =
-	    freerdp_settings_get_uint32(settings, FreeRDP_BitmapCacheV2NumCells);
-	const BITMAP_CACHE_V2_CELL_INFO* cellInfo =
-	    freerdp_settings_get_pointer(settings, FreeRDP_BitmapCacheV2CellInfo);
-	if (!freerdp_settings_set_pointer_len(_settings, FreeRDP_BitmapCacheV2CellInfo, cellInfo,
-	                                      cellInfoSize))
-		goto out_fail;
-
 	const UINT32 glyphCacheCount = 10;
 	const GLYPH_CACHE_DEFINITION* glyphCache =
 	    freerdp_settings_get_pointer(settings, FreeRDP_GlyphCache);
@@ -996,25 +1424,6 @@ static BOOL freerdp_settings_int_buffer_copy(rdpSettings* _settings, const rdpSe
 	if (!freerdp_settings_set_pointer_len(_settings, FreeRDP_ClientTimeZone, tz, 1))
 		goto out_fail;
 
-	if (!freerdp_settings_set_uint32(
-	        _settings, FreeRDP_RedirectionPasswordLength,
-	        freerdp_settings_get_uint32(settings, FreeRDP_RedirectionPasswordLength)))
-		goto out_fail;
-	const UINT32 redirectionPasswordLength =
-	    freerdp_settings_get_uint32(settings, FreeRDP_RedirectionPasswordLength);
-	const BYTE* pwd = freerdp_settings_get_pointer(settings, FreeRDP_RedirectionPassword);
-	if (!freerdp_settings_set_pointer_len(_settings, FreeRDP_RedirectionPassword, pwd,
-	                                      redirectionPasswordLength))
-		goto out_fail;
-
-	const UINT32 RedirectionTsvUrlLength =
-	    freerdp_settings_get_uint32(settings, FreeRDP_RedirectionTsvUrlLength);
-	const BYTE* RedirectionTsvUrl =
-	    freerdp_settings_get_pointer(settings, FreeRDP_RedirectionTsvUrl);
-	if (!freerdp_settings_set_pointer_len(_settings, FreeRDP_RedirectionTsvUrl, RedirectionTsvUrl,
-	                                      RedirectionTsvUrlLength))
-		goto out_fail;
-
 	const UINT32 nrports = freerdp_settings_get_uint32(settings, FreeRDP_TargetNetAddressCount);
 	if (!freerdp_target_net_adresses_reset(_settings, nrports))
 		return FALSE;
@@ -1131,6 +1540,7 @@ BOOL freerdp_settings_copy(rdpSettings* _settings, const rdpSettings* settings)
 	_settings->RedirectionTsvUrl = NULL;
 	_settings->TargetNetAddresses = NULL;
 	_settings->TargetNetPorts = NULL;
+	_settings->RedirectionGuid = NULL;
 	_settings->DeviceArray = NULL;
 	_settings->StaticChannelArray = NULL;
 	_settings->DynamicChannelArray = NULL;
@@ -1162,7 +1572,10 @@ rdpSettings* freerdp_settings_clone(const rdpSettings* settings)
 
 	return _settings;
 out_fail:
+	WINPR_PRAGMA_DIAG_PUSH
+	WINPR_PRAGMA_DIAG_IGNORED_MISMATCHED_DEALLOC
 	freerdp_settings_free(_settings);
+	WINPR_PRAGMA_DIAG_POP
 	return NULL;
 }
 #ifdef _MSC_VER
@@ -1222,8 +1635,9 @@ BOOL identity_set_from_smartcard_hash(SEC_WINNT_AUTH_IDENTITY_W* identity,
                                       const rdpSettings* settings,
                                       FreeRDP_Settings_Keys_String userId,
                                       FreeRDP_Settings_Keys_String domainId,
-                                      FreeRDP_Settings_Keys_String pwdId, const BYTE* certSha1,
-                                      size_t sha1len)
+                                      FreeRDP_Settings_Keys_String pwdId,
+                                      WINPR_ATTR_UNUSED const BYTE* certSha1,
+                                      WINPR_ATTR_UNUSED size_t sha1len)
 {
 #ifdef _WIN32
 	CERT_CREDENTIAL_INFO certInfo = { sizeof(CERT_CREDENTIAL_INFO), { 0 } };
@@ -1284,8 +1698,8 @@ BOOL freerdp_target_net_adresses_reset(rdpSettings* settings, size_t size)
 
 	if (size > 0)
 	{
-		if (!freerdp_settings_set_pointer_len_(settings, FreeRDP_TargetNetPorts, -1, NULL, size,
-		                                       sizeof(UINT32)))
+		if (!freerdp_settings_set_pointer_len_(settings, FreeRDP_TargetNetPorts,
+		                                       FreeRDP_UINT32_UNUSED, NULL, size, sizeof(UINT32)))
 			return FALSE;
 		if (!freerdp_settings_set_pointer_len_(settings, FreeRDP_TargetNetAddresses,
 		                                       FreeRDP_TargetNetAddressCount, NULL, size,
@@ -1294,3 +1708,58 @@ BOOL freerdp_target_net_adresses_reset(rdpSettings* settings, size_t size)
 	}
 	return TRUE;
 }
+
+BOOL freerdp_settings_enforce_monitor_exists(rdpSettings* settings)
+{
+	const UINT32 nrIds = freerdp_settings_get_uint32(settings, FreeRDP_NumMonitorIds);
+	const UINT32 count = freerdp_settings_get_uint32(settings, FreeRDP_MonitorCount);
+	const BOOL fullscreen = freerdp_settings_get_bool(settings, FreeRDP_Fullscreen);
+	const BOOL multimon = freerdp_settings_get_bool(settings, FreeRDP_UseMultimon);
+	const BOOL useMonitors = fullscreen || multimon;
+
+	const UINT32 size = freerdp_settings_get_uint32(settings, FreeRDP_MonitorDefArraySize);
+	if (size == 0)
+	{
+		if (!freerdp_settings_set_pointer_len(settings, FreeRDP_MonitorDefArray, NULL, 16))
+			return FALSE;
+	}
+
+	if (nrIds == 0)
+	{
+		if (!freerdp_settings_set_uint32(settings, FreeRDP_NumMonitorIds, 1))
+			return FALSE;
+	}
+	if (!useMonitors || (count == 0))
+	{
+		const UINT32 width = freerdp_settings_get_uint32(settings, FreeRDP_DesktopWidth);
+		const UINT32 height = freerdp_settings_get_uint32(settings, FreeRDP_DesktopHeight);
+		const UINT32 pwidth = freerdp_settings_get_uint32(settings, FreeRDP_DesktopPhysicalWidth);
+		const UINT32 pheight = freerdp_settings_get_uint32(settings, FreeRDP_DesktopPhysicalHeight);
+		const UINT16 orientation =
+		    freerdp_settings_get_uint16(settings, FreeRDP_DesktopOrientation);
+		const UINT32 desktopScaleFactor =
+		    freerdp_settings_get_uint32(settings, FreeRDP_DeviceScaleFactor);
+		const UINT32 deviceScaleFactor =
+		    freerdp_settings_get_uint32(settings, FreeRDP_DesktopScaleFactor);
+
+		if (!freerdp_settings_set_uint32(settings, FreeRDP_MonitorCount, 1))
+			return FALSE;
+
+		rdpMonitor monitor = { 0 };
+		monitor.x = 0;
+		monitor.y = 0;
+		monitor.width = WINPR_ASSERTING_INT_CAST(int32_t, width);
+		monitor.height = WINPR_ASSERTING_INT_CAST(int32_t, height);
+		monitor.is_primary = TRUE;
+		monitor.orig_screen = 0;
+		monitor.attributes.physicalWidth = pwidth;
+		monitor.attributes.physicalHeight = pheight;
+		monitor.attributes.orientation = orientation;
+		monitor.attributes.desktopScaleFactor = desktopScaleFactor;
+		monitor.attributes.deviceScaleFactor = deviceScaleFactor;
+		if (!freerdp_settings_set_monitor_def_array_sorted(settings, &monitor, 1))
+			return FALSE;
+	}
+
+	return TRUE;
+}
