@@ -47,7 +47,6 @@
 #if defined(WITH_SMARTCARD_EMULATE)
 #include <freerdp/emulate/scard/smartcard_emulate.h>
 
-#define str(x) #x
 #define wrap(ctx, fkt, ...)                                             \
 	ctx->useEmulatedCard ? Emulate_##fkt(ctx->emulation, ##__VA_ARGS__) \
 	                     : ctx->pWinSCardApi->pfn##fkt(__VA_ARGS__)
@@ -140,7 +139,8 @@ static LONG smartcard_EstablishContext_Call(scard_call_context* smartcard, wStre
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_ReleaseContext_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_ReleaseContext_Call(scard_call_context* smartcard,
+                                          WINPR_ATTR_UNUSED wStream* out,
                                           SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -162,7 +162,8 @@ static LONG smartcard_ReleaseContext_Call(scard_call_context* smartcard, wStream
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_IsValidContext_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_IsValidContext_Call(scard_call_context* smartcard,
+                                          WINPR_ATTR_UNUSED wStream* out,
                                           SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -227,7 +228,11 @@ static LONG smartcard_ListReaderGroupsW_Call(scard_call_context* smartcard, wStr
 		return SCARD_F_UNKNOWN_ERROR;
 
 	ret.msz = (BYTE*)mszGroups;
-	ret.cBytes = cchGroups * sizeof(WCHAR);
+
+	WINPR_ASSERT(cchGroups < SCARD_AUTOALLOCATE / sizeof(WCHAR));
+	const size_t blen = sizeof(WCHAR) * cchGroups;
+	WINPR_ASSERT(blen <= UINT32_MAX);
+	ret.cBytes = (UINT32)blen;
 
 	if (status != SCARD_S_SUCCESS)
 		return status;
@@ -332,26 +337,27 @@ static DWORD filter_device_by_name_w(wLinkedList* list, LPWSTR* mszReaders, DWOR
 static LONG smartcard_ListReadersA_Call(scard_call_context* smartcard, wStream* out,
                                         SMARTCARD_OPERATION* operation)
 {
-	LONG status = 0;
 	ListReaders_Return ret = { 0 };
 	LPSTR mszReaders = NULL;
-	DWORD cchReaders = 0;
-	ListReaders_Call* call = NULL;
 
 	WINPR_ASSERT(smartcard);
 	WINPR_ASSERT(out);
 	WINPR_ASSERT(operation);
 
-	call = &operation->call.listReaders;
-	cchReaders = SCARD_AUTOALLOCATE;
-	status = ret.ReturnCode = wrap(smartcard, SCardListReadersA, operation->hContext,
-	                               (LPCSTR)call->mszGroups, (LPSTR)&mszReaders, &cchReaders);
-	if (cchReaders == SCARD_AUTOALLOCATE)
-		return SCARD_F_UNKNOWN_ERROR;
+	ListReaders_Call* call = &operation->call.listReaders;
+	DWORD cchReaders = SCARD_AUTOALLOCATE;
+	LONG status = ret.ReturnCode = wrap(smartcard, SCardListReadersA, operation->hContext,
+	                                    (LPCSTR)call->mszGroups, (LPSTR)&mszReaders, &cchReaders);
+	if (status == SCARD_S_SUCCESS)
+	{
+		if (cchReaders == SCARD_AUTOALLOCATE)
+			status = SCARD_F_UNKNOWN_ERROR;
+	}
 
 	if (status != SCARD_S_SUCCESS)
 	{
-		return scard_log_status_error(TAG, "SCardListReadersA", status);
+		(void)scard_log_status_error(TAG, "SCardListReadersA", status);
+		return smartcard_pack_list_readers_return(out, &ret, FALSE);
 	}
 
 	cchReaders = filter_device_by_name_a(smartcard->names, &mszReaders, cchReaders);
@@ -359,16 +365,11 @@ static LONG smartcard_ListReadersA_Call(scard_call_context* smartcard, wStream*
 	ret.cBytes = cchReaders;
 
 	status = smartcard_pack_list_readers_return(out, &ret, FALSE);
-	if (status != SCARD_S_SUCCESS)
-	{
-		return scard_log_status_error(TAG, "smartcard_pack_list_readers_return", status);
-	}
-
 	if (mszReaders)
 		wrap(smartcard, SCardFreeMemory, operation->hContext, mszReaders);
 
 	if (status != SCARD_S_SUCCESS)
-		return status;
+		return scard_log_status_error(TAG, "smartcard_pack_list_readers_return", status);
 
 	return ret.ReturnCode;
 }
@@ -403,11 +404,17 @@ static LONG smartcard_ListReadersW_Call(scard_call_context* smartcard, wStream*
 	cchReaders = SCARD_AUTOALLOCATE;
 	status = ret.ReturnCode = wrap(smartcard, SCardListReadersW, operation->hContext, string.wz,
 	                               (LPWSTR)&mszReaders.pw, &cchReaders);
-	if (cchReaders == SCARD_AUTOALLOCATE)
-		return SCARD_F_UNKNOWN_ERROR;
+	if (status == SCARD_S_SUCCESS)
+	{
+		if (cchReaders == SCARD_AUTOALLOCATE)
+			status = SCARD_F_UNKNOWN_ERROR;
+	}
 
 	if (status != SCARD_S_SUCCESS)
-		return scard_log_status_error(TAG, "SCardListReadersW", status);
+	{
+		(void)scard_log_status_error(TAG, "SCardListReadersW", status);
+		return smartcard_pack_list_readers_return(out, &ret, TRUE);
+	}
 
 	cchReaders = filter_device_by_name_w(smartcard->names, &mszReaders.pw, cchReaders);
 	ret.msz = mszReaders.pb;
@@ -423,7 +430,8 @@ static LONG smartcard_ListReadersW_Call(scard_call_context* smartcard, wStream*
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_IntroduceReaderGroupA_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_IntroduceReaderGroupA_Call(scard_call_context* smartcard,
+                                                 WINPR_ATTR_UNUSED wStream* out,
                                                  SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -440,7 +448,8 @@ static LONG smartcard_IntroduceReaderGroupA_Call(scard_call_context* smartcard,
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_IntroduceReaderGroupW_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_IntroduceReaderGroupW_Call(scard_call_context* smartcard,
+                                                 WINPR_ATTR_UNUSED wStream* out,
                                                  SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -457,7 +466,8 @@ static LONG smartcard_IntroduceReaderGroupW_Call(scard_call_context* smartcard,
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_IntroduceReaderA_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_IntroduceReaderA_Call(scard_call_context* smartcard,
+                                            WINPR_ATTR_UNUSED wStream* out,
                                             SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -475,7 +485,8 @@ static LONG smartcard_IntroduceReaderA_Call(scard_call_context* smartcard, wStre
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_IntroduceReaderW_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_IntroduceReaderW_Call(scard_call_context* smartcard,
+                                            WINPR_ATTR_UNUSED wStream* out,
                                             SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -493,7 +504,8 @@ static LONG smartcard_IntroduceReaderW_Call(scard_call_context* smartcard, wStre
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_ForgetReaderA_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_ForgetReaderA_Call(scard_call_context* smartcard,
+                                         WINPR_ATTR_UNUSED wStream* out,
                                          SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -510,7 +522,8 @@ static LONG smartcard_ForgetReaderA_Call(scard_call_context* smartcard, wStream*
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_ForgetReaderW_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_ForgetReaderW_Call(scard_call_context* smartcard,
+                                         WINPR_ATTR_UNUSED wStream* out,
                                          SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -527,7 +540,8 @@ static LONG smartcard_ForgetReaderW_Call(scard_call_context* smartcard, wStream*
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_AddReaderToGroupA_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_AddReaderToGroupA_Call(scard_call_context* smartcard,
+                                             WINPR_ATTR_UNUSED wStream* out,
                                              SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -545,7 +559,8 @@ static LONG smartcard_AddReaderToGroupA_Call(scard_call_context* smartcard, wStr
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_AddReaderToGroupW_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_AddReaderToGroupW_Call(scard_call_context* smartcard,
+                                             WINPR_ATTR_UNUSED wStream* out,
                                              SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -563,7 +578,8 @@ static LONG smartcard_AddReaderToGroupW_Call(scard_call_context* smartcard, wStr
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_RemoveReaderFromGroupA_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_RemoveReaderFromGroupA_Call(scard_call_context* smartcard,
+                                                  WINPR_ATTR_UNUSED wStream* out,
                                                   SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -581,7 +597,8 @@ static LONG smartcard_RemoveReaderFromGroupA_Call(scard_call_context* smartcard,
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_RemoveReaderFromGroupW_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_RemoveReaderFromGroupW_Call(scard_call_context* smartcard,
+                                                  WINPR_ATTR_UNUSED wStream* out,
                                                   SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -772,7 +789,8 @@ static LONG smartcard_ReadCacheW_Call(scard_call_context* smartcard, wStream* ou
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_WriteCacheA_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_WriteCacheA_Call(scard_call_context* smartcard,
+                                       WINPR_ATTR_UNUSED wStream* out,
                                        SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -792,7 +810,8 @@ static LONG smartcard_WriteCacheA_Call(scard_call_context* smartcard, wStream* o
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_WriteCacheW_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_WriteCacheW_Call(scard_call_context* smartcard,
+                                       WINPR_ATTR_UNUSED wStream* out,
                                        SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -1021,7 +1040,7 @@ fail:
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_Cancel_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_Cancel_Call(scard_call_context* smartcard, WINPR_ATTR_UNUSED wStream* out,
                                   SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -1131,7 +1150,7 @@ static LONG smartcard_Reconnect_Call(scard_call_context* smartcard, wStream* out
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_Disconnect_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_Disconnect_Call(scard_call_context* smartcard, WINPR_ATTR_UNUSED wStream* out,
                                       SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -1150,7 +1169,8 @@ static LONG smartcard_Disconnect_Call(scard_call_context* smartcard, wStream* ou
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_BeginTransaction_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_BeginTransaction_Call(scard_call_context* smartcard,
+                                            WINPR_ATTR_UNUSED wStream* out,
                                             SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -1165,7 +1185,8 @@ static LONG smartcard_BeginTransaction_Call(scard_call_context* smartcard, wStre
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_EndTransaction_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_EndTransaction_Call(scard_call_context* smartcard,
+                                          WINPR_ATTR_UNUSED wStream* out,
                                           SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -1302,8 +1323,10 @@ static LONG smartcard_StatusW_Call(scard_call_context* smartcard, wStream* out,
 	}
 
 	/* SCardStatusW returns number of characters, we need number of bytes */
-	WINPR_ASSERT(ret.cBytes != SCARD_AUTOALLOCATE);
-	ret.cBytes *= sizeof(WCHAR);
+	WINPR_ASSERT(ret.cBytes < SCARD_AUTOALLOCATE / sizeof(WCHAR));
+	const size_t blen = sizeof(WCHAR) * ret.cBytes;
+	WINPR_ASSERT(blen <= UINT32_MAX);
+	ret.cBytes = (UINT32)blen;
 
 	status = smartcard_pack_status_return(out, &ret, TRUE);
 	if (status != SCARD_S_SUCCESS)
@@ -1434,7 +1457,7 @@ static LONG smartcard_GetAttrib_Call(scard_call_context* smartcard, wStream* out
 	return status;
 }
 
-static LONG smartcard_SetAttrib_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_SetAttrib_Call(scard_call_context* smartcard, WINPR_ATTR_UNUSED wStream* out,
                                      SMARTCARD_OPERATION* operation)
 {
 	Long_Return ret = { 0 };
@@ -1454,7 +1477,8 @@ static LONG smartcard_SetAttrib_Call(scard_call_context* smartcard, wStream* out
 	return ret.ReturnCode;
 }
 
-static LONG smartcard_AccessStartedEvent_Call(scard_call_context* smartcard, wStream* out,
+static LONG smartcard_AccessStartedEvent_Call(scard_call_context* smartcard,
+                                              WINPR_ATTR_UNUSED wStream* out,
                                               SMARTCARD_OPERATION* operation)
 {
 	LONG status = SCARD_S_SUCCESS;
@@ -1553,7 +1577,7 @@ static LONG smartcard_LocateCardsByATRA_Call(scard_call_context* smartcard, wStr
 }
 
 LONG smartcard_irp_device_control_call(scard_call_context* smartcard, wStream* out,
-                                       UINT32* pIoStatus, SMARTCARD_OPERATION* operation)
+                                       NTSTATUS* pIoStatus, SMARTCARD_OPERATION* operation)
 {
 	LONG result = 0;
 	UINT32 offset = 0;
@@ -1810,7 +1834,7 @@ LONG smartcard_irp_device_control_call(scard_call_context* smartcard, wStream* o
 	if ((result & 0xC0000000L) == 0xC0000000L)
 	{
 		/* NTSTATUS error */
-		*pIoStatus = (UINT32)result;
+		*pIoStatus = result;
 		WLog_WARN(TAG, "IRP failure: %s (0x%08" PRIX32 "), ntstatus: 0x%08" PRIX32 "",
 		          scard_get_ioctl_string(ioControlCode, TRUE), ioControlCode, result);
 	}
@@ -1959,7 +1983,7 @@ void smartcard_call_context_free(scard_call_context* ctx)
 	ctx->pWinSCardApi = NULL;
 
 	HashTable_Free(ctx->rgSCardContextList);
-	CloseHandle(ctx->stopEvent);
+	(void)CloseHandle(ctx->stopEvent);
 	free(ctx);
 }
 
