@@ -34,6 +34,7 @@
 
 #include <winpr/crt.h>
 #include <winpr/assert.h>
+#include <winpr/cast.h>
 #include <winpr/path.h>
 #include <winpr/synch.h>
 #include <winpr/image.h>
@@ -135,10 +136,10 @@ static BOOL x11_shadow_pam_get_service_name(SHADOW_PAM_AUTH_INFO* info)
 
 	for (size_t x = 0; x < ARRAYSIZE(hints); x++)
 	{
-		char path[MAX_PATH];
+		char path[MAX_PATH] = { 0 };
 		const char* hint = hints[x];
 
-		_snprintf(path, sizeof(path), "%s/%s", base, hint);
+		(void)_snprintf(path, sizeof(path), "%s/%s", base, hint);
 		if (winpr_PathFileExists(path))
 		{
 
@@ -195,8 +196,9 @@ static int x11_shadow_pam_authenticate(rdpShadowSubsystem* subsystem, rdpShadowC
 
 #endif
 
-static BOOL x11_shadow_input_synchronize_event(rdpShadowSubsystem* subsystem,
-                                               rdpShadowClient* client, UINT32 flags)
+static BOOL x11_shadow_input_synchronize_event(WINPR_ATTR_UNUSED rdpShadowSubsystem* subsystem,
+                                               WINPR_ATTR_UNUSED rdpShadowClient* client,
+                                               WINPR_ATTR_UNUSED UINT32 flags)
 {
 	/* TODO: Implement */
 	WLog_WARN(TAG, "not implemented");
@@ -249,9 +251,10 @@ static BOOL x11_shadow_input_keyboard_event(rdpShadowSubsystem* subsystem, rdpSh
 	return TRUE;
 }
 
-static BOOL x11_shadow_input_unicode_keyboard_event(rdpShadowSubsystem* subsystem,
-                                                    rdpShadowClient* client, UINT16 flags,
-                                                    UINT16 code)
+static BOOL x11_shadow_input_unicode_keyboard_event(WINPR_ATTR_UNUSED rdpShadowSubsystem* subsystem,
+                                                    WINPR_ATTR_UNUSED rdpShadowClient* client,
+                                                    WINPR_ATTR_UNUSED UINT16 flags,
+                                                    WINPR_ATTR_UNUSED UINT16 code)
 {
 	/* TODO: Implement */
 	WLog_WARN(TAG, "not implemented");
@@ -409,7 +412,7 @@ static int x11_shadow_pointer_position_update(x11ShadowSubsystem* subsystem)
 {
 	UINT32 msgId = SHADOW_MSG_OUT_POINTER_POSITION_UPDATE_ID;
 	rdpShadowServer* server = NULL;
-	SHADOW_MSG_OUT_POINTER_POSITION_UPDATE templateMsg;
+	SHADOW_MSG_OUT_POINTER_POSITION_UPDATE templateMsg = { 0 };
 	int count = 0;
 
 	if (!subsystem || !subsystem->common.server || !subsystem->common.server->clients)
@@ -463,8 +466,8 @@ static int x11_shadow_pointer_alpha_update(x11ShadowSubsystem* subsystem)
 	msg->width = subsystem->cursorWidth;
 	msg->height = subsystem->cursorHeight;
 
-	if (shadow_subsystem_pointer_convert_alpha_pointer_data(subsystem->cursorPixels, TRUE,
-	                                                        msg->width, msg->height, msg) < 0)
+	if (shadow_subsystem_pointer_convert_alpha_pointer_data_to_format(
+	        subsystem->cursorPixels, subsystem->format, TRUE, msg->width, msg->height, msg) < 0)
 	{
 		free(msg);
 		return -1;
@@ -558,8 +561,8 @@ static int x11_shadow_query_cursor(x11ShadowSubsystem* subsystem, BOOL getImage)
 
 	if ((x != (INT64)subsystem->common.pointerX) || (y != (INT64)subsystem->common.pointerY))
 	{
-		subsystem->common.pointerX = x;
-		subsystem->common.pointerY = y;
+		subsystem->common.pointerX = WINPR_ASSERTING_INT_CAST(UINT32, x);
+		subsystem->common.pointerY = WINPR_ASSERTING_INT_CAST(UINT32, y);
 		x11_shadow_pointer_position_update(subsystem);
 	}
 
@@ -594,10 +597,10 @@ static void x11_shadow_validate_region(x11ShadowSubsystem* subsystem, int x, int
 	if (!subsystem->use_xfixes || !subsystem->use_xdamage)
 		return;
 
-	region.x = x;
-	region.y = y;
-	region.width = width;
-	region.height = height;
+	region.x = WINPR_ASSERTING_INT_CAST(INT16, x);
+	region.y = WINPR_ASSERTING_INT_CAST(INT16, y);
+	region.width = WINPR_ASSERTING_INT_CAST(UINT16, width);
+	region.height = WINPR_ASSERTING_INT_CAST(UINT16, height);
 #if defined(WITH_XFIXES) && defined(WITH_XDAMAGE)
 	XLockDisplay(subsystem->display);
 	XFixesSetRegion(subsystem->display, subsystem->xdamage_region, &region, 1);
@@ -666,22 +669,23 @@ static int x11_shadow_blend_cursor(x11ShadowSubsystem* subsystem)
 	}
 
 	if ((nXDst + nWidth) > surface->width)
-		nWidth = surface->width - nXDst;
+		nWidth = (nXDst > surface->width) ? 0 : (UINT32)(surface->width - nXDst);
 
 	if ((nYDst + nHeight) > surface->height)
-		nHeight = surface->height - nYDst;
+		nHeight = (nYDst > surface->height) ? 0 : (UINT32)(surface->height - nYDst);
 
 	pSrcData = subsystem->cursorPixels;
 	nSrcStep = subsystem->cursorWidth * 4;
 	pDstData = surface->data;
 	nDstStep = surface->scanline;
 
-	for (int y = 0; y < nHeight; y++)
+	for (size_t y = 0; y < nHeight; y++)
 	{
-		const BYTE* pSrcPixel = &pSrcData[((nYSrc + y) * nSrcStep) + (nXSrc * 4)];
-		BYTE* pDstPixel = &pDstData[((nYDst + y) * nDstStep) + (nXDst * 4)];
+		const BYTE* pSrcPixel = &pSrcData[((nYSrc + y) * nSrcStep) + (4ULL * nXSrc)];
+		BYTE* pDstPixel = &pDstData[((WINPR_ASSERTING_INT_CAST(uint32_t, nYDst) + y) * nDstStep) +
+		                            (4ULL * WINPR_ASSERTING_INT_CAST(uint32_t, nXDst))];
 
-		for (int x = 0; x < nWidth; x++)
+		for (size_t x = 0; x < nWidth; x++)
 		{
 			B = *pSrcPixel++;
 			G = *pSrcPixel++;
@@ -722,14 +726,19 @@ static BOOL x11_shadow_check_resize(x11ShadowSubsystem* subsystem)
 
 		/* Screen size changed. Refresh monitor definitions and trigger screen resize */
 		subsystem->common.numMonitors = x11_shadow_enum_monitors(subsystem->common.monitors, 16);
-		shadow_screen_resize(subsystem->common.server->screen);
-		subsystem->width = attr.width;
-		subsystem->height = attr.height;
+		if (!shadow_screen_resize(subsystem->common.server->screen))
+			return FALSE;
+
+		WINPR_ASSERT(attr.width > 0);
+		WINPR_ASSERT(attr.height > 0);
+
+		subsystem->width = (UINT32)attr.width;
+		subsystem->height = (UINT32)attr.height;
 
 		virtualScreen->left = 0;
 		virtualScreen->top = 0;
-		virtualScreen->right = subsystem->width - 1;
-		virtualScreen->bottom = subsystem->height - 1;
+		virtualScreen->right = attr.width - 1;
+		virtualScreen->bottom = attr.height - 1;
 		virtualScreen->flags = 1;
 		return TRUE;
 	}
@@ -778,8 +787,9 @@ static int x11_shadow_screen_grab(x11ShadowSubsystem* subsystem)
 	EnterCriticalSection(&surface->lock);
 	surfaceRect.left = 0;
 	surfaceRect.top = 0;
-	surfaceRect.right = surface->width;
-	surfaceRect.bottom = surface->height;
+
+	surfaceRect.right = WINPR_ASSERTING_INT_CAST(UINT16, surface->width);
+	surfaceRect.bottom = WINPR_ASSERTING_INT_CAST(UINT16, surface->height);
 	LeaveCriticalSection(&surface->lock);
 
 	XLockDisplay(subsystem->display);
@@ -796,9 +806,10 @@ static int x11_shadow_screen_grab(x11ShadowSubsystem* subsystem)
 		          subsystem->xshm_gc, 0, 0, subsystem->width, subsystem->height, 0, 0);
 
 		EnterCriticalSection(&surface->lock);
-		status = shadow_capture_compare(surface->data, surface->scanline, surface->width,
-		                                surface->height, (BYTE*)&(image->data[surface->width * 4]),
-		                                image->bytes_per_line, &invalidRect);
+		status = shadow_capture_compare_with_format(
+		    surface->data, surface->format, surface->scanline, surface->width, surface->height,
+		    (BYTE*)&(image->data[surface->width * 4ull]), subsystem->format,
+		    WINPR_ASSERTING_INT_CAST(UINT32, image->bytes_per_line), &invalidRect);
 		LeaveCriticalSection(&surface->lock);
 	}
 	else
@@ -810,9 +821,10 @@ static int x11_shadow_screen_grab(x11ShadowSubsystem* subsystem)
 
 		if (image)
 		{
-			status = shadow_capture_compare(surface->data, surface->scanline, surface->width,
-			                                surface->height, (BYTE*)image->data,
-			                                image->bytes_per_line, &invalidRect);
+			status = shadow_capture_compare_with_format(
+			    surface->data, surface->format, surface->scanline, surface->width, surface->height,
+			    (BYTE*)image->data, subsystem->format,
+			    WINPR_ASSERTING_INT_CAST(UINT32, image->bytes_per_line), &invalidRect);
 		}
 		LeaveCriticalSection(&surface->lock);
 		if (!image)
@@ -852,10 +864,14 @@ static int x11_shadow_screen_grab(x11ShadowSubsystem* subsystem)
 			WINPR_ASSERT(image->bytes_per_line >= 0);
 			WINPR_ASSERT(width >= 0);
 			WINPR_ASSERT(height >= 0);
-			success = freerdp_image_copy(surface->data, surface->format, surface->scanline, x, y,
-			                             (UINT32)width, (UINT32)height, (BYTE*)image->data,
-			                             PIXEL_FORMAT_BGRX32, (UINT32)image->bytes_per_line, x, y,
-			                             NULL, FREERDP_FLIP_NONE);
+			success = freerdp_image_copy_no_overlap(
+			    surface->data, surface->format, surface->scanline,
+			    WINPR_ASSERTING_INT_CAST(uint32_t, x), WINPR_ASSERTING_INT_CAST(uint32_t, y),
+			    WINPR_ASSERTING_INT_CAST(uint32_t, width),
+			    WINPR_ASSERTING_INT_CAST(uint32_t, height), (BYTE*)image->data, subsystem->format,
+			    WINPR_ASSERTING_INT_CAST(uint32_t, image->bytes_per_line),
+			    WINPR_ASSERTING_INT_CAST(UINT32, x), WINPR_ASSERTING_INT_CAST(UINT32, y), NULL,
+			    FREERDP_FLIP_NONE);
 			LeaveCriticalSection(&surface->lock);
 			if (!success)
 				goto fail_capture;
@@ -920,8 +936,6 @@ static DWORD WINAPI x11_shadow_subsystem_thread(LPVOID arg)
 	XEvent xevent;
 	DWORD status = 0;
 	DWORD nCount = 0;
-	UINT64 cTime = 0;
-	DWORD dwTimeout = 0;
 	DWORD dwInterval = 0;
 	UINT64 frameTime = 0;
 	HANDLE events[32];
@@ -937,8 +951,9 @@ static DWORD WINAPI x11_shadow_subsystem_thread(LPVOID arg)
 
 	while (1)
 	{
-		cTime = GetTickCount64();
-		dwTimeout = (cTime > frameTime) ? 0 : frameTime - cTime;
+		const UINT64 cTime = GetTickCount64();
+		const DWORD dwTimeout =
+		    (DWORD)((cTime > frameTime) ? 0 : MIN(UINT32_MAX, frameTime - cTime));
 		status = WaitForMultipleObjects(nCount, events, FALSE, dwTimeout);
 
 		if (WaitForSingleObject(MessageQueue_Event(MsgPipe->In), 0) == WAIT_OBJECT_0)
@@ -984,8 +999,12 @@ static int x11_shadow_subsystem_base_init(x11ShadowSubsystem* subsystem)
 	if (subsystem->display)
 		return 1; /* initialize once */
 
+	// NOLINTNEXTLINE(concurrency-mt-unsafe)
 	if (!getenv("DISPLAY"))
+	{
+		// NOLINTNEXTLINE(concurrency-mt-unsafe)
 		setenv("DISPLAY", ":0", 1);
+	}
 
 	if (!XInitThreads())
 		return -1;
@@ -1001,9 +1020,9 @@ static int x11_shadow_subsystem_base_init(x11ShadowSubsystem* subsystem)
 	subsystem->xfds = ConnectionNumber(subsystem->display);
 	subsystem->number = DefaultScreen(subsystem->display);
 	subsystem->screen = ScreenOfDisplay(subsystem->display, subsystem->number);
-	subsystem->depth = DefaultDepthOfScreen(subsystem->screen);
-	subsystem->width = WidthOfScreen(subsystem->screen);
-	subsystem->height = HeightOfScreen(subsystem->screen);
+	subsystem->depth = WINPR_ASSERTING_INT_CAST(UINT32, DefaultDepthOfScreen(subsystem->screen));
+	subsystem->width = WINPR_ASSERTING_INT_CAST(UINT32, WidthOfScreen(subsystem->screen));
+	subsystem->height = WINPR_ASSERTING_INT_CAST(UINT32, HeightOfScreen(subsystem->screen));
 	subsystem->root_window = RootWindow(subsystem->display, subsystem->number);
 	return 1;
 }
@@ -1036,7 +1055,10 @@ static int x11_shadow_xinerama_init(x11ShadowSubsystem* subsystem)
 #ifdef WITH_XINERAMA
 	int xinerama_event = 0;
 	int xinerama_error = 0;
-	x11_shadow_subsystem_base_init(subsystem);
+
+	const int rc = x11_shadow_subsystem_base_init(subsystem);
+	if (rc < 0)
+		return rc;
 
 	if (!XineramaQueryExtension(subsystem->display, &xinerama_event, &xinerama_error))
 		return -1;
@@ -1126,9 +1148,11 @@ static int x11_shadow_xshm_init(x11ShadowSubsystem* subsystem)
 		return -1;
 	}
 
-	subsystem->fb_shm_info.shmid = shmget(
-	    IPC_PRIVATE, 1ull * subsystem->fb_image->bytes_per_line * subsystem->fb_image->height,
-	    IPC_CREAT | 0600);
+	subsystem->fb_shm_info.shmid =
+	    shmget(IPC_PRIVATE,
+	           1ull * WINPR_ASSERTING_INT_CAST(uint32_t, subsystem->fb_image->bytes_per_line) *
+	               WINPR_ASSERTING_INT_CAST(uint32_t, subsystem->fb_image->height),
+	           IPC_CREAT | 0600);
 
 	if (subsystem->fb_shm_info.shmid == -1)
 	{
@@ -1150,10 +1174,11 @@ static int x11_shadow_xshm_init(x11ShadowSubsystem* subsystem)
 
 	XSync(subsystem->display, False);
 	shmctl(subsystem->fb_shm_info.shmid, IPC_RMID, 0);
-	subsystem->fb_pixmap =
-	    XShmCreatePixmap(subsystem->display, subsystem->root_window, subsystem->fb_image->data,
-	                     &(subsystem->fb_shm_info), subsystem->fb_image->width,
-	                     subsystem->fb_image->height, subsystem->fb_image->depth);
+	subsystem->fb_pixmap = XShmCreatePixmap(
+	    subsystem->display, subsystem->root_window, subsystem->fb_image->data,
+	    &(subsystem->fb_shm_info), WINPR_ASSERTING_INT_CAST(uint32_t, subsystem->fb_image->width),
+	    WINPR_ASSERTING_INT_CAST(uint32_t, subsystem->fb_image->height),
+	    WINPR_ASSERTING_INT_CAST(uint32_t, subsystem->fb_image->depth));
 	XSync(subsystem->display, False);
 
 	if (!subsystem->fb_pixmap)
@@ -1177,15 +1202,19 @@ UINT32 x11_shadow_enum_monitors(MONITOR_DEF* monitors, UINT32 maxMonitors)
 	int displayHeight = 0;
 	int numMonitors = 0;
 
+	// NOLINTNEXTLINE(concurrency-mt-unsafe)
 	if (!getenv("DISPLAY"))
+	{
+		// NOLINTNEXTLINE(concurrency-mt-unsafe)
 		setenv("DISPLAY", ":0", 1);
+	}
 
 	display = XOpenDisplay(NULL);
 
 	if (!display)
 	{
 		WLog_ERR(TAG, "failed to open display: %s", XDisplayName(NULL));
-		return -1;
+		return 0;
 	}
 
 	displayWidth = WidthOfScreen(DefaultScreenOfDisplay(display));
@@ -1249,7 +1278,7 @@ UINT32 x11_shadow_enum_monitors(MONITOR_DEF* monitors, UINT32 maxMonitors)
 	}
 
 	errno = 0;
-	return numMonitors;
+	return WINPR_ASSERTING_INT_CAST(uint32_t, numMonitors);
 }
 
 static int x11_shadow_subsystem_init(rdpShadowSubsystem* sub)
@@ -1270,7 +1299,12 @@ static int x11_shadow_subsystem_init(rdpShadowSubsystem* sub)
 		return -1;
 
 	subsystem->common.numMonitors = x11_shadow_enum_monitors(subsystem->common.monitors, 16);
-	x11_shadow_subsystem_base_init(subsystem);
+	const int rc = x11_shadow_subsystem_base_init(subsystem);
+	if (rc < 0)
+		return rc;
+
+	subsystem->format = (ImageByteOrder(subsystem->display) == LSBFirst) ? PIXEL_FORMAT_BGRA32
+	                                                                     : PIXEL_FORMAT_ARGB32;
 
 	if ((subsystem->depth != 24) && (subsystem->depth != 32))
 	{
@@ -1308,8 +1342,8 @@ static int x11_shadow_subsystem_init(rdpShadowSubsystem* sub)
 
 		if (pf->depth == (INT64)subsystem->depth)
 		{
-			subsystem->bpp = pf->bits_per_pixel;
-			subsystem->scanline_pad = pf->scanline_pad;
+			subsystem->bpp = WINPR_ASSERTING_INT_CAST(uint32_t, pf->bits_per_pixel);
+			subsystem->scanline_pad = WINPR_ASSERTING_INT_CAST(uint32_t, pf->scanline_pad);
 			break;
 		}
 	}
@@ -1342,7 +1376,7 @@ static int x11_shadow_subsystem_init(rdpShadowSubsystem* sub)
 	subsystem->cursorMaxWidth = 256;
 	subsystem->cursorMaxHeight = 256;
 	subsystem->cursorPixels =
-	    winpr_aligned_malloc(subsystem->cursorMaxWidth * subsystem->cursorMaxHeight * 4ull, 16);
+	    winpr_aligned_malloc(4ULL * subsystem->cursorMaxWidth * subsystem->cursorMaxHeight, 16);
 
 	if (!subsystem->cursorPixels)
 		return -1;
@@ -1392,6 +1426,7 @@ static int x11_shadow_subsystem_init(rdpShadowSubsystem* sub)
 		          subsystem->use_xfixes, subsystem->use_xinerama, subsystem->use_xdamage,
 		          subsystem->use_xshm);
 	}
+
 	return 1;
 }
 
@@ -1410,7 +1445,7 @@ static int x11_shadow_subsystem_uninit(rdpShadowSubsystem* sub)
 
 	if (subsystem->common.event)
 	{
-		CloseHandle(subsystem->common.event);
+		(void)CloseHandle(subsystem->common.event);
 		subsystem->common.event = NULL;
 	}
 
@@ -1450,9 +1485,9 @@ static int x11_shadow_subsystem_stop(rdpShadowSubsystem* sub)
 	if (subsystem->thread)
 	{
 		if (MessageQueue_PostQuit(subsystem->common.MsgPipe->In, 0))
-			WaitForSingleObject(subsystem->thread, INFINITE);
+			(void)WaitForSingleObject(subsystem->thread, INFINITE);
 
-		CloseHandle(subsystem->thread);
+		(void)CloseHandle(subsystem->thread);
 		subsystem->thread = NULL;
 	}
 
