@@ -22,6 +22,7 @@
 
 #include <winpr/crt.h>
 #include <winpr/assert.h>
+#include <winpr/cast.h>
 #include <winpr/file.h>
 #include <winpr/path.h>
 #include <winpr/synch.h>
@@ -68,8 +69,13 @@ static BOOL BitmapUpdateProxyEx(rdpUpdate* update, rdpContext* context, const BI
 		const BOOL rc = IFCALLRESULT(FALSE, update->BitmapUpdate, context, &cur);
 		if (!rc)
 		{
-			WLog_Print(WLog_Get(TAG), WLOG_ERROR, line, file, fkt,
-			           "BitmapUpdate[%" PRIu32 "] failed", x);
+			const DWORD log_level = WLOG_ERROR;
+			wLog* log = WLog_Get(TAG);
+			if (WLog_IsLevelActive(log, log_level))
+			{
+				WLog_PrintMessage(log, WLOG_MESSAGE_TEXT, log_level, line, file, fkt,
+				                  "BitmapUpdate[%" PRIu32 "] failed", x);
+			}
 			return FALSE;
 		}
 	}
@@ -513,7 +519,8 @@ static BOOL shadow_client_post_connect(freerdp_peer* peer)
 		BOOL rc = freerdp_settings_set_bool(
 		    settings, FreeRDP_NSCodec,
 		    FALSE); /* NSCodec compressor does not support fragmentation yet */
-		WINPR_ASSERT(rc);
+		if (!rc)
+			return FALSE;
 	}
 
 	WLog_INFO(TAG, "Client from %s is activated (%" PRIu32 "x%" PRIu32 "@%" PRIu32 ")",
@@ -745,14 +752,21 @@ static BOOL shadow_client_logon(freerdp_peer* peer, const SEC_WINNT_AUTH_IDENTIT
 		goto fail;
 
 	if (user)
-		freerdp_settings_set_string(settings, FreeRDP_Username, user);
+	{
+		if (!freerdp_settings_set_string(settings, FreeRDP_Username, user))
+			goto fail;
+	}
 
 	if (domain)
-		freerdp_settings_set_string(settings, FreeRDP_Domain, domain);
-
+	{
+		if (!freerdp_settings_set_string(settings, FreeRDP_Domain, domain))
+			goto fail;
+	}
 	if (password)
-		freerdp_settings_set_string(settings, FreeRDP_Password, password);
-
+	{
+		if (!freerdp_settings_set_string(settings, FreeRDP_Password, password))
+			goto fail;
+	}
 	rc = TRUE;
 fail:
 	free(user);
@@ -768,7 +782,7 @@ static INLINE void shadow_client_common_frame_acknowledge(rdpShadowClient* clien
 	 * calculate how much frames are in progress.
 	 * Some rdp clients (win7 mstsc) skips frame ACK if it is
 	 * inactive, we should not expect ACK for each frame.
-	 * So it is OK to calculate inflight frame count according to
+	 * So it is OK to calculate in-flight frame count according to
 	 * a latest acknowledged frame id.
 	 */
 	WINPR_ASSERT(client);
@@ -958,9 +972,12 @@ static UINT shadow_client_rdpgfx_caps_advertise(RdpgfxServerContext* context,
 	h264 =
 	    (shadow_encoder_prepare(client->encoder, FREERDP_CODEC_AVC420 | FREERDP_CODEC_AVC444) >= 0);
 #else
-	freerdp_settings_set_bool(clientSettings, FreeRDP_GfxAVC444v2, FALSE);
-	freerdp_settings_set_bool(clientSettings, FreeRDP_GfxAVC444, FALSE);
-	freerdp_settings_set_bool(clientSettings, FreeRDP_GfxH264, FALSE);
+	if (!freerdp_settings_set_bool(clientSettings, FreeRDP_GfxAVC444v2, FALSE))
+		return rc;
+	if (!freerdp_settings_set_bool(clientSettings, FreeRDP_GfxAVC444, FALSE))
+		return rc;
+	if (!freerdp_settings_set_bool(clientSettings, FreeRDP_GfxH264, FALSE))
+		return rc;
 #endif
 
 	/* Request full screen update for new gfx channel */
@@ -1018,25 +1035,38 @@ static UINT shadow_client_rdpgfx_caps_advertise(RdpgfxServerContext* context,
 
 				flags = pdu.capsSet->flags;
 
-				freerdp_settings_set_bool(clientSettings, FreeRDP_GfxAVC444v2, FALSE);
-				freerdp_settings_set_bool(clientSettings, FreeRDP_GfxAVC444, FALSE);
+				if (!freerdp_settings_set_bool(clientSettings, FreeRDP_GfxAVC444v2, FALSE))
+					return rc;
+				if (!freerdp_settings_set_bool(clientSettings, FreeRDP_GfxAVC444, FALSE))
+					return rc;
 
-				freerdp_settings_set_bool(clientSettings, FreeRDP_GfxThinClient,
-				                          (flags & RDPGFX_CAPS_FLAG_THINCLIENT) ? TRUE : FALSE);
-				freerdp_settings_set_bool(clientSettings, FreeRDP_GfxSmallCache,
-				                          (flags & RDPGFX_CAPS_FLAG_SMALL_CACHE) ? TRUE : FALSE);
+				if (!freerdp_settings_set_bool(clientSettings, FreeRDP_GfxThinClient,
+				                               (flags & RDPGFX_CAPS_FLAG_THINCLIENT) ? TRUE
+				                                                                     : FALSE))
+					return rc;
+				if (!freerdp_settings_set_bool(clientSettings, FreeRDP_GfxSmallCache,
+				                               (flags & RDPGFX_CAPS_FLAG_SMALL_CACHE) ? TRUE
+				                                                                      : FALSE))
+					return rc;
 
 #ifndef WITH_GFX_H264
-				freerdp_settings_set_bool(clientSettings, FreeRDP_GfxH264, FALSE);
+				if (!freerdp_settings_set_bool(clientSettings, FreeRDP_GfxH264, FALSE))
+					return rc;
 				pdu.capsSet->flags &= ~RDPGFX_CAPS_FLAG_AVC420_ENABLED;
 #else
 
 				if (h264)
-					freerdp_settings_set_bool(clientSettings, FreeRDP_GfxH264,
-					                          (flags & RDPGFX_CAPS_FLAG_AVC420_ENABLED) ? TRUE
-					                                                                    : FALSE);
+				{
+					if (!freerdp_settings_set_bool(
+					        clientSettings, FreeRDP_GfxH264,
+					        (flags & RDPGFX_CAPS_FLAG_AVC420_ENABLED) ? TRUE : FALSE))
+						return rc;
+				}
 				else
-					freerdp_settings_set_bool(clientSettings, FreeRDP_GfxH264, FALSE);
+				{
+					if (!freerdp_settings_set_bool(clientSettings, FreeRDP_GfxH264, FALSE))
+						return rc;
+				}
 #endif
 
 				return shadow_client_send_caps_confirm(context, client, &pdu);
@@ -1057,14 +1087,21 @@ static UINT shadow_client_rdpgfx_caps_advertise(RdpgfxServerContext* context,
 				pdu.capsSet = &caps;
 				flags = pdu.capsSet->flags;
 
-				freerdp_settings_set_bool(clientSettings, FreeRDP_GfxAVC444v2, FALSE);
-				freerdp_settings_set_bool(clientSettings, FreeRDP_GfxAVC444, FALSE);
-				freerdp_settings_set_bool(clientSettings, FreeRDP_GfxH264, FALSE);
+				if (!freerdp_settings_set_bool(clientSettings, FreeRDP_GfxAVC444v2, FALSE))
+					return rc;
+				if (!freerdp_settings_set_bool(clientSettings, FreeRDP_GfxAVC444, FALSE))
+					return rc;
+				if (!freerdp_settings_set_bool(clientSettings, FreeRDP_GfxH264, FALSE))
+					return rc;
 
-				freerdp_settings_set_bool(clientSettings, FreeRDP_GfxThinClient,
-				                          (flags & RDPGFX_CAPS_FLAG_THINCLIENT) ? TRUE : FALSE);
-				freerdp_settings_set_bool(clientSettings, FreeRDP_GfxSmallCache,
-				                          (flags & RDPGFX_CAPS_FLAG_SMALL_CACHE) ? TRUE : FALSE);
+				if (!freerdp_settings_set_bool(clientSettings, FreeRDP_GfxThinClient,
+				                               (flags & RDPGFX_CAPS_FLAG_THINCLIENT) ? TRUE
+				                                                                     : FALSE))
+					return rc;
+				if (!freerdp_settings_set_bool(clientSettings, FreeRDP_GfxSmallCache,
+				                               (flags & RDPGFX_CAPS_FLAG_SMALL_CACHE) ? TRUE
+				                                                                      : FALSE))
+					return rc;
 
 				return shadow_client_send_caps_confirm(context, client, &pdu);
 			}
@@ -1255,8 +1292,8 @@ static BOOL shadow_client_send_surface_gfx(rdpShadowClient* client, const BYTE*
 		WINPR_ASSERT(cmd.bottom <= UINT16_MAX);
 		rect.x = (UINT16)cmd.left;
 		rect.y = (UINT16)cmd.top;
-		rect.width = (UINT16)cmd.right - cmd.left;
-		rect.height = (UINT16)cmd.bottom - cmd.top;
+		rect.width = WINPR_ASSERTING_INT_CAST(UINT16, cmd.right - cmd.left);
+		rect.height = WINPR_ASSERTING_INT_CAST(UINT16, cmd.bottom - cmd.top);
 
 		rc = rfx_compose_message(encoder->rfx, s, &rect, 1, pSrcData, nWidth, nHeight, nSrcStep);
 
@@ -1687,10 +1724,10 @@ static BOOL shadow_client_send_bitmap_update(rdpShadowClient* client, BYTE* pSrc
 			bitmap->destLeft = nXSrc + (xIdx * 64);
 			bitmap->destTop = nYSrc + (yIdx * 64);
 
-			if ((INT64)(bitmap->destLeft + bitmap->width) > (nXSrc + nWidth))
+			if (((INT64)bitmap->destLeft + bitmap->width) > (nXSrc + nWidth))
 				bitmap->width = (UINT32)(nXSrc + nWidth) - bitmap->destLeft;
 
-			if ((INT64)(bitmap->destTop + bitmap->height) > (nYSrc + nHeight))
+			if (((INT64)bitmap->destTop + bitmap->height) > (nYSrc + nHeight))
 				bitmap->height = (UINT32)(nYSrc + nHeight) - bitmap->destTop;
 
 			bitmap->destRight = bitmap->destLeft + bitmap->width - 1;
@@ -2105,12 +2142,12 @@ static int shadow_client_subsystem_process_message(rdpShadowClient* client, wMes
 			pointerNew.xorBpp = 24;
 			pointerColor = &(pointerNew.colorPtrAttr);
 			pointerColor->cacheIndex = 0;
-			pointerColor->hotSpotX = msg->xHot;
-			pointerColor->hotSpotY = msg->yHot;
-			pointerColor->width = msg->width;
-			pointerColor->height = msg->height;
-			pointerColor->lengthAndMask = msg->lengthAndMask;
-			pointerColor->lengthXorMask = msg->lengthXorMask;
+			pointerColor->hotSpotX = WINPR_ASSERTING_INT_CAST(UINT16, msg->xHot);
+			pointerColor->hotSpotY = WINPR_ASSERTING_INT_CAST(UINT16, msg->yHot);
+			pointerColor->width = WINPR_ASSERTING_INT_CAST(UINT16, msg->width);
+			pointerColor->height = WINPR_ASSERTING_INT_CAST(UINT16, msg->height);
+			pointerColor->lengthAndMask = WINPR_ASSERTING_INT_CAST(UINT16, msg->lengthAndMask);
+			pointerColor->lengthXorMask = WINPR_ASSERTING_INT_CAST(UINT16, msg->lengthXorMask);
 			pointerColor->xorMaskData = msg->xorMaskData;
 			pointerColor->andMaskData = msg->andMaskData;
 			pointerCached.cacheIndex = pointerColor->cacheIndex;
@@ -2555,7 +2592,7 @@ BOOL shadow_client_accepted(freerdp_listener* listener, freerdp_peer* peer)
 	else
 	{
 		/* Close the thread handle to make it detached. */
-		CloseHandle(client->thread);
+		(void)CloseHandle(client->thread);
 		client->thread = NULL;
 	}
 
