@@ -32,7 +32,7 @@
 #pragma clang diagnostic ignored "-Wreserved-id-macro"
 #endif
 
-#define _GNU_SOURCE // NOLINT(bugprone-reserved-identifier)
+#define _GNU_SOURCE // NOLINT(bugprone-reserved-identifier,cert-dcl37-c,cert-dcl51-cpp)
 
 #if defined(__clang__)
 #pragma clang diagnostic pop
@@ -228,10 +228,10 @@ int uwac_create_anonymous_file(off_t size)
 	static const char template[] = "/weston-shared-XXXXXX";
 	size_t length = 0;
 	char* name = NULL;
-	const char* path = NULL;
 	int fd = 0;
 	int ret = 0;
-	path = getenv("XDG_RUNTIME_DIR");
+	// NOLINTNEXTLINE(concurrency-mt-unsafe)
+	const char* path = getenv("XDG_RUNTIME_DIR");
 
 	if (!path)
 	{
