@@ -24,6 +24,7 @@
 #include <winpr/user.h>
 #include <winpr/image.h>
 
+#include "../utils/image.h"
 #include "clipboard.h"
 
 static const char* mime_bitmap[] = { "image/bmp", "image/x-bmp", "image/x-MS-bmp",
@@ -59,12 +60,15 @@ static void* clipboard_synthesize_cf_text(wClipboard* clipboard, UINT32 formatId
 	{
 		char* str = ConvertWCharNToUtf8Alloc(data, *pSize / sizeof(WCHAR), &size);
 
-		if (!str)
+		if (!str || (size > UINT32_MAX))
+		{
+			free(str);
 			return NULL;
+		}
 
 		pDstData = ConvertLineEndingToCRLF(str, &size);
 		free(str);
-		*pSize = size;
+		*pSize = (UINT32)size;
 		return pDstData;
 	}
 	else if ((formatId == CF_TEXT) || (formatId == CF_OEMTEXT) ||
@@ -73,10 +77,13 @@ static void* clipboard_synthesize_cf_text(wClipboard* clipboard, UINT32 formatId
 		size = *pSize;
 		pDstData = ConvertLineEndingToCRLF(data, &size);
 
-		if (!pDstData)
+		if (!pDstData || (size > *pSize))
+		{
+			free(pDstData);
 			return NULL;
+		}
 
-		*pSize = size;
+		*pSize = (UINT32)size;
 		return pDstData;
 	}
 
@@ -101,8 +108,10 @@ static void* clipboard_synthesize_cf_oemtext(wClipboard* clipboard, UINT32 forma
  * System locale identifier associated with CF_TEXT
  */
 
-static void* clipboard_synthesize_cf_locale(wClipboard* clipboard, UINT32 formatId,
-                                            const void* data, UINT32* pSize)
+static void* clipboard_synthesize_cf_locale(WINPR_ATTR_UNUSED wClipboard* clipboard,
+                                            WINPR_ATTR_UNUSED UINT32 formatId,
+                                            WINPR_ATTR_UNUSED const void* data,
+                                            WINPR_ATTR_UNUSED UINT32* pSize)
 {
 	UINT32* pDstData = NULL;
 	pDstData = (UINT32*)malloc(sizeof(UINT32));
@@ -143,13 +152,14 @@ static void* clipboard_synthesize_cf_unicodetext(wClipboard* clipboard, UINT32 f
 		pDstData = ConvertUtf8NToWCharAlloc(crlfStr, size, &len);
 		free(crlfStr);
 
-		if ((len < 1) || (len > UINT32_MAX / sizeof(WCHAR)))
+		if ((len < 1) || ((len + 1) > UINT32_MAX / sizeof(WCHAR)))
 		{
 			free(pDstData);
 			return NULL;
 		}
 
-		*pSize = (len + 1) * sizeof(WCHAR);
+		const size_t slen = (len + 1) * sizeof(WCHAR);
+		*pSize = (UINT32)slen;
 	}
 
 	return (void*)pDstData;
@@ -164,37 +174,31 @@ static void* clipboard_synthesize_cf_unicodetext(wClipboard* clipboard, UINT32 f
 static void* clipboard_synthesize_utf8_string(wClipboard* clipboard, UINT32 formatId,
                                               const void* data, UINT32* pSize)
 {
-	size_t size = 0;
-	char* pDstData = NULL;
-
 	if (formatId == CF_UNICODETEXT)
 	{
-		pDstData = ConvertWCharNToUtf8Alloc(data, *pSize / sizeof(WCHAR), &size);
+		size_t size = 0;
+		char* pDstData = ConvertWCharNToUtf8Alloc(data, *pSize / sizeof(WCHAR), &size);
 
 		if (!pDstData)
 			return NULL;
 
-		size = ConvertLineEndingToLF(pDstData, size);
-		*pSize = (UINT32)size;
+		const size_t rc = ConvertLineEndingToLF(pDstData, size);
+		WINPR_ASSERT(rc <= UINT32_MAX);
+		*pSize = (UINT32)rc;
 		return pDstData;
 	}
 	else if ((formatId == CF_TEXT) || (formatId == CF_OEMTEXT) ||
 	         (formatId == ClipboardGetFormatId(clipboard, mime_text_plain)))
 	{
-		int rc = 0;
-		size = *pSize;
-		pDstData = (char*)malloc(size);
+		const size_t size = *pSize;
+		char* pDstData = calloc(size + 1, sizeof(char));
 
 		if (!pDstData)
 			return NULL;
 
 		CopyMemory(pDstData, data, size);
-		rc = ConvertLineEndingToLF(pDstData, size);
-		if (rc < 0)
-		{
-			free(pDstData);
-			return NULL;
-		}
+		const size_t rc = ConvertLineEndingToLF(pDstData, size);
+		WINPR_ASSERT(rc <= UINT32_MAX);
 		*pSize = (UINT32)rc;
 		return pDstData;
 	}
@@ -234,14 +238,10 @@ static void* clipboard_synthesize_cf_dib(wClipboard* clipboard, UINT32 formatId,
 	}
 	else if (is_format_bitmap(clipboard, formatId))
 	{
-		const BITMAPFILEHEADER* pFileHeader = NULL;
-
-		if (SrcSize < (sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER)))
-			return NULL;
-
-		pFileHeader = (const BITMAPFILEHEADER*)data;
-
-		if (pFileHeader->bfType != 0x4D42)
+		WINPR_BITMAP_FILE_HEADER pFileHeader = { 0 };
+		wStream sbuffer = { 0 };
+		wStream* s = Stream_StaticConstInit(&sbuffer, data, SrcSize);
+		if (!readBitmapFileHeader(s, &pFileHeader))
 			return NULL;
 
 		DstSize = SrcSize - sizeof(BITMAPFILEHEADER);
@@ -265,8 +265,9 @@ static void* clipboard_synthesize_cf_dib(wClipboard* clipboard, UINT32 formatId,
  * BITMAPV5HEADER structure followed by the bitmap color space information and the bitmap bits.
  */
 
-static void* clipboard_synthesize_cf_dibv5(wClipboard* clipboard, UINT32 formatId, const void* data,
-                                           UINT32* pSize)
+static void* clipboard_synthesize_cf_dibv5(wClipboard* clipboard, UINT32 formatId,
+                                           WINPR_ATTR_UNUSED const void* data,
+                                           WINPR_ATTR_UNUSED UINT32* pSize)
 {
 	if (formatId == CF_DIB)
 	{
@@ -278,8 +279,8 @@ static void* clipboard_synthesize_cf_dibv5(wClipboard* clipboard, UINT32 formatI
 	return NULL;
 }
 
-static void* clipboard_prepend_bmp_header(const BITMAPINFOHEADER* pInfoHeader, const void* data,
-                                          size_t size, UINT32* pSize)
+static void* clipboard_prepend_bmp_header(const WINPR_BITMAP_INFO_HEADER* pInfoHeader,
+                                          const void* data, size_t size, UINT32* pSize)
 {
 	WINPR_ASSERT(pInfoHeader);
 	WINPR_ASSERT(pSize);
@@ -288,22 +289,37 @@ static void* clipboard_prepend_bmp_header(const BITMAPINFOHEADER* pInfoHeader, c
 	if ((pInfoHeader->biBitCount < 1) || (pInfoHeader->biBitCount > 32))
 		return NULL;
 
-	const size_t DstSize = sizeof(BITMAPFILEHEADER) + size;
-	BYTE* pDstData = (BYTE*)malloc(DstSize);
-
-	if (!pDstData)
+	const size_t DstSize = sizeof(WINPR_BITMAP_FILE_HEADER) + size;
+	if (DstSize > UINT32_MAX)
 		return NULL;
 
-	BITMAPFILEHEADER* pFileHeader = (BITMAPFILEHEADER*)pDstData;
-	pFileHeader->bfType = 0x4D42;
-	pFileHeader->bfSize = DstSize;
-	pFileHeader->bfReserved1 = 0;
-	pFileHeader->bfReserved2 = 0;
-	pFileHeader->bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
-	unsigned char* pDst = &pDstData[sizeof(BITMAPFILEHEADER)];
-	CopyMemory(pDst, data, size);
-	*pSize = DstSize;
-	return pDstData;
+	wStream* s = Stream_New(NULL, DstSize);
+	if (!s)
+		return NULL;
+
+	WINPR_BITMAP_FILE_HEADER fileHeader = { 0 };
+	fileHeader.bfType[0] = 'B';
+	fileHeader.bfType[1] = 'M';
+	fileHeader.bfSize = (UINT32)DstSize;
+	fileHeader.bfOffBits = sizeof(WINPR_BITMAP_FILE_HEADER) + sizeof(WINPR_BITMAP_INFO_HEADER);
+	if (!writeBitmapFileHeader(s, &fileHeader))
+		goto fail;
+
+	if (!Stream_EnsureRemainingCapacity(s, size))
+		goto fail;
+	Stream_Write(s, data, size);
+	const size_t len = Stream_GetPosition(s);
+	if (len != DstSize)
+		goto fail;
+	*pSize = (UINT32)DstSize;
+
+	BYTE* dst = Stream_Buffer(s);
+	Stream_Free(s, FALSE);
+	return dst;
+
+fail:
+	Stream_Free(s, TRUE);
+	return NULL;
 }
 
 /**
@@ -312,8 +328,8 @@ static void* clipboard_prepend_bmp_header(const BITMAPINFOHEADER* pInfoHeader, c
  * Bitmap file format.
  */
 
-static void* clipboard_synthesize_image_bmp(wClipboard* clipboard, UINT32 formatId,
-                                            const void* data, UINT32* pSize)
+static void* clipboard_synthesize_image_bmp(WINPR_ATTR_UNUSED wClipboard* clipboard,
+                                            UINT32 formatId, const void* data, UINT32* pSize)
 {
 	UINT32 SrcSize = *pSize;
 
@@ -322,8 +338,14 @@ static void* clipboard_synthesize_image_bmp(wClipboard* clipboard, UINT32 format
 		if (SrcSize < sizeof(BITMAPINFOHEADER))
 			return NULL;
 
-		const BITMAPINFOHEADER* pInfoHeader = (const BITMAPINFOHEADER*)data;
-		return clipboard_prepend_bmp_header(pInfoHeader, data, SrcSize, pSize);
+		wStream sbuffer = { 0 };
+		size_t offset = 0;
+		WINPR_BITMAP_INFO_HEADER header = { 0 };
+		wStream* s = Stream_StaticConstInit(&sbuffer, data, SrcSize);
+		if (!readBitmapInfoHeader(s, &header, &offset))
+			return NULL;
+
+		return clipboard_prepend_bmp_header(&header, data, SrcSize, pSize);
 	}
 	else if (formatId == CF_DIBV5)
 	{
@@ -332,6 +354,8 @@ static void* clipboard_synthesize_image_bmp(wClipboard* clipboard, UINT32 format
 	return NULL;
 }
 
+#if defined(WINPR_UTILS_IMAGE_PNG) || defined(WINPR_UTILS_IMAGE_WEBP) || \
+    defined(WINPR_UTILS_IMAGE_JPEG)
 static void* clipboard_synthesize_image_bmp_to_format(wClipboard* clipboard, UINT32 formatId,
                                                       UINT32 bmpFormat, const void* data,
                                                       UINT32* pSize)
@@ -356,13 +380,22 @@ static void* clipboard_synthesize_image_bmp_to_format(wClipboard* clipboard, UIN
 
 	result = winpr_image_write_buffer(img, bmpFormat, &dsize);
 	if (result)
-		*pSize = dsize;
+	{
+		if (dsize <= UINT32_MAX)
+			*pSize = (UINT32)dsize;
+		else
+		{
+			free(result);
+			result = NULL;
+		}
+	}
 
 fail:
 	free(bmp);
 	winpr_image_free(img, TRUE);
 	return result;
 }
+#endif
 
 #if defined(WINPR_UTILS_IMAGE_PNG)
 static void* clipboard_synthesize_image_bmp_to_png(wClipboard* clipboard, UINT32 formatId,
@@ -372,7 +405,8 @@ static void* clipboard_synthesize_image_bmp_to_png(wClipboard* clipboard, UINT32
 	                                                pSize);
 }
 
-static void* clipboard_synthesize_image_format_to_bmp(wClipboard* clipboard, UINT32 srcFormatId,
+static void* clipboard_synthesize_image_format_to_bmp(wClipboard* clipboard,
+                                                      WINPR_ATTR_UNUSED UINT32 srcFormatId,
                                                       const void* data, UINT32* pSize)
 {
 	WINPR_ASSERT(clipboard);
@@ -470,7 +504,7 @@ static void* clipboard_synthesize_html_format(wClipboard* clipboard, UINT32 form
 
 	if (formatId == ClipboardGetFormatId(clipboard, "text/html"))
 	{
-		const INT64 SrcSize = (INT64)*pSize;
+		const size_t SrcSize = (size_t)*pSize;
 		const size_t DstSize = SrcSize + 200;
 		char* body = NULL;
 		char num[20] = { 0 };
@@ -570,49 +604,43 @@ fail:
 static void* clipboard_synthesize_text_html(wClipboard* clipboard, UINT32 formatId,
                                             const void* data, UINT32* pSize)
 {
-	long beg = 0;
-	long end = 0;
-	const char* str = NULL;
-	char* begStr = NULL;
-	char* endStr = NULL;
-	long DstSize = -1;
-	BYTE* pDstData = NULL;
+	char* pDstData = NULL;
 
 	if (formatId == ClipboardGetFormatId(clipboard, "HTML Format"))
 	{
-		INT64 SrcSize = 0;
-		str = (const char*)data;
-		SrcSize = (INT64)*pSize;
-		begStr = strstr(str, "StartHTML:");
-		endStr = strstr(str, "EndHTML:");
+		const char* str = (const char*)data;
+		const size_t SrcSize = *pSize;
+		const char* begStr = strstr(str, "StartHTML:");
+		const char* endStr = strstr(str, "EndHTML:");
 
 		if (!begStr || !endStr)
 			return NULL;
 
 		errno = 0;
-		beg = strtol(&begStr[10], NULL, 10);
+		const long beg = strtol(&begStr[10], NULL, 10);
 
 		if (errno != 0)
 			return NULL;
 
-		end = strtol(&endStr[8], NULL, 10);
+		const long end = strtol(&endStr[8], NULL, 10);
 
-		if (beg < 0 || end < 0 || (beg > SrcSize) || (end > SrcSize) || (beg >= end) ||
-		    (errno != 0))
+		if ((beg < 0) || (end < 0) || ((size_t)beg > SrcSize) || ((size_t)end > SrcSize) ||
+		    (beg >= end) || (errno != 0))
 			return NULL;
 
-		DstSize = end - beg;
-		pDstData = (BYTE*)malloc((size_t)(SrcSize - beg + 1));
+		const size_t DstSize = (size_t)(end - beg);
+		pDstData = calloc(DstSize + 1, sizeof(char));
 
 		if (!pDstData)
 			return NULL;
 
 		CopyMemory(pDstData, &str[beg], DstSize);
-		DstSize = ConvertLineEndingToLF((char*)pDstData, DstSize);
-		*pSize = (UINT32)DstSize;
+		const size_t rc = ConvertLineEndingToLF(pDstData, DstSize);
+		WINPR_ASSERT(rc <= UINT32_MAX);
+		*pSize = (UINT32)rc;
 	}
 
-	return (void*)pDstData;
+	return pDstData;
 }
 
 BOOL ClipboardInitSynthesizers(wClipboard* clipboard)
