@@ -27,6 +27,9 @@
 #include <fcntl.h>
 #include <pthread.h>
 #include <stdarg.h>
+#if defined(WINPR_HAVE_SYS_EVENTFD_H)
+#include <sys/eventfd.h>
+#endif
 #include <sys/ioctl.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -62,15 +65,79 @@ struct comm_device
 typedef struct comm_device COMM_DEVICE;
 
 /* FIXME: get a clever data structure, see also io.h functions */
-/* _CommDevices is a NULL-terminated array with a maximun of COMM_DEVICE_MAX COMM_DEVICE */
+/* _CommDevices is a NULL-terminated array with a maximum of COMM_DEVICE_MAX COMM_DEVICE */
 #define COMM_DEVICE_MAX 128
 static COMM_DEVICE** sCommDevices = NULL;
-static CRITICAL_SECTION sCommDevicesLock;
-
-static HANDLE_CREATOR sCommHandleCreator = { 0 };
+static CRITICAL_SECTION sCommDevicesLock = { 0 };
 
 static pthread_once_t sCommInitialized = PTHREAD_ONCE_INIT;
 
+static const _SERIAL_IOCTL_NAME S_SERIAL_IOCTL_NAMES[] = {
+	{ IOCTL_SERIAL_SET_BAUD_RATE, "IOCTL_SERIAL_SET_BAUD_RATE" },
+	{ IOCTL_SERIAL_GET_BAUD_RATE, "IOCTL_SERIAL_GET_BAUD_RATE" },
+	{ IOCTL_SERIAL_SET_LINE_CONTROL, "IOCTL_SERIAL_SET_LINE_CONTROL" },
+	{ IOCTL_SERIAL_GET_LINE_CONTROL, "IOCTL_SERIAL_GET_LINE_CONTROL" },
+	{ IOCTL_SERIAL_SET_TIMEOUTS, "IOCTL_SERIAL_SET_TIMEOUTS" },
+	{ IOCTL_SERIAL_GET_TIMEOUTS, "IOCTL_SERIAL_GET_TIMEOUTS" },
+	{ IOCTL_SERIAL_GET_CHARS, "IOCTL_SERIAL_GET_CHARS" },
+	{ IOCTL_SERIAL_SET_CHARS, "IOCTL_SERIAL_SET_CHARS" },
+	{ IOCTL_SERIAL_SET_DTR, "IOCTL_SERIAL_SET_DTR" },
+	{ IOCTL_SERIAL_CLR_DTR, "IOCTL_SERIAL_CLR_DTR" },
+	{ IOCTL_SERIAL_RESET_DEVICE, "IOCTL_SERIAL_RESET_DEVICE" },
+	{ IOCTL_SERIAL_SET_RTS, "IOCTL_SERIAL_SET_RTS" },
+	{ IOCTL_SERIAL_CLR_RTS, "IOCTL_SERIAL_CLR_RTS" },
+	{ IOCTL_SERIAL_SET_XOFF, "IOCTL_SERIAL_SET_XOFF" },
+	{ IOCTL_SERIAL_SET_XON, "IOCTL_SERIAL_SET_XON" },
+	{ IOCTL_SERIAL_SET_BREAK_ON, "IOCTL_SERIAL_SET_BREAK_ON" },
+	{ IOCTL_SERIAL_SET_BREAK_OFF, "IOCTL_SERIAL_SET_BREAK_OFF" },
+	{ IOCTL_SERIAL_SET_QUEUE_SIZE, "IOCTL_SERIAL_SET_QUEUE_SIZE" },
+	{ IOCTL_SERIAL_GET_WAIT_MASK, "IOCTL_SERIAL_GET_WAIT_MASK" },
+	{ IOCTL_SERIAL_SET_WAIT_MASK, "IOCTL_SERIAL_SET_WAIT_MASK" },
+	{ IOCTL_SERIAL_WAIT_ON_MASK, "IOCTL_SERIAL_WAIT_ON_MASK" },
+	{ IOCTL_SERIAL_IMMEDIATE_CHAR, "IOCTL_SERIAL_IMMEDIATE_CHAR" },
+	{ IOCTL_SERIAL_PURGE, "IOCTL_SERIAL_PURGE" },
+	{ IOCTL_SERIAL_GET_HANDFLOW, "IOCTL_SERIAL_GET_HANDFLOW" },
+	{ IOCTL_SERIAL_SET_HANDFLOW, "IOCTL_SERIAL_SET_HANDFLOW" },
+	{ IOCTL_SERIAL_GET_MODEMSTATUS, "IOCTL_SERIAL_GET_MODEMSTATUS" },
+	{ IOCTL_SERIAL_GET_DTRRTS, "IOCTL_SERIAL_GET_DTRRTS" },
+	{ IOCTL_SERIAL_GET_COMMSTATUS, "IOCTL_SERIAL_GET_COMMSTATUS" },
+	{ IOCTL_SERIAL_GET_PROPERTIES, "IOCTL_SERIAL_GET_PROPERTIES" },
+	// {IOCTL_SERIAL_XOFF_COUNTER,	"IOCTL_SERIAL_XOFF_COUNTER"},
+	// {IOCTL_SERIAL_LSRMST_INSERT,	"IOCTL_SERIAL_LSRMST_INSERT"},
+	{ IOCTL_SERIAL_CONFIG_SIZE, "IOCTL_SERIAL_CONFIG_SIZE" },
+	// {IOCTL_SERIAL_GET_STATS,	"IOCTL_SERIAL_GET_STATS"},
+	// {IOCTL_SERIAL_CLEAR_STATS,	"IOCTL_SERIAL_CLEAR_STATS"},
+	// {IOCTL_SERIAL_GET_MODEM_CONTROL,"IOCTL_SERIAL_GET_MODEM_CONTROL"},
+	// {IOCTL_SERIAL_SET_MODEM_CONTROL,"IOCTL_SERIAL_SET_MODEM_CONTROL"},
+	// {IOCTL_SERIAL_SET_FIFO_CONTROL,	"IOCTL_SERIAL_SET_FIFO_CONTROL"},
+
+	// {IOCTL_PAR_QUERY_INFORMATION,	"IOCTL_PAR_QUERY_INFORMATION"},
+	// {IOCTL_PAR_SET_INFORMATION,	"IOCTL_PAR_SET_INFORMATION"},
+	// {IOCTL_PAR_QUERY_DEVICE_ID,	"IOCTL_PAR_QUERY_DEVICE_ID"},
+	// {IOCTL_PAR_QUERY_DEVICE_ID_SIZE,"IOCTL_PAR_QUERY_DEVICE_ID_SIZE"},
+	// {IOCTL_IEEE1284_GET_MODE,	"IOCTL_IEEE1284_GET_MODE"},
+	// {IOCTL_IEEE1284_NEGOTIATE,	"IOCTL_IEEE1284_NEGOTIATE"},
+	// {IOCTL_PAR_SET_WRITE_ADDRESS,	"IOCTL_PAR_SET_WRITE_ADDRESS"},
+	// {IOCTL_PAR_SET_READ_ADDRESS,	"IOCTL_PAR_SET_READ_ADDRESS"},
+	// {IOCTL_PAR_GET_DEVICE_CAPS,	"IOCTL_PAR_GET_DEVICE_CAPS"},
+	// {IOCTL_PAR_GET_DEFAULT_MODES,	"IOCTL_PAR_GET_DEFAULT_MODES"},
+	// {IOCTL_PAR_QUERY_RAW_DEVICE_ID, "IOCTL_PAR_QUERY_RAW_DEVICE_ID"},
+	// {IOCTL_PAR_IS_PORT_FREE,	"IOCTL_PAR_IS_PORT_FREE"},
+
+	{ IOCTL_USBPRINT_GET_1284_ID, "IOCTL_USBPRINT_GET_1284_ID" }
+};
+const char* _comm_serial_ioctl_name(ULONG number)
+{
+	for (size_t x = 0; x < ARRAYSIZE(S_SERIAL_IOCTL_NAMES); x++)
+	{
+		const _SERIAL_IOCTL_NAME* cur = &S_SERIAL_IOCTL_NAMES[x];
+		if (cur->number == number)
+			return cur->name;
+	}
+
+	return "(unknown ioctl name)";
+}
+
 static int CommGetFd(HANDLE handle)
 {
 	WINPR_COMM* comm = (WINPR_COMM*)handle;
@@ -81,11 +148,15 @@ static int CommGetFd(HANDLE handle)
 	return comm->fd;
 }
 
-HANDLE_CREATOR* GetCommHandleCreator(void)
+const HANDLE_CREATOR* GetCommHandleCreator(void)
 {
-	sCommHandleCreator.IsHandled = IsCommDevice;
-	sCommHandleCreator.CreateFileA = CommCreateFileA;
+#if defined(WINPR_HAVE_SERIAL_SUPPORT)
+	static const HANDLE_CREATOR sCommHandleCreator = { .IsHandled = IsCommDevice,
+		                                               .CreateFileA = CommCreateFileA };
 	return &sCommHandleCreator;
+#else
+	return NULL;
+#endif
 }
 
 static void CommInit(void)
@@ -100,7 +171,7 @@ static void CommInit(void)
 
 	if (!InitializeCriticalSectionEx(&sCommDevicesLock, 0, 0))
 	{
-		free(sCommDevices);
+		free((void*)sCommDevices);
 		sCommDevices = NULL;
 		return;
 	}
@@ -110,7 +181,7 @@ static void CommInit(void)
 }
 
 /**
- * Returns TRUE when the comm module is correctly intialized, FALSE otherwise
+ * Returns TRUE when the comm module is correctly initialized, FALSE otherwise
  * with ERROR_DLL_INIT_FAILED set as the last error.
  */
 static BOOL CommInitialized(void)
@@ -129,13 +200,13 @@ void CommLog_Print(DWORD level, ...)
 	if (!CommInitialized())
 		return;
 
-	va_list ap;
+	va_list ap = { 0 };
 	va_start(ap, level);
 	WLog_PrintVA(sLog, level, ap);
 	va_end(ap);
 }
 
-BOOL BuildCommDCBA(LPCSTR lpDef, LPDCB lpDCB)
+BOOL BuildCommDCBA(WINPR_ATTR_UNUSED LPCSTR lpDef, WINPR_ATTR_UNUSED LPDCB lpDCB)
 {
 	if (!CommInitialized())
 		return FALSE;
@@ -146,7 +217,7 @@ BOOL BuildCommDCBA(LPCSTR lpDef, LPDCB lpDCB)
 	return FALSE;
 }
 
-BOOL BuildCommDCBW(LPCWSTR lpDef, LPDCB lpDCB)
+BOOL BuildCommDCBW(WINPR_ATTR_UNUSED LPCWSTR lpDef, WINPR_ATTR_UNUSED LPDCB lpDCB)
 {
 	if (!CommInitialized())
 		return FALSE;
@@ -157,7 +228,8 @@ BOOL BuildCommDCBW(LPCWSTR lpDef, LPDCB lpDCB)
 	return FALSE;
 }
 
-BOOL BuildCommDCBAndTimeoutsA(LPCSTR lpDef, LPDCB lpDCB, LPCOMMTIMEOUTS lpCommTimeouts)
+BOOL BuildCommDCBAndTimeoutsA(WINPR_ATTR_UNUSED LPCSTR lpDef, WINPR_ATTR_UNUSED LPDCB lpDCB,
+                              WINPR_ATTR_UNUSED LPCOMMTIMEOUTS lpCommTimeouts)
 {
 	if (!CommInitialized())
 		return FALSE;
@@ -168,7 +240,8 @@ BOOL BuildCommDCBAndTimeoutsA(LPCSTR lpDef, LPDCB lpDCB, LPCOMMTIMEOUTS lpCommTi
 	return FALSE;
 }
 
-BOOL BuildCommDCBAndTimeoutsW(LPCWSTR lpDef, LPDCB lpDCB, LPCOMMTIMEOUTS lpCommTimeouts)
+BOOL BuildCommDCBAndTimeoutsW(WINPR_ATTR_UNUSED LPCWSTR lpDef, WINPR_ATTR_UNUSED LPDCB lpDCB,
+                              WINPR_ATTR_UNUSED LPCOMMTIMEOUTS lpCommTimeouts)
 {
 	if (!CommInitialized())
 		return FALSE;
@@ -179,7 +252,8 @@ BOOL BuildCommDCBAndTimeoutsW(LPCWSTR lpDef, LPDCB lpDCB, LPCOMMTIMEOUTS lpCommT
 	return FALSE;
 }
 
-BOOL CommConfigDialogA(LPCSTR lpszName, HWND hWnd, LPCOMMCONFIG lpCC)
+BOOL CommConfigDialogA(WINPR_ATTR_UNUSED LPCSTR lpszName, WINPR_ATTR_UNUSED HWND hWnd,
+                       WINPR_ATTR_UNUSED LPCOMMCONFIG lpCC)
 {
 	if (!CommInitialized())
 		return FALSE;
@@ -190,7 +264,8 @@ BOOL CommConfigDialogA(LPCSTR lpszName, HWND hWnd, LPCOMMCONFIG lpCC)
 	return FALSE;
 }
 
-BOOL CommConfigDialogW(LPCWSTR lpszName, HWND hWnd, LPCOMMCONFIG lpCC)
+BOOL CommConfigDialogW(WINPR_ATTR_UNUSED LPCWSTR lpszName, WINPR_ATTR_UNUSED HWND hWnd,
+                       WINPR_ATTR_UNUSED LPCOMMCONFIG lpCC)
 {
 	if (!CommInitialized())
 		return FALSE;
@@ -201,7 +276,8 @@ BOOL CommConfigDialogW(LPCWSTR lpszName, HWND hWnd, LPCOMMCONFIG lpCC)
 	return FALSE;
 }
 
-BOOL GetCommConfig(HANDLE hCommDev, LPCOMMCONFIG lpCC, LPDWORD lpdwSize)
+BOOL GetCommConfig(HANDLE hCommDev, WINPR_ATTR_UNUSED LPCOMMCONFIG lpCC,
+                   WINPR_ATTR_UNUSED LPDWORD lpdwSize)
 {
 	WINPR_COMM* pComm = (WINPR_COMM*)hCommDev;
 
@@ -218,7 +294,8 @@ BOOL GetCommConfig(HANDLE hCommDev, LPCOMMCONFIG lpCC, LPDWORD lpdwSize)
 	return FALSE;
 }
 
-BOOL SetCommConfig(HANDLE hCommDev, LPCOMMCONFIG lpCC, DWORD dwSize)
+BOOL SetCommConfig(HANDLE hCommDev, WINPR_ATTR_UNUSED LPCOMMCONFIG lpCC,
+                   WINPR_ATTR_UNUSED DWORD dwSize)
 {
 	WINPR_COMM* pComm = (WINPR_COMM*)hCommDev;
 
@@ -235,7 +312,7 @@ BOOL SetCommConfig(HANDLE hCommDev, LPCOMMCONFIG lpCC, DWORD dwSize)
 	return FALSE;
 }
 
-BOOL GetCommMask(HANDLE hFile, PDWORD lpEvtMask)
+BOOL GetCommMask(HANDLE hFile, WINPR_ATTR_UNUSED PDWORD lpEvtMask)
 {
 	WINPR_COMM* pComm = (WINPR_COMM*)hFile;
 
@@ -252,7 +329,7 @@ BOOL GetCommMask(HANDLE hFile, PDWORD lpEvtMask)
 	return FALSE;
 }
 
-BOOL SetCommMask(HANDLE hFile, DWORD dwEvtMask)
+BOOL SetCommMask(HANDLE hFile, WINPR_ATTR_UNUSED DWORD dwEvtMask)
 {
 	WINPR_COMM* pComm = (WINPR_COMM*)hFile;
 
@@ -269,7 +346,7 @@ BOOL SetCommMask(HANDLE hFile, DWORD dwEvtMask)
 	return FALSE;
 }
 
-BOOL GetCommModemStatus(HANDLE hFile, PDWORD lpModemStat)
+BOOL GetCommModemStatus(HANDLE hFile, WINPR_ATTR_UNUSED PDWORD lpModemStat)
 {
 	WINPR_COMM* pComm = (WINPR_COMM*)hFile;
 
@@ -426,7 +503,7 @@ BOOL GetCommState(HANDLE hFile, LPDCB lpDCB)
 	lpLocalDcb->wReserved = 0; /* must be zero */
 	lpLocalDcb->XonLim = handflow.XonLimit;
 	lpLocalDcb->XoffLim = handflow.XoffLimit;
-	SERIAL_LINE_CONTROL lineControl;
+	SERIAL_LINE_CONTROL lineControl = { 0 };
 
 	if (!CommDeviceIoControl(pComm, IOCTL_SERIAL_GET_LINE_CONTROL, NULL, 0, &lineControl,
 	                         sizeof(SERIAL_LINE_CONTROL), &bytesReturned, NULL))
@@ -651,7 +728,7 @@ BOOL SetCommState(HANDLE hFile, LPDCB lpDCB)
 
 	if (lpDCB->fBinary)
 	{
-		upcomingTermios.c_lflag &= ~ICANON;
+		upcomingTermios.c_lflag &= WINPR_ASSERTING_INT_CAST(tcflag_t, ~ICANON);
 	}
 	else
 	{
@@ -665,7 +742,7 @@ BOOL SetCommState(HANDLE hFile, LPDCB lpDCB)
 	}
 	else
 	{
-		upcomingTermios.c_iflag &= ~INPCK;
+		upcomingTermios.c_iflag &= WINPR_ASSERTING_INT_CAST(tcflag_t, ~INPCK);
 	}
 
 	/* http://msdn.microsoft.com/en-us/library/windows/desktop/aa363423%28v=vs.85%29.aspx
@@ -736,7 +813,8 @@ BOOL SetCommTimeouts(HANDLE hFile, LPCOMMTIMEOUTS lpCommTimeouts)
 	return TRUE;
 }
 
-BOOL GetDefaultCommConfigA(LPCSTR lpszName, LPCOMMCONFIG lpCC, LPDWORD lpdwSize)
+BOOL GetDefaultCommConfigA(WINPR_ATTR_UNUSED LPCSTR lpszName, WINPR_ATTR_UNUSED LPCOMMCONFIG lpCC,
+                           WINPR_ATTR_UNUSED LPDWORD lpdwSize)
 {
 	if (!CommInitialized())
 		return FALSE;
@@ -747,7 +825,8 @@ BOOL GetDefaultCommConfigA(LPCSTR lpszName, LPCOMMCONFIG lpCC, LPDWORD lpdwSize)
 	return FALSE;
 }
 
-BOOL GetDefaultCommConfigW(LPCWSTR lpszName, LPCOMMCONFIG lpCC, LPDWORD lpdwSize)
+BOOL GetDefaultCommConfigW(WINPR_ATTR_UNUSED LPCWSTR lpszName, WINPR_ATTR_UNUSED LPCOMMCONFIG lpCC,
+                           WINPR_ATTR_UNUSED LPDWORD lpdwSize)
 {
 	if (!CommInitialized())
 		return FALSE;
@@ -758,7 +837,8 @@ BOOL GetDefaultCommConfigW(LPCWSTR lpszName, LPCOMMCONFIG lpCC, LPDWORD lpdwSize
 	return FALSE;
 }
 
-BOOL SetDefaultCommConfigA(LPCSTR lpszName, LPCOMMCONFIG lpCC, DWORD dwSize)
+BOOL SetDefaultCommConfigA(WINPR_ATTR_UNUSED LPCSTR lpszName, WINPR_ATTR_UNUSED LPCOMMCONFIG lpCC,
+                           WINPR_ATTR_UNUSED DWORD dwSize)
 {
 	if (!CommInitialized())
 		return FALSE;
@@ -769,7 +849,8 @@ BOOL SetDefaultCommConfigA(LPCSTR lpszName, LPCOMMCONFIG lpCC, DWORD dwSize)
 	return FALSE;
 }
 
-BOOL SetDefaultCommConfigW(LPCWSTR lpszName, LPCOMMCONFIG lpCC, DWORD dwSize)
+BOOL SetDefaultCommConfigW(WINPR_ATTR_UNUSED LPCWSTR lpszName, WINPR_ATTR_UNUSED LPCOMMCONFIG lpCC,
+                           WINPR_ATTR_UNUSED DWORD dwSize)
 {
 	if (!CommInitialized())
 		return FALSE;
@@ -814,7 +895,8 @@ BOOL ClearCommBreak(HANDLE hFile)
 	return FALSE;
 }
 
-BOOL ClearCommError(HANDLE hFile, PDWORD lpErrors, LPCOMSTAT lpStat)
+BOOL ClearCommError(HANDLE hFile, WINPR_ATTR_UNUSED PDWORD lpErrors,
+                    WINPR_ATTR_UNUSED LPCOMSTAT lpStat)
 {
 	WINPR_COMM* pComm = (WINPR_COMM*)hFile;
 
@@ -871,7 +953,7 @@ BOOL SetupComm(HANDLE hFile, DWORD dwInQueue, DWORD dwOutQueue)
 	return TRUE;
 }
 
-BOOL EscapeCommFunction(HANDLE hFile, DWORD dwFunc)
+BOOL EscapeCommFunction(HANDLE hFile, WINPR_ATTR_UNUSED DWORD dwFunc)
 {
 	WINPR_COMM* pComm = (WINPR_COMM*)hFile;
 
@@ -888,7 +970,7 @@ BOOL EscapeCommFunction(HANDLE hFile, DWORD dwFunc)
 	return FALSE;
 }
 
-BOOL TransmitCommChar(HANDLE hFile, char cChar)
+BOOL TransmitCommChar(HANDLE hFile, WINPR_ATTR_UNUSED char cChar)
 {
 	WINPR_COMM* pComm = (WINPR_COMM*)hFile;
 
@@ -905,7 +987,8 @@ BOOL TransmitCommChar(HANDLE hFile, char cChar)
 	return FALSE;
 }
 
-BOOL WaitCommEvent(HANDLE hFile, PDWORD lpEvtMask, LPOVERLAPPED lpOverlapped)
+BOOL WaitCommEvent(HANDLE hFile, WINPR_ATTR_UNUSED PDWORD lpEvtMask,
+                   WINPR_ATTR_UNUSED LPOVERLAPPED lpOverlapped)
 {
 	WINPR_COMM* pComm = (WINPR_COMM*)hFile;
 
@@ -1073,15 +1156,16 @@ DWORD QueryCommDevice(LPCTSTR lpDeviceName, LPTSTR lpTargetPath, DWORD ucchMax)
 		return 0;
 	}
 
-	if (_tcslen(storedTargetPath) + 2 > ucchMax)
+	const size_t size = _tcsnlen(storedTargetPath, ucchMax);
+	if (size + 2 > ucchMax)
 	{
 		SetLastError(ERROR_INSUFFICIENT_BUFFER);
 		return 0;
 	}
 
-	_tcscpy(lpTargetPath, storedTargetPath);
-	lpTargetPath[_tcslen(storedTargetPath) + 1] = '\0'; /* 2nd final '\0' */
-	return _tcslen(lpTargetPath) + 2;
+	_tcsncpy(lpTargetPath, storedTargetPath, size + 1);
+	lpTargetPath[size + 2] = '\0'; /* 2nd final '\0' */
+	return (DWORD)size + 2UL;
 }
 
 /**
@@ -1261,8 +1345,10 @@ HANDLE CommCreateFileA(LPCSTR lpDeviceName, DWORD dwDesiredAccess, DWORD dwShare
 		goto error_handle;
 	}
 
+#if defined(WINPR_HAVE_SYS_EVENTFD_H)
 	pComm->fd_read_event = eventfd(
 	    0, EFD_NONBLOCK); /* EFD_NONBLOCK required because a read() is not always expected */
+#endif
 
 	if (pComm->fd_read_event < 0)
 	{
@@ -1281,8 +1367,10 @@ HANDLE CommCreateFileA(LPCSTR lpDeviceName, DWORD dwDesiredAccess, DWORD dwShare
 		goto error_handle;
 	}
 
+#if defined(WINPR_HAVE_SYS_EVENTFD_H)
 	pComm->fd_write_event = eventfd(
 	    0, EFD_NONBLOCK); /* EFD_NONBLOCK required because a read() is not always expected */
+#endif
 
 	if (pComm->fd_write_event < 0)
 	{
@@ -1296,6 +1384,7 @@ HANDLE CommCreateFileA(LPCSTR lpDeviceName, DWORD dwDesiredAccess, DWORD dwShare
 	pComm->serverSerialDriverId = SerialDriverUnknown;
 	InitializeCriticalSection(&pComm->EventsLock);
 
+#if defined(WINPR_HAVE_COMM_COUNTERS)
 	if (ioctl(pComm->fd, TIOCGICOUNT, &(pComm->counters)) < 0)
 	{
 		char ebuffer[256] = { 0 };
@@ -1310,6 +1399,7 @@ HANDLE CommCreateFileA(LPCSTR lpDeviceName, DWORD dwDesiredAccess, DWORD dwShare
 		 */
 		ZeroMemory(&(pComm->counters), sizeof(struct serial_icounter_struct));
 	}
+#endif
 
 	/* The binary/raw mode is required for the redirection but
 	 * only flags that are not handle somewhere-else, except
@@ -1322,8 +1412,8 @@ HANDLE CommCreateFileA(LPCSTR lpDeviceName, DWORD dwDesiredAccess, DWORD dwShare
 		goto error_handle;
 	}
 
-	upcomingTermios.c_iflag &=
-	    ~(/*IGNBRK |*/ BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL /*| IXON*/);
+	upcomingTermios.c_iflag &= WINPR_ASSERTING_INT_CAST(
+	    tcflag_t, ~(/*IGNBRK |*/ BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL /*| IXON*/));
 	upcomingTermios.c_oflag = 0; /* <=> &= ~OPOST */
 	upcomingTermios.c_lflag = 0; /* <=> &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN); */
 	/* upcomingTermios.c_cflag &= ~(CSIZE | PARENB); */
@@ -1345,8 +1435,7 @@ HANDLE CommCreateFileA(LPCSTR lpDeviceName, DWORD dwDesiredAccess, DWORD dwShare
 	return (HANDLE)pComm;
 error_handle:
 	WINPR_PRAGMA_DIAG_PUSH
-	WINPR_PRAGMA_DIAG_IGNORED_MISMATCHED_DEALLOC
-	CloseHandle(pComm);
+	WINPR_PRAGMA_DIAG_IGNORED_MISMATCHED_DEALLOC(void) CloseHandle(pComm);
 	WINPR_PRAGMA_DIAG_POP
 	return INVALID_HANDLE_VALUE;
 }
@@ -1402,6 +1491,7 @@ BOOL CommCloseHandle(HANDLE handle)
 	return TRUE;
 }
 
+#if defined(WINPR_HAVE_SYS_EVENTFD_H)
 #ifndef WITH_EVENTFD_READ_WRITE
 int eventfd_read(int fd, eventfd_t* value)
 {
@@ -1413,3 +1503,4 @@ int eventfd_write(int fd, eventfd_t value)
 	return (write(fd, &value, sizeof(value)) == sizeof(value)) ? 0 : -1;
 }
 #endif
+#endif
