@@ -28,9 +28,7 @@
 #include <unistd.h>
 
 #include "comm_serial_sys.h"
-#ifdef __UCLIBC__
 #include "comm.h"
-#endif
 
 #include <winpr/crt.h>
 #include <winpr/wlog.h>
@@ -156,7 +154,7 @@ static const speed_t BAUD_TABLE[][3] = {
 
 static BOOL commstatus_error(WINPR_COMM* pComm, const char* ctrl);
 
-static BOOL get_properties(WINPR_COMM* pComm, COMMPROP* pProperties)
+static BOOL get_properties(WINPR_ATTR_UNUSED WINPR_COMM* pComm, COMMPROP* pProperties)
 {
 	WINPR_ASSERT(pComm);
 	/* http://msdn.microsoft.com/en-us/library/windows/hardware/jj680684%28v=vs.85%29.aspx
@@ -339,7 +337,7 @@ static BOOL set_serial_chars(WINPR_COMM* pComm, const SERIAL_CHARS* pSerialChars
 		result = FALSE; /* but keep on */
 	}
 
-	/* According the Linux's n_tty discipline, charaters with a
+	/* According the Linux's n_tty discipline, characters with a
 	 * parity error can only be let unchanged, replaced by \0 or
 	 * get the prefix the prefix \377 \0
 	 */
@@ -425,7 +423,7 @@ static BOOL set_line_control(WINPR_COMM* pComm, const SERIAL_LINE_CONTROL* pLine
 	 * The use of 5 data bits with 2 stop bits is an invalid
 	 * combination, as is 6, 7, or 8 data bits with 1.5 stop bits.
 	 *
-	 * FIXME: prefered to let the underlying driver to deal with
+	 * FIXME: preferred to let the underlying driver to deal with
 	 * this issue. At least produce a warning message?
 	 */
 
@@ -440,7 +438,7 @@ static BOOL set_line_control(WINPR_COMM* pComm, const SERIAL_LINE_CONTROL* pLine
 	switch (pLineControl->StopBits)
 	{
 		case STOP_BIT_1:
-			upcomingTermios.c_cflag &= ~CSTOPB;
+			upcomingTermios.c_cflag &= (uint32_t)~CSTOPB;
 			break;
 
 		case STOP_BITS_1_5:
@@ -461,16 +459,16 @@ static BOOL set_line_control(WINPR_COMM* pComm, const SERIAL_LINE_CONTROL* pLine
 	switch (pLineControl->Parity)
 	{
 		case NO_PARITY:
-			upcomingTermios.c_cflag &= ~(PARENB | PARODD | CMSPAR);
+			upcomingTermios.c_cflag &= (uint32_t)~(PARENB | PARODD | CMSPAR);
 			break;
 
 		case ODD_PARITY:
-			upcomingTermios.c_cflag &= ~CMSPAR;
+			upcomingTermios.c_cflag &= (uint32_t)~CMSPAR;
 			upcomingTermios.c_cflag |= PARENB | PARODD;
 			break;
 
 		case EVEN_PARITY:
-			upcomingTermios.c_cflag &= ~(PARODD | CMSPAR);
+			upcomingTermios.c_cflag &= (uint32_t)~(PARODD | CMSPAR);
 			upcomingTermios.c_cflag |= PARENB;
 			break;
 
@@ -479,7 +477,7 @@ static BOOL set_line_control(WINPR_COMM* pComm, const SERIAL_LINE_CONTROL* pLine
 			break;
 
 		case SPACE_PARITY:
-			upcomingTermios.c_cflag &= ~PARODD;
+			upcomingTermios.c_cflag &= (uint32_t)~PARODD;
 			upcomingTermios.c_cflag |= PARENB | CMSPAR;
 			break;
 
@@ -493,22 +491,22 @@ static BOOL set_line_control(WINPR_COMM* pComm, const SERIAL_LINE_CONTROL* pLine
 	switch (pLineControl->WordLength)
 	{
 		case 5:
-			upcomingTermios.c_cflag &= ~CSIZE;
+			upcomingTermios.c_cflag &= (uint32_t)~CSIZE;
 			upcomingTermios.c_cflag |= CS5;
 			break;
 
 		case 6:
-			upcomingTermios.c_cflag &= ~CSIZE;
+			upcomingTermios.c_cflag &= (uint32_t)~CSIZE;
 			upcomingTermios.c_cflag |= CS6;
 			break;
 
 		case 7:
-			upcomingTermios.c_cflag &= ~CSIZE;
+			upcomingTermios.c_cflag &= (uint32_t)~CSIZE;
 			upcomingTermios.c_cflag |= CS7;
 			break;
 
 		case 8:
-			upcomingTermios.c_cflag &= ~CSIZE;
+			upcomingTermios.c_cflag &= (uint32_t)~CSIZE;
 			upcomingTermios.c_cflag |= CS8;
 			break;
 
@@ -613,7 +611,7 @@ static BOOL set_handflow(WINPR_COMM* pComm, const SERIAL_HANDFLOW* pHandflow)
 	}
 	else
 	{
-		upcomingTermios.c_cflag &= ~HUPCL;
+		upcomingTermios.c_cflag &= (uint32_t)~HUPCL;
 
 		/* FIXME: is the DTR line also needs to be forced to a disable state according
 		 * SERIAL_DTR_CONTROL? */
@@ -698,7 +696,7 @@ static BOOL set_handflow(WINPR_COMM* pComm, const SERIAL_HANDFLOW* pHandflow)
 	}
 	else
 	{
-		upcomingTermios.c_iflag &= ~IXON;
+		upcomingTermios.c_iflag &= (uint32_t)~IXON;
 	}
 
 	if (pHandflow->FlowReplace & SERIAL_AUTO_RECEIVE)
@@ -707,14 +705,14 @@ static BOOL set_handflow(WINPR_COMM* pComm, const SERIAL_HANDFLOW* pHandflow)
 	}
 	else
 	{
-		upcomingTermios.c_iflag &= ~IXOFF;
+		upcomingTermios.c_iflag &= (uint32_t)~IXOFF;
 	}
 
 	// FIXME: could be implemented during read/write I/O, as of today ErrorChar is necessary '\0'
 	if (pHandflow->FlowReplace & SERIAL_ERROR_CHAR)
 	{
 		/* errors will be replaced by the character '\0'. */
-		upcomingTermios.c_iflag &= ~IGNPAR;
+		upcomingTermios.c_iflag &= (uint32_t)~IGNPAR;
 	}
 	else
 	{
@@ -727,7 +725,7 @@ static BOOL set_handflow(WINPR_COMM* pComm, const SERIAL_HANDFLOW* pHandflow)
 	}
 	else
 	{
-		upcomingTermios.c_iflag &= ~IGNBRK;
+		upcomingTermios.c_iflag &= (uint32_t)~IGNBRK;
 	}
 
 	// FIXME: could be implemented during read/write I/O
@@ -1075,7 +1073,7 @@ static BOOL set_wait_mask(WINPR_COMM* pComm, const ULONG* pWaitMask)
 			Sleep(10); /* 10ms */
 
 		EnterCriticalSection(&pComm->EventsLock);
-		pComm->PendingEvents &= ~SERIAL_EV_WINPR_STOP;
+		pComm->PendingEvents &= (uint32_t)~SERIAL_EV_WINPR_STOP;
 		LeaveCriticalSection(&pComm->EventsLock);
 	}
 
@@ -1085,7 +1083,7 @@ static BOOL set_wait_mask(WINPR_COMM* pComm, const ULONG* pWaitMask)
 	if (*pWaitMask == 0)
 	{
 		/* clearing pending events */
-
+#if defined(WINPR_HAVE_COMM_COUNTERS)
 		if (ioctl(pComm->fd, TIOCGICOUNT, &(pComm->counters)) < 0)
 		{
 			if (!commstatus_error(pComm, "TIOCGICOUNT"))
@@ -1095,7 +1093,7 @@ static BOOL set_wait_mask(WINPR_COMM* pComm, const ULONG* pWaitMask)
 			}
 			ZeroMemory(&(pComm->counters), sizeof(struct serial_icounter_struct));
 		}
-
+#endif
 		pComm->PendingEvents = 0;
 	}
 
@@ -1130,7 +1128,7 @@ static BOOL get_wait_mask(WINPR_COMM* pComm, ULONG* pWaitMask)
 	return TRUE;
 }
 
-static BOOL set_queue_size(WINPR_COMM* pComm, const SERIAL_QUEUE_SIZE* pQueueSize)
+static BOOL set_queue_size(WINPR_ATTR_UNUSED WINPR_COMM* pComm, const SERIAL_QUEUE_SIZE* pQueueSize)
 {
 	WINPR_ASSERT(pComm);
 	WINPR_ASSERT(pQueueSize);
@@ -1161,8 +1159,8 @@ static BOOL purge(WINPR_COMM* pComm, const ULONG* pPurgeMask)
 	WINPR_ASSERT(pComm);
 	WINPR_ASSERT(pPurgeMask);
 
-	if ((*pPurgeMask & ~(SERIAL_PURGE_TXABORT | SERIAL_PURGE_RXABORT | SERIAL_PURGE_TXCLEAR |
-	                     SERIAL_PURGE_RXCLEAR)) > 0)
+	if ((*pPurgeMask & (uint32_t)~(SERIAL_PURGE_TXABORT | SERIAL_PURGE_RXABORT |
+	                               SERIAL_PURGE_TXCLEAR | SERIAL_PURGE_RXCLEAR)) > 0)
 	{
 		CommLog_Print(WLOG_WARN, "Invalid purge mask: 0x%" PRIX32 "\n", *pPurgeMask);
 		SetLastError(ERROR_INVALID_PARAMETER);
@@ -1177,7 +1175,7 @@ static BOOL purge(WINPR_COMM* pComm, const ULONG* pPurgeMask)
 	if (*pPurgeMask & SERIAL_PURGE_TXABORT)
 	{
 		/* Purges all write (IRP_MJ_WRITE) requests. */
-
+#if defined(WINPR_HAVE_SYS_EVENTFD_H)
 		if (eventfd_write(pComm->fd_write_event, WINPR_PURGE_TXABORT) < 0)
 		{
 			if (errno != EAGAIN)
@@ -1189,12 +1187,13 @@ static BOOL purge(WINPR_COMM* pComm, const ULONG* pPurgeMask)
 
 			WINPR_ASSERT(errno == EAGAIN); /* no reader <=> no pending IRP_MJ_WRITE */
 		}
+#endif
 	}
 
 	if (*pPurgeMask & SERIAL_PURGE_RXABORT)
 	{
 		/* Purges all read (IRP_MJ_READ) requests. */
-
+#if defined(WINPR_HAVE_SYS_EVENTFD_H)
 		if (eventfd_write(pComm->fd_read_event, WINPR_PURGE_RXABORT) < 0)
 		{
 			if (errno != EAGAIN)
@@ -1206,6 +1205,7 @@ static BOOL purge(WINPR_COMM* pComm, const ULONG* pPurgeMask)
 
 			WINPR_ASSERT(errno == EAGAIN); /* no reader <=> no pending IRP_MJ_READ */
 		}
+#endif
 	}
 
 	if (*pPurgeMask & SERIAL_PURGE_TXCLEAR)
@@ -1261,9 +1261,9 @@ static BOOL get_commstatus(WINPR_COMM* pComm, SERIAL_STATUS* pCommstatus)
 {
 	BOOL rc = FALSE;
 	/* http://msdn.microsoft.com/en-us/library/jj673022%28v=vs.85%29.aspx */
-
+#if defined(WINPR_HAVE_COMM_COUNTERS)
 	struct serial_icounter_struct currentCounters = { 0 };
-
+#endif
 	WINPR_ASSERT(pComm);
 	WINPR_ASSERT(pCommstatus);
 
@@ -1284,6 +1284,7 @@ static BOOL get_commstatus(WINPR_COMM* pComm, SERIAL_STATUS* pCommstatus)
 		status = 0;
 	}
 
+#if defined(WINPR_HAVE_COMM_COUNTERS)
 	if (ioctl(pComm->fd, TIOCGICOUNT, &currentCounters) < 0)
 	{
 		if (!commstatus_error(pComm, "TIOCGICOUNT"))
@@ -1324,6 +1325,7 @@ static BOOL get_commstatus(WINPR_COMM* pComm, SERIAL_STATUS* pCommstatus)
 		pCommstatus->Errors |= SERIAL_ERROR_FRAMING;
 		pComm->PendingEvents |= SERIAL_EV_ERR;
 	}
+#endif
 
 	/* HoldReasons */
 
@@ -1341,11 +1343,13 @@ static BOOL get_commstatus(WINPR_COMM* pComm, SERIAL_STATUS* pCommstatus)
 
 	/* AmountInInQueue */
 
+#if defined(__linux__)
 	if (ioctl(pComm->fd, TIOCINQ, &(pCommstatus->AmountInInQueue)) < 0)
 	{
 		if (!commstatus_error(pComm, "TIOCINQ"))
 			goto fail;
 	}
+#endif
 
 	/*  AmountInOutQueue */
 
@@ -1360,14 +1364,14 @@ static BOOL get_commstatus(WINPR_COMM* pComm, SERIAL_STATUS* pCommstatus)
 	/*  BOOLEAN WaitForImmediate; TODO: once IOCTL_SERIAL_IMMEDIATE_CHAR fully supported */
 
 	/* other events based on counters */
-
+#if defined(WINPR_HAVE_COMM_COUNTERS)
 	if (currentCounters.rx != pComm->counters.rx)
 	{
 		pComm->PendingEvents |= SERIAL_EV_RXFLAG | SERIAL_EV_RXCHAR;
 	}
 
 	if ((currentCounters.tx != pComm->counters.tx) && /* at least a transmission occurred AND ...*/
-	    (pCommstatus->AmountInOutQueue == 0))         /* output bufer is now empty */
+	    (pCommstatus->AmountInOutQueue == 0))         /* output buffer is now empty */
 	{
 		pComm->PendingEvents |= SERIAL_EV_TXEMPTY;
 	}
@@ -1375,7 +1379,7 @@ static BOOL get_commstatus(WINPR_COMM* pComm, SERIAL_STATUS* pCommstatus)
 	{
 		/* FIXME: "now empty" from the specs is ambiguous, need to track previous completed
 		 * transmission? */
-		pComm->PendingEvents &= ~SERIAL_EV_TXEMPTY;
+		pComm->PendingEvents &= (uint32_t)~SERIAL_EV_TXEMPTY;
 	}
 
 	if (currentCounters.cts != pComm->counters.cts)
@@ -1398,6 +1402,9 @@ static BOOL get_commstatus(WINPR_COMM* pComm, SERIAL_STATUS* pCommstatus)
 		pComm->PendingEvents |= SERIAL_EV_RING;
 	}
 
+	pComm->counters = currentCounters;
+#endif
+
 	if (pCommstatus->AmountInInQueue > (0.8 * N_TTY_BUF_SIZE))
 	{
 		pComm->PendingEvents |= SERIAL_EV_RX80FULL;
@@ -1406,11 +1413,9 @@ static BOOL get_commstatus(WINPR_COMM* pComm, SERIAL_STATUS* pCommstatus)
 	{
 		/* FIXME: "is 80 percent full" from the specs is ambiguous, need to track when it previously
 		 * * occurred? */
-		pComm->PendingEvents &= ~SERIAL_EV_RX80FULL;
+		pComm->PendingEvents &= (uint32_t)~SERIAL_EV_RX80FULL;
 	}
 
-	pComm->counters = currentCounters;
-
 	rc = TRUE;
 fail:
 	LeaveCriticalSection(&pComm->EventsLock);
@@ -1447,7 +1452,7 @@ static void consume_event(WINPR_COMM* pComm, ULONG* pOutputMask, ULONG event)
 static BOOL unlock_return(WINPR_COMM* pComm, BOOL res)
 {
 	EnterCriticalSection(&pComm->EventsLock);
-	pComm->PendingEvents &= ~SERIAL_EV_WINPR_WAITING;
+	pComm->PendingEvents &= (uint32_t)~SERIAL_EV_WINPR_WAITING;
 	LeaveCriticalSection(&pComm->EventsLock);
 	return res;
 }
@@ -1483,7 +1488,7 @@ static BOOL wait_on_mask(WINPR_COMM* pComm, ULONG* pOutputMask)
 			WINPR_ASSERT(*pOutputMask == 0);
 
 			LeaveCriticalSection(&pComm->EventsLock);
-			return unlock_return(pComm, TRUE);
+			break;
 		}
 
 		consume_event(pComm, pOutputMask, SERIAL_EV_RXCHAR);
@@ -1503,7 +1508,7 @@ static BOOL wait_on_mask(WINPR_COMM* pComm, ULONG* pOutputMask)
 		 * not pOutputMask */
 
 		if (*pOutputMask != 0)
-			return unlock_return(pComm, TRUE);
+			break;
 
 		/* waiting for a modification of PendingEvents.
 		 *
@@ -1516,7 +1521,7 @@ static BOOL wait_on_mask(WINPR_COMM* pComm, ULONG* pOutputMask)
 		Sleep(100); /* 100 ms */
 	}
 
-	return unlock_return(pComm, FALSE);
+	return unlock_return(pComm, TRUE);
 }
 
 static BOOL set_break_on(WINPR_COMM* pComm)
@@ -1552,6 +1557,7 @@ static BOOL set_break_off(WINPR_COMM* pComm)
 static BOOL set_xoff(WINPR_COMM* pComm)
 {
 	WINPR_ASSERT(pComm);
+	// NOLINTNEXTLINE(concurrency-mt-unsafe)
 	if (tcflow(pComm->fd, TCIOFF) < 0)
 	{
 		char ebuffer[256] = { 0 };
@@ -1567,6 +1573,7 @@ static BOOL set_xoff(WINPR_COMM* pComm)
 static BOOL set_xon(WINPR_COMM* pComm)
 {
 	WINPR_ASSERT(pComm);
+	// NOLINTNEXTLINE(concurrency-mt-unsafe)
 	if (tcflow(pComm->fd, TCION) < 0)
 	{
 		char ebuffer[256] = { 0 };
@@ -1599,7 +1606,7 @@ static BOOL get_dtrrts(WINPR_COMM* pComm, ULONG* pMask)
 	return TRUE;
 }
 
-static BOOL config_size(WINPR_COMM* pComm, ULONG* pSize)
+static BOOL config_size(WINPR_ATTR_UNUSED WINPR_COMM* pComm, ULONG* pSize)
 {
 	WINPR_ASSERT(pComm);
 	WINPR_ASSERT(pSize);
@@ -1615,7 +1622,7 @@ static BOOL config_size(WINPR_COMM* pComm, ULONG* pSize)
 static BOOL immediate_char(WINPR_COMM* pComm, const UCHAR* pChar)
 {
 	BOOL result = 0;
-	DWORD nbBytesWritten = -1;
+	DWORD nbBytesWritten = 0;
 
 	WINPR_ASSERT(pComm);
 	WINPR_ASSERT(pChar);
@@ -1633,13 +1640,13 @@ static BOOL immediate_char(WINPR_COMM* pComm, const UCHAR* pChar)
 	return result;
 }
 
-static BOOL reset_device(WINPR_COMM* pComm)
+static BOOL reset_device(WINPR_ATTR_UNUSED WINPR_COMM* pComm)
 {
 	/* http://msdn.microsoft.com/en-us/library/dn265347%28v=vs.85%29.aspx */
 	return TRUE;
 }
 
-static SERIAL_DRIVER SerialSys = {
+static const SERIAL_DRIVER SerialSys = {
 	.id = SerialDriverSerialSys,
 	.name = _T("Serial.sys"),
 	.set_baud_rate = set_baud_rate,
@@ -1674,7 +1681,7 @@ static SERIAL_DRIVER SerialSys = {
 	.reset_device = reset_device,
 };
 
-SERIAL_DRIVER* SerialSys_s(void)
+const SERIAL_DRIVER* SerialSys_s(void)
 {
 	return &SerialSys;
 }
