@@ -47,6 +47,19 @@
 #endif
 #endif
 
+struct winpr_cipher_ctx_private_st
+{
+	WINPR_CIPHER_TYPE cipher;
+	WINPR_CRYPTO_OPERATION op;
+
+#ifdef WITH_OPENSSL
+	EVP_CIPHER_CTX* ectx;
+#endif
+#ifdef WITH_MBEDTLS
+	mbedtls_cipher_context_t* mctx;
+#endif
+};
+
 /**
  * RC4
  */
@@ -172,8 +185,100 @@ extern const EVP_MD* winpr_openssl_get_evp_md(WINPR_MD_TYPE md);
 extern mbedtls_md_type_t winpr_mbedtls_get_md_type(int md);
 #endif
 
+struct cipher_map
+{
+	WINPR_CIPHER_TYPE md;
+	const char* name;
+};
+static const struct cipher_map s_cipher_map[] = {
+	{ WINPR_CIPHER_NONE, "none" },
+	{ WINPR_CIPHER_NULL, "null" },
+	{ WINPR_CIPHER_AES_128_ECB, "aes-128-ecb" },
+	{ WINPR_CIPHER_AES_192_ECB, "aes-192-ecb" },
+	{ WINPR_CIPHER_AES_256_ECB, "aes-256-ecb" },
+	{ WINPR_CIPHER_AES_128_CBC, "aes-128-cbc" },
+	{ WINPR_CIPHER_AES_192_CBC, "aes-192-cbc" },
+	{ WINPR_CIPHER_AES_256_CBC, "aes-256-cbc" },
+	{ WINPR_CIPHER_AES_128_CFB128, "aes-128-cfb128" },
+	{ WINPR_CIPHER_AES_192_CFB128, "aes-192-cfb128" },
+	{ WINPR_CIPHER_AES_256_CFB128, "aes-256-cfb128" },
+	{ WINPR_CIPHER_AES_128_CTR, "aes-128-ctr" },
+	{ WINPR_CIPHER_AES_192_CTR, "aes-192-ctr" },
+	{ WINPR_CIPHER_AES_256_CTR, "aes-256-ctr" },
+	{ WINPR_CIPHER_AES_128_GCM, "aes-128-gcm" },
+	{ WINPR_CIPHER_AES_192_GCM, "aes-192-gcm" },
+	{ WINPR_CIPHER_AES_256_GCM, "aes-256-gcm" },
+	{ WINPR_CIPHER_CAMELLIA_128_ECB, "camellia-128-ecb" },
+	{ WINPR_CIPHER_CAMELLIA_192_ECB, "camellia-192-ecb" },
+	{ WINPR_CIPHER_CAMELLIA_256_ECB, "camellia-256-ecb" },
+	{ WINPR_CIPHER_CAMELLIA_128_CBC, "camellia-128-cbc" },
+	{ WINPR_CIPHER_CAMELLIA_192_CBC, "camellia-192-cbc" },
+	{ WINPR_CIPHER_CAMELLIA_256_CBC, "camellia-256-cbc" },
+	{ WINPR_CIPHER_CAMELLIA_128_CFB128, "camellia-128-cfb128" },
+	{ WINPR_CIPHER_CAMELLIA_192_CFB128, "camellia-192-cfb128" },
+	{ WINPR_CIPHER_CAMELLIA_256_CFB128, "camellia-256-cfb128" },
+	{ WINPR_CIPHER_CAMELLIA_128_CTR, "camellia-128-ctr" },
+	{ WINPR_CIPHER_CAMELLIA_192_CTR, "camellia-192-ctr" },
+	{ WINPR_CIPHER_CAMELLIA_256_CTR, "camellia-256-ctr" },
+	{ WINPR_CIPHER_CAMELLIA_128_GCM, "camellia-128-gcm" },
+	{ WINPR_CIPHER_CAMELLIA_192_GCM, "camellia-192-gcm" },
+	{ WINPR_CIPHER_CAMELLIA_256_GCM, "camellia-256-gcm" },
+	{ WINPR_CIPHER_DES_ECB, "des-ecb" },
+	{ WINPR_CIPHER_DES_CBC, "des-cbc" },
+	{ WINPR_CIPHER_DES_EDE_ECB, "des-ede-ecb" },
+	{ WINPR_CIPHER_DES_EDE_CBC, "des-ede-cbc" },
+	{ WINPR_CIPHER_DES_EDE3_ECB, "des-ede3-ecb" },
+	{ WINPR_CIPHER_DES_EDE3_CBC, "des-ede3-cbc" },
+	{ WINPR_CIPHER_BLOWFISH_ECB, "blowfish-ecb" },
+	{ WINPR_CIPHER_BLOWFISH_CBC, "blowfish-cbc" },
+	{ WINPR_CIPHER_BLOWFISH_CFB64, "blowfish-cfb64" },
+	{ WINPR_CIPHER_BLOWFISH_CTR, "blowfish-ctr" },
+	{ WINPR_CIPHER_ARC4_128, "rc4" },
+	{ WINPR_CIPHER_AES_128_CCM, "aes-128-ccm" },
+	{ WINPR_CIPHER_AES_192_CCM, "aes-192-ccm" },
+	{ WINPR_CIPHER_AES_256_CCM, "aes-256-ccm" },
+	{ WINPR_CIPHER_CAMELLIA_128_CCM, "camellia-128-ccm" },
+	{ WINPR_CIPHER_CAMELLIA_192_CCM, "camellia-192-ccm" },
+	{ WINPR_CIPHER_CAMELLIA_256_CCM, "camellia-256-ccm" },
+};
+
+static int cipher_compare(const void* a, const void* b)
+{
+	const WINPR_CIPHER_TYPE* cipher = a;
+	const struct cipher_map* map = b;
+	if (*cipher == map->md)
+		return 0;
+	return *cipher > map->md ? 1 : -1;
+}
+
+const char* winpr_cipher_type_to_string(WINPR_CIPHER_TYPE md)
+{
+	WINPR_CIPHER_TYPE lc = md;
+	const struct cipher_map* ret = bsearch(&lc, s_cipher_map, ARRAYSIZE(s_cipher_map),
+	                                       sizeof(struct cipher_map), cipher_compare);
+	if (!ret)
+		return "unknown";
+	return ret->name;
+}
+
+static int cipher_string_compare(const void* a, const void* b)
+{
+	const char* cipher = a;
+	const struct cipher_map* map = b;
+	return strcmp(cipher, map->name);
+}
+
+WINPR_CIPHER_TYPE winpr_cipher_type_from_string(const char* name)
+{
+	const struct cipher_map* ret = bsearch(name, s_cipher_map, ARRAYSIZE(s_cipher_map),
+	                                       sizeof(struct cipher_map), cipher_string_compare);
+	if (!ret)
+		return WINPR_CIPHER_NONE;
+	return ret->md;
+}
+
 #if defined(WITH_OPENSSL)
-static const EVP_CIPHER* winpr_openssl_get_evp_cipher(int cipher)
+static const EVP_CIPHER* winpr_openssl_get_evp_cipher(WINPR_CIPHER_TYPE cipher)
 {
 	const EVP_CIPHER* evp = NULL;
 
@@ -320,11 +425,11 @@ static const EVP_CIPHER* winpr_openssl_get_evp_cipher(int cipher)
 			break;
 
 		case WINPR_CIPHER_CAMELLIA_192_CCM:
-			evp = EVP_get_cipherbyname("camellia-192-gcm");
+			evp = EVP_get_cipherbyname("camellia-192-ccm");
 			break;
 
 		case WINPR_CIPHER_CAMELLIA_256_CCM:
-			evp = EVP_get_cipherbyname("camellia-256-gcm");
+			evp = EVP_get_cipherbyname("camellia-256-ccm");
 			break;
 
 		case WINPR_CIPHER_DES_ECB:
@@ -370,6 +475,8 @@ static const EVP_CIPHER* winpr_openssl_get_evp_cipher(int cipher)
 		case WINPR_CIPHER_BLOWFISH_CTR:
 			evp = EVP_get_cipherbyname("blowfish-ctr");
 			break;
+		default:
+			break;
 	}
 
 	return evp;
@@ -467,71 +574,98 @@ mbedtls_cipher_type_t winpr_mbedtls_get_cipher_type(int cipher)
 }
 #endif
 
-WINPR_CIPHER_CTX* winpr_Cipher_New(int cipher, int op, const void* key, const void* iv)
+WINPR_CIPHER_CTX* winpr_Cipher_New(WINPR_CIPHER_TYPE cipher, WINPR_CRYPTO_OPERATION op,
+                                   const void* key, const void* iv)
 {
-	WINPR_CIPHER_CTX* ctx = NULL;
-#if defined(WITH_OPENSSL)
-	int operation = 0;
-	const EVP_CIPHER* evp = NULL;
-	EVP_CIPHER_CTX* octx = NULL;
+	return winpr_Cipher_NewEx(cipher, op, key, 0, iv, 0);
+}
 
-	if (!(evp = winpr_openssl_get_evp_cipher(cipher)))
-		return NULL;
-
-	if (!(octx = EVP_CIPHER_CTX_new()))
-		return NULL;
-
-	operation = (op == WINPR_ENCRYPT) ? 1 : 0;
-
-	if (EVP_CipherInit_ex(octx, evp, NULL, key, iv, operation) != 1)
+WINPR_API WINPR_CIPHER_CTX* winpr_Cipher_NewEx(WINPR_CIPHER_TYPE cipher, WINPR_CRYPTO_OPERATION op,
+                                               const void* key, WINPR_ATTR_UNUSED size_t keylen,
+                                               const void* iv, WINPR_ATTR_UNUSED size_t ivlen)
+{
+	if (cipher == WINPR_CIPHER_ARC4_128)
 	{
-		EVP_CIPHER_CTX_free(octx);
+		WLog_ERR(TAG,
+		         "WINPR_CIPHER_ARC4_128 (RC4) cipher not supported, use winpr_RC4_new instead");
 		return NULL;
 	}
 
-	EVP_CIPHER_CTX_set_padding(octx, 0);
-	ctx = (WINPR_CIPHER_CTX*)octx;
+	WINPR_CIPHER_CTX* ctx = calloc(1, sizeof(WINPR_CIPHER_CTX));
+	if (!ctx)
+		return NULL;
+
+	ctx->cipher = cipher;
+	ctx->op = op;
+
+#if defined(WITH_OPENSSL)
+	const EVP_CIPHER* evp = winpr_openssl_get_evp_cipher(cipher);
+	if (!evp)
+		goto fail;
+
+	ctx->ectx = EVP_CIPHER_CTX_new();
+	if (!ctx->ectx)
+		goto fail;
+
+#if 0
+	if (keylen != 0)
+	{
+		WINPR_ASSERT(keylen <= INT32_MAX);
+		const int len = EVP_CIPHER_CTX_key_length(ctx->ectx);
+		if ((len > 0) && (len != keylen))
+		{
+			if (EVP_CIPHER_CTX_set_key_length(ctx->ectx, (int)keylen) != 1)
+				goto fail;
+		}
+	}
+
+	if (ivlen != 0)
+	{
+		WINPR_ASSERT(ivlen <= INT32_MAX);
+		const int len = EVP_CIPHER_CTX_iv_length(ctx->ectx);
+		if ((len > 0) && (ivlen != len))
+			goto fail;
+	}
+#endif
+
+	const int operation = (op == WINPR_ENCRYPT) ? 1 : 0;
+
+	if (EVP_CipherInit_ex(ctx->ectx, evp, NULL, key, iv, operation) != 1)
+		goto fail;
+
+	EVP_CIPHER_CTX_set_padding(ctx->ectx, 0);
+
 #elif defined(WITH_MBEDTLS)
-	int key_bitlen;
-	mbedtls_operation_t operation;
-	mbedtls_cipher_context_t* mctx;
 	mbedtls_cipher_type_t cipher_type = winpr_mbedtls_get_cipher_type(cipher);
 	const mbedtls_cipher_info_t* cipher_info = mbedtls_cipher_info_from_type(cipher_type);
 
 	if (!cipher_info)
-		return NULL;
+		goto fail;
 
-	if (!(mctx = (mbedtls_cipher_context_t*)calloc(1, sizeof(mbedtls_cipher_context_t))))
-		return NULL;
+	ctx->mctx = calloc(1, sizeof(mbedtls_cipher_context_t));
+	if (!ctx->mctx)
+		goto fail;
 
-	operation = (op == WINPR_ENCRYPT) ? MBEDTLS_ENCRYPT : MBEDTLS_DECRYPT;
-	mbedtls_cipher_init(mctx);
+	const mbedtls_operation_t operation = (op == WINPR_ENCRYPT) ? MBEDTLS_ENCRYPT : MBEDTLS_DECRYPT;
+	mbedtls_cipher_init(ctx->mctx);
 
-	if (mbedtls_cipher_setup(mctx, cipher_info) != 0)
-	{
-		free(mctx);
-		return NULL;
-	}
+	if (mbedtls_cipher_setup(ctx->mctx, cipher_info) != 0)
+		goto fail;
 
-	key_bitlen = mbedtls_cipher_get_key_bitlen(mctx);
+	const int key_bitlen = mbedtls_cipher_get_key_bitlen(ctx->mctx);
 
-	if (mbedtls_cipher_setkey(mctx, key, key_bitlen, operation) != 0)
-	{
-		mbedtls_cipher_free(mctx);
-		free(mctx);
-		return NULL;
-	}
+	if (mbedtls_cipher_setkey(ctx->mctx, key, key_bitlen, operation) != 0)
+		goto fail;
 
-	if (mbedtls_cipher_set_padding_mode(mctx, MBEDTLS_PADDING_NONE) != 0)
-	{
-		mbedtls_cipher_free(mctx);
-		free(mctx);
-		return NULL;
-	}
+	if (mbedtls_cipher_set_padding_mode(ctx->mctx, MBEDTLS_PADDING_NONE) != 0)
+		goto fail;
 
-	ctx = (WINPR_CIPHER_CTX*)mctx;
 #endif
 	return ctx;
+
+fail:
+	winpr_Cipher_Free(ctx);
+	return NULL;
 }
 
 BOOL winpr_Cipher_SetPadding(WINPR_CIPHER_CTX* ctx, BOOL enabled)
@@ -539,7 +673,9 @@ BOOL winpr_Cipher_SetPadding(WINPR_CIPHER_CTX* ctx, BOOL enabled)
 	WINPR_ASSERT(ctx);
 
 #if defined(WITH_OPENSSL)
-	EVP_CIPHER_CTX_set_padding((EVP_CIPHER_CTX*)ctx, enabled);
+	if (!ctx->ectx)
+		return FALSE;
+	EVP_CIPHER_CTX_set_padding(ctx->ectx, enabled);
 #elif defined(WITH_MBEDTLS)
 	mbedtls_cipher_padding_t option = enabled ? MBEDTLS_PADDING_PKCS7 : MBEDTLS_PADDING_NONE;
 	if (mbedtls_cipher_set_padding_mode((mbedtls_cipher_context_t*)ctx, option) != 0)
@@ -553,6 +689,9 @@ BOOL winpr_Cipher_SetPadding(WINPR_CIPHER_CTX* ctx, BOOL enabled)
 BOOL winpr_Cipher_Update(WINPR_CIPHER_CTX* ctx, const void* input, size_t ilen, void* output,
                          size_t* olen)
 {
+	WINPR_ASSERT(ctx);
+	WINPR_ASSERT(olen);
+
 #if defined(WITH_OPENSSL)
 	int outl = (int)*olen;
 
@@ -562,16 +701,16 @@ BOOL winpr_Cipher_Update(WINPR_CIPHER_CTX* ctx, const void* input, size_t ilen,
 		return FALSE;
 	}
 
-	WINPR_ASSERT(ctx);
-	if (EVP_CipherUpdate((EVP_CIPHER_CTX*)ctx, output, &outl, input, (int)ilen) == 1)
+	WINPR_ASSERT(ctx->ectx);
+	if (EVP_CipherUpdate(ctx->ectx, output, &outl, input, (int)ilen) == 1)
 	{
 		*olen = (size_t)outl;
 		return TRUE;
 	}
 
 #elif defined(WITH_MBEDTLS)
-
-	if (mbedtls_cipher_update((mbedtls_cipher_context_t*)ctx, input, ilen, output, olen) == 0)
+	WINPR_ASSERT(ctx->mctx);
+	if (mbedtls_cipher_update(ctx->mctx, input, ilen, output, olen) == 0)
 		return TRUE;
 
 #endif
@@ -582,10 +721,13 @@ BOOL winpr_Cipher_Update(WINPR_CIPHER_CTX* ctx, const void* input, size_t ilen,
 
 BOOL winpr_Cipher_Final(WINPR_CIPHER_CTX* ctx, void* output, size_t* olen)
 {
+	WINPR_ASSERT(ctx);
+
 #if defined(WITH_OPENSSL)
 	int outl = (int)*olen;
 
-	if (EVP_CipherFinal_ex((EVP_CIPHER_CTX*)ctx, output, &outl) == 1)
+	WINPR_ASSERT(ctx->ectx);
+	if (EVP_CipherFinal_ex(ctx->ectx, output, &outl) == 1)
 	{
 		*olen = (size_t)outl;
 		return TRUE;
@@ -593,10 +735,12 @@ BOOL winpr_Cipher_Final(WINPR_CIPHER_CTX* ctx, void* output, size_t* olen)
 
 #elif defined(WITH_MBEDTLS)
 
-	if (mbedtls_cipher_finish((mbedtls_cipher_context_t*)ctx, output, olen) == 0)
+	WINPR_ASSERT(ctx->mctx);
+	if (mbedtls_cipher_finish(ctx->mctx, output, olen) == 0)
 		return TRUE;
 
 #endif
+
 	return FALSE;
 }
 
@@ -606,11 +750,17 @@ void winpr_Cipher_Free(WINPR_CIPHER_CTX* ctx)
 		return;
 
 #if defined(WITH_OPENSSL)
-	EVP_CIPHER_CTX_free((EVP_CIPHER_CTX*)ctx);
+	if (ctx->ectx)
+		EVP_CIPHER_CTX_free(ctx->ectx);
 #elif defined(WITH_MBEDTLS)
-	mbedtls_cipher_free((mbedtls_cipher_context_t*)ctx);
-	free(ctx);
+	if (ctx->mctx)
+	{
+		mbedtls_cipher_free(ctx->mctx);
+		free(ctx->mctx);
+	}
 #endif
+
+	free(ctx);
 }
 
 /**
@@ -628,8 +778,10 @@ int winpr_Cipher_BytesToKey(int cipher, WINPR_MD_TYPE md, const void* salt, cons
 	const EVP_MD* evp_md = NULL;
 	const EVP_CIPHER* evp_cipher = NULL;
 	evp_md = winpr_openssl_get_evp_md(md);
-	evp_cipher = winpr_openssl_get_evp_cipher(cipher);
-	return EVP_BytesToKey(evp_cipher, evp_md, salt, data, datal, count, key, iv);
+	evp_cipher = winpr_openssl_get_evp_cipher(WINPR_ASSERTING_INT_CAST(WINPR_CIPHER_TYPE, cipher));
+	WINPR_ASSERT(datal <= INT_MAX);
+	WINPR_ASSERT(count <= INT_MAX);
+	return EVP_BytesToKey(evp_cipher, evp_md, salt, data, (int)datal, (int)count, key, iv);
 #elif defined(WITH_MBEDTLS)
 	int rv = 0;
 	BYTE md_buf[64];
@@ -694,7 +846,7 @@ int winpr_Cipher_BytesToKey(int cipher, WINPR_MD_TYPE md, const void* salt, cons
 				goto err;
 		}
 
-		i = 0;
+		unsigned int i = 0;
 
 		if (nkey)
 		{
