@@ -139,6 +139,8 @@ static BOOL PipeRead(PVOID Object, LPVOID lpBuffer, DWORD nNumberOfBytesToRead,
 			case EWOULDBLOCK:
 				SetLastError(ERROR_NO_DATA);
 				break;
+			default:
+				break;
 		}
 	}
 
@@ -330,8 +332,8 @@ BOOL NamedPipeRead(PVOID Object, LPVOID lpBuffer, DWORD nNumberOfBytesToRead,
 		/* synchronous behavior */
 		lpOverlapped->Internal = 0;
 		lpOverlapped->InternalHigh = (ULONG_PTR)nNumberOfBytesToRead;
-		lpOverlapped->Pointer = (PVOID)lpBuffer;
-		SetEvent(lpOverlapped->hEvent);
+		lpOverlapped->DUMMYUNIONNAME.Pointer = (PVOID)lpBuffer;
+		(void)SetEvent(lpOverlapped->hEvent);
 #endif
 	}
 
@@ -424,9 +426,9 @@ BOOL NamedPipeWrite(PVOID Object, LPCVOID lpBuffer, DWORD nNumberOfBytesToWrite,
 				PVOID pv;
 			} cnv;
 			cnv.cpv = lpBuffer;
-			lpOverlapped->Pointer = cnv.pv;
+			lpOverlapped->DUMMYUNIONNAME.Pointer = cnv.pv;
 		}
-		SetEvent(lpOverlapped->hEvent);
+		(void)SetEvent(lpOverlapped->hEvent);
 #endif
 	}
 
@@ -726,9 +728,12 @@ out:
 	return INVALID_HANDLE_VALUE;
 }
 
-HANDLE CreateNamedPipeW(LPCWSTR lpName, DWORD dwOpenMode, DWORD dwPipeMode, DWORD nMaxInstances,
-                        DWORD nOutBufferSize, DWORD nInBufferSize, DWORD nDefaultTimeOut,
-                        LPSECURITY_ATTRIBUTES lpSecurityAttributes)
+HANDLE CreateNamedPipeW(WINPR_ATTR_UNUSED LPCWSTR lpName, WINPR_ATTR_UNUSED DWORD dwOpenMode,
+                        WINPR_ATTR_UNUSED DWORD dwPipeMode, WINPR_ATTR_UNUSED DWORD nMaxInstances,
+                        WINPR_ATTR_UNUSED DWORD nOutBufferSize,
+                        WINPR_ATTR_UNUSED DWORD nInBufferSize,
+                        WINPR_ATTR_UNUSED DWORD nDefaultTimeOut,
+                        WINPR_ATTR_UNUSED LPSECURITY_ATTRIBUTES lpSecurityAttributes)
 {
 	WLog_ERR(TAG, "is not implemented");
 	SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
@@ -780,8 +785,8 @@ BOOL ConnectNamedPipe(HANDLE hNamedPipe, LPOVERLAPPED lpOverlapped)
 		/* synchronous behavior */
 		lpOverlapped->Internal = 2;
 		lpOverlapped->InternalHigh = (ULONG_PTR)0;
-		lpOverlapped->Pointer = (PVOID)NULL;
-		SetEvent(lpOverlapped->hEvent);
+		lpOverlapped->DUMMYUNIONNAME.Pointer = (PVOID)NULL;
+		(void)SetEvent(lpOverlapped->hEvent);
 	}
 
 	return TRUE;
@@ -801,17 +806,21 @@ BOOL DisconnectNamedPipe(HANDLE hNamedPipe)
 	return TRUE;
 }
 
-BOOL PeekNamedPipe(HANDLE hNamedPipe, LPVOID lpBuffer, DWORD nBufferSize, LPDWORD lpBytesRead,
-                   LPDWORD lpTotalBytesAvail, LPDWORD lpBytesLeftThisMessage)
+BOOL PeekNamedPipe(WINPR_ATTR_UNUSED HANDLE hNamedPipe, WINPR_ATTR_UNUSED LPVOID lpBuffer,
+                   WINPR_ATTR_UNUSED DWORD nBufferSize, WINPR_ATTR_UNUSED LPDWORD lpBytesRead,
+                   WINPR_ATTR_UNUSED LPDWORD lpTotalBytesAvail,
+                   WINPR_ATTR_UNUSED LPDWORD lpBytesLeftThisMessage)
 {
 	WLog_ERR(TAG, "Not implemented");
 	SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
 	return FALSE;
 }
 
-BOOL TransactNamedPipe(HANDLE hNamedPipe, LPVOID lpInBuffer, DWORD nInBufferSize,
-                       LPVOID lpOutBuffer, DWORD nOutBufferSize, LPDWORD lpBytesRead,
-                       LPOVERLAPPED lpOverlapped)
+BOOL TransactNamedPipe(WINPR_ATTR_UNUSED HANDLE hNamedPipe, WINPR_ATTR_UNUSED LPVOID lpInBuffer,
+                       WINPR_ATTR_UNUSED DWORD nInBufferSize, WINPR_ATTR_UNUSED LPVOID lpOutBuffer,
+                       WINPR_ATTR_UNUSED DWORD nOutBufferSize,
+                       WINPR_ATTR_UNUSED LPDWORD lpBytesRead,
+                       WINPR_ATTR_UNUSED LPOVERLAPPED lpOverlapped)
 {
 	WLog_ERR(TAG, "Not implemented");
 	SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
@@ -856,15 +865,17 @@ BOOL WaitNamedPipeA(LPCSTR lpNamedPipeName, DWORD nTimeOut)
 	return status;
 }
 
-BOOL WaitNamedPipeW(LPCWSTR lpNamedPipeName, DWORD nTimeOut)
+BOOL WaitNamedPipeW(WINPR_ATTR_UNUSED LPCWSTR lpNamedPipeName, WINPR_ATTR_UNUSED DWORD nTimeOut)
 {
 	WLog_ERR(TAG, "Not implemented");
 	SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
 	return FALSE;
 }
 
+// NOLINTBEGIN(readability-non-const-parameter)
 BOOL SetNamedPipeHandleState(HANDLE hNamedPipe, LPDWORD lpMode, LPDWORD lpMaxCollectionCount,
                              LPDWORD lpCollectDataTimeout)
+// NOLINTEND(readability-non-const-parameter)
 {
 	int fd = 0;
 	int flags = 0;
@@ -904,23 +915,25 @@ BOOL SetNamedPipeHandleState(HANDLE hNamedPipe, LPDWORD lpMode, LPDWORD lpMaxCol
 	return TRUE;
 }
 
-BOOL ImpersonateNamedPipeClient(HANDLE hNamedPipe)
+BOOL ImpersonateNamedPipeClient(WINPR_ATTR_UNUSED HANDLE hNamedPipe)
 {
 	WLog_ERR(TAG, "Not implemented");
 	SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
 	return FALSE;
 }
 
-BOOL GetNamedPipeClientComputerNameA(HANDLE Pipe, LPCSTR ClientComputerName,
-                                     ULONG ClientComputerNameLength)
+BOOL GetNamedPipeClientComputerNameA(WINPR_ATTR_UNUSED HANDLE Pipe,
+                                     WINPR_ATTR_UNUSED LPCSTR ClientComputerName,
+                                     WINPR_ATTR_UNUSED ULONG ClientComputerNameLength)
 {
 	WLog_ERR(TAG, "Not implemented");
 	SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
 	return FALSE;
 }
 
-BOOL GetNamedPipeClientComputerNameW(HANDLE Pipe, LPCWSTR ClientComputerName,
-                                     ULONG ClientComputerNameLength)
+BOOL GetNamedPipeClientComputerNameW(WINPR_ATTR_UNUSED HANDLE Pipe,
+                                     WINPR_ATTR_UNUSED LPCWSTR ClientComputerName,
+                                     WINPR_ATTR_UNUSED ULONG ClientComputerNameLength)
 {
 	WLog_ERR(TAG, "Not implemented");
 	SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
