@@ -104,7 +104,7 @@ static CONTEXT_BUFFER_ALLOC_TABLE ContextBufferAllocTable = { 0 };
 
 static int sspi_ContextBufferAllocTableNew(void)
 {
-	size_t size;
+	size_t size = 0;
 	ContextBufferAllocTable.entries = NULL;
 	ContextBufferAllocTable.cEntries = 0;
 	ContextBufferAllocTable.cMaxEntries = 4;
@@ -119,8 +119,8 @@ static int sspi_ContextBufferAllocTableNew(void)
 
 static int sspi_ContextBufferAllocTableGrow(void)
 {
-	size_t size;
-	CONTEXT_BUFFER_ALLOC_ENTRY* entries;
+	size_t size = 0;
+	CONTEXT_BUFFER_ALLOC_ENTRY* entries = NULL;
 	ContextBufferAllocTable.cEntries = 0;
 	ContextBufferAllocTable.cMaxEntries *= 2;
 	size = sizeof(CONTEXT_BUFFER_ALLOC_ENTRY) * ContextBufferAllocTable.cMaxEntries;
@@ -155,10 +155,9 @@ static void sspi_ContextBufferAllocTableFree(void)
 
 static void* sspi_ContextBufferAlloc(UINT32 allocatorIndex, size_t size)
 {
-	UINT32 index;
-	void* contextBuffer;
+	void* contextBuffer = NULL;
 
-	for (index = 0; index < ContextBufferAllocTable.cMaxEntries; index++)
+	for (UINT32 index = 0; index < ContextBufferAllocTable.cMaxEntries; index++)
 	{
 		if (!ContextBufferAllocTable.entries[index].contextBuffer)
 		{
@@ -185,7 +184,7 @@ static void* sspi_ContextBufferAlloc(UINT32 allocatorIndex, size_t size)
 
 SSPI_CREDENTIALS* sspi_CredentialsNew(void)
 {
-	SSPI_CREDENTIALS* credentials;
+	SSPI_CREDENTIALS* credentials = NULL;
 	credentials = (SSPI_CREDENTIALS*)calloc(1, sizeof(SSPI_CREDENTIALS));
 	return credentials;
 }
@@ -268,7 +267,7 @@ SecHandle* sspi_SecureHandleAlloc(void)
 
 void* sspi_SecureHandleGetLowerPointer(SecHandle* handle)
 {
-	void* pointer;
+	void* pointer = NULL;
 
 	if (!handle || !SecIsValidHandle(handle) || !handle->dwLower)
 		return NULL;
@@ -296,7 +295,7 @@ void sspi_SecureHandleSetLowerPointer(SecHandle* handle, void* pointer)
 
 void* sspi_SecureHandleGetUpperPointer(SecHandle* handle)
 {
-	void* pointer;
+	void* pointer = NULL;
 
 	if (!handle || !SecIsValidHandle(handle) || !handle->dwUpper)
 		return NULL;
@@ -334,11 +333,21 @@ static BOOL copy(WCHAR** dst, ULONG* dstLen, const WCHAR* what, size_t len)
 	*dst = NULL;
 	*dstLen = 0;
 
+	if (len > UINT32_MAX)
+		return FALSE;
+
+	/* Case what="" and len=0 should allocate an empty string */
+	if (!what && (len != 0))
+		return FALSE;
+	if (!what && (len == 0))
+		return TRUE;
+
 	*dst = calloc(sizeof(WCHAR), len + 1);
 	if (!*dst)
 		return FALSE;
+
 	memcpy(*dst, what, len * sizeof(WCHAR));
-	*dstLen = len;
+	*dstLen = WINPR_ASSERTING_INT_CAST(UINT32, len);
 	return TRUE;
 }
 
@@ -348,7 +357,7 @@ int sspi_SetAuthIdentityWithLengthW(SEC_WINNT_AUTH_IDENTITY* identity, const WCH
 {
 	WINPR_ASSERT(identity);
 	sspi_FreeAuthIdentity(identity);
-	identity->Flags &= ~SEC_WINNT_AUTH_IDENTITY_ANSI;
+	identity->Flags &= (uint32_t)~SEC_WINNT_AUTH_IDENTITY_ANSI;
 	identity->Flags |= SEC_WINNT_AUTH_IDENTITY_UNICODE;
 
 	if (!copy(&identity->User, &identity->UserLength, user, userLen))
@@ -373,13 +382,22 @@ static void zfree(WCHAR* str, size_t len)
 int sspi_SetAuthIdentityA(SEC_WINNT_AUTH_IDENTITY* identity, const char* user, const char* domain,
                           const char* password)
 {
-	int rc;
+	int rc = 0;
 	size_t unicodeUserLenW = 0;
 	size_t unicodeDomainLenW = 0;
 	size_t unicodePasswordLenW = 0;
-	LPWSTR unicodeUser = ConvertUtf8ToWCharAlloc(user, &unicodeUserLenW);
-	LPWSTR unicodeDomain = ConvertUtf8ToWCharAlloc(domain, &unicodeDomainLenW);
-	LPWSTR unicodePassword = ConvertUtf8ToWCharAlloc(password, &unicodePasswordLenW);
+	LPWSTR unicodeUser = NULL;
+	LPWSTR unicodeDomain = NULL;
+	LPWSTR unicodePassword = NULL;
+
+	if (user)
+		unicodeUser = ConvertUtf8ToWCharAlloc(user, &unicodeUserLenW);
+
+	if (domain)
+		unicodeDomain = ConvertUtf8ToWCharAlloc(domain, &unicodeDomainLenW);
+
+	if (password)
+		unicodePassword = ConvertUtf8ToWCharAlloc(password, &unicodePasswordLenW);
 
 	rc = sspi_SetAuthIdentityWithLengthW(identity, unicodeUser, unicodeUserLenW, unicodeDomain,
 	                                     unicodeDomainLenW, unicodePassword, unicodePasswordLenW);
@@ -392,7 +410,7 @@ int sspi_SetAuthIdentityA(SEC_WINNT_AUTH_IDENTITY* identity, const char* user, c
 
 UINT32 sspi_GetAuthIdentityVersion(const void* identity)
 {
-	UINT32 version;
+	UINT32 version = 0;
 
 	if (!identity)
 		return 0;
@@ -410,7 +428,7 @@ UINT32 sspi_GetAuthIdentityVersion(const void* identity)
 
 UINT32 sspi_GetAuthIdentityFlags(const void* identity)
 {
-	UINT32 version;
+	UINT32 version = 0;
 	UINT32 flags = 0;
 
 	if (!identity)
@@ -437,7 +455,7 @@ UINT32 sspi_GetAuthIdentityFlags(const void* identity)
 BOOL sspi_GetAuthIdentityUserDomainW(const void* identity, const WCHAR** pUser, UINT32* pUserLength,
                                      const WCHAR** pDomain, UINT32* pDomainLength)
 {
-	UINT32 version;
+	UINT32 version = 0;
 
 	if (!identity)
 		return FALSE;
@@ -477,7 +495,7 @@ BOOL sspi_GetAuthIdentityUserDomainW(const void* identity, const WCHAR** pUser,
 BOOL sspi_GetAuthIdentityUserDomainA(const void* identity, const char** pUser, UINT32* pUserLength,
                                      const char** pDomain, UINT32* pDomainLength)
 {
-	UINT32 version;
+	UINT32 version = 0;
 
 	if (!identity)
 		return FALSE;
@@ -517,7 +535,7 @@ BOOL sspi_GetAuthIdentityUserDomainA(const void* identity, const char** pUser, U
 BOOL sspi_GetAuthIdentityPasswordW(const void* identity, const WCHAR** pPassword,
                                    UINT32* pPasswordLength)
 {
-	UINT32 version;
+	UINT32 version = 0;
 
 	if (!identity)
 		return FALSE;
@@ -547,7 +565,7 @@ BOOL sspi_GetAuthIdentityPasswordW(const void* identity, const WCHAR** pPassword
 BOOL sspi_GetAuthIdentityPasswordA(const void* identity, const char** pPassword,
                                    UINT32* pPasswordLength)
 {
-	UINT32 version;
+	UINT32 version = 0;
 
 	if (!identity)
 		return FALSE;
@@ -767,8 +785,8 @@ cleanup:
 
 BOOL sspi_CopyAuthPackageListA(const SEC_WINNT_AUTH_IDENTITY_INFO* identity, char** pPackageList)
 {
-	UINT32 version;
-	UINT32 identityFlags;
+	UINT32 version = 0;
+	UINT32 identityFlags = 0;
 	char* PackageList = NULL;
 	const char* PackageListA = NULL;
 	const WCHAR* PackageListW = NULL;
@@ -828,8 +846,8 @@ BOOL sspi_CopyAuthPackageListA(const SEC_WINNT_AUTH_IDENTITY_INFO* identity, cha
 int sspi_CopyAuthIdentity(SEC_WINNT_AUTH_IDENTITY* identity,
                           const SEC_WINNT_AUTH_IDENTITY_INFO* srcIdentity)
 {
-	int status;
-	UINT32 identityFlags;
+	int status = 0;
+	UINT32 identityFlags = 0;
 	const char* UserA = NULL;
 	const char* DomainA = NULL;
 	const char* PasswordA = NULL;
@@ -864,7 +882,7 @@ int sspi_CopyAuthIdentity(SEC_WINNT_AUTH_IDENTITY* identity,
 		if (status <= 0)
 			return -1;
 
-		identity->Flags &= ~SEC_WINNT_AUTH_IDENTITY_ANSI;
+		identity->Flags &= (uint32_t)~SEC_WINNT_AUTH_IDENTITY_ANSI;
 		identity->Flags |= SEC_WINNT_AUTH_IDENTITY_UNICODE;
 		return 1;
 	}
@@ -931,10 +949,9 @@ int sspi_CopyAuthIdentity(SEC_WINNT_AUTH_IDENTITY* identity,
 
 PSecBuffer sspi_FindSecBuffer(PSecBufferDesc pMessage, ULONG BufferType)
 {
-	ULONG index;
 	PSecBuffer pSecBuffer = NULL;
 
-	for (index = 0; index < pMessage->cBuffers; index++)
+	for (UINT32 index = 0; index < pMessage->cBuffers; index++)
 	{
 		if (pMessage->pBuffers[index].BufferType == BufferType)
 		{
@@ -951,14 +968,15 @@ static BOOL WINPR_init(void)
 
 	for (size_t x = 0; x < ARRAYSIZE(SecurityFunctionTableA_NAME_LIST); x++)
 	{
-		const SecurityFunctionTableA_NAME* const cur = &SecurityFunctionTableA_NAME_LIST[x];
+		const SecurityFunctionTableA_NAME* cur = &SecurityFunctionTableA_NAME_LIST[x];
 		InitializeConstWCharFromUtf8(cur->Name, BUFFER_NAME_LIST_W[x],
 		                             ARRAYSIZE(BUFFER_NAME_LIST_W[x]));
 	}
 	return TRUE;
 }
 
-static BOOL CALLBACK sspi_init(PINIT_ONCE InitOnce, PVOID Parameter, PVOID* Context)
+static BOOL CALLBACK sspi_init(WINPR_ATTR_UNUSED PINIT_ONCE InitOnce,
+                               WINPR_ATTR_UNUSED PVOID Parameter, WINPR_ATTR_UNUSED PVOID* Context)
 {
 	winpr_InitializeSSL(WINPR_SSL_INIT_DEFAULT);
 	sspi_ContextBufferAllocTableNew();
@@ -989,16 +1007,13 @@ void sspi_GlobalFinish(void)
 
 static const SecurityFunctionTableA* sspi_GetSecurityFunctionTableAByNameA(const SEC_CHAR* Name)
 {
-	int index;
-	UINT32 cPackages;
-	cPackages = sizeof(SecPkgInfoA_LIST) / sizeof(*(SecPkgInfoA_LIST));
+	size_t cPackages = ARRAYSIZE(SecPkgInfoA_LIST);
 
-	for (index = 0; index < (int)cPackages; index++)
+	for (size_t index = 0; index < cPackages; index++)
 	{
 		if (strcmp(Name, SecurityFunctionTableA_NAME_LIST[index].Name) == 0)
 		{
-			return (const SecurityFunctionTableA*)SecurityFunctionTableA_NAME_LIST[index]
-			    .SecurityFunctionTable;
+			return SecurityFunctionTableA_NAME_LIST[index].SecurityFunctionTable;
 		}
 	}
 
@@ -1007,16 +1022,13 @@ static const SecurityFunctionTableA* sspi_GetSecurityFunctionTableAByNameA(const
 
 static const SecurityFunctionTableW* sspi_GetSecurityFunctionTableWByNameW(const SEC_WCHAR* Name)
 {
-	int index;
-	UINT32 cPackages;
-	cPackages = sizeof(SecPkgInfoW_LIST) / sizeof(*(SecPkgInfoW_LIST));
+	size_t cPackages = ARRAYSIZE(SecPkgInfoW_LIST);
 
-	for (index = 0; index < (int)cPackages; index++)
+	for (size_t index = 0; index < cPackages; index++)
 	{
 		if (_wcscmp(Name, SecurityFunctionTableW_NAME_LIST[index].Name) == 0)
 		{
-			return (const SecurityFunctionTableW*)SecurityFunctionTableW_NAME_LIST[index]
-			    .SecurityFunctionTable;
+			return SecurityFunctionTableW_NAME_LIST[index].SecurityFunctionTable;
 		}
 	}
 
@@ -1046,10 +1058,9 @@ static void FreeContextBuffer_QuerySecurityPackageInfo(void* contextBuffer);
 
 static void sspi_ContextBufferFree(void* contextBuffer)
 {
-	UINT32 index;
-	UINT32 allocatorIndex;
+	UINT32 allocatorIndex = 0;
 
-	for (index = 0; index < ContextBufferAllocTable.cMaxEntries; index++)
+	for (size_t index = 0; index < ContextBufferAllocTable.cMaxEntries; index++)
 	{
 		if (contextBuffer == ContextBufferAllocTable.entries[index].contextBuffer)
 		{
@@ -1068,6 +1079,8 @@ static void sspi_ContextBufferFree(void* contextBuffer)
 				case QuerySecurityPackageInfoIndex:
 					FreeContextBuffer_QuerySecurityPackageInfo(contextBuffer);
 					break;
+				default:
+					break;
 			}
 		}
 	}
@@ -1082,18 +1095,17 @@ static void sspi_ContextBufferFree(void* contextBuffer)
 static SECURITY_STATUS SEC_ENTRY winpr_EnumerateSecurityPackagesW(ULONG* pcPackages,
                                                                   PSecPkgInfoW* ppPackageInfo)
 {
-	int index;
-	size_t size;
-	UINT32 cPackages;
-	SecPkgInfoW* pPackageInfo;
-	cPackages = sizeof(SecPkgInfoW_LIST) / sizeof(*(SecPkgInfoW_LIST));
-	size = sizeof(SecPkgInfoW) * cPackages;
-	pPackageInfo = (SecPkgInfoW*)sspi_ContextBufferAlloc(EnumerateSecurityPackagesIndex, size);
+	const size_t cPackages = ARRAYSIZE(SecPkgInfoW_LIST);
+	const size_t size = sizeof(SecPkgInfoW) * cPackages;
+	SecPkgInfoW* pPackageInfo =
+	    (SecPkgInfoW*)sspi_ContextBufferAlloc(EnumerateSecurityPackagesIndex, size);
+
+	WINPR_ASSERT(cPackages <= UINT32_MAX);
 
 	if (!pPackageInfo)
 		return SEC_E_INSUFFICIENT_MEMORY;
 
-	for (index = 0; index < (int)cPackages; index++)
+	for (size_t index = 0; index < cPackages; index++)
 	{
 		pPackageInfo[index].fCapabilities = SecPkgInfoW_LIST[index]->fCapabilities;
 		pPackageInfo[index].wVersion = SecPkgInfoW_LIST[index]->wVersion;
@@ -1103,7 +1115,7 @@ static SECURITY_STATUS SEC_ENTRY winpr_EnumerateSecurityPackagesW(ULONG* pcPacka
 		pPackageInfo[index].Comment = _wcsdup(SecPkgInfoW_LIST[index]->Comment);
 	}
 
-	*(pcPackages) = cPackages;
+	*(pcPackages) = (UINT32)cPackages;
 	*(ppPackageInfo) = pPackageInfo;
 	return SEC_E_OK;
 }
@@ -1111,18 +1123,17 @@ static SECURITY_STATUS SEC_ENTRY winpr_EnumerateSecurityPackagesW(ULONG* pcPacka
 static SECURITY_STATUS SEC_ENTRY winpr_EnumerateSecurityPackagesA(ULONG* pcPackages,
                                                                   PSecPkgInfoA* ppPackageInfo)
 {
-	int index;
-	size_t size;
-	UINT32 cPackages;
-	SecPkgInfoA* pPackageInfo;
-	cPackages = sizeof(SecPkgInfoA_LIST) / sizeof(*(SecPkgInfoA_LIST));
-	size = sizeof(SecPkgInfoA) * cPackages;
-	pPackageInfo = (SecPkgInfoA*)sspi_ContextBufferAlloc(EnumerateSecurityPackagesIndex, size);
+	const size_t cPackages = ARRAYSIZE(SecPkgInfoA_LIST);
+	const size_t size = sizeof(SecPkgInfoA) * cPackages;
+	SecPkgInfoA* pPackageInfo =
+	    (SecPkgInfoA*)sspi_ContextBufferAlloc(EnumerateSecurityPackagesIndex, size);
+
+	WINPR_ASSERT(cPackages <= UINT32_MAX);
 
 	if (!pPackageInfo)
 		return SEC_E_INSUFFICIENT_MEMORY;
 
-	for (index = 0; index < (int)cPackages; index++)
+	for (size_t index = 0; index < cPackages; index++)
 	{
 		pPackageInfo[index].fCapabilities = SecPkgInfoA_LIST[index]->fCapabilities;
 		pPackageInfo[index].wVersion = SecPkgInfoA_LIST[index]->wVersion;
@@ -1138,22 +1149,20 @@ static SECURITY_STATUS SEC_ENTRY winpr_EnumerateSecurityPackagesA(ULONG* pcPacka
 		}
 	}
 
-	*(pcPackages) = cPackages;
+	*(pcPackages) = (UINT32)cPackages;
 	*(ppPackageInfo) = pPackageInfo;
 	return SEC_E_OK;
 }
 
 static void FreeContextBuffer_EnumerateSecurityPackages(void* contextBuffer)
 {
-	int index;
-	UINT32 cPackages;
 	SecPkgInfoA* pPackageInfo = (SecPkgInfoA*)contextBuffer;
-	cPackages = sizeof(SecPkgInfoA_LIST) / sizeof(*(SecPkgInfoA_LIST));
+	size_t cPackages = ARRAYSIZE(SecPkgInfoA_LIST);
 
 	if (!pPackageInfo)
 		return;
 
-	for (index = 0; index < (int)cPackages; index++)
+	for (size_t index = 0; index < cPackages; index++)
 	{
 		free(pPackageInfo[index].Name);
 		free(pPackageInfo[index].Comment);
@@ -1175,18 +1184,14 @@ SecurityFunctionTableA* SEC_ENTRY winpr_InitSecurityInterfaceA(void)
 static SECURITY_STATUS SEC_ENTRY winpr_QuerySecurityPackageInfoW(SEC_WCHAR* pszPackageName,
                                                                  PSecPkgInfoW* ppPackageInfo)
 {
-	int index;
-	size_t size;
-	UINT32 cPackages;
-	SecPkgInfoW* pPackageInfo;
-	cPackages = sizeof(SecPkgInfoW_LIST) / sizeof(*(SecPkgInfoW_LIST));
+	size_t cPackages = ARRAYSIZE(SecPkgInfoW_LIST);
 
-	for (index = 0; index < (int)cPackages; index++)
+	for (size_t index = 0; index < cPackages; index++)
 	{
 		if (_wcscmp(pszPackageName, SecPkgInfoW_LIST[index]->Name) == 0)
 		{
-			size = sizeof(SecPkgInfoW);
-			pPackageInfo =
+			size_t size = sizeof(SecPkgInfoW);
+			SecPkgInfoW* pPackageInfo =
 			    (SecPkgInfoW*)sspi_ContextBufferAlloc(QuerySecurityPackageInfoIndex, size);
 
 			if (!pPackageInfo)
@@ -1210,18 +1215,14 @@ static SECURITY_STATUS SEC_ENTRY winpr_QuerySecurityPackageInfoW(SEC_WCHAR* pszP
 static SECURITY_STATUS SEC_ENTRY winpr_QuerySecurityPackageInfoA(SEC_CHAR* pszPackageName,
                                                                  PSecPkgInfoA* ppPackageInfo)
 {
-	int index;
-	size_t size;
-	UINT32 cPackages;
-	SecPkgInfoA* pPackageInfo;
-	cPackages = sizeof(SecPkgInfoA_LIST) / sizeof(*(SecPkgInfoA_LIST));
+	size_t cPackages = ARRAYSIZE(SecPkgInfoA_LIST);
 
-	for (index = 0; index < (int)cPackages; index++)
+	for (size_t index = 0; index < cPackages; index++)
 	{
 		if (strcmp(pszPackageName, SecPkgInfoA_LIST[index]->Name) == 0)
 		{
-			size = sizeof(SecPkgInfoA);
-			pPackageInfo =
+			size_t size = sizeof(SecPkgInfoA);
+			SecPkgInfoA* pPackageInfo =
 			    (SecPkgInfoA*)sspi_ContextBufferAlloc(QuerySecurityPackageInfoIndex, size);
 
 			if (!pPackageInfo)
@@ -1268,7 +1269,7 @@ static SECURITY_STATUS SEC_ENTRY winpr_AcquireCredentialsHandleW(
     void* pAuthData, SEC_GET_KEY_FN pGetKeyFn, void* pvGetKeyArgument, PCredHandle phCredential,
     PTimeStamp ptsExpiry)
 {
-	SECURITY_STATUS status;
+	SECURITY_STATUS status = 0;
 	const SecurityFunctionTableW* table = sspi_GetSecurityFunctionTableWByNameW(pszPackage);
 
 	if (!table)
@@ -1298,7 +1299,7 @@ static SECURITY_STATUS SEC_ENTRY winpr_AcquireCredentialsHandleA(
     void* pAuthData, SEC_GET_KEY_FN pGetKeyFn, void* pvGetKeyArgument, PCredHandle phCredential,
     PTimeStamp ptsExpiry)
 {
-	SECURITY_STATUS status;
+	SECURITY_STATUS status = 0;
 	const SecurityFunctionTableA* table = sspi_GetSecurityFunctionTableAByNameA(pszPackage);
 
 	if (!table)
@@ -1327,9 +1328,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_ExportSecurityContext(PCtxtHandle phConte
                                                              PSecBuffer pPackedContext,
                                                              HANDLE* pToken)
 {
-	SEC_CHAR* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableW* table;
+	SEC_CHAR* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableW* table = NULL;
 	Name = (SEC_CHAR*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -1359,9 +1360,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_ExportSecurityContext(PCtxtHandle phConte
 
 static SECURITY_STATUS SEC_ENTRY winpr_FreeCredentialsHandle(PCredHandle phCredential)
 {
-	char* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableA* table;
+	char* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableA* table = NULL;
 	Name = (char*)sspi_SecureHandleGetUpperPointer(phCredential);
 
 	if (!Name)
@@ -1393,9 +1394,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_ImportSecurityContextW(SEC_WCHAR* pszPack
                                                               PSecBuffer pPackedContext,
                                                               HANDLE pToken, PCtxtHandle phContext)
 {
-	SEC_CHAR* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableW* table;
+	SEC_CHAR* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableW* table = NULL;
 	Name = (SEC_CHAR*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -1428,8 +1429,8 @@ static SECURITY_STATUS SEC_ENTRY winpr_ImportSecurityContextA(SEC_CHAR* pszPacka
                                                               HANDLE pToken, PCtxtHandle phContext)
 {
 	char* Name = NULL;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableA* table;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableA* table = NULL;
 	Name = (char*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -1460,9 +1461,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_ImportSecurityContextA(SEC_CHAR* pszPacka
 static SECURITY_STATUS SEC_ENTRY winpr_QueryCredentialsAttributesW(PCredHandle phCredential,
                                                                    ULONG ulAttribute, void* pBuffer)
 {
-	SEC_WCHAR* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableW* table;
+	SEC_WCHAR* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableW* table = NULL;
 	Name = (SEC_WCHAR*)sspi_SecureHandleGetUpperPointer(phCredential);
 
 	if (!Name)
@@ -1493,9 +1494,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_QueryCredentialsAttributesW(PCredHandle p
 static SECURITY_STATUS SEC_ENTRY winpr_QueryCredentialsAttributesA(PCredHandle phCredential,
                                                                    ULONG ulAttribute, void* pBuffer)
 {
-	char* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableA* table;
+	char* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableA* table = NULL;
 	Name = (char*)sspi_SecureHandleGetUpperPointer(phCredential);
 
 	if (!Name)
@@ -1527,9 +1528,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_SetCredentialsAttributesW(PCredHandle phC
                                                                  ULONG ulAttribute, void* pBuffer,
                                                                  ULONG cbBuffer)
 {
-	SEC_WCHAR* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableW* table;
+	SEC_WCHAR* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableW* table = NULL;
 	Name = (SEC_WCHAR*)sspi_SecureHandleGetUpperPointer(phCredential);
 
 	if (!Name)
@@ -1561,9 +1562,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_SetCredentialsAttributesA(PCredHandle phC
                                                                  ULONG ulAttribute, void* pBuffer,
                                                                  ULONG cbBuffer)
 {
-	char* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableA* table;
+	char* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableA* table = NULL;
 	Name = (char*)sspi_SecureHandleGetUpperPointer(phCredential);
 
 	if (!Name)
@@ -1598,9 +1599,9 @@ winpr_AcceptSecurityContext(PCredHandle phCredential, PCtxtHandle phContext, PSe
                             ULONG fContextReq, ULONG TargetDataRep, PCtxtHandle phNewContext,
                             PSecBufferDesc pOutput, PULONG pfContextAttr, PTimeStamp ptsTimeStamp)
 {
-	char* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableA* table;
+	char* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableA* table = NULL;
 	Name = (char*)sspi_SecureHandleGetUpperPointer(phCredential);
 
 	if (!Name)
@@ -1634,8 +1635,8 @@ static SECURITY_STATUS SEC_ENTRY winpr_ApplyControlToken(PCtxtHandle phContext,
                                                          PSecBufferDesc pInput)
 {
 	char* Name = NULL;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableA* table;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableA* table = NULL;
 	Name = (char*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -1667,8 +1668,8 @@ static SECURITY_STATUS SEC_ENTRY winpr_CompleteAuthToken(PCtxtHandle phContext,
                                                          PSecBufferDesc pToken)
 {
 	char* Name = NULL;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableA* table;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableA* table = NULL;
 	Name = (char*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -1714,7 +1715,7 @@ static SECURITY_STATUS SEC_ENTRY winpr_DeleteSecurityContext(PCtxtHandle phConte
 		return SEC_E_UNSUPPORTED_FUNCTION;
 	}
 
-	const UINT32 status = table->DeleteSecurityContext(phContext);
+	const SECURITY_STATUS status = table->DeleteSecurityContext(phContext);
 
 	if (IsSecurityStatusError(status))
 	{
@@ -1736,9 +1737,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_FreeContextBuffer(void* pvContextBuffer)
 
 static SECURITY_STATUS SEC_ENTRY winpr_ImpersonateSecurityContext(PCtxtHandle phContext)
 {
-	SEC_CHAR* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableW* table;
+	SEC_CHAR* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableW* table = NULL;
 	Name = (SEC_CHAR*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -1771,9 +1772,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_InitializeSecurityContextW(
     ULONG Reserved1, ULONG TargetDataRep, PSecBufferDesc pInput, ULONG Reserved2,
     PCtxtHandle phNewContext, PSecBufferDesc pOutput, PULONG pfContextAttr, PTimeStamp ptsExpiry)
 {
-	SEC_CHAR* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableW* table;
+	SEC_CHAR* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableW* table = NULL;
 	Name = (SEC_CHAR*)sspi_SecureHandleGetUpperPointer(phCredential);
 
 	if (!Name)
@@ -1808,9 +1809,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_InitializeSecurityContextA(
     ULONG Reserved1, ULONG TargetDataRep, PSecBufferDesc pInput, ULONG Reserved2,
     PCtxtHandle phNewContext, PSecBufferDesc pOutput, PULONG pfContextAttr, PTimeStamp ptsExpiry)
 {
-	SEC_CHAR* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableA* table;
+	SEC_CHAR* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableA* table = NULL;
 	Name = (SEC_CHAR*)sspi_SecureHandleGetUpperPointer(phCredential);
 
 	if (!Name)
@@ -1843,9 +1844,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_InitializeSecurityContextA(
 static SECURITY_STATUS SEC_ENTRY winpr_QueryContextAttributesW(PCtxtHandle phContext,
                                                                ULONG ulAttribute, void* pBuffer)
 {
-	SEC_CHAR* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableW* table;
+	SEC_CHAR* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableW* table = NULL;
 	Name = (SEC_CHAR*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -1876,9 +1877,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_QueryContextAttributesW(PCtxtHandle phCon
 static SECURITY_STATUS SEC_ENTRY winpr_QueryContextAttributesA(PCtxtHandle phContext,
                                                                ULONG ulAttribute, void* pBuffer)
 {
-	SEC_CHAR* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableA* table;
+	SEC_CHAR* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableA* table = NULL;
 	Name = (SEC_CHAR*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -1909,9 +1910,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_QueryContextAttributesA(PCtxtHandle phCon
 static SECURITY_STATUS SEC_ENTRY winpr_QuerySecurityContextToken(PCtxtHandle phContext,
                                                                  HANDLE* phToken)
 {
-	SEC_CHAR* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableW* table;
+	SEC_CHAR* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableW* table = NULL;
 	Name = (SEC_CHAR*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -1943,9 +1944,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_SetContextAttributesW(PCtxtHandle phConte
                                                              ULONG ulAttribute, void* pBuffer,
                                                              ULONG cbBuffer)
 {
-	SEC_CHAR* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableW* table;
+	SEC_CHAR* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableW* table = NULL;
 	Name = (SEC_CHAR*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -1977,9 +1978,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_SetContextAttributesA(PCtxtHandle phConte
                                                              ULONG ulAttribute, void* pBuffer,
                                                              ULONG cbBuffer)
 {
-	char* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableA* table;
+	char* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableA* table = NULL;
 	Name = (char*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -2009,9 +2010,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_SetContextAttributesA(PCtxtHandle phConte
 
 static SECURITY_STATUS SEC_ENTRY winpr_RevertSecurityContext(PCtxtHandle phContext)
 {
-	SEC_CHAR* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableW* table;
+	SEC_CHAR* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableW* table = NULL;
 	Name = (SEC_CHAR*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -2045,9 +2046,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_DecryptMessage(PCtxtHandle phContext,
                                                       PSecBufferDesc pMessage, ULONG MessageSeqNo,
                                                       PULONG pfQOP)
 {
-	char* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableA* table;
+	char* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableA* table = NULL;
 	Name = (char*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -2078,9 +2079,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_DecryptMessage(PCtxtHandle phContext,
 static SECURITY_STATUS SEC_ENTRY winpr_EncryptMessage(PCtxtHandle phContext, ULONG fQOP,
                                                       PSecBufferDesc pMessage, ULONG MessageSeqNo)
 {
-	char* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableA* table;
+	char* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableA* table = NULL;
 	Name = (char*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -2111,9 +2112,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_EncryptMessage(PCtxtHandle phContext, ULO
 static SECURITY_STATUS SEC_ENTRY winpr_MakeSignature(PCtxtHandle phContext, ULONG fQOP,
                                                      PSecBufferDesc pMessage, ULONG MessageSeqNo)
 {
-	char* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableA* table;
+	char* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableA* table = NULL;
 	Name = (char*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
@@ -2145,9 +2146,9 @@ static SECURITY_STATUS SEC_ENTRY winpr_VerifySignature(PCtxtHandle phContext,
                                                        PSecBufferDesc pMessage, ULONG MessageSeqNo,
                                                        PULONG pfQOP)
 {
-	char* Name;
-	SECURITY_STATUS status;
-	const SecurityFunctionTableA* table;
+	char* Name = NULL;
+	SECURITY_STATUS status = 0;
+	const SecurityFunctionTableA* table = NULL;
 	Name = (char*)sspi_SecureHandleGetUpperPointer(phContext);
 
 	if (!Name)
