@@ -20,6 +20,7 @@
 
 #include <winpr/config.h>
 
+#include <winpr/assert.h>
 #include <winpr/sysinfo.h>
 #include <winpr/platform.h>
 
@@ -33,9 +34,43 @@
 #include <fcntl.h>
 #endif
 
+#if !defined(_WIN32)
+#if defined(_POSIX_C_SOURCE) && (_POSIX_C_SOURCE >= 199309L)
+#include <time.h>
+#elif !defined(__APPLE__)
+#include <sys/time.h>
+#include <sys/sysinfo.h>
+#endif
+#endif
+
 #include "../log.h"
 #define TAG WINPR_TAG("sysinfo")
 
+#define FILETIME_TO_UNIX_OFFSET_S 11644473600UL
+
+#if defined(__MACH__) && defined(__APPLE__)
+
+#include <mach/mach_time.h>
+
+static UINT64 scaleHighPrecision(UINT64 i, UINT32 numer, UINT32 denom)
+{
+	UINT64 high = (i >> 32) * numer;
+	UINT64 low = (i & 0xffffffffull) * numer / denom;
+	UINT64 highRem = ((high % denom) << 32) / denom;
+	high /= denom;
+	return (high << 32) + highRem + low;
+}
+
+static UINT64 mac_get_time_ns(void)
+{
+	mach_timebase_info_data_t timebase = { 0 };
+	mach_timebase_info(&timebase);
+	UINT64 t = mach_absolute_time();
+	return scaleHighPrecision(t, timebase.numer, timebase.denom);
+}
+
+#endif
+
 /**
  * api-ms-win-core-sysinfo-l1-1-1.dll:
  *
@@ -73,9 +108,9 @@
 #include <sys/sysctl.h>
 #endif
 
-static DWORD GetProcessorArchitecture(void)
+static WORD GetProcessorArchitecture(void)
 {
-	DWORD cpuArch = PROCESSOR_ARCHITECTURE_UNKNOWN;
+	WORD cpuArch = PROCESSOR_ARCHITECTURE_UNKNOWN;
 #if defined(ANDROID)
 	AndroidCpuFamily family = android_getCpuFamily();
 
@@ -192,17 +227,14 @@ static DWORD GetSystemPageSize(void)
 
 void GetSystemInfo(LPSYSTEM_INFO lpSystemInfo)
 {
-	lpSystemInfo->wProcessorArchitecture = GetProcessorArchitecture();
-	lpSystemInfo->wReserved = 0;
+	const SYSTEM_INFO empty = { 0 };
+	WINPR_ASSERT(lpSystemInfo);
+
+	*lpSystemInfo = empty;
+	lpSystemInfo->DUMMYUNIONNAME.DUMMYSTRUCTNAME.wProcessorArchitecture =
+	    GetProcessorArchitecture();
 	lpSystemInfo->dwPageSize = GetSystemPageSize();
-	lpSystemInfo->lpMinimumApplicationAddress = NULL;
-	lpSystemInfo->lpMaximumApplicationAddress = NULL;
-	lpSystemInfo->dwActiveProcessorMask = 0;
 	lpSystemInfo->dwNumberOfProcessors = GetNumberOfProcessors();
-	lpSystemInfo->dwProcessorType = 0;
-	lpSystemInfo->dwAllocationGranularity = 0;
-	lpSystemInfo->wProcessorLevel = 0;
-	lpSystemInfo->wProcessorRevision = 0;
 }
 
 void GetNativeSystemInfo(LPSYSTEM_INFO lpSystemInfo)
@@ -234,9 +266,10 @@ void GetSystemTime(LPSYSTEMTIME lpSystemTime)
 	}
 }
 
-BOOL SetSystemTime(CONST SYSTEMTIME* lpSystemTime)
+BOOL SetSystemTime(WINPR_ATTR_UNUSED CONST SYSTEMTIME* lpSystemTime)
 {
 	/* TODO: Implement */
+	WLog_ERR("TODO", "TODO: Implement");
 	return FALSE;
 }
 
@@ -264,27 +297,31 @@ VOID GetLocalTime(LPSYSTEMTIME lpSystemTime)
 	}
 }
 
-BOOL SetLocalTime(CONST SYSTEMTIME* lpSystemTime)
+BOOL SetLocalTime(WINPR_ATTR_UNUSED CONST SYSTEMTIME* lpSystemTime)
 {
 	/* TODO: Implement */
+	WLog_ERR("TODO", "TODO: Implement");
 	return FALSE;
 }
 
 VOID GetSystemTimeAsFileTime(LPFILETIME lpSystemTimeAsFileTime)
 {
-	ULARGE_INTEGER time64;
-	time64.u.HighPart = 0;
-	/* time represented in tenths of microseconds since midnight of January 1, 1601 */
-	time64.QuadPart = time(NULL) + 11644473600LL; /* Seconds since January 1, 1601 */
-	time64.QuadPart *= 10000000;                  /* Convert timestamp to tenths of a microsecond */
-	lpSystemTimeAsFileTime->dwLowDateTime = time64.u.LowPart;
-	lpSystemTimeAsFileTime->dwHighDateTime = time64.u.HighPart;
+	union
+	{
+		UINT64 u64;
+		FILETIME ft;
+	} t;
+
+	t.u64 = (winpr_GetUnixTimeNS() / 100ull) + FILETIME_TO_UNIX_OFFSET_S * 10000000ull;
+	*lpSystemTimeAsFileTime = t.ft;
 }
 
-BOOL GetSystemTimeAdjustment(PDWORD lpTimeAdjustment, PDWORD lpTimeIncrement,
-                             PBOOL lpTimeAdjustmentDisabled)
+BOOL GetSystemTimeAdjustment(WINPR_ATTR_UNUSED PDWORD lpTimeAdjustment,
+                             WINPR_ATTR_UNUSED PDWORD lpTimeIncrement,
+                             WINPR_ATTR_UNUSED PBOOL lpTimeAdjustmentDisabled)
 {
 	/* TODO: Implement */
+	WLog_ERR("TODO", "TODO: Implement");
 	return FALSE;
 }
 
@@ -294,25 +331,7 @@ BOOL GetSystemTimeAdjustment(PDWORD lpTimeAdjustment, PDWORD lpTimeIncrement,
 
 DWORD GetTickCount(void)
 {
-	DWORD ticks = 0;
-#ifdef __linux__
-	struct timespec ts;
-
-	if (!clock_gettime(CLOCK_MONOTONIC_RAW, &ts))
-		ticks = (ts.tv_sec * 1000) + (ts.tv_nsec / 1000000);
-
-#else
-	/**
-	 * FIXME: this is relative to the Epoch time, and we
-	 * need to return a value relative to the system uptime.
-	 */
-	struct timeval tv;
-
-	if (!gettimeofday(&tv, NULL))
-		ticks = (tv.tv_sec * 1000) + (tv.tv_usec / 1000);
-
-#endif
-	return ticks;
+	return (DWORD)GetTickCount64();
 }
 #endif // _WIN32
 
@@ -419,24 +438,24 @@ BOOL GetComputerNameW(LPWSTR lpBuffer, LPDWORD lpnSize)
 
 BOOL GetComputerNameA(LPSTR lpBuffer, LPDWORD lpnSize)
 {
-	char* dot = NULL;
-	size_t length = 0;
-	char hostname[256] = { 0 };
-
 	if (!lpnSize)
 	{
 		SetLastError(ERROR_BAD_ARGUMENTS);
 		return FALSE;
 	}
 
-	if (gethostname(hostname, sizeof(hostname)) == -1)
+	char hostname[256 + 1] = { 0 };
+	if (gethostname(hostname, ARRAYSIZE(hostname) - 1) == -1)
 		return FALSE;
 
-	length = strnlen(hostname, sizeof(hostname));
-	dot = strchr(hostname, '.');
-
+	size_t length = strnlen(hostname, MAX_COMPUTERNAME_LENGTH);
+	const char* dot = strchr(hostname, '.');
 	if (dot)
-		length = (dot - hostname);
+	{
+		const size_t dotlen = WINPR_ASSERTING_INT_CAST(size_t, (dot - hostname));
+		if (dotlen < length)
+			length = dotlen;
+	}
 
 	if ((*lpnSize <= (DWORD)length) || !lpBuffer)
 	{
@@ -445,7 +464,7 @@ BOOL GetComputerNameA(LPSTR lpBuffer, LPDWORD lpnSize)
 		return FALSE;
 	}
 
-	CopyMemory(lpBuffer, hostname, length);
+	strncpy(lpBuffer, hostname, length);
 	lpBuffer[length] = '\0';
 	*lpnSize = (DWORD)length;
 	return TRUE;
@@ -553,35 +572,82 @@ DWORD GetTickCount(void)
 
 ULONGLONG winpr_GetTickCount64(void)
 {
-	ULONGLONG ticks = 0;
-#if defined(__linux__)
-	struct timespec ts;
+	const UINT64 ns = winpr_GetTickCount64NS();
+	return WINPR_TIME_NS_TO_MS(ns);
+}
 
-	if (!clock_gettime(CLOCK_MONOTONIC_RAW, &ts))
-		ticks = (ts.tv_sec * 1000) + (ts.tv_nsec / 1000000);
+#endif
 
+UINT64 winpr_GetTickCount64NS(void)
+{
+	UINT64 ticks = 0;
+#if defined(_POSIX_C_SOURCE) && (_POSIX_C_SOURCE >= 199309L)
+	struct timespec ts = { 0 };
+
+	if (clock_gettime(CLOCK_MONOTONIC_RAW, &ts) == 0)
+		ticks = (WINPR_ASSERTING_INT_CAST(uint64_t, ts.tv_sec) * 1000000000ull) +
+		        WINPR_ASSERTING_INT_CAST(uint64_t, ts.tv_nsec);
+#elif defined(__MACH__) && defined(__APPLE__)
+	ticks = mac_get_time_ns();
 #elif defined(_WIN32)
-	FILETIME ft;
-	ULARGE_INTEGER ul;
-	GetSystemTimeAsFileTime(&ft);
-	ul.LowPart = ft.dwLowDateTime;
-	ul.HighPart = ft.dwHighDateTime;
-	ticks = ul.QuadPart;
+	LARGE_INTEGER li = { 0 };
+	LARGE_INTEGER freq = { 0 };
+	if (QueryPerformanceFrequency(&freq) && QueryPerformanceCounter(&li))
+		ticks = li.QuadPart * 1000000000ull / freq.QuadPart;
 #else
-	/**
-	 * FIXME: this is relative to the Epoch time, and we
-	 * need to return a value relative to the system uptime.
+	struct timeval tv = { 0 };
+
+	if (gettimeofday(&tv, NULL) == 0)
+		ticks = (tv.tv_sec * 1000000000ull) + (tv.tv_usec * 1000ull);
+
+	/* We need to trick here:
+	 * this function should return the system uptime, but we need higher resolution.
+	 * so on first call get the actual timestamp along with the system uptime.
+	 *
+	 * return the uptime measured from now on (e.g. current measure - first measure + uptime at
+	 * first measure)
 	 */
-	struct timeval tv;
-
-	if (!gettimeofday(&tv, NULL))
-		ticks = (tv.tv_sec * 1000) + (tv.tv_usec / 1000);
+	static UINT64 first = 0;
+	static UINT64 uptime = 0;
+	if (first == 0)
+	{
+		struct sysinfo info = { 0 };
+		if (sysinfo(&info) == 0)
+		{
+			first = ticks;
+			uptime = 1000000000ull * info.uptime;
+		}
+	}
 
+	ticks = ticks - first + uptime;
 #endif
 	return ticks;
 }
 
+UINT64 winpr_GetUnixTimeNS(void)
+{
+#if defined(_WIN32)
+
+	union
+	{
+		UINT64 u64;
+		FILETIME ft;
+	} t = { 0 };
+	GetSystemTimeAsFileTime(&t.ft);
+	return (t.u64 - FILETIME_TO_UNIX_OFFSET_S * 10000000ull) * 100ull;
+#elif defined(_POSIX_C_SOURCE) && (_POSIX_C_SOURCE >= 199309L)
+	struct timespec ts = { 0 };
+	if (clock_gettime(CLOCK_REALTIME, &ts) != 0)
+		return 0;
+	return WINPR_ASSERTING_INT_CAST(uint64_t, ts.tv_sec) * 1000000000ull +
+	       WINPR_ASSERTING_INT_CAST(uint64_t, ts.tv_nsec);
+#else
+	struct timeval tv = { 0 };
+	if (gettimeofday(&tv, NULL) != 0)
+		return 0;
+	return tv.tv_sec * 1000000000ULL + tv.tv_usec * 1000ull;
 #endif
+}
 
 /* If x86 */
 #ifdef _M_IX86_AMD64
@@ -648,7 +714,7 @@ static void cpuid(unsigned info, unsigned* eax, unsigned* ebx, unsigned* ecx, un
 	*edx = a[3];
 #endif
 }
-#elif defined(_M_ARM)
+#elif defined(_M_ARM) || defined(_M_ARM64)
 #if defined(__linux__)
 // HWCAP flags from linux kernel - uapi/asm/hwcap.h
 #define HWCAP_SWP (1 << 0)
@@ -713,6 +779,12 @@ static unsigned GetARMCPUCaps(void)
 
 #ifndef _WIN32
 
+#if defined(_M_ARM) || defined(_M_ARM64)
+#ifdef __linux__
+#include <sys/auxv.h>
+#endif
+#endif
+
 BOOL IsProcessorFeaturePresent(DWORD ProcessorFeature)
 {
 	BOOL ret = FALSE;
@@ -726,17 +798,19 @@ BOOL IsProcessorFeaturePresent(DWORD ProcessorFeature)
 			return features & ANDROID_CPU_ARM_FEATURE_NEON;
 
 		default:
+			WLog_WARN(TAG, "feature 0x%08" PRIx32 " check not implemented", ProcessorFeature);
 			return FALSE;
 	}
 
-#elif defined(_M_ARM)
+#elif defined(_M_ARM) || defined(_M_ARM64)
 #ifdef __linux__
-	const unsigned caps = GetARMCPUCaps();
+	const unsigned long caps = getauxval(AT_HWCAP);
 
 	switch (ProcessorFeature)
 	{
 		case PF_ARM_NEON_INSTRUCTIONS_AVAILABLE:
 		case PF_ARM_NEON:
+
 			if (caps & HWCAP_NEON)
 				ret = TRUE;
 
@@ -795,8 +869,15 @@ BOOL IsProcessorFeaturePresent(DWORD ProcessorFeature)
 				ret = TRUE;
 
 			break;
-
+		case PF_ARM_V8_INSTRUCTIONS_AVAILABLE:
+		case PF_ARM_V8_CRYPTO_INSTRUCTIONS_AVAILABLE:
+		case PF_ARM_V8_CRC32_INSTRUCTIONS_AVAILABLE:
+		case PF_ARM_V81_ATOMIC_INSTRUCTIONS_AVAILABLE:
+		case PF_ARM_V82_DP_INSTRUCTIONS_AVAILABLE:
+		case PF_ARM_V83_JSCVT_INSTRUCTIONS_AVAILABLE:
+		case PF_ARM_V83_LRCPC_INSTRUCTIONS_AVAILABLE:
 		default:
+			WLog_WARN(TAG, "feature 0x%08" PRIx32 " check not implemented", ProcessorFeature);
 			break;
 	}
 
@@ -810,12 +891,22 @@ BOOL IsProcessorFeaturePresent(DWORD ProcessorFeature)
 			ret = TRUE;
 #endif
 			break;
+		case PF_ARM_V8_INSTRUCTIONS_AVAILABLE:
+		case PF_ARM_V8_CRYPTO_INSTRUCTIONS_AVAILABLE:
+		case PF_ARM_V8_CRC32_INSTRUCTIONS_AVAILABLE:
+		case PF_ARM_V81_ATOMIC_INSTRUCTIONS_AVAILABLE:
+		case PF_ARM_V82_DP_INSTRUCTIONS_AVAILABLE:
+		case PF_ARM_V83_JSCVT_INSTRUCTIONS_AVAILABLE:
+		case PF_ARM_V83_LRCPC_INSTRUCTIONS_AVAILABLE:
 		default:
+			WLog_WARN(TAG, "feature 0x%08" PRIx32 " check not implemented", ProcessorFeature);
 			break;
 	}
 
 #endif // __linux__
-#elif defined(_M_IX86_AMD64)
+#endif
+
+#if defined(_M_IX86_AMD64)
 #ifdef __GNUC__
 	unsigned a = 0;
 	unsigned b = 0;
@@ -850,17 +941,41 @@ BOOL IsProcessorFeaturePresent(DWORD ProcessorFeature)
 			break;
 
 		case PF_SSE3_INSTRUCTIONS_AVAILABLE:
-			if (c & C_BIT_SSE3)
-				ret = TRUE;
-
+			ret = __builtin_cpu_supports("sse3");
 			break;
 
+		case PF_SSSE3_INSTRUCTIONS_AVAILABLE:
+			ret = __builtin_cpu_supports("ssse3");
+			break;
+		case PF_SSE4_1_INSTRUCTIONS_AVAILABLE:
+			ret = __builtin_cpu_supports("sse4.1");
+			break;
+		case PF_SSE4_2_INSTRUCTIONS_AVAILABLE:
+			ret = __builtin_cpu_supports("sse4.2");
+			break;
+		case PF_AVX_INSTRUCTIONS_AVAILABLE:
+			ret = __builtin_cpu_supports("avx");
+			break;
+		case PF_AVX2_INSTRUCTIONS_AVAILABLE:
+			ret = __builtin_cpu_supports("avx2");
+			break;
+		case PF_AVX512F_INSTRUCTIONS_AVAILABLE:
+			ret = __builtin_cpu_supports("avx512f");
+			break;
+		case PF_ARM_NEON_INSTRUCTIONS_AVAILABLE:
+#if defined(__ARM_NEON__)
+			ret = TRUE;
+#endif
+			break;
 		default:
+			WLog_WARN(TAG, "feature 0x%08" PRIx32 " check not implemented", ProcessorFeature);
 			break;
 	}
 
 #endif // __GNUC__
-#elif defined(_M_E2K)
+#endif
+
+#if defined(_M_E2K)
 	/* compiler flags on e2k arch determine CPU features */
 	switch (ProcessorFeature)
 	{
@@ -895,8 +1010,8 @@ BOOL IsProcessorFeaturePresent(DWORD ProcessorFeature)
 DWORD GetTickCountPrecise(void)
 {
 #ifdef _WIN32
-	LARGE_INTEGER freq;
-	LARGE_INTEGER current;
+	LARGE_INTEGER freq = { 0 };
+	LARGE_INTEGER current = { 0 };
 	QueryPerformanceFrequency(&freq);
 	QueryPerformanceCounter(&current);
 	return (DWORD)(current.QuadPart * 1000LL / freq.QuadPart);
@@ -908,7 +1023,7 @@ DWORD GetTickCountPrecise(void)
 BOOL IsProcessorFeaturePresentEx(DWORD ProcessorFeature)
 {
 	BOOL ret = FALSE;
-#ifdef _M_ARM
+#if defined(_M_ARM) || defined(_M_ARM64)
 #ifdef __linux__
 	unsigned caps;
 	caps = GetARMCPUCaps();
@@ -1058,6 +1173,8 @@ BOOL IsProcessorFeaturePresentEx(DWORD ProcessorFeature)
 						if (c & C_BIT_PCLMULQDQ)
 							ret = TRUE;
 
+						break;
+					default:
 						break;
 				}
 			}
