@@ -28,6 +28,7 @@
 #include <winpr/assert.h>
 #include <winpr/file.h>
 #include "../log.h"
+#include "timezone.h"
 
 #define TAG WINPR_TAG("timezone")
 
@@ -49,24 +50,29 @@
 static char* winpr_read_unix_timezone_identifier_from_file(FILE* fp)
 {
 	const INT CHUNK_SIZE = 32;
-	INT64 rc = 0;
-	INT64 read = 0;
-	INT64 length = CHUNK_SIZE;
-	char* tzid = NULL;
+	size_t rc = 0;
+	size_t read = 0;
+	size_t length = CHUNK_SIZE;
 
-	tzid = (char*)malloc((size_t)length);
+	char* tzid = malloc(length);
 	if (!tzid)
 		return NULL;
 
 	do
 	{
-		rc = fread(tzid + read, 1, length - read - 1, fp);
+		rc = fread(tzid + read, 1, length - read - 1UL, fp);
 		if (rc > 0)
 			read += rc;
 
-		if (read < (length - 1))
+		if (read < (length - 1UL))
 			break;
 
+		if (read > length - 1UL)
+		{
+			free(tzid);
+			return NULL;
+		}
+
 		length += CHUNK_SIZE;
 		char* tmp = (char*)realloc(tzid, length);
 		if (!tmp)
@@ -253,7 +259,7 @@ static char* winpr_get_unix_timezone_identifier_from_file(void)
 		return NULL;
 
 	tzid = winpr_read_unix_timezone_identifier_from_file(fp);
-	fclose(fp);
+	(void)fclose(fp);
 	if (tzid != NULL)
 		WLog_DBG(TAG, "tzid: %s", tzid);
 	return tzid;
@@ -343,7 +349,7 @@ static SYSTEMTIME tm2systemtime(const struct tm* t)
 
 	if (t)
 	{
-		st.wYear = (WORD)1900 + t->tm_year;
+		st.wYear = (WORD)(1900 + t->tm_year);
 		st.wMonth = (WORD)t->tm_mon + 1;
 		st.wDay = (WORD)t->tm_mday;
 		st.wDayOfWeek = (WORD)t->tm_wday;
@@ -388,7 +394,7 @@ static struct tm next_day(const struct tm* start)
 	cur.tm_isdst = -1;
 	cur.tm_mday++;
 	const time_t t = mktime(&cur);
-	localtime_r(&t, &cur);
+	(void)localtime_r(&t, &cur);
 	return cur;
 }
 
@@ -400,7 +406,7 @@ static struct tm adjust_time(const struct tm* start, int hour, int minute)
 	cur.tm_sec = 0;
 	cur.tm_isdst = -1;
 	const time_t t = mktime(&cur);
-	localtime_r(&t, &cur);
+	(void)localtime_r(&t, &cur);
 	return cur;
 }
 
@@ -421,7 +427,7 @@ static WORD get_transition_weekday_occurrence(const SYSTEMTIME* st)
 		next.tm_mday++;
 
 		struct tm cur = { 0 };
-		localtime_r(&t, &cur);
+		(void)localtime_r(&t, &cur);
 
 		if (cur.tm_mon + 1 != st->wMonth)
 			break;
@@ -541,11 +547,11 @@ static BOOL map_iana_id(const char* iana, LPDYNAMIC_TIME_ZONE_INFORMATION tz)
 	const char* winDst = TimeZoneIanaToWindows(iana, TIME_ZONE_NAME_DAYLIGHT);
 
 	if (winStd)
-		ConvertUtf8ToWChar(winStd, tz->StandardName, ARRAYSIZE(tz->StandardName));
+		(void)ConvertUtf8ToWChar(winStd, tz->StandardName, ARRAYSIZE(tz->StandardName));
 	if (winDst)
-		ConvertUtf8ToWChar(winDst, tz->DaylightName, ARRAYSIZE(tz->DaylightName));
+		(void)ConvertUtf8ToWChar(winDst, tz->DaylightName, ARRAYSIZE(tz->DaylightName));
 	if (winId)
-		ConvertUtf8ToWChar(winId, tz->TimeZoneKeyName, ARRAYSIZE(tz->TimeZoneKeyName));
+		(void)ConvertUtf8ToWChar(winId, tz->TimeZoneKeyName, ARRAYSIZE(tz->TimeZoneKeyName));
 
 	return winId != NULL;
 }
@@ -578,14 +584,14 @@ static char* systemtime2str(const SYSTEMTIME* t, char* buffer, size_t len)
 	const SYSTEMTIME empty = { 0 };
 
 	if (memcmp(t, &empty, sizeof(SYSTEMTIME)) == 0)
-		_snprintf(buffer, len, "{ not set }");
+		(void)_snprintf(buffer, len, "{ not set }");
 	else
 	{
-		_snprintf(buffer, len,
-		          "{ %" PRIu16 "-%" PRIu16 "-%" PRIu16 " [%s] %" PRIu16 ":%" PRIu16 ":%" PRIu16
-		          ".%" PRIu16 "}",
-		          t->wYear, t->wMonth, t->wDay, weekday2str(t->wDayOfWeek), t->wHour, t->wMinute,
-		          t->wSecond, t->wMilliseconds);
+		(void)_snprintf(buffer, len,
+		                "{ %" PRIu16 "-%" PRIu16 "-%" PRIu16 " [%s] %" PRIu16 ":%" PRIu16
+		                ":%" PRIu16 ".%" PRIu16 "}",
+		                t->wYear, t->wMonth, t->wDay, weekday2str(t->wDayOfWeek), t->wHour,
+		                t->wMinute, t->wSecond, t->wMilliseconds);
 	}
 	return buffer;
 }
@@ -595,7 +601,7 @@ static void log_print(wLog* log, DWORD level, const char* file, const char* fkt,
 	if (!WLog_IsLevelActive(log, level))
 		return;
 
-	va_list ap;
+	va_list ap = { 0 };
 	va_start(ap, line);
 	WLog_PrintMessageVA(log, WLOG_MESSAGE_TEXT, level, line, file, fkt, ap);
 	va_end(ap);
@@ -613,21 +619,21 @@ static void log_timezone_(const DYNAMIC_TIME_ZONE_INFORMATION* tzif, DWORD resul
 	log_print(log, level, file, fkt, line, "DYNAMIC_TIME_ZONE_INFORMATION {");
 
 	log_print(log, level, file, fkt, line, "  Bias=%" PRIu32, tzif->Bias);
-	ConvertWCharNToUtf8(tzif->StandardName, ARRAYSIZE(tzif->StandardName), buffer,
-	                    ARRAYSIZE(buffer));
+	(void)ConvertWCharNToUtf8(tzif->StandardName, ARRAYSIZE(tzif->StandardName), buffer,
+	                          ARRAYSIZE(buffer));
 	log_print(log, level, file, fkt, line, "  StandardName=%s", buffer);
 	log_print(log, level, file, fkt, line, "  StandardDate=%s",
 	          systemtime2str(&tzif->StandardDate, buffer, sizeof(buffer)));
 	log_print(log, level, file, fkt, line, "  StandardBias=%" PRIu32, tzif->StandardBias);
 
-	ConvertWCharNToUtf8(tzif->DaylightName, ARRAYSIZE(tzif->DaylightName), buffer,
-	                    ARRAYSIZE(buffer));
+	(void)ConvertWCharNToUtf8(tzif->DaylightName, ARRAYSIZE(tzif->DaylightName), buffer,
+	                          ARRAYSIZE(buffer));
 	log_print(log, level, file, fkt, line, "  DaylightName=%s", buffer);
 	log_print(log, level, file, fkt, line, "  DaylightDate=%s",
 	          systemtime2str(&tzif->DaylightDate, buffer, sizeof(buffer)));
 	log_print(log, level, file, fkt, line, "  DaylightBias=%" PRIu32, tzif->DaylightBias);
-	ConvertWCharNToUtf8(tzif->TimeZoneKeyName, ARRAYSIZE(tzif->TimeZoneKeyName), buffer,
-	                    ARRAYSIZE(buffer));
+	(void)ConvertWCharNToUtf8(tzif->TimeZoneKeyName, ARRAYSIZE(tzif->TimeZoneKeyName), buffer,
+	                          ARRAYSIZE(buffer));
 	log_print(log, level, file, fkt, line, "  TimeZoneKeyName=%s", buffer);
 	log_print(log, level, file, fkt, line, "  DynamicDaylightTimeDisabled=DST-%s",
 	          tzif->DynamicDaylightTimeDisabled ? "disabled" : "enabled");
@@ -725,33 +731,40 @@ static int dynamic_time_zone_from_localtime(const struct tm* local_time,
 	int rc = HAVE_TRANSITION_DATES;
 
 	tz->Bias = get_bias(local_time, FALSE);
+
+	/* If the current time has (or had) DST */
 	if (local_time->tm_isdst >= 0)
 	{
 		/* DST bias is the difference between standard time and DST in minutes */
 		const LONG d = get_bias(local_time, TRUE);
-		tz->DaylightBias = -1 * labs(tz->Bias - d);
+		tz->DaylightBias = -1 * (tz->Bias - d);
 		if (!get_transition_date(local_time, FALSE, &tz->StandardDate))
+		{
 			rc |= HAVE_NO_STANDARD_TRANSITION_DATE;
+			tz->StandardBias = 0;
+		}
 		if (!get_transition_date(local_time, TRUE, &tz->DaylightDate))
+		{
 			rc |= HAVE_NO_DAYLIGHT_TRANSITION_DATE;
+			tz->DaylightBias = 0;
+		}
 	}
 	return rc;
 }
 
-DWORD GetDynamicTimeZoneInformation(PDYNAMIC_TIME_ZONE_INFORMATION tz)
+DWORD GetDynamicTimeZoneInformation(PDYNAMIC_TIME_ZONE_INFORMATION pTimeZoneInformation)
 {
-	BOOL doesNotHaveStandardDate = FALSE;
-	BOOL doesNotHaveDaylightDate = FALSE;
 	const char** list = NULL;
 	char* tzid = NULL;
 	const char* defaultName = "Client Local Time";
 	DWORD res = TIME_ZONE_ID_UNKNOWN;
 	const DYNAMIC_TIME_ZONE_INFORMATION empty = { 0 };
 
-	WINPR_ASSERT(tz);
+	WINPR_ASSERT(pTimeZoneInformation);
 
-	*tz = empty;
-	ConvertUtf8ToWChar(defaultName, tz->StandardName, ARRAYSIZE(tz->StandardName));
+	*pTimeZoneInformation = empty;
+	(void)ConvertUtf8ToWChar(defaultName, pTimeZoneInformation->StandardName,
+	                         ARRAYSIZE(pTimeZoneInformation->StandardName));
 
 	const time_t t = time(NULL);
 	struct tm tres = { 0 };
@@ -759,28 +772,22 @@ DWORD GetDynamicTimeZoneInformation(PDYNAMIC_TIME_ZONE_INFORMATION tz)
 	if (!local_time)
 		goto out_error;
 
-	tz->Bias = get_bias(local_time, FALSE);
+	pTimeZoneInformation->Bias = get_bias(local_time, FALSE);
 	if (local_time->tm_isdst >= 0)
-	{
-		const int rc = dynamic_time_zone_from_localtime(local_time, tz);
-		if (rc & HAVE_NO_STANDARD_TRANSITION_DATE)
-			doesNotHaveStandardDate = TRUE;
-		if (rc & HAVE_NO_DAYLIGHT_TRANSITION_DATE)
-			doesNotHaveDaylightDate = TRUE;
-	}
+		dynamic_time_zone_from_localtime(local_time, pTimeZoneInformation);
 
 	tzid = winpr_guess_time_zone();
-	if (!map_iana_id(tzid, tz))
+	if (!map_iana_id(tzid, pTimeZoneInformation))
 	{
 		const size_t len = TimeZoneIanaAbbrevGet(local_time->tm_zone, NULL, 0);
-		list = calloc(len, sizeof(char*));
+		list = (const char**)calloc(len, sizeof(const char*));
 		if (!list)
 			goto out_error;
 		const size_t size = TimeZoneIanaAbbrevGet(local_time->tm_zone, list, len);
 		for (size_t x = 0; x < size; x++)
 		{
 			const char* id = list[x];
-			if (map_iana_id(id, tz))
+			if (map_iana_id(id, pTimeZoneInformation))
 			{
 				res = (local_time->tm_isdst) ? TIME_ZONE_ID_DAYLIGHT : TIME_ZONE_ID_STANDARD;
 				break;
@@ -790,17 +797,11 @@ DWORD GetDynamicTimeZoneInformation(PDYNAMIC_TIME_ZONE_INFORMATION tz)
 	else
 		res = (local_time->tm_isdst) ? TIME_ZONE_ID_DAYLIGHT : TIME_ZONE_ID_STANDARD;
 
-	if (doesNotHaveDaylightDate)
-		tz->DaylightBias = 0;
-
-	if (doesNotHaveStandardDate)
-		tz->StandardBias = 0;
-
 out_error:
 	free(tzid);
-	free(list);
+	free((void*)list);
 
-	log_timezone(tz, res);
+	log_timezone(pTimeZoneInformation, res);
 	return res;
 }
 
@@ -858,51 +859,35 @@ DWORD EnumDynamicTimeZoneInformation(const DWORD dwIndex,
 		return ERROR_NO_MORE_ITEMS;
 
 	if (entry->DaylightName)
-		ConvertUtf8ToWChar(entry->DaylightName, lpTimeZoneInformation->DaylightName,
-		                   ARRAYSIZE(lpTimeZoneInformation->DaylightName));
+		(void)ConvertUtf8ToWChar(entry->DaylightName, lpTimeZoneInformation->DaylightName,
+		                         ARRAYSIZE(lpTimeZoneInformation->DaylightName));
 	if (entry->StandardName)
-		ConvertUtf8ToWChar(entry->StandardName, lpTimeZoneInformation->StandardName,
-		                   ARRAYSIZE(lpTimeZoneInformation->StandardName));
+		(void)ConvertUtf8ToWChar(entry->StandardName, lpTimeZoneInformation->StandardName,
+		                         ARRAYSIZE(lpTimeZoneInformation->StandardName));
 	if (entry->Id)
-		ConvertUtf8ToWChar(entry->Id, lpTimeZoneInformation->TimeZoneKeyName,
-		                   ARRAYSIZE(lpTimeZoneInformation->TimeZoneKeyName));
+		(void)ConvertUtf8ToWChar(entry->Id, lpTimeZoneInformation->TimeZoneKeyName,
+		                         ARRAYSIZE(lpTimeZoneInformation->TimeZoneKeyName));
 
 	const time_t t = time(NULL);
 	struct tm tres = { 0 };
 
-	const char* tz = getenv("TZ");
-	char* tzcopy = NULL;
-	if (tz)
-	{
-		size_t tzianalen = 0;
-		winpr_asprintf(&tzcopy, &tzianalen, "TZ=%s", tz);
-	}
+	char* tzcopy = entry->Iana ? setNewAndSaveOldTZ(entry->Iana) : NULL;
 
-	char* tziana = NULL;
-	{
-		size_t tzianalen = 0;
-		winpr_asprintf(&tziana, &tzianalen, "TZ=%s", entry->Iana);
-	}
-	if (tziana)
-		putenv(tziana);
-
-	tzset();
 	struct tm* local_time = localtime_r(&t, &tres);
-	free(tziana);
-	if (tzcopy)
-		putenv(tzcopy);
-	else
-		unsetenv("TZ");
-	free(tzcopy);
 
 	if (local_time)
 		dynamic_time_zone_from_localtime(local_time, lpTimeZoneInformation);
 
+	if (entry->Iana)
+		restoreSavedTZ(tzcopy);
+
 	return ERROR_SUCCESS;
 }
 
+// NOLINTBEGIN(readability-non-const-parameter)
 DWORD GetDynamicTimeZoneInformationEffectiveYears(
     const PDYNAMIC_TIME_ZONE_INFORMATION lpTimeZoneInformation, LPDWORD FirstYear, LPDWORD LastYear)
+// NOLINTEND(readability-non-const-parameter)
 {
 	WINPR_UNUSED(lpTimeZoneInformation);
 	WINPR_UNUSED(FirstYear);
@@ -928,3 +913,32 @@ DWORD GetDynamicTimeZoneInformationEffectiveYears(
 	return ERROR_FILE_NOT_FOUND;
 }
 #endif
+
+#if !defined(_WIN32)
+char* setNewAndSaveOldTZ(const char* val)
+{
+	// NOLINTBEGIN(concurrency-mt-unsafe)
+	const char* otz = getenv("TZ");
+	char* oldtz = NULL;
+	if (otz)
+		oldtz = _strdup(otz);
+	setenv("TZ", val, 1);
+	tzset();
+	// NOLINTEND(concurrency-mt-unsafe)
+	return oldtz;
+}
+
+void restoreSavedTZ(char* saved)
+{
+	// NOLINTBEGIN(concurrency-mt-unsafe)
+	if (saved)
+	{
+		setenv("TZ", saved, 1);
+		free(saved);
+	}
+	else
+		unsetenv("TZ");
+	tzset();
+	// NOLINTEND(concurrency-mt-unsafe)
+}
+#endif
