@@ -19,8 +19,16 @@
  */
 
 #include <winpr/config.h>
+#include <winpr/platform.h>
+
+WINPR_PRAGMA_DIAG_PUSH
+WINPR_PRAGMA_DIAG_IGNORED_RESERVED_ID_MACRO
+WINPR_PRAGMA_DIAG_IGNORED_UNUSED_MACRO
+
+#define __STDC_WANT_LIB_EXT1__ 1 // NOLINT(bugprone-reserved-identifier,cert-dcl37-c,cert-dcl51-cpp)
+
+WINPR_PRAGMA_DIAG_POP
 
-#define __STDC_WANT_LIB_EXT1__ 1 // NOLINT(bugprone-reserved-identifier)
 #include <stdio.h>
 #include <string.h>
 #include <fcntl.h>
@@ -117,7 +125,7 @@ void* winpr_backtrace(DWORD size)
 	return winpr_win_backtrace(size);
 #else
 	LOGF(support_msg);
-	/* return a non NULL buffer to allow the backtrace function familiy to succeed without failing
+	/* return a non NULL buffer to allow the backtrace function family to succeed without failing
 	 */
 	return _strdup(support_msg);
 #endif
@@ -184,8 +192,8 @@ void winpr_backtrace_symbols_fd(void* buffer, int fd)
 			return;
 
 		for (size_t i = 0; i < used; i++)
-			_write(fd, lines[i], (unsigned)strnlen(lines[i], UINT32_MAX));
-		free(lines);
+			(void)_write(fd, lines[i], (unsigned)strnlen(lines[i], UINT32_MAX));
+		free((void*)lines);
 	}
 #else
 	LOGF(support_msg);
@@ -197,7 +205,7 @@ void winpr_log_backtrace(const char* tag, DWORD level, DWORD size)
 	winpr_log_backtrace_ex(WLog_Get(tag), level, size);
 }
 
-void winpr_log_backtrace_ex(wLog* log, DWORD level, DWORD size)
+void winpr_log_backtrace_ex(wLog* log, DWORD level, WINPR_ATTR_UNUSED DWORD size)
 {
 	size_t used = 0;
 	char** msg = NULL;
@@ -216,20 +224,20 @@ void winpr_log_backtrace_ex(wLog* log, DWORD level, DWORD size)
 		for (size_t x = 0; x < used; x++)
 			WLog_Print(log, level, "%" PRIuz ": %s", x, msg[x]);
 	}
-	free(msg);
+	free((void*)msg);
 
 fail:
 	winpr_backtrace_free(stack);
 }
 
-char* winpr_strerror(DWORD dw, char* dmsg, size_t size)
+char* winpr_strerror(INT32 dw, char* dmsg, size_t size)
 {
 #ifdef __STDC_LIB_EXT1__
-	strerror_s((int)dw, dmsg, size);
+	(void)strerror_s(dw, dmsg, size);
 #elif defined(WINPR_HAVE_STRERROR_R)
-	strerror_r((int)dw, dmsg, size);
+	(void)strerror_r(dw, dmsg, size);
 #else
-	(void)_snprintf(dmsg, size, "%s", strerror((int)dw));
+	(void)_snprintf(dmsg, size, "%s", strerror(dw));
 #endif
 	return dmsg;
 }
