@@ -26,6 +26,7 @@
 #include <winpr/wtypes.h>
 #include <winpr/crt.h>
 #include <winpr/file.h>
+#include <winpr/cast.h>
 
 #include <winpr/image.h>
 
@@ -49,6 +50,7 @@
 #endif
 #include <winpr/stream.h>
 
+#include "image.h"
 #include "../log.h"
 #define TAG WINPR_TAG("utils.image")
 
@@ -59,7 +61,7 @@ static SSIZE_T winpr_convert_from_png(const BYTE* comp_data, size_t comp_data_by
 static SSIZE_T winpr_convert_from_webp(const BYTE* comp_data, size_t comp_data_bytes, UINT32* width,
                                        UINT32* height, UINT32* bpp, BYTE** ppdecomp_data);
 
-static BOOL writeBitmapFileHeader(wStream* s, const WINPR_BITMAP_FILE_HEADER* bf)
+BOOL writeBitmapFileHeader(wStream* s, const WINPR_BITMAP_FILE_HEADER* bf)
 {
 	if (!Stream_EnsureRemainingCapacity(s, sizeof(WINPR_BITMAP_FILE_HEADER)))
 		return FALSE;
@@ -73,9 +75,14 @@ static BOOL writeBitmapFileHeader(wStream* s, const WINPR_BITMAP_FILE_HEADER* bf
 	return TRUE;
 }
 
-static BOOL readBitmapFileHeader(wStream* s, WINPR_BITMAP_FILE_HEADER* bf)
+BOOL readBitmapFileHeader(wStream* s, WINPR_BITMAP_FILE_HEADER* bf)
 {
-	if (!s || !bf || (!Stream_CheckAndLogRequiredLength(TAG, s, sizeof(WINPR_BITMAP_FILE_HEADER))))
+	static wLog* log = NULL;
+	if (!log)
+		log = WLog_Get(TAG);
+
+	if (!s || !bf ||
+	    (!Stream_CheckAndLogRequiredLengthWLog(log, s, sizeof(WINPR_BITMAP_FILE_HEADER))))
 		return FALSE;
 
 	Stream_Read_UINT8(s, bf->bfType[0]);
@@ -87,15 +94,22 @@ static BOOL readBitmapFileHeader(wStream* s, WINPR_BITMAP_FILE_HEADER* bf)
 
 	if (bf->bfSize < sizeof(WINPR_BITMAP_FILE_HEADER))
 	{
-		WLog_ERR(TAG, "");
+		WLog_Print(log, WLOG_ERROR, "Invalid bitmap::bfSize=%" PRIu32 ", require at least %" PRIuz,
+		           bf->bfSize, sizeof(WINPR_BITMAP_FILE_HEADER));
 		return FALSE;
 	}
 
-	return Stream_CheckAndLogRequiredCapacity(TAG, s,
-	                                          bf->bfSize - sizeof(WINPR_BITMAP_FILE_HEADER));
+	if ((bf->bfType[0] != 'B') || (bf->bfType[1] != 'M'))
+	{
+		WLog_Print(log, WLOG_ERROR, "Invalid bitmap header [%c%c], expected [BM]", bf->bfType[0],
+		           bf->bfType[1]);
+		return FALSE;
+	}
+	return Stream_CheckAndLogRequiredCapacityWLog(log, s,
+	                                              bf->bfSize - sizeof(WINPR_BITMAP_FILE_HEADER));
 }
 
-static BOOL writeBitmapInfoHeader(wStream* s, const WINPR_BITMAP_INFO_HEADER* bi)
+BOOL writeBitmapInfoHeader(wStream* s, const WINPR_BITMAP_INFO_HEADER* bi)
 {
 	if (!Stream_EnsureRemainingCapacity(s, sizeof(WINPR_BITMAP_INFO_HEADER)))
 		return FALSE;
@@ -114,7 +128,7 @@ static BOOL writeBitmapInfoHeader(wStream* s, const WINPR_BITMAP_INFO_HEADER* bi
 	return TRUE;
 }
 
-static BOOL readBitmapInfoHeader(wStream* s, WINPR_BITMAP_INFO_HEADER* bi, size_t* poffset)
+BOOL readBitmapInfoHeader(wStream* s, WINPR_BITMAP_INFO_HEADER* bi, size_t* poffset)
 {
 	if (!s || !bi || (!Stream_CheckAndLogRequiredLength(TAG, s, sizeof(WINPR_BITMAP_INFO_HEADER))))
 		return FALSE;
@@ -152,8 +166,9 @@ static BOOL readBitmapInfoHeader(wStream* s, WINPR_BITMAP_INFO_HEADER* bi, size_
 			}
 			if (bi->biSizeImage == 0)
 			{
-				UINT32 stride = ((((bi->biWidth * bi->biBitCount) + 31) & ~31) >> 3);
-				bi->biSizeImage = abs(bi->biHeight) * stride;
+				UINT32 stride = WINPR_ASSERTING_INT_CAST(
+				    uint32_t, ((((bi->biWidth * bi->biBitCount) + 31) & ~31) >> 3));
+				bi->biSizeImage = WINPR_ASSERTING_INT_CAST(uint32_t, abs(bi->biHeight)) * stride;
 			}
 			break;
 		case BI_BITFIELDS:
@@ -231,7 +246,7 @@ BYTE* winpr_bitmap_construct_header(size_t width, size_t height, size_t bpp)
 			offset += sizeof(DWORD) * 3; // 3 DWORD color masks
 			break;
 		default:
-			return FALSE;
+			return NULL;
 	}
 
 	if (!writeBitmapFileHeader(s, &bf))
@@ -254,17 +269,22 @@ fail:
  * Refer to "Compressed Image File Formats: JPEG, PNG, GIF, XBM, BMP" book
  */
 
-static void* winpr_bitmap_write_buffer(const BYTE* data, size_t size, UINT32 width, UINT32 height,
-                                       UINT32 stride, UINT32 bpp, UINT32* pSize)
+WINPR_ATTR_MALLOC(free, 1)
+static void* winpr_bitmap_write_buffer(const BYTE* data, WINPR_ATTR_UNUSED size_t size,
+                                       UINT32 width, UINT32 height, UINT32 stride, UINT32 bpp,
+                                       UINT32* pSize)
 {
 	WINPR_ASSERT(data || (size == 0));
 
 	void* result = NULL;
 	const size_t bpp_stride = 1ull * width * (bpp / 8);
+	if (bpp_stride > UINT32_MAX)
+		return NULL;
+
 	wStream* s = Stream_New(NULL, 1024);
 
 	if (stride == 0)
-		stride = bpp_stride;
+		stride = (UINT32)bpp_stride;
 
 	BYTE* bmp_header = winpr_bitmap_construct_header(width, height, bpp);
 	if (!bmp_header)
@@ -284,7 +304,10 @@ static void* winpr_bitmap_write_buffer(const BYTE* data, size_t size, UINT32 wid
 	}
 
 	result = Stream_Buffer(s);
-	*pSize = Stream_GetPosition(s);
+	const size_t pos = Stream_GetPosition(s);
+	if (pos > UINT32_MAX)
+		goto fail;
+	*pSize = (UINT32)pos;
 fail:
 	Stream_Free(s, result == NULL);
 	free(bmp_header);
@@ -302,14 +325,20 @@ int winpr_bitmap_write_ex(const char* filename, const BYTE* data, size_t stride,
 {
 	FILE* fp = NULL;
 	int ret = -1;
-	const size_t bpp_stride = ((((width * bpp) + 31) & ~31) >> 3);
+	void* bmpdata = NULL;
+	const size_t bpp_stride = ((((width * bpp) + 31) & (size_t)~31) >> 3);
+
+	if ((stride > UINT32_MAX) || (width > UINT32_MAX) || (height > UINT32_MAX) ||
+	    (bpp > UINT32_MAX))
+		goto fail;
 
 	if (stride == 0)
 		stride = bpp_stride;
 
 	UINT32 bmpsize = 0;
 	const size_t size = stride * 1ull * height;
-	void* bmpdata = winpr_bitmap_write_buffer(data, size, width, height, stride, bpp, &bmpsize);
+	bmpdata = winpr_bitmap_write_buffer(data, size, (UINT32)width, (UINT32)height, (UINT32)stride,
+	                                    (UINT32)bpp, &bmpsize);
 	if (!bmpdata)
 		goto fail;
 
@@ -323,6 +352,7 @@ int winpr_bitmap_write_ex(const char* filename, const BYTE* data, size_t stride,
 	if (fwrite(bmpdata, bmpsize, 1, fp) != 1)
 		goto fail;
 
+	ret = 0;
 fail:
 	if (fp)
 		(void)fclose(fp);
@@ -352,7 +382,7 @@ fail:
 int winpr_image_write(wImage* image, const char* filename)
 {
 	WINPR_ASSERT(image);
-	return winpr_image_write_ex(image, image->type, filename);
+	return winpr_image_write_ex(image, WINPR_ASSERTING_INT_CAST(uint32_t, image->type), filename);
 }
 
 int winpr_image_write_ex(wImage* image, UINT32 format, const char* filename)
@@ -429,16 +459,19 @@ static int winpr_image_bitmap_read_buffer(wImage* image, const BYTE* buffer, siz
 	}
 
 	image->bitsPerPixel = bi.biBitCount;
-	image->bytesPerPixel = (image->bitsPerPixel / 8);
-	const size_t bpp = (bi.biBitCount + 7) / 8;
-	image->scanline = bi.biWidth * bpp;
-	const size_t bmpsize = 1ull * image->scanline * image->height;
+	image->bytesPerPixel = (image->bitsPerPixel / 8UL);
+	const size_t bpp = (bi.biBitCount + 7UL) / 8UL;
+	image->scanline =
+	    WINPR_ASSERTING_INT_CAST(uint32_t, bi.biWidth) * WINPR_ASSERTING_INT_CAST(uint32_t, bpp);
+	const size_t bmpsize = 1ULL * image->scanline * image->height;
 	if (bmpsize != bi.biSizeImage)
 		WLog_WARN(TAG, "bmpsize=%" PRIuz " != bi.biSizeImage=%" PRIu32, bmpsize, bi.biSizeImage);
 	if (bi.biSizeImage < bmpsize)
 		goto fail;
 
-	image->data = (BYTE*)malloc(bi.biSizeImage);
+	image->data = NULL;
+	if (bi.biSizeImage > 0)
+		image->data = (BYTE*)malloc(bi.biSizeImage);
 
 	if (!image->data)
 		goto fail;
@@ -580,8 +613,10 @@ void winpr_image_free(wImage* image, BOOL bFreeBuffer)
 	free(image);
 }
 
-static void* winpr_convert_to_jpeg(const void* data, size_t size, UINT32 width, UINT32 height,
-                                   UINT32 stride, UINT32 bpp, UINT32* pSize)
+static void* winpr_convert_to_jpeg(WINPR_ATTR_UNUSED const void* data,
+                                   WINPR_ATTR_UNUSED size_t size, WINPR_ATTR_UNUSED UINT32 width,
+                                   WINPR_ATTR_UNUSED UINT32 height, WINPR_ATTR_UNUSED UINT32 stride,
+                                   WINPR_ATTR_UNUSED UINT32 bpp, WINPR_ATTR_UNUSED UINT32* pSize)
 {
 	WINPR_ASSERT(data || (size == 0));
 	WINPR_ASSERT(pSize);
@@ -612,7 +647,8 @@ static void* winpr_convert_to_jpeg(const void* data, size_t size, UINT32 width,
 
 	cinfo.image_width = width;
 	cinfo.image_height = height;
-	cinfo.input_components = (bpp + 7) / 8;
+	WINPR_ASSERT(bpp <= INT32_MAX / 8);
+	cinfo.input_components = (int)(bpp + 7) / 8;
 	cinfo.in_color_space = (bpp > 24) ? JCS_EXT_BGRA : JCS_EXT_BGR;
 	cinfo.data_precision = 8;
 
@@ -626,11 +662,12 @@ static void* winpr_convert_to_jpeg(const void* data, size_t size, UINT32 width,
 	const JSAMPLE* cdata = data;
 	for (size_t x = 0; x < height; x++)
 	{
-		const JDIMENSION offset = x * stride;
+		WINPR_ASSERT(x * stride <= UINT32_MAX);
+		const JDIMENSION offset = (JDIMENSION)x * stride;
 
 		/* libjpeg is not const correct, we must cast here to avoid issues
 		 * with newer C compilers type check errors */
-		JSAMPLE* coffset = (JSAMPLE*)&cdata[offset];
+		JSAMPLE* coffset = WINPR_CAST_CONST_PTR_AWAY(&cdata[offset], JSAMPLE*);
 		if (jpeg_write_scanlines(&cinfo, &coffset, 1) != 1)
 			goto fail;
 	}
@@ -639,13 +676,19 @@ fail:
 	jpeg_finish_compress(&cinfo);
 	jpeg_destroy_compress(&cinfo);
 
-	*pSize = outsize;
+	WINPR_ASSERT(outsize <= UINT32_MAX);
+	*pSize = (UINT32)outsize;
 	return outbuffer;
 #endif
 }
 
-SSIZE_T winpr_convert_from_jpeg(const BYTE* comp_data, size_t comp_data_bytes, UINT32* width,
-                                UINT32* height, UINT32* bpp, BYTE** ppdecomp_data)
+// NOLINTBEGIN(readability-non-const-parameter)
+SSIZE_T winpr_convert_from_jpeg(WINPR_ATTR_UNUSED const BYTE* comp_data,
+                                WINPR_ATTR_UNUSED size_t comp_data_bytes,
+                                WINPR_ATTR_UNUSED UINT32* width, WINPR_ATTR_UNUSED UINT32* height,
+                                WINPR_ATTR_UNUSED UINT32* bpp,
+                                WINPR_ATTR_UNUSED BYTE** ppdecomp_data)
+// NOLINTEND(readability-non-const-parameter)
 {
 	WINPR_ASSERT(comp_data || (comp_data_bytes == 0));
 	WINPR_ASSERT(width);
@@ -670,14 +713,18 @@ SSIZE_T winpr_convert_from_jpeg(const BYTE* comp_data, size_t comp_data_bytes, U
 
 	cinfo.out_color_space = cinfo.num_components > 3 ? JCS_EXT_RGBA : JCS_EXT_BGR;
 
-	*width = cinfo.image_width;
-	*height = cinfo.image_height;
-	*bpp = cinfo.num_components * 8;
+	*width = WINPR_ASSERTING_INT_CAST(uint32_t, cinfo.image_width);
+	*height = WINPR_ASSERTING_INT_CAST(uint32_t, cinfo.image_height);
+	*bpp = WINPR_ASSERTING_INT_CAST(uint32_t, cinfo.num_components * 8);
 
 	if (!jpeg_start_decompress(&cinfo))
 		goto fail;
 
-	size_t stride = 1ULL * cinfo.image_width * cinfo.num_components;
+	size_t stride =
+	    1ULL * cinfo.image_width * WINPR_ASSERTING_INT_CAST(uint32_t, cinfo.num_components);
+
+	if ((stride == 0) || (cinfo.image_height == 0))
+		goto fail;
 
 	decomp_data = calloc(stride, cinfo.image_height);
 	if (decomp_data)
@@ -701,8 +748,10 @@ fail:
 #endif
 }
 
-static void* winpr_convert_to_webp(const void* data, size_t size, UINT32 width, UINT32 height,
-                                   UINT32 stride, UINT32 bpp, UINT32* pSize)
+static void* winpr_convert_to_webp(WINPR_ATTR_UNUSED const void* data,
+                                   WINPR_ATTR_UNUSED size_t size, WINPR_ATTR_UNUSED UINT32 width,
+                                   WINPR_ATTR_UNUSED UINT32 height, WINPR_ATTR_UNUSED UINT32 stride,
+                                   WINPR_ATTR_UNUSED UINT32 bpp, UINT32* pSize)
 {
 	WINPR_ASSERT(data || (size == 0));
 	WINPR_ASSERT(pSize);
@@ -733,14 +782,17 @@ static void* winpr_convert_to_webp(const void* data, size_t size, UINT32 width,
 	if (rc)
 	{
 		memcpy(rc, pDstData, dstSize);
-		*pSize = dstSize;
+
+		WINPR_ASSERT(dstSize <= UINT32_MAX);
+		*pSize = (UINT32)dstSize;
 	}
 	WebPFree(pDstData);
 	return rc;
 #endif
 }
 
-SSIZE_T winpr_convert_from_webp(const BYTE* comp_data, size_t comp_data_bytes, UINT32* width,
+SSIZE_T winpr_convert_from_webp(WINPR_ATTR_UNUSED const BYTE* comp_data,
+                                WINPR_ATTR_UNUSED size_t comp_data_bytes, UINT32* width,
                                 UINT32* height, UINT32* bpp, BYTE** ppdecomp_data)
 {
 	WINPR_ASSERT(comp_data || (comp_data_bytes == 0));
@@ -766,8 +818,8 @@ SSIZE_T winpr_convert_from_webp(const BYTE* comp_data, size_t comp_data_bytes, U
 		return -1;
 	}
 
-	*width = w;
-	*height = h;
+	*width = WINPR_ASSERTING_INT_CAST(uint32_t, w);
+	*height = WINPR_ASSERTING_INT_CAST(uint32_t, h);
 	*bpp = 32;
 	*ppdecomp_data = dst;
 	return 4ll * w * h;
@@ -790,7 +842,11 @@ static void png_write_data(png_structp png_ptr, png_bytep data, png_size_t lengt
 
 	/* allocate or grow buffer */
 	if (p->buffer)
-		p->buffer = realloc(p->buffer, nsize);
+	{
+		char* tmp = realloc(p->buffer, nsize);
+		if (tmp)
+			p->buffer = tmp;
+	}
 	else
 		p->buffer = malloc(nsize);
 
@@ -803,7 +859,7 @@ static void png_write_data(png_structp png_ptr, png_bytep data, png_size_t lengt
 }
 
 /* This is optional but included to show how png_set_write_fn() is called */
-static void png_flush(png_structp png_ptr)
+static void png_flush(WINPR_ATTR_UNUSED png_structp png_ptr)
 {
 }
 
@@ -855,7 +911,7 @@ static SSIZE_T save_png_to_buffer(UINT32 bpp, UINT32 width, UINT32 height, const
 	             PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
 
 	/* Initialize rows of PNG. */
-	row_pointers = png_malloc(png_ptr, height * sizeof(png_byte*));
+	row_pointers = (png_byte**)png_malloc(png_ptr, height * sizeof(png_byte*));
 	for (size_t y = 0; y < height; ++y)
 	{
 		uint8_t* row = png_malloc(png_ptr, sizeof(uint8_t) * bytes_per_row);
@@ -881,10 +937,12 @@ static SSIZE_T save_png_to_buffer(UINT32 bpp, UINT32 width, UINT32 height, const
 	/* Cleanup. */
 	for (size_t y = 0; y < height; y++)
 		png_free(png_ptr, row_pointers[y]);
-	png_free(png_ptr, row_pointers);
+	png_free(png_ptr, (void*)row_pointers);
 
 	/* Finish writing. */
-	rc = state.size;
+	if (state.size > SSIZE_MAX)
+		goto fail;
+	rc = (SSIZE_T)state.size;
 	*pDstData = state.buffer;
 fail:
 	png_destroy_write_struct(&png_ptr, &info_ptr);
@@ -925,6 +983,9 @@ static void* winpr_read_png_from_buffer(const void* data, size_t SrcSize, size_t
 	MEMORY_READER_STATE memory_reader_state = { 0 };
 	png_bytepp row_pointers = NULL;
 	png_infop info_ptr = NULL;
+	if (SrcSize > UINT32_MAX)
+		return NULL;
+
 	png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
 	if (!png_ptr)
 		goto fail;
@@ -932,8 +993,8 @@ static void* winpr_read_png_from_buffer(const void* data, size_t SrcSize, size_t
 	if (!info_ptr)
 		goto fail;
 
-	memory_reader_state.buffer = (png_bytep)data;
-	memory_reader_state.bufsize = SrcSize;
+	memory_reader_state.buffer = WINPR_CAST_CONST_PTR_AWAY(data, png_bytep);
+	memory_reader_state.bufsize = (UINT32)SrcSize;
 	memory_reader_state.current_pos = 0;
 
 	png_set_read_fn(png_ptr, &memory_reader_state, read_data_memory);
@@ -961,7 +1022,7 @@ static void* winpr_read_png_from_buffer(const void* data, size_t SrcSize, size_t
 		if (rc)
 		{
 			char* cur = rc;
-			for (int i = 0; i < height; i++)
+			for (png_uint_32 i = 0; i < height; i++)
 			{
 				memcpy(cur, row_pointers[i], copybytes);
 				cur += stride;
@@ -969,7 +1030,8 @@ static void* winpr_read_png_from_buffer(const void* data, size_t SrcSize, size_t
 			*pSize = size;
 			*pWidth = width;
 			*pHeight = height;
-			*pBpp = bpp;
+			WINPR_ASSERT(bpp <= UINT32_MAX);
+			*pBpp = (UINT32)bpp;
 		}
 	}
 fail:
@@ -979,8 +1041,10 @@ fail:
 }
 #endif
 
-static void* winpr_convert_to_png(const void* data, size_t size, UINT32 width, UINT32 height,
-                                  UINT32 stride, UINT32 bpp, UINT32* pSize)
+static void* winpr_convert_to_png(WINPR_ATTR_UNUSED const void* data, WINPR_ATTR_UNUSED size_t size,
+                                  WINPR_ATTR_UNUSED UINT32 width, WINPR_ATTR_UNUSED UINT32 height,
+                                  WINPR_ATTR_UNUSED UINT32 stride, WINPR_ATTR_UNUSED UINT32 bpp,
+                                  UINT32* pSize)
 {
 	WINPR_ASSERT(data || (size == 0));
 	WINPR_ASSERT(pSize);
@@ -1021,8 +1085,11 @@ static void* winpr_convert_to_png(const void* data, size_t size, UINT32 width, U
 #endif
 }
 
-SSIZE_T winpr_convert_from_png(const BYTE* comp_data, size_t comp_data_bytes, UINT32* width,
-                               UINT32* height, UINT32* bpp, BYTE** ppdecomp_data)
+SSIZE_T winpr_convert_from_png(WINPR_ATTR_UNUSED const BYTE* comp_data,
+                               WINPR_ATTR_UNUSED size_t comp_data_bytes,
+                               WINPR_ATTR_UNUSED UINT32* width, WINPR_ATTR_UNUSED UINT32* height,
+                               WINPR_ATTR_UNUSED UINT32* bpp,
+                               WINPR_ATTR_UNUSED BYTE** ppdecomp_data)
 {
 #if defined(WINPR_UTILS_IMAGE_PNG)
 	size_t len = 0;
@@ -1060,7 +1127,7 @@ BOOL winpr_image_format_is_supported(UINT32 format)
 	}
 }
 
-static BYTE* convert(const wImage* image, size_t* pstride, UINT32 flags)
+static BYTE* convert(const wImage* image, size_t* pstride, WINPR_ATTR_UNUSED UINT32 flags)
 {
 	WINPR_ASSERT(image);
 	WINPR_ASSERT(pstride);
